package org.climax.demo;

import java.io.FileReader;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.climax.ArgGroupComplete;
import org.climax.ArgParser;
import org.climax.ArgParserException;
import org.climax.CLI;
import org.climax.Option;
import org.climax.OptionGroupComplete;
import org.climax.OptionType;
import org.climax.argparser.IntegerArgParser;
import org.climax.argparser.ParameterlessArgParser;
import org.climax.argsvalidator.NoneOrSingleArgValidator;
import org.climax.reflect.ArgGroupCompleteSpec;
import org.climax.reflect.CLIInterface;
import org.climax.reflect.CLISpec;
import org.climax.reflect.OptionCompleteSpec;
import org.climax.reflect.OptionGroupCompleteSpec;

/**
 * This demo provides a command line interface exactly like GNU base64's 
 * command line interface and prints the results to standard output. The 
 * purpose of this demo is to provide an example of how Climax can be used to 
 * customize the help information.
 */
public final class Base64 {
	
	/**
	 * A {@code CLI} interface for the command line interface. 
	 */
	@CLISpec(
		helpOptionGroupDocDescription = "display this help and exit",
		helpOptionGroupDocHelp = 
			"      ${format:%-9s;p=usage}  ${description}",
		helpOptionGroupDocOrdinal = 3,
		optionsDocUsage = "[OPTION]...",
		programDescription = 
			"Base64 encode or decode ${argGroupDoc0.argumentType}"
			+ ", or standard input, to standard output.",	
		programHelpBody =
			"Usage: ${programUsage}\n"
			+ "${programDescription}\n\n"
			+ "${optionsDoc.help}\n\n"
			+ "${format:With no %1$s, or when %1$s is -," 
			+ " read standard input." 
			+ ";p=argGroupDoc0.argumentType}",
		programVersion = "${programName}\n" 
			+ "Command line interface provided by" 
			+ " ${org.climax.specTitle} ${org.climax.specVersion}",
		versionOptionGroupDocDescription = 
			"output version information and exit",
		versionOptionGroupDocHelp = 
			"      ${format:%-9s;p=usage}  ${description}",
		versionOptionGroupDocOrdinal = 4
	)
	interface base64 extends CLI {

		/**
		 * Returns the {@code OptionGroupComplete} for the options -d and 
		 * --decode.
		 * 
		 * @return the {@code OptionGroupComplete} for the options -d and 
		 * --decode
		 */
		@OptionGroupCompleteSpec(
			description = "decode data",
			help = "  ${format:%-20s;p=usage}  ${description}",
			options = { 
				@OptionCompleteSpec(optionType = OptionType.POSIX),
				@OptionCompleteSpec(optionType = OptionType.GNU_LONG) 
			},
			ordinal = 0,
			usage = "${optionDoc0.option}, ${optionDoc1.usage}"
		)
		OptionGroupComplete<Void> getDecode();

		/**
		 * Returns the {@code ArgGroupComplete} for the file argument group.
		 * 
		 * @return the {@code ArgGroupComplete} for the file argument group
		 */
		@ArgGroupCompleteSpec(
			argsValidatorClass = NoneOrSingleArgValidator.class,
			defaultArguments = { "-" }
		)
		ArgGroupComplete<FileReader> getFile();

		/**
		 * Returns the {@code OptionGroupComplete} for the options -i and 
		 * --ignore-garbage.
		 * 
		 * @return the {@code OptionGroupComplete} for the options -i and 
		 * --ignore-garbage
		 */
		@OptionGroupCompleteSpec(
			description = "when decoding, ignore non-alphabet characters",
			help = "  ${format:%-20s;p=usage}  ${description}",
			options = { 
				@OptionCompleteSpec(optionType = OptionType.POSIX),
				@OptionCompleteSpec(optionType = OptionType.GNU_LONG) 
			},
			ordinal = 1,
			usage = "${optionDoc0.option}, ${optionDoc1.usage}"
		)	
		OptionGroupComplete<Void> getIgnoreGarbage();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the options -w and 
		 * --wrap.
		 * 
		 * @return the {@code OptionGroupComplete} for the options -w and 
		 * --wrap
		 */
		@OptionGroupCompleteSpec(
			argParserClass = ColumnLimitArgParser.class,
			defaultArguments = { "76" },
			description = 
				"wrap encoded lines after COLS character (default 76).",
			help = "  ${format:%-20s;p=usage}  ${description}\n"
				+ "${format:%-22s;v= }     Use 0 to disable line wrapping\n",
			options = { 
				@OptionCompleteSpec(
					optionType = OptionType.POSIX,
					usage = "${option} COLS" 
				),
				@OptionCompleteSpec(
					optionType = OptionType.GNU_LONG,
					usage = "${option}${optionTypeDoc.assignmentOperator}COLS" 
				) 
			},
			ordinal = 2,
			usage = "${optionDoc0.option}, ${optionDoc1.usage}"
		)
		OptionGroupComplete<Integer> getWrap();
		
	}
	
	/**
	 * A {@code ParameterlessArgParser} implementation that parses arguments 
	 * as {@code Integer}s for the options -w and --wrap.
	 */
	static class ColumnLimitArgParser 
		implements ParameterlessArgParser<Integer> {

		/** The {@code ArgParser}. */
		private final ArgParser<Integer> argParser;
		
		/**
		 * Constructs a {@code ColumnLimitArgParser}.
		 */
		public ColumnLimitArgParser() { 
			
			this.argParser = new IntegerArgParser(
					Integer.valueOf(0),
					Integer.valueOf(Integer.MAX_VALUE));
		}
		
		@Override
		public Integer parse(final String arg) throws ArgParserException {
			return this.argParser.parse(arg);
		}
	}	

	/** A table for display as a {@code String}. */
	private static class Table {
		
		/**
		 * Appends the provided {@code String} multiple times based on the 
		 * provided integer indicating how many times to append.
		 *   
		 * @param string the provided {@code String}
		 * 
		 * @param times the provided integer indicating how many times to
		 * append
		 * 
		 * @return the appended String
		 */
		private static String append(final String string, final int times) {
			StringBuilder stringBuilder = new StringBuilder();
			for (int i = 0; i < times; i++) {
				stringBuilder.append(string);
			}
			return stringBuilder.toString();
		}
		
		/**
		 * Returns an {@code int} array of the maximum width of each column 
		 * based on the provided two-dimensional {@code Object} array of the 
		 * content.
		 * 
		 * @param cells the provided two-dimensional {@code Object} array of 
		 * the content
		 * 
		 * @return an {@code int} array of the maximum width of each column
		 */
		private static int[] getMaxColumnWidths(final Object[][] cells) {

			int maxColumnCount = 0;
			
			for (Object[] row : cells) {
				
				if (row.length > maxColumnCount) {
					maxColumnCount = row.length;
				}
			}
			
			int[] maxColumnWidths = new int[maxColumnCount];
			
			Arrays.fill(maxColumnWidths, 0);
			
			for (Object[] row : cells) {
				
				for (int i = 0; i < row.length; i++) {
					
					if (i < maxColumnWidths.length) {
						
						Object cell = row[i];
						
						int cellWidth = 0;
						
						if (cell != null) {
							
							cellWidth = cell.toString().length(); 
						}
						
						int maxColumnWidth = maxColumnWidths[i];
						
						if (cellWidth > maxColumnWidth) {
							
							maxColumnWidths[i] = cellWidth;
						}
					}
				}
			}
			
			return maxColumnWidths;
		}
		
		/**
		 * Creates a row based on the provided {@code Object} array of the 
		 * content and the provided {@code int} array of the maximum width of 
		 * each column.
		 * 
		 * @param row the provided {@code Object} array of the content
		 * 
		 * @param maxColumnWidths the provided {@code int} array of the maximum
		 * width of each column
		 * 
		 * @return a new row
		 */
		private static String newRow(
				final Object[] row, final int[] maxColumnWidths) {
			
			StringBuilder rowBuilder = new StringBuilder();
			
			rowBuilder.append('|');
			
			for (int i = 0; i < maxColumnWidths.length; i++) {
				
				int columnWidth = maxColumnWidths[i];
				
				if (i < row.length) {
					
					Object cell = row[i];
					
					if (cell != null) {
						
						rowBuilder.append(cell);
						
						int remainingSpaceLength = 
								columnWidth - cell.toString().length();
						
						if (remainingSpaceLength > 0) {
							
							rowBuilder.append(
									append(" ", remainingSpaceLength));
						}
						
					} else {
						
						rowBuilder.append(append(" ", columnWidth));
					}
					
				} else {
					
					rowBuilder.append(append(" ", columnWidth));
				}
				
				if (i < maxColumnWidths.length - 1) {
					
					rowBuilder.append('|');
				}
			}
			
			rowBuilder.append('|');
			
			return rowBuilder.toString();
		}
		
		/**
		 * Creates a new row separator based on the provided {@code int} array 
		 * of the maximum width of each column.
		 *  
		 * @param maxColumnWidths the provided {@code int} array of the maximum
		 * width of each column
		 * 
		 * @return a new row separator
		 */
		private static String newRowSeparator(final int[] maxColumnWidths) {
			
			StringBuilder rowSeparatorBuilder = new StringBuilder();
			
			rowSeparatorBuilder.append('+');
			
			for (int i = 0; i < maxColumnWidths.length; i++) {
				
				int maxColumnWidth = maxColumnWidths[i];
				
				rowSeparatorBuilder.append(append("-", maxColumnWidth));
				
				if (i < maxColumnWidths.length - 1) {
					
					rowSeparatorBuilder.append('+');
				}
			}
			
			rowSeparatorBuilder.append('+');
			
			return rowSeparatorBuilder.toString();
		}
		
		/** The {@code String} representation of this {@code Table}. */
		private final String string;
		
		/**
		 * Constructs a {@code Table} based on the provided 
		 * two-dimensional {@code Object} array of the content.
		 * 
		 * @param cells the provided two-dimensional {@code Object} array of 
		 * the content
		 */
		public Table(final Object[][] cells) {
			
			int[] maxColumnWidths = getMaxColumnWidths(cells);
			
			String rowSeparator = newRowSeparator(maxColumnWidths);
			
			StringBuilder tableBuilder = new StringBuilder();

			if (cells.length > 0) {
				
				tableBuilder.append(rowSeparator).append('\n');
				
				for (int i = 0; i < cells.length; i++) {
					
					Object[] row = cells[i];
										
					tableBuilder.append(
							newRow(row, maxColumnWidths)).append('\n');
					
					if (i < cells.length - 1) {
						
						tableBuilder.append(rowSeparator).append('\n');
					}
				}
				
				tableBuilder.append(rowSeparator);
			}
			
			this.string = tableBuilder.toString();
		}
		
		@Override
		public String toString() {
			return this.string;
		}
	}	

	/**
	 * Processes the supplied command line arguments and prints the results 
	 * to standard output.
	 * 
	 * @param args the supplied command line arguments
	 */
	public static void main(final String[] args) {

		CLIInterface<base64> c = CLIInterface.getInstance(base64.class);
		
		base64 b = c.newCLI();
		
		switch(b.process(args, System.out, System.err)) {
		
		case ERROR:
			System.exit(1);
			
		case HELP_REQUESTED:
			return;
			
		case VERSION_REQUESTED:
			return;
			
		default:
			break;
		}
		
		Object[][] cells = new Object[][] {
				new Object[] {
					"OptionGroupComplete/ArgGroupComplete",
					"hasParsedOptionSelection/hasParsedArgument",
					"parsedArguments"
				},
				toTableRow(b.getDecode()),
				toTableRow(b.getIgnoreGarbage()),
				toTableRow(b.getWrap()),
				new Object[] {
					b.getFile().getArgumentType(),
					b.getFile().hasParsedArgument(),
					b.getFile().getParsedArguments()
				}
		};
		
		System.out.println(new Table(cells));
	}
	
	/**
	 * Returns a {@code String} representation of a provided {@code List} of 
	 * {@code Option}s.
	 * 
	 * @param optionList the provided {@code List} of {@code Option}s.
	 * 
	 * @return a {@code String} representation of a provided {@code List} of 
	 * {@code Option}s
	 */
	private static String toString(final List<Option> optionList) {
		StringBuilder builder = new StringBuilder();
		for (Iterator<Option> it = optionList.iterator(); it.hasNext();) {
			Option option = it.next();
			builder.append(option.toOptionString());
			if (it.hasNext()) {
				builder.append(", ");
			}
		}
		return builder.toString();
	}
	
	/**
	 * Returns a {@code Object} array as a table row of the provided 
	 * {@code OptionGroupComplete}.
	 * 
	 * @param optionGroupComplete the provided {@code OptionGroupComplete}
	 * 
	 * @return the arguments
	 */
	private static Object[] toTableRow(
			final OptionGroupComplete<?> optionGroupComplete) {
		
		return new Object[] { 
				toString(optionGroupComplete.getOptions()),
				optionGroupComplete.hasParsedOptionSelection(),
				optionGroupComplete.getParsedArguments()
		};
	}

	/**
	 * Constructs an instance of {@code Base64}. This constructor is 
	 * suppressed for noninstantiability outside this class.
	 */
	private Base64() {
	}
}

