package org.climax.demo;

import java.io.CharConversionException;
import java.util.List;

import org.climax.ArgGroupComplete;
import org.climax.CLI;
import org.climax.OptionGroupComplete;
import org.climax.OptionType;
import org.climax.argsvalidator.PassiveArgsValidator;
import org.climax.reflect.ArgGroupCompleteSpec;
import org.climax.reflect.CLIInterface;
import org.climax.reflect.CLISpec;
import org.climax.reflect.OptionCompleteSpec;
import org.climax.reflect.OptionGroupCompleteSpec;

/**
 * This demo provides a command line interface almost exact as GNU echo's 
 * command line interface and prints the provided strings to standard output. 
 * Unlike GNU echo, this implementation of echo recognizes the sequence 
 * <code>{@literal \}uHHHHHH</code> where {@code HHHHHH} is the Unicode code 
 * point of a character. The purpose of this demo is to provide an example of 
 * how Climax can be used to customize the help information.
 */
public final class Echo {
	
	/**
	 * A {@code CLI} interface for the command line interface. 
	 */	
	@CLISpec(
		helpOptionGroupDocDescription = "display this help and exit",
		helpOptionGroupDocHelp = "      ${format:%-10s;p=usage} ${description}",
		programDescription = 
			"Echo the ${argGroupDoc0.argumentType}(s) to standard output.",
		programHelpBody = 
			"Usage: ${programUsage}\n"
			+ "${programDescription}\n\n"
			+ "${optionsDoc.help}\n\n",
		programHelpFooter = 
			"If ${optionDoc1.option} is in effect, the following sequences are "
			+ "recognized:\n\n"
			+ "  \\\\       backslash\n"
			+ "  \\a       alert (BEL)\n"
			+ "  \\b       backspace\n"
			+ "  \\c       produce no further output\n"
			+ "  \\f       form feed\n"
			+ "  \\n       new line\n"
			+ "  \\r       carriage return\n"
			+ "  \\t       horizontal tab\n"
			+ "  \\uHHHHHH the character whose Unicode is HHHHHH (hexadecimal)" 
			+ "\n"
			+ "  \\v       vertical tab\n",
		programUsage = "${programName}"
			+ " [SHORT-OPTION]... ${argsDoc.syntax}\n"
			+ "  or:  ${programName} LONG-OPTION",
		programVersion = "${programName}" 
			+ " (${org.climax.demo.specTitle}"  
			+ " ${org.climax.demo.specVersion})",
		versionOptionGroupDocDescription = 
			"output version information and exit",
		versionOptionGroupDocHelp = 
			"      ${format:%-10s;p=usage} ${description}"
	)
	private interface echo extends CLI {

		/**
		 * Returns the {@code OptionGroupComplete} for the option -E.
		 * 
		 * @return the {@code OptionGroupComplete} for the option -E
		 */
		@OptionGroupCompleteSpec(
			description = 
				"disable interpretation of backslash escapes (default)",
			help = "  ${format:%-14s;p=usage} ${description}",
			options = { 
				@OptionCompleteSpec(name = "E", optionType = OptionType.POSIX) 
			},
			ordinal = 2
		) 
		OptionGroupComplete<Void> disableBackslashEscapes();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option -e.
		 * 
		 * @return the {@code OptionGroupComplete} for the option -e
		 */
		@OptionGroupCompleteSpec(
			description = "enable interpretation of backslash escapes",
			help = "  ${format:%-14s;p=usage} ${description}",
			options = { 
				@OptionCompleteSpec(name = "e", optionType = OptionType.POSIX) 
			},
			ordinal = 1			
		) 
		OptionGroupComplete<Void> enableBackslashEscapes();
		
		/**
		 * Returns the {@code ArgGroupComplete} for the string argument 
		 * group.
		 * 
		 * @return the {@code ArgGroupComplete} for the string argument 
		 * group
		 */
		@ArgGroupCompleteSpec(
			argsValidatorClass = PassiveArgsValidator.class,
			syntax = "[${argumentType}]..."
		)
		ArgGroupComplete<String> getString();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option -n.
		 * 
		 * @return the {@code OptionGroupComplete} for the option -n
		 */
		@OptionGroupCompleteSpec(
			description = "do not output the trailing new line",
			help = "  ${format:%-14s;p=usage} ${description}",
			options = { 
				@OptionCompleteSpec(name = "n", optionType = OptionType.POSIX) 
			},
			ordinal = 0
		) 
		OptionGroupComplete<Void> removeTrailingNewline();
	}
	
	/**
	 * Interprets the backslash escapes within the provided 
	 * {@code StringBuilder}.
	 * 
	 * @param builder the provided {@code StringBuilder}
	 *  
	 * @throws CharConversionException if the Unicode code point for the 
	 * backslash escape <code>{@literal \}uHHHHHH</code> is invalid
	 */
	private static void interpretBackslashEscapes(
			final StringBuilder builder) throws CharConversionException {
		
		for (int i = 0; i < builder.length(); i++) {
			
			char ch = builder.charAt(i);
			
			if (ch == '\\' && i < builder.length() - 1) {
				
				char nextChar = builder.charAt(i + 1);
				
				switch(nextChar) {
				
				case '\\':
					builder.replace(i, i + 2, "\\");
					break;
					
				case 'a':
					builder.replace(i, i + 2, "\u0007");
					break;
					
				case 'b':
					builder.replace(i, i + 2, "\b");
					break;
					
				case 'c':
					builder.replace(i, builder.length(), "");
					break;
					
				case 'f':
					builder.replace(i, i + 2, "\f");
					break;
					
				case 'n':
					builder.replace(i, i + 2, "\n");
					break;
					
				case 'r':
					builder.replace(i, i + 2, "\r");
					break;
					
				case 't':
					builder.replace(i, i + 2, "\t");
					break;
					
				case 'u':
					
					StringBuilder b = new StringBuilder();
					
					final int codePointLength = 6;
					
					for (int j = i + 2; 
						j < i + 2 + codePointLength && j < builder.length(); 
						j++) {
						
						char c = builder.charAt(j);

						if (c >= '0' && c <= '9') {
							
							b.append(c);
							
						} else if (c >= 'A' && c <= 'F') {
							
							b.append(c);
							
						} else if (c >= 'a' && c <= 'f') {
							
							b.append(c);
							
						} else {
							
							break;
						}
					}
					
					if (b.length() > 0) {
						
						final int radix = 16;
						
						int codePoint = Integer.parseInt(b.toString(), radix);
						
						char[] charArray = null;
						
						try {
							
							charArray = Character.toChars(codePoint);
							
						} catch (IllegalArgumentException ex) {
							
							String message = String.format(
									"invalid Unicode code point: "
									+ "%1$s : code point must be in " 
									+ "the range of %2$s to %3$s " 
									+ "inclusive",
									b.toString(),
									Integer.toString(
											Character.MIN_CODE_POINT, 
											radix),
									Integer.toString(
											Character.MAX_CODE_POINT, 
											radix));
							
							throw new CharConversionException(message);
						}
						
						String string = String.valueOf(charArray);

						builder.replace(i, i + 2 + b.length(), string);
						
						i += string.length() - 1;
						
					} else {
						
						builder.replace(i, i + 2, "");
						
						i--;
					}
					
					break;
					
				case 'v':
					builder.replace(i, i + 2, "\u000B");
					break;
					
				default:
					break;
				}
			}
		}
	}

	/**
	 * Prints from the supplied command line arguments the provided strings 
	 * to standard output.
	 *   
	 * @param args the supplied command line arguments
	 */
	public static void main(final String[] args) {
		
		CLIInterface<echo> c = CLIInterface.getInstance(echo.class);
		
		echo e = c.newCLI();
		
		switch(e.process(args, System.out, System.err)) {
		
		case ERROR:
			System.exit(1);
			
		case HELP_REQUESTED:
			return;
			
		case VERSION_REQUESTED:
			return;
			
		default:
			break;
		}
		
		String programName = e.getProgramName();

		boolean removeTrailingNewline = 
			e.removeTrailingNewline().hasParsedOptionSelection();
		boolean enableBackslashEscapes = 
			e.enableBackslashEscapes().hasParsedOptionSelection();
		boolean disableBackslashEscapes = 
			e.disableBackslashEscapes().hasParsedOptionSelection();
		List<String> parsedArgumentList = 
			e.getString().getParsedArguments();
		String[] tokens = 
			parsedArgumentList.toArray(new String[parsedArgumentList.size()]);
		
		StringBuilder builder = new StringBuilder();
		
		for (int i = 0; i < tokens.length; i++) {
			
			String token = tokens[i];
			
			builder.append(token);
			
			if (i < tokens.length - 1) {
			
				builder.append(" ");
			}
		}
		
		if (!removeTrailingNewline) {
			
			builder.append("\n");
		}
		
		if (enableBackslashEscapes && !disableBackslashEscapes) {
			
			try {

				Echo.interpretBackslashEscapes(builder);
				
			} catch (CharConversionException ex) {
				
				String message = String.format(
						"%1$s: %2$s", programName, ex.getMessage());
				
				System.err.println(message);
				
				System.exit(1);
			}
		}
		
		System.out.print(builder.toString());
	}

	/**
	 * Constructs an instance of {@code Echo}. This constructor is 
	 * suppressed for noninstantiability outside this class.
	 */
	private Echo() {
	}
}
