import org.climax.ArgGroupComplete;
import org.climax.ArgParserException;
import org.climax.CLI;
import org.climax.OptionGroupComplete;
import org.climax.OptionType;
import org.climax.argacceptor.UnlimitedArgAcceptor;
import org.climax.argparser.ParameterlessArgParser;
import org.climax.argsvalidator.SingleOrMultipleArgsValidator;
import org.climax.reflect.ArgGroupCompleteSpec;
import org.climax.reflect.CLIInterface;
import org.climax.reflect.CLISpec;
import org.climax.reflect.OptionCompleteSpec;
import org.climax.reflect.OptionGroupCompleteSpec;

/**
 * This demo prints to standard output greetings to the provided names. The 
 * purpose of this demo is to show the resulting source code based on the 
 * tutorial. 
 */
public final class Hello {

	/**
	 * A {@code CLI} interface for the command line interface. 
	 */
	@CLISpec(
		programDescription = 
			"Greets each provided ${argGroupDoc0.argumentType} " 
			+ "(and NICKNAME if provided)",
		programHelpBody = "Usage: ${programUsage}\n"
			+ "${programDescription}\n"
			+ "Example: " 
			+ "${programName} " 
			+ "${optionDoc3.option}${optionDoc3.optionTypeDoc.assignmentOperator}ES "
			+ "Lawrence:Larry Curly Moe\n\n"
			+ "OPTIONS:\n"
			+ "${optionsDoc.help}\n"
	)
	private interface hello extends CLI {
	
		/**
		 * Returns the {@code OptionGroupComplete} for the options -i 
		 * and --inline.
		 * 
		 * @return the {@code OptionGroupComplete} for the options -i 
		 * and --inline
		 */
		@OptionGroupCompleteSpec(
			description = "Greet all in one sentence",
			options = {
				@OptionCompleteSpec(optionType = OptionType.POSIX),
				@OptionCompleteSpec	
			}
		)
		OptionGroupComplete<Void> getInline();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the options -lang and 
		 * --language.
		 * 
		 * @return the {@code OptionGroupComplete} for the options -lang and 
		 * --language
		 */
		@OptionGroupCompleteSpec(
			defaultArguments = { "EN" },
			description = "Greet in a specified language (EN is the default)",
			options = {
				@OptionCompleteSpec(
					name = "lang", 
					optionType = OptionType.GNU_SHORT_LONG
				),
				@OptionCompleteSpec
			}
		)
		OptionGroupComplete<Language> getLanguage();
		
		/**
		 * Returns the {@code ArgGroupComplete} for the name argument group.
		 * 
		 * @return the {@code ArgGroupComplete} for the name argument group
		 */
		@ArgGroupCompleteSpec(
			argAcceptorClass = UnlimitedArgAcceptor.class,
			argParserClass = NameArgParser.class,
			argsValidatorClass = SingleOrMultipleArgsValidator.class,
			syntax = "${format:%1$s1[:%2$s1] [%1$s2[:%2$s2] [...]]"
				+ ";p=argumentType;v=NICKNAME}"
		)
		ArgGroupComplete<Name> getName();
	}
	
	/**
	 * A language type. The constants of this enumerated type provides a 
	 * classification of language types. 
	 */
	private static enum Language {
		
		/** English. */
		EN {
			@Override
			public String getGreeting() {
				return "Hello";
			}
		},
		
		/** Spanish. */
		ES {
			@Override
			public String getGreeting() {
				return "Hola";
			}
		},
		
		/** French. */
		FR {
			@Override
			public String getGreeting() {
				return "Bonjour";
			}
		};
		
		/**
		 * Returns the greeting of this {@code Language}.
		 * 
		 * @return the greeting of this {@code Language}
		 */
		public abstract String getGreeting();
	}

	/**
	 * An {@code Object} representation of a name and an optional nickname.
	 */
	private static final class Name {

		/** The name. */
		private final String name;
		
		/** The optional nickname. */
		private final String nickname;
		
		/** The {@code String} representation of this {@code Name}. */
		private final String string;

		/**
		 * Constructs a {@code Name} with the provided name and the optional 
		 * nickname.
		 * 
		 * @param n the provided name
		 * @param nn the optional nickname (can be {@code null})
		 */
		public Name(final String n, final String nn) {

			this.name = n;
			
			this.nickname = nn;
			
			String s = this.name;

			if (this.nickname != null && !this.nickname.isEmpty()) {
				s = s.concat(" (" + this.nickname + ")");
			}

			this.string = s;
		}
		
		@Override
		public String toString() {
			return this.string;
		}
	}

	/**
	 * A {@code ParameterlessArgParser} implementation that parses arguments as 
	 * {@code Name}s for the NAME argument group.
	 */
	private static final class NameArgParser 
		implements ParameterlessArgParser<Name> {
	
		/**
		 * Constructs a {@code NameArgParser}.
		 */
		// this constructor is used via reflection
		@SuppressWarnings("unused") 
		public NameArgParser() {
		}
	
		@Override
		public Name parse(final String arg) throws ArgParserException {
	
			String[] splitArg = arg.split(":", 2);
			
			String name = splitArg[0];
	
			if (name.isEmpty()) {
				throw new ArgParserException(
					arg, "must be either NAME or NAME:NICKNAME");
			}	
	
			String nickname = null;
	
			if (splitArg.length == 2) {
				nickname = splitArg[1];
			}
	
			return new Name(name, nickname);
		}
	}	
	
	/**
	 * Prints from the supplied command line arguments to standard output 
	 * greetings to the provided names.
	 * 
	 * @param args the supplied command line arguments
	 */
	public static void main(final String[] args) {

		CLIInterface<hello> c = CLIInterface.getInstance(hello.class);
		
		hello h = c.newCLI();
		
		switch(h.process(args, System.out, System.err)) {
		
		case ERROR:
			System.exit(1);
			
		case HELP_REQUESTED:
			return;
			
		default:
			break;
		}
		
		ArgGroupComplete<Name> name = h.getName();
		
		OptionGroupComplete<Void> inline = h.getInline();
		
		OptionGroupComplete<Language> language = h.getLanguage();
		
		String greeting = language.getParsedArguments().get(0).getGreeting();

		if (inline.hasParsedOptionSelection()) {
			
			System.out.print(greeting);

			for (Name n : name.getParsedArguments()) {
				
				System.out.print(" " + n);
			}
			
			System.out.print("\n");
			
		} else {
			
			for (Name n : name.getParsedArguments()) {
				
				System.out.println(greeting + " " + n);
			}			
		}
	}
	
	/**
	 * Constructs an instance of {@code Hello}. This constructor is 
	 * suppressed for noninstantiability outside this class.
	 */
	private Hello() {
	}
}
