import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.climax.ArgGroupComplete;
import org.climax.CLI;
import org.climax.Option;
import org.climax.OptionGroupComplete;
import org.climax.reflect.CLIInterface;

/**
 * This demo writes the results of the provided command line options to 
 * a specified file or to standard output. The purpose of this demo is to 
 * provide an example of how Climax provides the default display and behavior 
 * without the use of annotations.
 */
public final class Results {

	/**
	 * A decision type. The constants of this enumerated type provides a 
	 * classification of decision types. 
	 */
	private static enum Decision {
		
		/** Yes. */
		YES,
		
		/** No. */
		NO 
	}
	
	/**
	 * A {@code CLI} interface for the command line interface. 
	 */	
	private interface results extends CLI {
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --big-decimal.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --big-decimal
		 */
		OptionGroupComplete<BigDecimal> getBigDecimal();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --big-integer.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --big-integer
		 */
		OptionGroupComplete<BigInteger> getBigInteger();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --boolean.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --boolean
		 */
		OptionGroupComplete<Boolean> getBoolean();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --byte.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --byte
		 */
		OptionGroupComplete<Byte> getByte();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --character.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --character
		 */		
		OptionGroupComplete<Character> getCharacter();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --classname.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --classname
		 */
		OptionGroupComplete<Class<?>> getClassname();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --date.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --date
		 */		
		OptionGroupComplete<Date> getDate();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --decision.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --decision
		 */		
		OptionGroupComplete<Decision> getDecision();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --double.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --double
		 */		
		OptionGroupComplete<Double> getDouble();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --file.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --file
		 */		
		OptionGroupComplete<File> getFile();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --float.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --float
		 */		
		OptionGroupComplete<Float> getFloat();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --integer.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --integer
		 */		
		OptionGroupComplete<Integer> getInteger();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --long.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --long
		 */		
		OptionGroupComplete<Long> getLong();

		/**
		 * Returns the {@code ArgGroupComplete} for the output file argument 
		 * group.
		 * 
		 * @return the {@code ArgGroupComplete} for the output file argument 
		 * group
		 */		
		ArgGroupComplete<FileWriter> getOutputFile();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --short.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --short
		 */		
		OptionGroupComplete<Short> getShort();
		
		/**
		 * Returns the {@code OptionGroupComplete} for the option --string.
		 * 
		 * @return the {@code OptionGroupComplete} for the option --string
		 */		
		OptionGroupComplete<String> getString();
	}

	/** A table for display as a {@code String}. */
	private static class Table {
		
		/**
		 * Appends the provided {@code String} multiple times based on the 
		 * provided integer indicating how many times to append.
		 *   
		 * @param string the provided {@code String}
		 * 
		 * @param times the provided integer indicating how many times to
		 * append
		 * 
		 * @return the appended String
		 */
		private static String append(final String string, final int times) {
			StringBuilder stringBuilder = new StringBuilder();
			for (int i = 0; i < times; i++) {
				stringBuilder.append(string);
			}
			return stringBuilder.toString();
		}
		
		/**
		 * Returns an {@code int} array of the maximum width of each column 
		 * based on the provided two-dimensional {@code Object} array of the 
		 * content.
		 * 
		 * @param cells the provided two-dimensional {@code Object} array of 
		 * the content
		 * 
		 * @return an {@code int} array of the maximum width of each column
		 */
		private static int[] getMaxColumnWidths(final Object[][] cells) {

			int maxColumnCount = 0;
			
			for (Object[] row : cells) {
				
				if (row.length > maxColumnCount) {
					maxColumnCount = row.length;
				}
			}
			
			int[] maxColumnWidths = new int[maxColumnCount];
			
			Arrays.fill(maxColumnWidths, 0);
			
			for (Object[] row : cells) {
				
				for (int i = 0; i < row.length; i++) {
					
					if (i < maxColumnWidths.length) {
						
						Object cell = row[i];
						
						int cellWidth = 0;
						
						if (cell != null) {
							
							cellWidth = cell.toString().length(); 
						}
						
						int maxColumnWidth = maxColumnWidths[i];
						
						if (cellWidth > maxColumnWidth) {
							
							maxColumnWidths[i] = cellWidth;
						}
					}
				}
			}
			
			return maxColumnWidths;
		}
		
		/**
		 * Creates a row based on the provided {@code Object} array of the 
		 * content and the provided {@code int} array of the maximum width of 
		 * each column.
		 * 
		 * @param row the provided {@code Object} array of the content
		 * 
		 * @param maxColumnWidths the provided {@code int} array of the maximum
		 * width of each column
		 * 
		 * @return a new row
		 */
		private static String newRow(
				final Object[] row, final int[] maxColumnWidths) {
			
			StringBuilder rowBuilder = new StringBuilder();
			
			rowBuilder.append('|');
			
			for (int i = 0; i < maxColumnWidths.length; i++) {
				
				int columnWidth = maxColumnWidths[i];
				
				if (i < row.length) {
					
					Object cell = row[i];
					
					if (cell != null) {
						
						rowBuilder.append(cell);
						
						int remainingSpaceLength = 
								columnWidth - cell.toString().length();
						
						if (remainingSpaceLength > 0) {
							
							rowBuilder.append(
									append(" ", remainingSpaceLength));
						}
						
					} else {
						
						rowBuilder.append(append(" ", columnWidth));
					}
					
				} else {
					
					rowBuilder.append(append(" ", columnWidth));
				}
				
				if (i < maxColumnWidths.length - 1) {
					
					rowBuilder.append('|');
				}
			}
			
			rowBuilder.append('|');
			
			return rowBuilder.toString();
		}
		
		/**
		 * Creates a new row separator based on the provided {@code int} array 
		 * of the maximum width of each column.
		 *  
		 * @param maxColumnWidths the provided {@code int} array of the maximum
		 * width of each column
		 * 
		 * @return a new row separator
		 */
		private static String newRowSeparator(final int[] maxColumnWidths) {
			
			StringBuilder rowSeparatorBuilder = new StringBuilder();
			
			rowSeparatorBuilder.append('+');
			
			for (int i = 0; i < maxColumnWidths.length; i++) {
				
				int maxColumnWidth = maxColumnWidths[i];
				
				rowSeparatorBuilder.append(append("-", maxColumnWidth));
				
				if (i < maxColumnWidths.length - 1) {
					
					rowSeparatorBuilder.append('+');
				}
			}
			
			rowSeparatorBuilder.append('+');
			
			return rowSeparatorBuilder.toString();
		}
		
		/** The {@code String} representation of this {@code Table}. */
		private final String string;
		
		/**
		 * Constructs a {@code Table} based on the provided 
		 * two-dimensional {@code Object} array of the content.
		 * 
		 * @param cells the provided two-dimensional {@code Object} array of 
		 * the content
		 */
		public Table(final Object[][] cells) {
			
			int[] maxColumnWidths = getMaxColumnWidths(cells);
			
			String rowSeparator = newRowSeparator(maxColumnWidths);
			
			StringBuilder tableBuilder = new StringBuilder();

			if (cells.length > 0) {
				
				tableBuilder.append(rowSeparator).append('\n');
				
				for (int i = 0; i < cells.length; i++) {
					
					Object[] row = cells[i];
										
					tableBuilder.append(
							newRow(row, maxColumnWidths)).append('\n');
					
					if (i < cells.length - 1) {
						
						tableBuilder.append(rowSeparator).append('\n');
					}
				}
				
				tableBuilder.append(rowSeparator);
			}
			
			this.string = tableBuilder.toString();
		}
		
		@Override
		public String toString() {
			return this.string;
		}
	}	

	/**
	 * Writes from the supplied command line arguments the results of the 
	 * provided command line options.
	 * 
	 * @param args the supplied command line arguments
	 * @throws IOException if an error occurs in writing the results
	 */
	public static void main(final String[] args) throws IOException {

		CLIInterface<results> c = CLIInterface.getInstance(results.class);
		
		results r = c.newCLI();
		
		switch(r.process(args, System.out, System.err)) {
		
		case ERROR:
			System.exit(1);
			
		case HELP_REQUESTED:
			return;
			
		default:
			break;
		}
		
		Object[][] cells = new Object[][] {
				new Object[] {
					"OptionGroupComplete",
					"hasParsedOptionSelection",
					"parsedArguments"
				},
				toTableRow(r.getBigDecimal()),
				toTableRow(r.getBigInteger()),
				toTableRow(r.getBoolean()),
				toTableRow(r.getByte()),
				toTableRow(r.getCharacter()),
				toTableRow(r.getClassname()),
				toTableRow(r.getDate()),
				toTableRow(r.getDecision()),
				toTableRow(r.getDouble()),
				toTableRow(r.getFile()),
				toTableRow(r.getFloat()),
				toTableRow(r.getInteger()),
				toTableRow(r.getLong()),
				toTableRow(r.getShort()),
				toTableRow(r.getString()),				
		};
		
		FileWriter fw = r.getOutputFile().getParsedArguments().get(0);
		
		fw.write(new Table(cells).toString());
		
		fw.write("\n");
		
		fw.close();
	}
	
	/**
	 * Returns a {@code String} representation of a provided {@code List} of 
	 * {@code Option}s.
	 * 
	 * @param optionList the provided {@code List} of {@code Option}s.
	 * 
	 * @return a {@code String} representation of a provided {@code List} of 
	 * {@code Option}s
	 */
	private static String toString(final List<Option> optionList) {
		StringBuilder builder = new StringBuilder();
		for (Iterator<Option> it = optionList.iterator(); it.hasNext();) {
			Option option = it.next();
			builder.append(option.toOptionString());
			if (it.hasNext()) {
				builder.append(", ");
			}
		}
		return builder.toString();
	}

	/**
	 * Returns a {@code Object} array as a table row of the provided 
	 * {@code OptionGroupComplete}.
	 * 
	 * @param optionGroupComplete the provided {@code OptionGroupComplete}
	 * 
	 * @return the arguments
	 */
	private static Object[] toTableRow(
			final OptionGroupComplete<?> optionGroupComplete) {
		
		return new Object[] { 
				toString(optionGroupComplete.getOptions()),
				optionGroupComplete.hasParsedOptionSelection(),
				optionGroupComplete.getParsedArguments()
		};
	}
	
	/**
	 * Constructs an instance of {@code Results}. This constructor is 
	 * suppressed for noninstantiability outside this class.
	 */
	private Results() {
	}
}
