/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * The abstract factory for providing instances.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public abstract class AbstractFactory {

	/** 
	 * Lazy initialization holder class idiom for static field for
	 * {@code DefaultInstanceHolder.DEFAULT_INSTANCE}.
	 */
	private static final class DefaultInstanceHolder {

		/** The default instance of {@code AbstractFactory}. */
		static final AbstractFactory DEFAULT_INSTANCE = 
				new AbstractFactoryImpl();
		
		/**
		 * Constructs a {@code DefaultInstanceHolder}. This constructor 
		 * is suppressed for noninstantiability outside of this class.
		 */
		private DefaultInstanceHolder() {
		}
	}
	
	/** The {@code AbstractFactory}. */
	private static AbstractFactory defaultInstance;
	
	/**
	 * Returns the default {@code AbstractFactory}. 
	 * <p>
	 * If no {@code AbstractFactory} or {@code null} has been supplied to 
	 * {@link org.climax.AbstractFactory#setDefault(AbstractFactory)}, the 
	 * default {@code AbstractFactory} is created and then provided.
	 * </p>
	 * 
	 * @return the default {@code AbstractFactory}
	 */
	public static AbstractFactory getDefault() {
		if (AbstractFactory.defaultInstance == null) {
			AbstractFactory.setDefault(
					DefaultInstanceHolder.DEFAULT_INSTANCE);
		}
		return AbstractFactory.defaultInstance;
	}
	
	/**
	 * Sets the default {@code AbstractFactory} with the provided 
	 * {@code AbstractFactory}.
	 * 
	 * @param abstractFactory the provided {@code AbstractFactory}
	 */
	public static void setDefault(final AbstractFactory abstractFactory) {
		AbstractFactory.defaultInstance = abstractFactory;
	}
	
	/**
	 * Constructs an {@code AbstractFactory}.
	 */
	protected AbstractFactory() {
	}

	/**
	 * Returns a {@code Comparator} that compares two {@code ArgGroup}s 
	 * first by their ordinals and then by their argument types.
	 * 
	 * @return a {@code Comparator} that compares two {@code ArgGroup}s 
	 * first by their ordinals and then by their argument types
	 */
	public abstract Comparator<ArgGroup<?>> getArgGroupComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code ArgGroupDoc}s 
	 * by their {@code ArgGroup}s.
	 * 
	 * @return a {@code Comparator} that compares two {@code ArgGroupDoc}s 
	 * by their {@code ArgGroup}s
	 */
	public abstract Comparator<ArgGroupDoc> getArgGroupDocComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code Option}s 
	 * by their names.
	 * 
	 * @return a {@code Comparator} that compares two {@code Option}s 
	 * by their names
	 */
	public abstract Comparator<Option> getOptionByNameComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code Option}s 
	 * first by their {@code OptionType}s and then by their names.
	 * 
	 * @return a {@code Comparator} that compares two {@code Option}s 
	 * first by their {@code OptionType}s and then by their names
	 */
	public abstract Comparator<Option> getOptionComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code OptionDoc}s 
	 * by their {@code Option}s' names.
	 * 
	 * @return a {@code Comparator} that compares two {@code OptionDoc}s 
	 * by their {@code Option}s' names
	 */
	public abstract Comparator<OptionDoc> getOptionDocByOptionNameComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code OptionDoc}s 
	 * first by their ordinals and then by their {@code Option}s.
	 * 
	 * @return a {@code Comparator} that compares two {@code OptionDoc}s 
	 * first by their ordinals and then by their {@code Option}s
	 */
	public abstract Comparator<OptionDoc> getOptionDocComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code OptionGroupDoc}s 
	 * first by their ordinals and then by their {@code OptionDoc}s' 
	 * {@code Option}s' names.
	 * 
	 * @return a {@code Comparator} that compares two {@code OptionGroupDoc}s 
	 * first by their ordinals and then by their {@code OptionDoc}s' 
	 * {@code Option}s' names
	 */
	public abstract Comparator<OptionGroupDoc> getOptionGroupDocComparator();
	
	/**
	 * Returns a {@code Comparator} that compares two {@code Package}s by their
	 * names.
	 * 
	 * @return a {@code Comparator} that compares two {@code Package}s by their
	 * names
	 */
	public abstract Comparator<Package> getPackageComparator();
	
	/**
	 * Returns a {@code PackageDoc} based on the provided {@code Package} 
	 * that exists from the provided {@code List} of {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been provided to 
	 * {@link org.climax.AbstractFactory#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 * 
	 * @param pkg the provided {@code Package} that exists from the 
	 * {@code List} of {@code Package}s
	 * 
	 * @return a {@code PackageDoc} based on the provided {@code Package} that 
	 * exists from the provided {@code List} of {@code Package}s or 
	 * {@code null} if the provided {@code Package} does not exist from the 
	 * {@code List} of {@code Package}s
	 */
	public abstract PackageDoc getPackageDoc(final Package pkg);
	
	/**
	 * Returns a {@code Comparator} that compares two {@code PackageDoc}s by 
	 * their {@code Package}s.
	 * 
	 * @return a {@code Comparator} that compares two {@code PackageDoc}s by 
	 * their {@code Package}s
	 */
	public abstract Comparator<PackageDoc> getPackageDocComparator();
	
	/**
	 * Returns a sorted unmodifiable {@code List} of all of the 
	 * {@code PackageDoc}s based on the provided {@code List} of 
	 * {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been supplied to 
	 * {@link org.climax.AbstractFactory#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 *  
	 * @return a sorted unmodifiable {@code List} of all of the 
	 * {@code PackageDoc}s based on the provided {@code List} of
	 * {@code Package}s
	 */
	public abstract List<PackageDoc> getPackageDocs();
	
	/**
	 * Returns an unmodifiable {@code Map} for the documentation properties of 
	 * all of the {@code PackageDoc}s based on the provided {@code List} of
	 * {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been supplied to 
	 * {@link org.climax.AbstractFactory#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 * 
	 * @return an unmodifiable {@code Map} for the documentation properties of 
	 * all of the {@code PackageDoc}s based on the provided {@code List} of
	 * {@code Package}s
	 */
	public abstract Map<String, String> getPackageDocsProperties();
	
	/**
	 * Returns an unmodifiable {@code List} of {@code Package}s based on the 
	 * provided {@code List} of {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been supplied to 
	 * {@link org.climax.AbstractFactory#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 * 
	 * @return an unmodifiable {@code List} of {@code Package}s based on the 
	 * provided {@code List} of {@code Package}s
	 */
	public abstract List<Package> getPackages();
	
	/**
	 * Returns the {@code SystemDoc}.
	 * 
	 * @return the {@code SystemDoc}
	 */
	public abstract SystemDoc getSystemDoc();
	
	/**
	 * Returns an unmodifiable {@code Map} for the documentation properties of 
	 * the {@code SystemDoc}.
	 *  
	 * @return an unmodifiable {@code Map} of system properties
	 */
	public abstract Map<String, String> getSystemDocProperties();
	
	/**
	 * Creates a new {@code ArgGroupBuilder} of the specified parsed argument 
	 * type with the provided argument type and the provided {@code ArgParser}.
	 * 
	 * @param <T> the specified parsed argument type
	 * 
	 * @param argumentType the provided argument type
	 * 
	 * @param argParser the provided {@code ArgParser}
	 * 
	 * @return a new {@code ArgGroupBuilder} of the specified parsed argument 
	 * type with the provided argument type and the provided {@code ArgParser} 
	 */	
	public abstract <T> ArgGroupBuilder<T> newArgGroupBuilder(
			final String argumentType, 
			final ArgParser<? extends T> argParser);

	/**
	 * Creates a new {@code ArgGroupCompleteBuilder} of the specified parsed 
	 * argument type with the provided argument type and the provided 
	 * {@code ArgParser}.
	 * 
	 * @param <T> the specified parsed argument type
	 * 
	 * @param argumentType the provided argument type
	 * 
	 * @param argParser the provided {@code ArgParser}
	 * 
	 * @return a new {@code ArgGroupCompleteBuilder} of the specified parsed 
	 * argument type with the provided argument type and the provided
	 * {@code ArgParser} 
	 */
	public abstract <T> ArgGroupCompleteBuilder<T> newArgGroupCompleteBuilder(
			final String argumentType,
			final ArgParser<? extends T> argParser);

	/**
	 * Creates a new {@code ArgGroupDocBuilder} with the provided 
	 * {@code ArgGroup}.
	 * 
	 * @param argGroup the provided {@code ArgGroup}
	 * 
	 * @return a new {@code ArgGroupDocBuilder} with the provided 
	 * {@code ArgGroup}
	 */
	public abstract ArgGroupDocBuilder newArgGroupDocBuilder(
			final ArgGroup<? extends Object> argGroup);
	
	/**
	 * Creates a new instance of {@code Args}.
	 * 
	 * @return a new instance of {@code Args}
	 */
	public abstract Args newArgs();

	/**
	 * Creates a new instance of {@code Args} with the provided {@code Args} 
	 * instance.
	 * 
	 * @param args the provided {@code Args} instance
	 * 
	 * @return a new instance of {@code Args} with the provided {@code Args} 
	 * instance
	 */
	public abstract Args newArgs(final Args args);

	/**
	 * Creates a new {@code ArgsComplete}.
	 * 
	 * @return a new {@code ArgsComplete}
	 */
	public abstract ArgsComplete newArgsComplete();

	/**
	 * Creates a new {@code ArgsComplete} with the provided
	 * {@code ArgsComplete}.
	 * 
	 * @param argsComplete the provided {@code ArgsComplete}
	 * 
	 * @return a new instance of {@code ArgsComplete} with the provided
	 * {@code ArgsComplete}
	 */
	public abstract ArgsComplete newArgsComplete(
			final ArgsComplete argsComplete);

	/**
	 * Creates a new {@code ArgsDoc}.
	 * 
	 * @return a new {@code ArgsDoc}
	 */
	public abstract ArgsDoc newArgsDoc();

	/**
	 * Creates a new {@code ArgsDoc} with the provided {@code ArgsDoc}.
	 * 
	 * @param argsDoc the provided {@code ArgsDoc}
	 * 
	 * @return a new {@code ArgsDoc} with the provided {@code ArgsDoc}
	 */
	public abstract ArgsDoc newArgsDoc(final ArgsDoc argsDoc);

	/**
	 * Creates a new {@code CLIBuilder} with the provided format string
	 * for the program name.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <ul>
	 * <li>Property keys defined in {@link System#getProperties()}</li>
	 * </ul>
	 *  
	 * @param programName the provided format string for the program name
	 * 
	 * @return a new {@code CLIBuilder} with the provided format string for
	 * the program name
	 */
	public abstract CLIBuilder newCLIBuilder(final String programName);

	/**
	 * Returns a new {@code Option} based on the provided {@code OptionType} 
	 * and the specified name.
	 * 
	 * @param optionType the provided {@code OptionType}
	 * 
	 * @param name the specified name
	 * 
	 * @return a new {@code Option} based on the provided {@code OptionType} 
	 * and the specified name
	 */
	public abstract Option newOption(
			final OptionType optionType, final String name);
	
	/**
	 * Creates a new {@code OptionCompleteBuilder} with the provided 
	 * {@code OptionType} and the specified name.
	 *  
	 * @param optionType the provided {@code OptionType}
	 *  
	 * @param name the specified name
	 * 
	 * @return a new {@code OptionCompleteBuilder} with the provided 
	 * {@code OptionType} and the specified name
	 */
	public abstract OptionCompleteBuilder newOptionCompleteBuilder(
			final OptionType optionType, final String name);

	/**
	 * Creates a new {@code OptionDocBuilder} with the provided
	 * {@code Option}.
	 * 
	 * @param option the provided {@code Option}
	 * 
	 * @return a new {@code OptionDocBuilder} with the provided
	 * {@code Option}
	 */
	public abstract OptionDocBuilder newOptionDocBuilder(final Option option);

	/**
	 * Creates a new {@code OptionGroupBuilder} of the specified parsed 
	 * argument type with the provided {@code List} of {@code Option}s.
	 * 
	 * @param <T> the specified parsed argument type
	 * 
	 * @param optionList the provided {@code List} of {@code Option}s
	 * 
	 * @return a new {@code OptionGroupBuilder} of the specified parsed 
	 * argument type with the provided {@code List} of {@code Option}s
	 */
	 public abstract <T> OptionGroupBuilder<T> newOptionGroupBuilder(
			final List<Option> optionList);

	/**
	 * Creates a new {@code OptionGroupCompleteBuilder} of the specified 
	 * parsed argument type with the provided {@code List} of 
	 * {@code OptionComplete}s.
	 * 
	 * @param <T> the specified parsed argument type
	 * 
	 * @param optionCompleteList the provided {@code List} of 
	 * {@code OptionComplete}s
	 * 
	 * @return a new {@code OptionGroupCompleteBuilder} of the specified 
	 * parsed argument type with the provided {@code List} of 
	 * {@code OptionComplete}s
	 */
	 public abstract <T> OptionGroupCompleteBuilder<T> newOptionGroupCompleteBuilder(
			final List<OptionComplete> optionCompleteList);

	/**
	 * Creates a new {@code OptionGroupDocBuilder} with the provided 
	 * {@code List} of {@code OptionDoc}s.
	 * 
	 * @param optionDocList the provided {@code List} of {@code OptionDoc}s
	 * 
	 * @return a new {@code OptionGroupDocBuilder} with the provided 
	 * {@code List} of {@code OptionDoc}s
	 */
	 public abstract OptionGroupDocBuilder newOptionGroupDocBuilder(
			final List<OptionDoc> optionDocList);

	/**
	 * Creates a new instance of {@code Options}.
	 * 
	 * @return a new instance of {@code Options}
	 */
	public abstract Options newOptions();

	/**
	 * Creates a new instance of {@code Options} with the provided 
	 * {@code Options} instance.
	 * 
	 * @param options the provided {@code Options} instance
	 * 
	 * @return a new instance of {@code Options} with the provided 
	 * {@code Options} instance
	 */
	public abstract Options newOptions(final Options options);

	/**
	 * Creates a new {@code OptionsComplete}.
	 * 
	 * @return a new {@code OptionsComplete}
	 */
	public abstract OptionsComplete newOptionsComplete();

	/**
	 * Creates a new {@code OptionsComplete} with the provided 
	 * {@code OptionsComplete}.
	 * 
	 * @param optionsComplete the provided {@code OptionsComplete}
	 * 
	 * @return a new {@code OptionsComplete} with the provided 
	 * {@code OptionsComplete}
	 */
	public abstract OptionsComplete newOptionsComplete(
			final OptionsComplete optionsComplete);

	/**
	 * Creates a new {@code OptionsDoc}.
	 * 
	 * @return a new {@code OptionsDoc}
	 */
	public abstract OptionsDoc newOptionsDoc();

	/**
	 * Creates a new {@code OptionsDoc} with the provided {@code OptionsDoc}.
	 * 
	 * @param optionsDoc the provided {@code OptionsDoc}
	 * 
	 * @return a new {@code OptionsDoc} with the provided {@code OptionsDoc}
	 */
	public abstract OptionsDoc newOptionsDoc(final OptionsDoc optionsDoc);

	/**
	 * Creates a new {@code OptionSelection} of the specified type 
	 * with the provided {@code Option} from the {@code OptionGroup} and 
	 * the optional argument of the specified type.
	 * 
	 * @param <T> the specified type
	 * 
	 * @param option the provided {@code Option} from the 
	 * {@code OptionGroup}
	 * 
	 * @param argument the provided argument of the specified type
	 * 
	 * @return a new {@code OptionSelection} of the specified type 
	 * with the provided {@code Option} from the {@code OptionGroup} and 
	 * the optional argument of the specified type
	 */
	public abstract <T> OptionSelection<T> newOptionSelection(
			final Option option, final T argument);

	/**
	 * Sets the {@code Package}s with the provided {@code List} of 
	 * {@code Package}s.
	 * 
	 * @param pkgList the provided {@code List} of {@code Package}s
	 */
	public abstract void setPackages(final List<Package> pkgList);
}
