/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * An implementation of an {@code ArgGroupComplete} of the specified parsed 
 * argument type. 
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <T> the specified parsed argument type
 */
final class ArgGroupCompleteImpl<T> implements ArgGroupComplete<T> {

	/**
	 * A builder for an {@code ArgGroupCompleteImpl} of the specified parsed 
	 * argument type.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 *
	 * @param <T> the specified parsed argument type
	 */
	static final class Builder<T> implements ArgGroupCompleteBuilder<T> {

		/**
		 * The ordinal of the {@code ArgGroup} in relation to other 
		 * {@code ArgGroup}s.
		 */
		private int ordinal;
		
		/** The argument type. */
		private final String argumentType;
		
		/** 
		 * The boolean value to indicate if the {@code ArgGroupDoc} is 
		 * unused from documentation. 
		 */
		private boolean hidden;
		
		/**
		 * The boolean value to indicate if the {@code ArgGroup} is 
		 * unused from parsing.
		 */
		private boolean unused;
		
		/** The format string for the syntax for the {@code ArgGroup}. */
		private String syntax;
		
		/** The {@code ArgAcceptor}. */
		private ArgAcceptor argAcceptor;
		
		/** The {@code ArgsValidator}. */
		private ArgsValidator argsValidator;
		
		/** The {@code ArgParser}. */
		private final ArgParser<? extends T> argParser;

		/** The {@code List} of default argument format strings. */
		private final List<String> defaultArgumentList;
		
		/**
		 * Constructs an {@code ArgGroupCompleteImpl.Builder} with the provided
		 * argument type and the provided {@code ArgParser}.
		 * 
		 * @param argType the provided argument type
		 * 
		 * @param p the provided {@code ArgParser}
		 */
		public Builder(
				final String argType, final ArgParser<? extends T> p) {
			
			this.ordinal = 0;
			
			this.argumentType = argType;
			
			this.hidden = false;
			
			this.unused = false;
			
			this.syntax = null;
			
			this.argAcceptor = null;
			
			this.argsValidator = null;
			
			this.argParser = p;
			
			this.defaultArgumentList = new ArrayList<String>();
		}
		
		@Override
		public ArgGroupCompleteBuilder<T> argAcceptor(final ArgAcceptor a) {
			this.argAcceptor = a;
			return this;
		}
		
		@Override
		public ArgGroupCompleteBuilder<T> argsValidator(final ArgsValidator v) {
			this.argsValidator = v;
			return this;
		}		

		@Override
		public ArgGroup<T> buildArgGroup() {
			return new ArgGroupCompleteImpl<T>(this);
		}

		@Override
		public ArgGroupComplete<T> buildArgGroupComplete() {
			return new ArgGroupCompleteImpl<T>(this);
		}

		@Override
		public ArgGroupDoc buildArgGroupDoc() {
			return new ArgGroupCompleteImpl<T>(this);
		}

		@Override
		public ArgGroupCompleteBuilder<T> defaultArguments(
				final List<String> l) {
			
			this.defaultArgumentList.clear();
			this.defaultArgumentList.addAll(l);
			return this;
		}

		@Override
		public ArgGroupCompleteBuilder<T> hidden(final boolean b) {
			this.hidden = b;
			return this;
		}

		@Override
		public ArgGroupCompleteBuilder<T> ordinal(final int i) {
			this.ordinal = i;
			return this;
		}

		@Override
		public ArgGroupCompleteBuilder<T> syntax(final String s) {
			this.syntax = s;
			return this;
		}

		@Override
		public ArgGroupCompleteBuilder<T> unused(final boolean b) {
			this.unused = b;
			return this;
		}
	}
	
	/** 
	 * The boolean value to indicate if the {@code ArgGroupDoc} is 
	 * unused from documentation. 
	 */
	private final boolean hidden;
	
	/**
	 * The boolean value to indicate if the {@code ArgGroup} is 
	 * unused from parsing.
	 */
	private final boolean unused;
	
	/** The {@code ArgGroup}. */
	private final ArgGroup<T> argGroup;
	
	/** The {@code ArgGroupDoc}. */
	private final ArgGroupDoc argGroupDoc;
	
	/**
	 * Constructs an {@code ArgGroupCompleteImpl} with the provided
	 * {@code ArgGroupCompleteImpl.Builder}.
	 * 
	 * @param builder the provided {@code ArgGroupCompleteImpl.Builder}
	 */
	private ArgGroupCompleteImpl(final Builder<T> builder) {

		String argumentType = builder.argumentType;
		int ordinal = builder.ordinal;
		boolean hide = builder.hidden;
		boolean unuse = builder.unused;		
		String syntax = builder.syntax;
		ArgAcceptor acceptor = builder.argAcceptor;
		ArgsValidator validator = builder.argsValidator;
		ArgParser<? extends T> parser = builder.argParser;
		List<String> defaultArgList = new ArrayList<String>(
				builder.defaultArgumentList);
		
		ArgGroupBuilder<T> argGroupBuilder = 
				AbstractFactory.getDefault().newArgGroupBuilder(
						argumentType, parser);
		argGroupBuilder.argAcceptor(acceptor);
		argGroupBuilder.argsValidator(validator);
		argGroupBuilder.defaultArguments(defaultArgList);
		argGroupBuilder.ordinal(ordinal);
		
		ArgGroup<T> arGroup = argGroupBuilder.buildArgGroup();
		
		ArgGroupDocBuilder argGroupDocBuilder = 
				AbstractFactory.getDefault().newArgGroupDocBuilder(arGroup);
		argGroupDocBuilder.syntax(syntax);
		
		ArgGroupDoc arGroupDoc = argGroupDocBuilder.buildArgGroupDoc();
		
		this.hidden = hide;
		this.unused = unuse;
		this.argGroup = arGroup;
		this.argGroupDoc = arGroupDoc;
	}
	
	@Override
	public void addArgument(final String argument) {
		this.argGroup.addArgument(argument);
	}
	
	@Override
	public void clearArguments() {
		this.argGroup.clearArguments();
	}

	@Override
	public void clearParsedArguments() {
		this.argGroup.clearArguments();
	}

	@Override
	public ArgAcceptor getArgAcceptor() {
		return this.argGroup.getArgAcceptor();
	}

	@Override
	public ArgGroup<Object> getArgGroup() {
		return this.argGroupDoc.getArgGroup();
	}

	@Override
	public ArgParser<T> getArgParser() {
		return this.argGroup.getArgParser();
	}

	@Override
	public ArgsValidator getArgsValidator() {
		return this.argGroup.getArgsValidator();
	}

	@Override
	public List<String> getArguments() {
		return this.argGroup.getArguments();
	}

	@Override
	public String getArgumentType() {
		return this.argGroup.getArgumentType();
	}

	@Override
	public int getOrdinal() {
		return this.argGroup.getOrdinal();
	}

	@Override
	public List<T> getParsedArguments() {
		return this.argGroup.getParsedArguments();
	}

	@Override
	public Map<String, String> getProperties() {
		return this.argGroupDoc.getProperties();
	}

	@Override
	public String getSyntax() {
		return this.argGroupDoc.getSyntax();
	}

	@Override
	public boolean hasArgument() {
		return this.argGroup.hasArgument();
	}

	@Override
	public boolean hasParsedArgument() {
		return this.argGroup.hasParsedArgument();
	}

	@Override
	public boolean isHidden() {
		return this.hidden;
	}

	@Override
	public boolean isUnused() {
		return this.unused;
	}

	@Override
	public void parse() throws ArgGroupException {
		this.argGroup.parse();
	}

	@Override
	public String toString() {
		return String.format("ArgGroupComplete [" 
				+ "hidden=%s, " 
				+ "unused=%s, " 
				+ "argGroup=%s, " 
				+ "argGroupDoc=%s]",
				this.hidden, 
				this.unused, 
				this.argGroup, 
				this.argGroupDoc);
	}
}
