/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.Collections;
import java.util.Map;


/**
 * An implementation of an {@code ArgGroupDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ArgGroupDocImpl implements ArgGroupDoc {

	/**
	 * A builder for an {@code ArgGroupDocImpl}.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	static final class Builder implements ArgGroupDocBuilder {

		/** The {@code ArgGroup}. */
		private final ArgGroup<Object> argGroup;
		
		/** The format string for the syntax for the {@code ArgGroup}. */
		private String syntax;
		
		/**
		 * Constructs an {@code ArgGroupDocImpl.Builder} with the provided
		 * {@code ArgGroup}.
		 * 
		 * @param arGroup the provided {@code ArgGroup} 
		 */
		public Builder(final ArgGroup<? extends Object> arGroup) {
			
			@SuppressWarnings("unchecked")
			ArgGroup<Object> o = (ArgGroup<Object>) arGroup;
			
			this.argGroup = o;
			
			this.syntax = null;
		}
		
		@Override
		public ArgGroupDoc buildArgGroupDoc() {
			
			ArgGroupDoc argGroupDoc = null;
			
			try {
				argGroupDoc = new ArgGroupDocImpl(this);
			} catch (RuntimeException e) {
				throw new IllegalStateException(e);
			}
			
			return argGroupDoc;
		}
		
		@Override
		public ArgGroupDocBuilder syntax(final String s) {
			this.syntax = s;
			return this;
		}		
	}
	
	/**
	 * Updates the provided {@code ArgGroupDocImpl}'s documentation properties.
	 * 
	 * @param argGroupDocImpl the provided {@code ArgGroupDocImpl}
	 */
	private static void updateProperties(
			final ArgGroupDocImpl argGroupDocImpl) {
		
		Map<String, String> properties = argGroupDocImpl.properties;
		
		ArgGroup<Object> argGroup = argGroupDocImpl.argGroup; 
		
		String syntax = argGroupDocImpl.syntax;
		
		if (argGroup != null) {
			
			if (!properties.containsKey(
					ArgGroupDocPropertyKeys.ARGUMENT_TYPE)) {
			
				properties.put(
						ArgGroupDocPropertyKeys.ARGUMENT_TYPE, 
						argGroup.getArgumentType());
			}			
		}
		
		if (syntax != null) {
			
			if (!properties.containsKey(ArgGroupDocPropertyKeys.SYNTAX)) {
			
				properties.put(ArgGroupDocPropertyKeys.SYNTAX, syntax);
			}
		}
	}
	
	/** 
	 * The {@code Map} for the documentation properties of this 
	 * {@code ArgGroupDocImpl}. 
	 */
	private final Map<String, String> properties;
	
	/** The {@code ArgGroup}. */
	private final ArgGroup<Object> argGroup;
	
	/** The syntax for the {@code ArgGroup}. */
	private final String syntax;
	
	/**
	 * Constructs an {@code ArgGroupDocImpl} with the provided 
	 * {@code ArgGroupDocImpl.Builder}.
	 * 
	 * @param builder the provided {@code ArgGroupDocImpl.Builder}
	 */
	private ArgGroupDocImpl(final Builder builder) {
		
		ArgGroup<Object> arGroup = builder.argGroup;
		String syn = builder.syntax;
		
		if (arGroup == null) {
			throw new NullPointerException("argGroup must not be null");
		}
		
		this.properties = new Properties();
		
		this.argGroup = arGroup;
		
		ArgGroupDocImpl.updateProperties(this);
		
		if (syn == null || syn.isEmpty()) {
			syn = new Properties.Property(
					ArgGroupDocPropertyKeys.ARGUMENT_TYPE).toExpandableString();
		}
		
		this.syntax = Util.expand3(syn, this.properties);
		
		ArgGroupDocImpl.updateProperties(this);
	}
	
	@Override
	public ArgGroup<Object> getArgGroup() {
		return this.argGroup;
	}

	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}

	@Override
	public String getSyntax() {
		return this.syntax;
	}

	@Override
	public String toString() {
		return String.format("ArgGroupDoc [" 
				+ "properties=%s, " 
				+ "argGroup=%s, " 
				+ "syntax=%s]",
				this.properties, 
				this.argGroup, 
				this.syntax);
	}
}
