/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Thrown when an {@code ArgGroup}'s arguments are invalid or cannot be parsed.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class ArgGroupException extends Exception {

	/** The default serial version UID. */
	private static final long serialVersionUID = 1L;
	
	/** The argument type. */
	private String argumentType;
	
	/** The {@code List} of arguments that are invalid or cannot be parsed. */
	private List<String> argumentsList;

	/**
	 * Constructs an {@code ArgGroupException} with the provided argument 
	 * type, the provided {@code List} of arguments that are invalid or cannot 
	 * be parsed, and the specified detail message. 
	 * 
	 * @param argType the provided argument type
	 * 
	 * @param argList the provided {@code List} of arguments that are 
	 * invalid or cannot be parsed
	 * 
	 * @param message the detail message
	 */
	public ArgGroupException(
			final String argType,
			final List<String> argList, 
			final String message) {
		
		this(argType, argList, message, null);
	}
	
	/**
	 * Constructs an {@code ArgGroupException} with the provided argument 
	 * type, the provided {@code List} of arguments that are invalid or cannot 
	 * be parsed, the specified detail message, and the optional 
	 * {@code Throwable} that was raised. 
	 * 
	 * @param argType the provided argument type
	 * 
	 * @param argList the provided {@code List} of arguments that are 
	 * invalid or cannot be parsed
	 * 
	 * @param message the detail message
	 * 
	 * @param cause the {@code Throwable} that was raised
	 */
	public ArgGroupException(
			final String argType,
			final List<String> argList, 
			final String message, 
			final Throwable cause) {
		
		super(message, cause);
		
		this.argumentType = argType;
		
		this.argumentsList = new ArrayList<String>();
		
		this.argumentsList.addAll(argList);
	}
	
	/**
	 * Returns an unmodifiable {@code List} of arguments that are invalid 
	 * or cannot be parsed.
	 * 
	 * @return an unmodifiable {@code List} of arguments that are invalid 
	 * or cannot be parsed
	 */
	public List<String> getArguments() {
		return Collections.unmodifiableList(this.argumentsList);
	}
	
	/**
	 * Returns the argument type.
	 * 
	 * @return the argument type
	 */
	public String getArgumentType() {
		return this.argumentType;
	}
}
