/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.climax.argacceptor.SingleArgAcceptor;
import org.climax.argparser.NullArgParser;
import org.climax.argsvalidator.SingleArgValidator;


/**
 * An implemenation of an {@code ArgGroup} of the specified parsed argument 
 * type.
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <T> the specified parsed argument type
 */
final class ArgGroupImpl<T> implements ArgGroup<T> {
	
	/**
	 * A builder for an {@code ArgGroupImpl} of the specified parsed argument 
	 * type.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 *
	 * @param <T> the specified parsed argument type
	 */
	static final class Builder<T> implements ArgGroupBuilder<T> {
		
		/**
		 * The ordinal of the {@code ArgGroup} in relation to other 
		 * {@code ArgGroup}s.
		 */
		private int ordinal;
		
		/** The argument type. */
		private final String argumentType;
		
		/** The {@code ArgAcceptor}. */
		private ArgAcceptor argAcceptor;
		
		/** The {@code ArgsValidator}. */
		private ArgsValidator argsValidator;
		
		/** The {@code ArgParser}. */
		private final ArgParser<? extends T> argParser;
		
		/** The {@code List} of default argument format strings. */
		private final List<String> defaultArgumentList;
		
		/**
		 * Constructs an {@code ArgGroupImpl.Builder} with the provided 
		 * argument type and the provided {@code ArgParser}.
		 * 
		 * @param argType the provided argument type
		 * 
		 * @param p the provided {@code ArgParser}
		 */
		public Builder(
				final String argType, final ArgParser<? extends T> p) {
			
			this.ordinal = 0;
			
			this.argumentType = argType;
			
			this.argAcceptor = null;
			
			this.argsValidator = null;
			
			this.argParser = p;
			
			this.defaultArgumentList = new ArrayList<String>();
		}
		
		@Override
		public ArgGroupBuilder<T> argAcceptor(final ArgAcceptor a) {
			this.argAcceptor = a;
			return this;
		}
		
		@Override
		public ArgGroupBuilder<T> argsValidator(final ArgsValidator v) {
			this.argsValidator = v;
			return this;
		}

		@Override
		public ArgGroup<T> buildArgGroup() {
			
			ArgGroup<T> argGroup = null;
			
			try {
				argGroup = new ArgGroupImpl<T>(this);
			} catch (RuntimeException e) {
				throw new IllegalStateException(e);
			}
			
			return argGroup;
		}

		@Override
		public ArgGroupBuilder<T> defaultArguments(final List<String> l) {
			this.defaultArgumentList.clear();
			this.defaultArgumentList.addAll(l);
			return this;
		}

		@Override
		public ArgGroupBuilder<T> ordinal(final int i) {
			this.ordinal = i;
			return this;
		}
	}
	
	/**
	 * Returns the detail message specifying if any of the arguments that are 
	 * invalid or cannot be parsed, the argument type, and the additional 
	 * specified detail message.
	 * 
	 * @param argumentType the provided argument type
	 * 
	 * @param argumentList the provided {@code List} of arguments that are 
	 * invalid or cannot be parsed
	 * 
	 * @param message the additional detail message
	 * 
	 * @return the detail message specifying if any of the arguments that are 
	 * invalid or cannot be parsed, the argument type, and the additional 
	 * specified detail message
	 */
	private static String getMessage(
			final String argumentType,
			final List<String> argumentList,
			final String message) {
		
		String argumentListString = Lists.toString(argumentList);
		
		String m = String.format(
				"no %1$s argument : %2$s",
				argumentType,
				message);
		
		if (argumentList.size() == 1) {
			
			m = String.format(
					"invalid %1$s argument %2$s : %3$s",
					argumentType,
					argumentListString,
					message);
		}
		
		if (argumentList.size() > 1) {
			
			m = String.format(
					"invalid %1$s arguments %2$s : %3$s",
					argumentType,
					argumentListString,
					message);
		}				
		
		return m;
	}
	
	/** 
	 * The ordinal of this {@code ArgGroup} in relation to other 
	 * {@code ArgGroup}s.
	 */
	private final int ordinal;
	
	/** The argument type. */
	private final String argumentType;
	
	/** The {@code ArgAcceptor}. */
	private final ArgAcceptor argAcceptor;
	
	/** The {@code ArgsValidator}. */
	private final ArgsValidator argsValidator;
	
	/** The {@code ArgParser}. */
	private final ArgParser<? extends T> argParser;
	
	/** The {@code List} of default arguments. */
	private final List<String> defaultArgumentList;
	
	/** 
	 * The {@code List} of parsed default arguments of the specified parsed 
	 * argument type.
	 */
	private final List<T> parsedDefaultArgumentList;
	
	/** A {@code List} of arguments of type {@code String}. */
	private final List<String> argumentList;
	
	/** A {@code List} of arguments of the specified parsed argument type. */
	private final List<T> parsedArgumentList;
	
	/**
	 * Constructs an {@code ArgGroupImpl} with the provided
	 * {@code ArgGroupImpl.Builder}.
	 * 
	 * @param builder the provided {@code ArgGroupImpl.Builder}
	 */
	private ArgGroupImpl(final Builder<T> builder) {
		
		int ord = builder.ordinal;
		String argType = builder.argumentType;
		ArgAcceptor acceptor = builder.argAcceptor;
		ArgsValidator validator = builder.argsValidator;
		ArgParser<? extends T> parser = builder.argParser;
		List<String> defaultArgList = new ArrayList<String>();
		
		for (String defaultArgument : builder.defaultArgumentList) {
			
			if (defaultArgument == null) {
				throw new NullPointerException(
						"default arguments must not be null");
			}
			
			String defaultArg = Util.expand1(
					defaultArgument, 
					AbstractFactory.getDefault().getSystemDocProperties());
			
			defaultArgList.add(defaultArg);
		}
		
		List<T> parsedDefaultArgList = new ArrayList<T>();
		
		if (argType == null) {
			throw new NullPointerException(
					"name of an argument must not be null");
		}
		
		if (argType.isEmpty()) {
			throw new IllegalArgumentException(
					"argument type must be a non-empty string");
		}
		
		if (acceptor == null) {
			acceptor = SingleArgAcceptor.newInstance();
		}
		
		if (validator == null) {
			validator = SingleArgValidator.getInstance();
		}
		
		if (parser == null) {
			parser = NullArgParser.newInstance();
		}
		
		if (defaultArgList.size() > 0) {
			
			parsedDefaultArgList = DefaultArgs.parse(
					defaultArgList, 
					validator, 
					parser);
		}
		
		this.ordinal = ord;
		
		this.argumentType = argType;
		
		this.argAcceptor = acceptor;
		
		this.argsValidator = validator;
		
		this.argParser = parser;
		
		this.defaultArgumentList = new ArrayList<String>(defaultArgList);

		this.parsedDefaultArgumentList = new ArrayList<T>(parsedDefaultArgList);
		
		this.argumentList = new ArrayList<String>();
		
		this.parsedArgumentList = new ArrayList<T>();
	}

	@Override
	public void addArgument(final String argument) {
		
		if (argument == null) {
			throw new NullPointerException("argument must not be null");
		}
		
		this.argumentList.add(argument);
	}

	@Override
	public void clearArguments() {
		this.argumentList.clear();
	}

	@Override
	public void clearParsedArguments() {
		this.parsedArgumentList.clear();
	}
	
	@Override
	public ArgAcceptor getArgAcceptor() {
		return this.argAcceptor;
	}

	@Override
	public ArgParser<T> getArgParser() {
		@SuppressWarnings("unchecked")
		ArgParser<T> aParser = (ArgParser<T>) this.argParser;
		return aParser;
	}

	@Override
	public ArgsValidator getArgsValidator() {
		return this.argsValidator;
	}

	@Override
	public List<String> getArguments() {
		
		List<String> argList = new ArrayList<String>();
		
		if (this.argumentList.size() > 0) {
			
			argList.addAll(this.argumentList);
			
		} else {
			
			argList.addAll(this.defaultArgumentList);
		}
		
		return Collections.unmodifiableList(argList);
	}

	@Override
	public String getArgumentType() {
		return this.argumentType;
	}
	
	@Override
	public int getOrdinal() {
		return this.ordinal;
	}

	@Override
	public List<T> getParsedArguments() {
		
		List<T> parsedArgList = new ArrayList<T>();
		
		if (this.parsedArgumentList.size() > 0) {
			
			parsedArgList.addAll(this.parsedArgumentList);
			
		} else {
			
			parsedArgList.addAll(this.parsedDefaultArgumentList);
		}
		
		return Collections.unmodifiableList(parsedArgList);
	}
	
	@Override
	public boolean hasArgument() {
		return this.argumentList.size() > 0 
				|| this.defaultArgumentList.size() > 0;
	}

	@Override
	public boolean hasParsedArgument() {
		return this.parsedArgumentList.size() > 0 
				|| this.parsedDefaultArgumentList.size() > 0;
	}

	@Override
	public void parse() throws ArgGroupException {
		
		List<String> argList = new ArrayList<String>(this.argumentList);

		try {

			this.argsValidator.validate(argList);

		} catch (ArgsValidatorException e) {
			
			String message = ArgGroupImpl.getMessage(
					this.getArgumentType(), 
					e.getArguments(), 
					e.getMessage());
			
			throw new ArgGroupException(
					this.getArgumentType(),
					e.getArguments(),
					message, 
					e);
		}

		List<T> parsedArgList = new ArrayList<T>();

		for (String arg : argList) {

			try {

				T parsedArg = this.argParser.parse(arg);
				
				parsedArgList.add(parsedArg);

			} catch (ArgParserException e) {
				
				String message = ArgGroupImpl.getMessage(
						this.getArgumentType(), 
						Arrays.asList(e.getArgument()), 
						e.getMessage());
				
				throw new ArgGroupException(
						this.getArgumentType(),
						Arrays.asList(e.getArgument()),
						message, 
						e);
			}
		}

		
		this.parsedArgumentList.clear();
		this.parsedArgumentList.addAll(parsedArgList);
	}

	@Override
	public String toString() {
		return String.format("ArgGroup [" 
				+ "ordinal=%s, " 
				+ "argumentType=%s, " 
				+ "argAcceptor=%s, " 
				+ "argsValidator=%s, " 
				+ "argParser=%s, " 
				+ "arguments=%s, " 
				+ "parsedArguments=%s]",
				this.ordinal, this.argumentType, 
				this.argAcceptor, this.argsValidator, this.argParser, 
				this.argumentList, this.parsedArgumentList);
	}
}
