/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An implementation of an {@code ArgsDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ArgsDocImpl implements ArgsDoc {
	
	/**
	 * Returns a default format string for the syntax for the {@code Args}.
	 * 
	 * @param argsDocImpl the provided {@code ArgsDocImpl}
	 *  
	 * @return a default format string for the syntax for the {@code Args}
	 */
	private static String getDefaultSyntax(final ArgsDocImpl argsDocImpl) {
		
		List<ArgGroupDoc> argGroupDocList = argsDocImpl.argGroupDocList;
		
		StringBuilder builder = new StringBuilder();
		
		for (int i = 0; i < argGroupDocList.size(); i++) {
			
			String argGroupDocSyntaxProperty =
				new Properties.Property(Strings.concat(
						ArgsDocPropertyKeys.ARGGROUPDOC_PREFIX,
						Integer.toString(i),
						".", 
						ArgGroupDocPropertyKeys.SYNTAX)).toExpandableString();
			
			builder.append(argGroupDocSyntaxProperty);
			
			if (i < argGroupDocList.size() - 1) {
			
				builder.append('\n');
			}
		}
		
		return builder.toString();
	}
	
	/**
	 * Updates the provided {@code ArgsDocImpl}'s documentation properties.
	 * 
	 * @param argsDocImpl the provided {@code ArgsDocImpl}
	 */
	private static void updateProperties(final ArgsDocImpl argsDocImpl) {
		
		Map<String, String> properties = argsDocImpl.properties;
		
		List<ArgGroupDoc> argGroupDocList = argsDocImpl.argGroupDocList;
		
		String syntax = argsDocImpl.syntax;
		
		if (argGroupDocList != null) {
			
			for (int i = 0; i < argGroupDocList.size(); i++) {
				
				ArgGroupDoc argGroupDoc = argGroupDocList.get(i);
				
				Map<String, String> argGroupDocProperties =
					argGroupDoc.getProperties();
				
				for (Map.Entry<String, String> argGroupDocProperty
						: argGroupDocProperties.entrySet()) {
					
					String argGroupDocPropertyKey =
						ArgsDocPropertyKeys.ARGGROUPDOC_PREFIX
						+ Integer.toString(i)
						+ "."
						+ argGroupDocProperty.getKey();
					
					properties.put(
							argGroupDocPropertyKey, 
							argGroupDocProperty.getValue());
				}
			}
		}
		
		if (syntax != null) {
			
			properties.put(ArgsDocPropertyKeys.SYNTAX, syntax);
		}
	}
	
	/** 
	 * The {@code Map} for the documentation properties of this 
	 * {@code ArgsDocImpl}. 
	 */
	private final Map<String, String> properties;
	
	/** The {@code List} of {@code ArgGroupDoc}s. */
	private final List<ArgGroupDoc> argGroupDocList;
	
	/**
	 * The default format string for the syntax for the {@code Args}.
	 */
	private String defaultSyntax;
	
	/**
	 * The specified format string for the syntax for the {@code Args}.
	 */	
	private String specifiedSyntax;
	
	/**
	 * The syntax for the {@code Args}.
	 */
	private String syntax;
	
	/**
	 * Constructs an {@code ArgsDocImpl}.
	 */
	ArgsDocImpl() {
		
		this.properties = new Properties();
		this.argGroupDocList = new ArrayList<ArgGroupDoc>();
		this.defaultSyntax = ArgsDocImpl.getDefaultSyntax(this);
		this.specifiedSyntax = null;
		this.syntax = Util.expand3(this.defaultSyntax, this.properties);
	}
	
	/**
	 * Constructs an {@code ArgsDocImpl} with the provided {@code ArgsDoc}.
	 * 
	 * @param argsDoc the provided {@code ArgsDoc}
	 */
	ArgsDocImpl(final ArgsDoc argsDoc) {
		
		this();
		
		this.properties.putAll(argsDoc.getProperties());
		this.argGroupDocList.addAll(argsDoc.getArgGroupDocs());
		this.syntax = argsDoc.getSyntax();
	}

	@Override
	public void addArgGroupDoc(final ArgGroupDoc argGroupDoc) {
		
		if (argGroupDoc == null) {
			throw new NullPointerException("argGroupDoc must not be null");
		}
		
		this.argGroupDocList.add(argGroupDoc);
		
		Collections.sort(
				this.argGroupDocList, 
				AbstractFactory.getDefault().getArgGroupDocComparator());
		
		this.update();
	}

	@Override
	public List<ArgGroupDoc> getArgGroupDocs() {
		return Collections.unmodifiableList(this.argGroupDocList);
	}

	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}

	@Override
	public String getSyntax() {
		return this.syntax;
	}

	@Override
	public void setSyntax(final String s) {
		
		this.specifiedSyntax = s;
		
		this.update();
	}

	@Override
	public String toString() {
		return String.format("ArgsDoc [" 
				+ "properties=%s, " 
				+ "argGroupDocs=%s, " 
				+ "syntax=%s]",
				this.properties, 
				this.argGroupDocList, 
				this.syntax);
	}

	/**
	 * Updates the syntax and help information for the {@code Args}.
	 */
	private void update() {
		
		this.syntax = null;
		
		ArgsDocImpl.updateProperties(this);
		
		this.defaultSyntax = ArgsDocImpl.getDefaultSyntax(this);
		
		if (this.specifiedSyntax != null && !this.specifiedSyntax.isEmpty()) {
			
			this.syntax = Util.expand3(
					this.specifiedSyntax, this.properties);
			
		} else {
			
			this.syntax = Util.expand3(
					this.defaultSyntax, this.properties);
		}
		
		ArgsDocImpl.updateProperties(this);
	}
}
