/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * An implementation of an {@code Args}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ArgsImpl implements Args {

	/**
	 * An implementation of an {@code ArgAcceptorContext}.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static final class ArgAcceptorContextImpl 
		implements ArgAcceptorContext {

		/** The arguments. */
		private String[] arguments;
		
		/** The arguments' availability statuses. */
		private boolean[] argumentsAvailability;
		
		/** The target index. */
		private int index;
		
		/**
		 * Constructs an {@code ArgAcceptorContextImpl} with 
		 * the provided arguments.
		 * 
		 * @param args the provided arguments
		 */
		ArgAcceptorContextImpl(final String[] args) {
			this.arguments = Arrays.copyOf(args, args.length);
			this.argumentsAvailability = new boolean[this.arguments.length];
			Arrays.fill(this.argumentsAvailability, true);
			int i = -1;
			if (this.arguments.length > 0) {
				i = 0;
			}
			this.index = i; 
		}
		
		@Override
		public String[] getArguments() {
			return Arrays.copyOf(this.arguments, this.arguments.length);
		}

		@Override
		public boolean[] getArgumentsAvailability() {
			return Arrays.copyOf(
					this.argumentsAvailability, 
					this.argumentsAvailability.length);
		}

		@Override
		public int getIndex() {
			return this.index;
		}
		
		/**
		 * Sets the argument's availability status at the specified index 
		 * with the specified availability status.
		 * 
		 * @param i the specified index
		 * @param availability the specified availability status
		 */
		void setArgumentAvailability(
				final int i, final boolean availability) {
			
			this.argumentsAvailability[i] = availability;
		}
		
		/**
		 * Sets the index of the argument in question with the specified 
		 * index of another argument in question.
		 *  
		 * @param i the specified index of another argument in question
		 */
		void setIndex(final int i) {
			this.index = i;
		}
		
	}

	/** The group of {@code ArgGroup}s. */
	private List<ArgGroup<Object>> argGroupList;
	
	/**
	 * Constructs an instance of {@code ArgsImpl}.
	 */
	ArgsImpl() {
		
		this.argGroupList = new ArrayList<ArgGroup<Object>>();
	}
	
	/**
	 * Constructs an instance of {@code ArgsImpl} with the provided 
	 * {@code Args}.
	 * 
	 * @param args the provided {@code Args}
	 */
	ArgsImpl(final Args args) {
		
		this();
		
		this.argGroupList.addAll(args.getArgGroups());
	}

	@Override
	public void addArgGroup(
			final ArgGroup<? extends Object> argGroup) {
		
		if (argGroup == null) {
			throw new NullPointerException("argGroup must not be null");
		}
		
		@SuppressWarnings("unchecked")
		ArgGroup<Object> arGroup = (ArgGroup<Object>) argGroup;
		
		this.argGroupList.add(arGroup);
		
		Collections.sort(
				this.argGroupList, 
				AbstractFactory.getDefault().getArgGroupComparator());
	}

	@Override
	public List<ArgGroup<Object>> getArgGroups() {
		return Collections.unmodifiableList(this.argGroupList);
	}
	
	@Override
	public void parse(final String[] args) throws ArgsException {
		
		List<ArgGroup<Object>> argGroups = this.argGroupList;
		
		for (ArgGroup<Object> argGroup : argGroups) {
			
			argGroup.clearArguments();
			argGroup.clearParsedArguments();
		}
		
		ArgAcceptorContextImpl argAcceptorContextImpl = 
				new ArgAcceptorContextImpl(args); 
		
		for (ArgGroup<Object> argGroup : argGroups) {
			
			for (int i = 0; i < args.length; i++) {
				
				String arg = args[i];
				
				argAcceptorContextImpl.setIndex(i);
				
				if (argAcceptorContextImpl.getArgumentsAvailability()[i]) {
					
					ArgAcceptor acceptor = argGroup.getArgAcceptor();
					
					if (acceptor.canAccept(argAcceptorContextImpl)) {
						
						argGroup.addArgument(arg);
						
						argAcceptorContextImpl.setArgumentAvailability(
								i, false);
					}
				}
			}
		}
		
		for (ArgGroup<Object> argGroup : argGroups) {
			
			try {
				
				argGroup.parse();
				
			} catch (ArgGroupException e) {

				throw new ArgsException(e.getMessage(), e);
			}
			
		}
		
		List<String> remainingArgumentList = new ArrayList<String>();
		
		for (int i = 0; i < args.length; i++) {
			
			String arg = args[i];
			
			if (argAcceptorContextImpl.getArgumentsAvailability()[i]) {
				
				remainingArgumentList.add(arg);
			}
		}
		
		if (remainingArgumentList.size() > 0) {
			
			String remainingArgumentsString = Lists.toString(
					remainingArgumentList);
			
			String message = String.format(
					"unknown argument(s) : %s",
					remainingArgumentsString);
			
			throw new ArgsException(message);
		}		
	}

	@Override
	public String toString() {
		return String.format("Args [argGroups=%s]", this.argGroupList);
	}
}
