/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.io.PrintStream;

/**
 * A program's command line interface.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public interface CLI extends Doc {
	
	/**
	 * Returns a new copy of the {@code Args}.
	 * 
	 * @return a new copy of the {@code Args}
	 */
	Args getArgs();
	
	/**
	 * Returns a new copy of the {@code ArgsDoc}.
	 * 
	 * @return a new copy of the {@code ArgsDoc}
	 */
	ArgsDoc getArgsDoc();
	
	/**
	 * Returns the help {@code OptionGroup}.
	 * 
	 * @return the help {@code OptionGroup}
	 */
	OptionGroup<Void> getHelpOptionGroup();
	
	/**
	 * Returns the help {@code OptionGroupDoc}.
	 * 
	 * @return the help {@code OptionGroupDoc}
	 */
	OptionGroupDoc getHelpOptionGroupDoc();
	
	/**
	 * Returns a new copy of the {@code Options}.
	 * 
	 * @return a new copy of the {@code Options}
	 */	
	Options getOptions();
	
	/**
	 * Returns a new copy of the {@code OptionsDoc}.
	 * 
	 * @return a new copy of the {@code OptionsDoc}
	 */	
	OptionsDoc getOptionsDoc();

	/**
	 * Returns the program description.
	 * 
	 * @return the program description
	 */
	String getProgramDescription();

	/**
	 * Returns the program help information.
	 * 
	 * @return the program help information
	 */
	String getProgramHelp();

	/**
	 * Returns the program name.
	 * 
	 * @return the program name
	 */
	String getProgramName();

	/**
	 * Returns the program usage.
	 * 
	 * @return the program usage
	 */
	String getProgramUsage();
	
	/**
	 * Returns the program version information.
	 * 
	 * @return the program version information
	 */
	String getProgramVersion();
	
	/**
	 * Returns the version {@code OptionGroup}.
	 * 
	 * @return the version {@code OptionGroup}
	 */
	OptionGroup<Void> getVersionOptionGroup();

	/**
	 * Returns the version {@code OptionGroupDoc}.
	 * 
	 * @return the version {@code OptionGroupDoc} 
	 */
	OptionGroupDoc getVersionOptionGroupDoc();
	
	/**
	 * Indicates if the help {@code OptionGroupDoc} has been excluded from 
	 * documentation.
	 * 
	 * @return {@code true} if the help {@code OptionGroupDoc} has been 
	 * excluded from documentation; otherwise {@code false}
	 */
	boolean isHelpOptionGroupHidden();
	
	/**
	 * Indicates if the help {@code OptionGroup} has been excluded from parsing.
	 * 
	 * @return {@code true} if the help {@code OptionGroup} has been excluded 
	 * from parsing; otherwise {@code false} 
	 */
	boolean isHelpOptionGroupUnused();
	
	/**
	 * Indicates if the version {@code OptionGroupDoc} has been excluded from 
	 * documentation.
	 * 
	 * @return {@code true} if the version {@code OptionGroupDoc} has been 
	 * excluded from documentation; otherwise {@code false}
	 */
	boolean isVersionOptionGroupHidden();
	
	/**
	 * Indicates if the version {@code OptionGroup} has been excluded from 
	 * parsing.
	 * 
	 * @return {@code true} if the version {@code OptionGroup} has been 
	 * excluded from parsing; otherwise {@code false}
	 */
	boolean isVersionOptionGroupUnused();

	/**
	 * Processes the supplied command line arguments.
	 * 
	 * @param args the supplied command line arguments
	 * 
	 * @return the {@code CLIProcessResult} 
	 * 
	 * @throws CLIException if the supplied command line arguments 
	 * cannot be processed
	 */
	CLIProcessResult process(final String[] args) throws CLIException;
	
	/**
	 * Processes the supplied command line arguments and in turn--depending 
	 * on the arguments--prints the appropriate response using the provided
	 * {@code PrintStream}.
	 * <p>
	 * If one of the supplied command line arguments is one of the help 
	 * {@code Option}s and that {@code Option} is not excluded from parsing, 
	 * the provided {@code PrintStream} will print the program help information.
	 * </p>
	 * <p>
	 * If one of the supplied command line arguments is one of the version 
	 * {@code Option}s and that {@code Option} is not excluded from parsing and 
	 * the program version information is provided, the provided 
	 * {@code PrintStream} will print the program version information.
	 * </p>
	 * <p>
	 * If both of the aforementioned {@code Option}s are of the supplied 
	 * command line arguments, the provided {@code PrintStream} will either 
	 * print the program help information or the program version information 
	 * depending on which is first.
	 * </p>
	 * 
	 * @param args the supplied command line arguments
	 * 
	 * @param outStream the provided {@code PrintStream} to print out the 
	 * appropriate response
	 * 
	 * @return the {@code CLIProcessResult} 
	 * 
	 * @throws CLIException if the supplied command line arguments 
	 * cannot be processed
	 */
	CLIProcessResult process(
			final String[] args, 
			final PrintStream outStream) 
			throws CLIException;

	/**
	 * Processes the supplied command line arguments and in turn--depending 
	 * on the arguments--prints the appropriate response using the provided
	 * {@code PrintStream} or the provided error {@code PrintStream}.
	 * <p>
	 * If one of the supplied command line arguments is one of the help 
	 * {@code Option}s and that {@code Option} is not excluded from parsing, 
	 * the provided {@code PrintStream} will print the program help information.
	 * </p>
	 * <p>
	 * If one of the supplied command line arguments is one of the version 
	 * {@code Option}s and that {@code Option} is not excluded from parsing and 
	 * the program version information is provided, the provided 
	 * {@code PrintStream} will print the program version information.
	 * </p>
	 * <p>
	 * If both of the aforementioned options are of the supplied command line 
	 * arguments, the provided {@code PrintStream} will either print the 
	 * program help information or the program version information depending on 
	 * which is first.
	 * </p>
	 * <p>
	 * If one of the command line arguments cannot be processed, the provided
	 * error {@code PrintStream} will print the error message. If any or all of 
	 * the help {@code Option}s are not excluded from documentation and 
	 * parsing, the provided error {@code PrintStream} will print a message 
	 * suggesting to use the first included help {@code Option} for more 
	 * information.
	 * </p>
	 * 
	 * @param args the supplied command line arguments
	 * 
	 * @param outStream the provided {@code PrintStream} to print out the 
	 * appropriate response
	 * 
	 * @param errStream the provided error {@code PrintStream} to print out the 
	 * appropriate error response
	 * 
	 * @return the {@code CLIProcessResult} 
	 */
	CLIProcessResult process(
			final String[] args, 
			final PrintStream outStream, 
			final PrintStream errStream);
}