/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.List;

/**
 * A builder for a {@code CLI}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 */
public interface CLIBuilder {

	/**
	 * Sets the {@code Args} using the provided {@code Args}.
	 * 
	 * @param a the provided {@code Args}
	 * 
	 * @return this builder
	 */
	CLIBuilder args(final Args a);
	
	/**
	 * Sets the {@code Args} and its {@code ArgsDoc} using the provided 
	 * {@code ArgsComplete}.
	 * 
	 * @param a the provided {@code ArgsComplete}
	 * 
	 * @return this builder
	 */
	CLIBuilder argsComplete(final ArgsComplete a);
	
	/**
	 * Sets the {@code ArgsDoc} using the provided {@code ArgsDoc}.
	 * 
	 * @param a the provided {@code ArgsDoc}
	 * 
	 * @return this builder
	 */
	CLIBuilder argsDoc(final ArgsDoc a);
	
	/**
	 * Sets the format string for the syntax for the {@code Args}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.ArgsDoc#setSyntax(String)} method.
	 * </p>
	 * 
	 * @param s the provided format string for the syntax for the 
	 * {@code Args}
	 * 
	 * @return this builder
	 */		
	CLIBuilder argsDocSyntax(final String s);
	
	/**
	 * Builds a new {@code CLI}.
	 * 
	 * @return a new {@code CLI}
	 */
	CLI buildCLI();
	
	/**
	 * Sets the help {@code Option}s and their {@code OptionDoc}s using the 
	 * provided {@code List} of help {@code OptionComplete}s.
	 * 
	 * @param l the provided {@code List} of help {@code OptionComplete}s
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionCompletes(final List<OptionComplete> l);
	
	/**
	 * Sets the help {@code OptionDoc}s using the provided {@code List} of 
	 * help {@code OptionDoc}s.
	 * 
	 * @param l the provided {@code List} of help {@code OptionDoc}s
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionDocs(final List<OptionDoc> l);
	
	/**
	 * Sets the format string for the description of the help 
	 * {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#description(String)} method.
	 * </p>
	 *  
	 * @param s the format string for the description of the help 
	 * {@code OptionGroup}
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionGroupDocDescription(final String s);
	
	/**
	 * Sets the format string for the help {@code OptionGroup}'s help 
	 * information.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#help(String)} method.
	 * </p>
	 *   
	 * @param s the provided format string for the help 
	 * {@code OptionGroup}'s help information
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionGroupDocHelp(final String s);
	
	/**
	 * Sets the ordinal of the help {@code OptionGroupDoc} in relation to 
	 * other {@code OptionGroupDoc}s.
	 * 
	 * @param i the provided ordinal of the help {@code OptionGroupDoc} in 
	 * relation to other {@code OptionGroupDoc}s
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionGroupDocOrdinal(final int i);
	
	/**
	 * Sets the format string for the usage of the help {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#usage(String)} method.
	 * </p>
	 *  
	 * @param s the provided format string for the usage of the help 
	 * {@code OptionGroup}
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionGroupDocUsage(final String s);
	
	/**
	 * Sets the boolean value to indicate if the help 
	 * {@code OptionGroupDoc} is excluded from documentation.
	 * 
	 * @param b the provided boolean value to indicate if the help 
	 * {@code OptionGroupDoc} is excluded from documentation
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionGroupHidden(final boolean b);
	
	/**
	 * Sets the boolean value to indicate if the help {@code OptionGroup} 
	 * is excluded from parsing.
	 * 
	 * @param b the provided boolean value to indicate if the help 
	 * {@code OptionGroup} is excluded from parsing
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptionGroupUnused(final boolean b);
	
	/**
	 * Sets the help {@code Option}s using the provided {@code List} of help 
	 * {@code Option}s.
	 * 
	 * @param l the provided {@code List} of help {@code Option}s
	 * 
	 * @return this builder
	 */
	CLIBuilder helpOptions(final List<Option> l);
	
	/**
	 * Sets the {@code Options} using the provided {@code Options}.
	 * 
	 * @param o the provided {@code Options}
	 * 
	 * @return this builder
	 */
	CLIBuilder options(final Options o);
	
	/**
	 * Sets the {@code Options} and its {@code OptionsDoc} using the provided 
	 * {@code OptionsComplete}.
	 * 
	 * @param o the provided {@code OptionsComplete}
	 * 
	 * @return this builder
	 */
	CLIBuilder optionsComplete(final OptionsComplete o);
	
	/**
	 * Sets the {@code OptionsDoc} using the provided {@code OptionsDoc}.
	 * 
	 * @param o the provided {@code OptionsDoc}
	 * 
	 * @return this builder
	 */
	CLIBuilder optionsDoc(final OptionsDoc o);
	
	/**
	 * Sets the format string for the help information for the 
	 * {@code Options}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionsDoc#setHelp(String)} method.
	 * </p>
	 *  
	 * @param s the provided format string for the help information for the 
	 * {@code Options}
	 * 
	 * @return this builder
	 */		
	CLIBuilder optionsDocHelp(final String s);
	
	/**
	 * Sets the format string for the usage of the {@code Options}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionsDoc#setUsage(String)} method.
	 * </p>
	 *  
	 * @param s the provided format string for the usage of the 
	 * {@code Options}
	 * 
	 * @return this builder
	 */		
	CLIBuilder optionsDocUsage(final String s);
	
	/**
	 * Sets the format string for the program description.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by 
	 * a dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code ArgGroupDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code ArgGroupDoc} and <i>n</i> is the last {@code ArgGroupDoc}), 
	 * a dot (.), and the {@code ArgGroupDoc}'s property key defined in 
	 * {@link org.climax.ArgGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}) followed by a 
	 * dot (.) and the {@code ArgsDoc}'s property key defined in
	 * {@link org.climax.ArgsDocPropertyKeys}.</td></tr> 
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code OptionDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code OptionDoc} of the first {@code OptionGroupDoc} and <i>n</i> 
	 * is the last {@code OptionDoc} of the last {@code OptionGroupDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property key defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}) followed 
	 * by an integer representing the index of an {@code OptionGroupDoc} 
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionGroupDoc} and <i>n</i> is the last 
	 * {@code OptionGroupDoc}), a dot (.), and the 
	 * {@code OptionGroupDoc}'s property key defined in 
	 * {@link org.climax.OptionGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}) followed by 
	 * a dot (.) and the {@code OptionsDoc}'s property key defined in 
	 * {@link org.climax.OptionsDocPropertyKeys}.</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_NAME}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_NAME})</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_USAGE}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_USAGE})</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_VERSION}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_VERSION})</td></tr> 
	 * </table>
	 * 
	 * @param s the provided format string for the program description
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a>
	 */		
	CLIBuilder programDescription(final String s);
	
	/**
	 * Sets the format string for the program help information body.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by 
	 * a dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr> 
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code ArgGroupDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code ArgGroupDoc} and <i>n</i> is the last {@code ArgGroupDoc}), 
	 * a dot (.), and the {@code ArgGroupDoc}'s property key defined in 
	 * {@link org.climax.ArgGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}) followed by a 
	 * dot (.) and the {@code ArgsDoc}'s property key defined in
	 * {@link org.climax.ArgsDocPropertyKeys}.</td></tr> 
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code OptionDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code OptionDoc} of the first {@code OptionGroupDoc} and <i>n</i> 
	 * is the last {@code OptionDoc} of the last {@code OptionGroupDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property key defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}) followed 
	 * by an integer representing the index of an {@code OptionGroupDoc} 
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionGroupDoc} and <i>n</i> is the last 
	 * {@code OptionGroupDoc}), a dot (.), and the 
	 * {@code OptionGroupDoc}'s property key defined in 
	 * {@link org.climax.OptionGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}) followed by 
	 * a dot (.) and the {@code OptionsDoc}'s property key defined in 
	 * {@link org.climax.OptionsDocPropertyKeys}.</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_DESCRIPTION}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_DESCRIPTION})</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_NAME}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_NAME})</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_USAGE}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_USAGE})</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_VERSION}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_VERSION})</td></tr>
	 * </table>
	 * 
	 * @param s the provided format string for the program help information
	 * body
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */
	CLIBuilder programHelpBody(final String s);
	
	/**
	 * Sets the format string for the program help information footer.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link #programHelpBody} method.
	 * </p>
	 * 
	 * @param s the provided format string for the program help information
	 * footer
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */		
	CLIBuilder programHelpFooter(final String s);
	
	/**
	 * Sets the format string for the program help information header.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link #programHelpBody} method.
	 * </p>
	 * 
	 * @param s the provided format string for the program help information
	 * header
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */		
	CLIBuilder programHelpHeader(final String s);
	
	/**
	 * Sets the format string for the program usage.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by 
	 * a dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code ArgGroupDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code ArgGroupDoc} and <i>n</i> is the last {@code ArgGroupDoc}), 
	 * a dot (.), and the {@code ArgGroupDoc}'s property key defined in 
	 * {@link org.climax.ArgGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}) followed by a 
	 * dot (.) and the {@code ArgsDoc}'s property key defined in
	 * {@link org.climax.ArgsDocPropertyKeys}.</td></tr> 
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code OptionDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code OptionDoc} of the first {@code OptionGroupDoc} and <i>n</i> 
	 * is the last {@code OptionDoc} of the last {@code OptionGroupDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property key defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}) followed 
	 * by an integer representing the index of an {@code OptionGroupDoc} 
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionGroupDoc} and <i>n</i> is the last 
	 * {@code OptionGroupDoc}), a dot (.), and the 
	 * {@code OptionGroupDoc}'s property key defined in 
	 * {@link org.climax.OptionGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}) followed by 
	 * a dot (.) and the {@code OptionsDoc}'s property key defined in 
	 * {@link org.climax.OptionsDocPropertyKeys}.</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_NAME}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_NAME})</td></tr>
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_VERSION}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_VERSION})</td></tr>
	 * </table>
	 * 
	 * @param s the provided format string for the program usage
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */		
	CLIBuilder programUsage(final String s);
	
	/**
	 * Sets the format string for the program version information.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by 
	 * a dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGGROUPDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code ArgGroupDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code ArgGroupDoc} and <i>n</i> is the last {@code ArgGroupDoc}), 
	 * a dot (.), and the {@code ArgGroupDoc}'s property key defined in 
	 * {@link org.climax.ArgGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#ARGSDOC_PREFIX}) followed by a 
	 * dot (.) and the {@code ArgsDoc}'s property key defined in
	 * {@link org.climax.ArgsDocPropertyKeys}.</td></tr> 
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONDOC_PREFIX}) followed by 
	 * an integer representing the index of an {@code OptionDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code OptionDoc} of the first {@code OptionGroupDoc} and <i>n</i> 
	 * is the last {@code OptionDoc} of the last {@code OptionGroupDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property key defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONGROUPDOC_PREFIX}) followed 
	 * by an integer representing the index of an {@code OptionGroupDoc} 
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionGroupDoc} and <i>n</i> is the last 
	 * {@code OptionGroupDoc}), a dot (.), and the 
	 * {@code OptionGroupDoc}'s property key defined in 
	 * {@link org.climax.OptionGroupDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with
	 * {@link org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}
	 * ({@value org.climax.CLIPropertyKeys#OPTIONSDOC_PREFIX}) followed by 
	 * a dot (.) and the {@code OptionsDoc}'s property key defined in 
	 * {@link org.climax.OptionsDocPropertyKeys}.</td></tr> 
	 * <tr><td>{@link org.climax.CLIPropertyKeys#PROGRAM_NAME}
	 * ({@value org.climax.CLIPropertyKeys#PROGRAM_NAME})</td></tr>
	 * </table>
	 * <p> 
	 * If the provided format string is not {@code null} or empty, the 
	 * included version {@code Option}s and their included 
	 * {@code OptionDoc}s are added to the command line interface.
	 * </p> 
	 * 
	 * @param s the provided format string for the program version
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */
	CLIBuilder programVersion(final String s);
	
	/**
	 * Sets the version {@code Option}s and their {@code OptionDoc}s using the 
	 * provided {@code List} of version {@code OptionComplete}s.
	 * 
	 * @param l the provided {@code List} of version {@code OptionComplete}s
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionCompletes(final List<OptionComplete> l);
	
	/**
	 * Sets the version {@code OptionDoc}s using the provided {@code List} of 
	 * version {@code OptionDoc}s.
	 * 
	 * @param l the provided {@code List} of version {@code OptionDoc}s
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionDocs(final List<OptionDoc> l);
	
	/**
	 * Sets the format string for the description of the version 
	 * {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#description(String)} method.
	 * </p>
	 * 
	 * @param s the provided format string for the description of the 
	 * version {@code OptionGroup}
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionGroupDocDescription(final String s);
	
	/**
	 * Sets the format string for the version {@code OptionGroup}'s help 
	 * information.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#help(String)} method.
	 * </p>
	 * 
	 * @param s the provided format string for the version 
	 * {@code OptionGroup}'s help information
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionGroupDocHelp(final String s);
	
	/**
	 * Sets the ordinal of the version {@code OptionGroupDoc} in relation 
	 * to other {@code OptionGroupDoc}s.
	 * 
	 * @param i the provided ordinal of the version {@code OptionGroupDoc} 
	 * in relation to other {@code OptionGroupDoc}s
	 * 
	 * @return this builder
	 */		
	CLIBuilder versionOptionGroupDocOrdinal(final int i);
	
	/**
	 * Sets the format string for the usage of the version 
	 * {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#usage(String)} method.
	 * </p>
	 *  
	 * @param s the provided format string for the usage of the version 
	 * {@code OptionGroup}
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionGroupDocUsage(final String s);
	
	/**
	 * Sets the boolean value to indicate if the version 
	 * {@code OptionGroupDoc} is excluded from documentation.
	 * 
	 * @param b the provided boolean value to indicate if the version 
	 * {@code OptionGroup}'s documentation is excluded
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionGroupHidden(final boolean b);
	
	/**
	 * Sets the boolean value to indicate if the version 
	 * {@code OptionGroup} is excluded from parsing.
	 * 
	 * @param b the provided boolean value to indicate if the version 
	 * {@code OptionGroup} is excluded from parsing
	 * 
	 * @return this builder
	 */
	CLIBuilder versionOptionGroupUnused(final boolean b);
	
	/**
	 * Sets the version {@code Option}s using the provided {@code List} of 
	 * version {@code Option}s.
	 * 
	 * @param l the provided {@code List} of version {@code Option}s
	 * 
	 * @return this builder
	 */	
	CLIBuilder versionOptions(final List<Option> l);
}
