/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An implementation of a {@code CLI}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class CLIImpl implements CLI {

	/**
	 * A builder for a {@code CLIImpl}. 
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	static final class Builder implements CLIBuilder {
		
		/** The format string for the program name. */
		private final String programName;
		
		/** The format string for the program usage. */
		private String programUsage;
		
		/** The format string for the program description. */
		private String programDescription;
		
		/** The format string for the program help information header. */
		private String programHelpHeader;
		
		/** The format string for the program help information body. */
		private String programHelpBody;
		
		/** The format string for the program help information footer. */
		private String programHelpFooter;
		
		/** The format string for the program version information. */
		private String programVersion;
		
		/**
		 * The {@code List} of help {@code Option}s.
		 */
		private List<Option> helpOptionList;
		
		/**
		 * The {@code List} of help {@code OptionDoc}s.
		 */
		private List<OptionDoc> helpOptionDocList;
		
		/** 
		 * The format string for the description of the help 
		 * {@code OptionGroup}. 
		 */
		private String helpOptionGroupDocDescription;
		
		/** 
		 * The format string for the help {@code OptionGroup}'s help 
		 * information. 
		 */
		private String helpOptionGroupDocHelp;
		
		/** 
		 * The ordinal of the help {@code OptionGroupDoc} in relation to other 
		 * {@code OptionGroupDoc}s. 
		 */
		private int helpOptionGroupDocOrdinal;
		
		/**
		 * The format string for the usage of the help {@code OptionGroup}.
		 */
		private String helpOptionGroupDocUsage;
		
		/**
		 * The boolean value to indicate if the help {@code OptionGroupDoc} 
		 * is excluded from documentation.
		 */
		private boolean helpOptionGroupHidden;
		
		/**
		 * The boolean value to indicate if the help {@code OptionGroup} is 
		 * excluded from parsing.
		 */
		private boolean helpOptionGroupUnused;

		/**
		 * The {@code List} of version {@code Option}s.
		 */
		private List<Option> versionOptionList;
		
		/**
		 * The {@code List} of version {@code OptionDoc}s.
		 */
		private List<OptionDoc> versionOptionDocList;
		
		/** 
		 * The format string for the description of the version 
		 * {@code OptionGroup}.
		 */
		private String versionOptionGroupDocDescription;
		
		/** 
		 * The format string for the version {@code OptionGroup}'s help 
		 * information. 
		 */
		private String versionOptionGroupDocHelp;
		
		/** 
		 * The ordinal of the version {@code OptionGroupDoc} in relation to 
		 * other {@code OptionGroupDoc}s. 
		 */
		private int versionOptionGroupDocOrdinal;
		
		/**
		 * The format string for the usage of the version {@code OptionGroup}.
		 */
		private String versionOptionGroupDocUsage;
		
		/**
		 * The boolean value to indicate if the version {@code OptionGroupDoc} 
		 * is excluded from documentation.
		 */
		private boolean versionOptionGroupHidden;
		
		/**
		 * The boolean value to indicate if the version {@code OptionGroup} is 
		 * excluded from parsing.
		 */
		private boolean versionOptionGroupUnused;
		
		/**
		 * The {@code Options}.
		 */
		private Options options;
		
		/**
		 * The {@code OptionsDoc}.
		 */
		private OptionsDoc optionsDoc;
		
		/** 
		 * The format string for the help information for the 
		 * {@code Options}. 
		 */
		private String optionsDocHelp;
		
		/**
		 * The format string for the usage of the {@code Options}.
		 */
		private String optionsDocUsage;
		
		/**
		 * The {@code Args}.
		 */
		private Args args;
		
		/**
		 * The {@code ArgsDoc}.
		 */
		private ArgsDoc argsDoc;
		
		/**
		 * The format string for the syntax for the {@code Args}.
		 */
		private String argsDocSyntax;

		/**
		 * Constructs a {@code CLIImpl.Builder} with the provided format string
		 * for the program name.
		 * <p>
		 * The following are property keys that can be applied as expandable 
		 * properties within the provided format string:
		 * </p>
		 * <ul>
		 * <li>Property keys defined in {@link System#getProperties()}</li>
		 * </ul>
		 *  
		 * @param progName the provided format string for the program name
		 */
		public Builder(final String progName) {
			this.programName = progName;
			this.programUsage = null;
			this.programDescription = null;
			this.programHelpHeader = null;
			this.programHelpBody = null;
			this.programHelpFooter = null;
			this.programVersion = null;
			this.helpOptionList = new ArrayList<Option>();
			this.helpOptionDocList = new ArrayList<OptionDoc>();
			this.helpOptionGroupDocDescription = null;
			this.helpOptionGroupDocHelp = null;
			this.helpOptionGroupDocOrdinal = Integer.MAX_VALUE;
			this.helpOptionGroupDocUsage = null;			
			this.helpOptionGroupHidden = false;			
			this.helpOptionGroupUnused = false;
			this.versionOptionList = new ArrayList<Option>();
			this.versionOptionDocList = new ArrayList<OptionDoc>();
			this.versionOptionGroupDocDescription = null;
			this.versionOptionGroupDocHelp = null;
			this.versionOptionGroupDocOrdinal = Integer.MAX_VALUE;
			this.versionOptionGroupDocUsage = null;
			this.versionOptionGroupHidden = false;
			this.versionOptionGroupUnused = false;
			this.options = null;
			this.optionsDoc = null;
			this.optionsDocHelp = null;
			this.optionsDocUsage = null;
			this.args = null;
			this.argsDoc = null;
			this.argsDocSyntax = null;
		}

		@Override
		public CLIBuilder args(final Args a) {
			this.args = AbstractFactory.getDefault().newArgs(a);
			return this;
		}
		
		@Override
		public CLIBuilder argsComplete(final ArgsComplete a) {
			this.args = AbstractFactory.getDefault().newArgs(a);
			this.argsDoc = AbstractFactory.getDefault().newArgsDoc(a);
			return this;
		}
		
		@Override
		public CLIBuilder argsDoc(final ArgsDoc a) {
			this.argsDoc = AbstractFactory.getDefault().newArgsDoc(a);
			return this;
		}
		
		@Override		
		public CLIBuilder argsDocSyntax(final String s) {
			this.argsDocSyntax = s;
			return this;
		}
		
		@Override
		public CLI buildCLI() {
			
			CLI cli = null;
			
			try {
				cli = new CLIImpl(this);
			} catch (RuntimeException e) {
				throw new IllegalStateException(e);
			}
			
			return cli;
		}
		
		@Override
		public CLIBuilder helpOptionCompletes(final List<OptionComplete> l) {
			this.helpOptionDocList.clear();
			this.helpOptionList.clear();
			for (OptionComplete o : l) {
				if (!o.isHidden()) {
					this.helpOptionDocList.add(o);
				}
				if (!o.isUnused()) {
					this.helpOptionList.add(o);
				}
			}
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionDocs(final List<OptionDoc> l) {
			this.helpOptionDocList.clear();
			this.helpOptionDocList.addAll(l);
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionGroupDocDescription(final String s) {
			this.helpOptionGroupDocDescription = s;
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionGroupDocHelp(final String s) {
			this.helpOptionGroupDocHelp = s;
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionGroupDocOrdinal(final int i) {
			this.helpOptionGroupDocOrdinal = i;
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionGroupDocUsage(final String s) {
			this.helpOptionGroupDocUsage = s;
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionGroupHidden(final boolean b) {
			this.helpOptionGroupHidden = b;
			return this;
		}
		
		@Override
		public CLIBuilder helpOptionGroupUnused(final boolean b) {
			this.helpOptionGroupUnused = b;
			return this;
		}
		
		@Override
		public CLIBuilder helpOptions(final List<Option> l) {
			this.helpOptionList.clear();
			this.helpOptionList.addAll(l);
			return this;
		}
		
		@Override
		public CLIBuilder options(final Options o) {
			this.options = AbstractFactory.getDefault().newOptions(o);
			return this;
		}
		
		@Override
		public CLIBuilder optionsComplete(final OptionsComplete o) {
			this.options = AbstractFactory.getDefault().newOptions(o);
			this.optionsDoc = AbstractFactory.getDefault().newOptionsDoc(o);
			return this;
		}
		
		@Override
		public CLIBuilder optionsDoc(final OptionsDoc o) {
			this.optionsDoc = AbstractFactory.getDefault().newOptionsDoc(o);
			return this;
		}
		
		@Override		
		public CLIBuilder optionsDocHelp(final String s) {
			this.optionsDocHelp = s;
			return this;
		}

		@Override		
		public CLIBuilder optionsDocUsage(final String s) {
			this.optionsDocUsage = s;
			return this;
		}

		@Override		
		public CLIBuilder programDescription(final String s) {
			this.programDescription = s;
			return this;
		}

		@Override
		public CLIBuilder programHelpBody(final String s) {
			this.programHelpBody = s;
			return this;
		}

		@Override		
		public CLIBuilder programHelpFooter(final String s) {
			this.programHelpFooter = s;
			return this;
		}

		@Override		
		public CLIBuilder programHelpHeader(final String s) {
			this.programHelpHeader = s;
			return this;
		}

		@Override		
		public CLIBuilder programUsage(final String s) {
			this.programUsage = s;
			return this;
		}

		@Override
		public CLIBuilder programVersion(final String s) {
			this.programVersion = s;
			return this;
		}

		@Override
		public CLIBuilder versionOptionCompletes(final List<OptionComplete> l) {
			this.versionOptionDocList.clear();
			this.versionOptionList.clear();
			for (OptionComplete o : l) {
				if (!o.isHidden()) {
					this.versionOptionDocList.add(o);
				}
				if (!o.isUnused()) {
					this.versionOptionList.add(o);
				}
			}
			return this;
		}


		@Override
		public CLIBuilder versionOptionDocs(final List<OptionDoc> l) {
			this.versionOptionDocList.clear();
			this.versionOptionDocList.addAll(l);
			return this;
		}

		@Override
		public CLIBuilder versionOptionGroupDocDescription(final String s) {
			this.versionOptionGroupDocDescription = s;
			return this;
		}

		@Override
		public CLIBuilder versionOptionGroupDocHelp(final String s) {
			this.versionOptionGroupDocHelp = s;
			return this;
		}

		@Override		
		public CLIBuilder versionOptionGroupDocOrdinal(final int i) {
			this.versionOptionGroupDocOrdinal = i;
			return this;
		}

		@Override
		public CLIBuilder versionOptionGroupDocUsage(final String s) {
			this.versionOptionGroupDocUsage = s;
			return this;
		}

		@Override
		public CLIBuilder versionOptionGroupHidden(final boolean b) {
			this.versionOptionGroupHidden = b;
			return this;
		}

		@Override
		public CLIBuilder versionOptionGroupUnused(final boolean b) {
			this.versionOptionGroupUnused = b;
			return this;
		}

		@Override
		public CLIBuilder versionOptions(final List<Option> l) {
			this.versionOptionList.clear();
			this.versionOptionList.addAll(l);
			return this;
		}

	}
	

	/**
	 * Returns an {@code Args} instance based on the provided 
	 * {@code CLIImpl.Builder}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 * 
	 * @return an {@code Args} instance based on the provided
	 * {@code CLIImpl.Builder}
	 */
	private static Args getArgs(final Builder builder) {
		
		Args ars = builder.args;
		
		if (ars == null) { ars = AbstractFactory.getDefault().newArgs(); }
		
		Args args = AbstractFactory.getDefault().newArgs(ars);
		
		return args;
	}
	
	/**
	 * Returns an {@code ArgsDoc} based on the provided 
	 * {@code CLIImpl.Builder}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 * 
	 * @return an {@code ArgsDoc} based on the provided
	 * {@code CLIImpl.Builder}
	 */
	private static ArgsDoc getArgsDoc(final Builder builder) {
		
		ArgsDoc arsDoc = builder.argsDoc;
		String arsDocSyntax = builder.argsDocSyntax;
		
		if (arsDoc == null) { 
			arsDoc = AbstractFactory.getDefault().newArgsDoc(); 
		}
		
		ArgsDoc argsDoc = AbstractFactory.getDefault().newArgsDoc(arsDoc);
		argsDoc.setSyntax(arsDocSyntax);
		
		return argsDoc;
	}
	
	/**
	 * Returns a concatenated format string for the program help information.
	 *  
	 * @param programHelpHeader the provided format string for the program
	 * help information header
	 * 
	 * @param programHelpBody the provided format string for the program
	 * help information body
	 * 
	 * @param programHelpFooter the provided format string for the program
	 * help information footer
	 * 
	 * @return a concatenated format string for the program help information
	 */
	private static String getConcatProgramHelp(
			final String programHelpHeader,
			final String programHelpBody,
			final String programHelpFooter) {
		
		StringBuilder builder = new StringBuilder();
		
		if (programHelpHeader != null && !programHelpHeader.isEmpty()) {
			builder.append(programHelpHeader);
		}
		
		builder.append(programHelpBody);
		
		if (programHelpFooter != null && !programHelpFooter.isEmpty()) {
			builder.append(programHelpFooter);
		}
		
		return builder.toString();
	}
	
	/**
	 * Returns a default format string for the program help information body.
	 * 
	 * @param cliImpl the provided {@code CLIImpl}
	 * 
	 * @return a default format string for the program help information body
	 */
	private static String getDefaultProgramHelpBody(final CLIImpl cliImpl) {

		OptionsDoc optionsDoc = cliImpl.optionsDoc;
		
		OptionGroupDoc helpOptionGroupDoc = cliImpl.helpOptionGroupDoc;
		OptionGroupDoc versionOptionGroupDoc = cliImpl.versionOptionGroupDoc;
		
		String programVersion = cliImpl.programVersion;
		String programUsage = cliImpl.programUsage;
		String programDescription = cliImpl.programDescription;
		
		String optionsDocHelp = optionsDoc.getHelp();

		String helpOptionGroupDocHelp = helpOptionGroupDoc.getHelp();
		
		String versionOptionGroupDocHelp = "";
		
		if (programVersion.isEmpty()) { 
			versionOptionGroupDocHelp = versionOptionGroupDoc.getHelp();
		}

		boolean optionsDocHelpEmpty = 
				optionsDocHelp.isEmpty() 
				&& helpOptionGroupDocHelp.isEmpty() 
				&& versionOptionGroupDocHelp.isEmpty();

		String optionsDocHelpFormat = "";
		
		if (!optionsDocHelpEmpty) { 
			optionsDocHelpFormat = String.format(
					"\nOPTIONS:\n%s\n", 
					new Properties.Property(Strings.concat(
							CLIPropertyKeys.OPTIONSDOC_PREFIX, 
							".", 
							OptionsDocPropertyKeys.HELP)).toExpandableString());
		}
		
		String newLine = "";
		
		if (!programDescription.isEmpty()) {
			newLine = "\n";
		}

		String anotherNewLine = "";

		if (!optionsDocHelpFormat.isEmpty()) {
			anotherNewLine = "\n";
		}
		
		String defaultProgramHelpBody = String.format(
				"Usage: %1$s%2$s%3$s%4$s%5$s",
				programUsage,
				newLine,
				programDescription,
				anotherNewLine, 
				optionsDocHelpFormat);
		
		return defaultProgramHelpBody;
	}
	
	/**
	 * Returns a default format string for the program usage.
	 * 
	 * @param cliImpl the provided {@code CLIImpl}
	 * 
	 * @return a default format string for the program usage
	 */	
	private static String getDefaultProgramUsage(final CLIImpl cliImpl) {
		
		OptionsDoc optionsDoc = cliImpl.optionsDoc;
		ArgsDoc argsDoc = cliImpl.argsInstanceDoc;
		
		OptionGroupDoc helpOptionGroupDoc = cliImpl.helpOptionGroupDoc;
		OptionGroupDoc versionOptionGroupDoc = cliImpl.versionOptionGroupDoc;
		
		String programVersion = cliImpl.programVersion;
		
		String optionsDocUsage = optionsDoc.getUsage();
		
		String argsDocSyntax = argsDoc.getSyntax();

		String helpOptionGroupDocUsage = helpOptionGroupDoc.getUsage();
		
		String versionOptionGroupDocUsage = "";
		
		if (programVersion.isEmpty()) {
			versionOptionGroupDocUsage = 
				versionOptionGroupDoc.getUsage();
		}

		boolean optionsDocUsageEmpty = 
				optionsDocUsage.isEmpty() 
				&& helpOptionGroupDocUsage.isEmpty() 
				&& versionOptionGroupDocUsage.isEmpty();

		boolean argsDocSyntaxEmpty = argsDocSyntax.isEmpty();

		String optionsDocUsageFormat = "";
		
		if (!optionsDocUsageEmpty) { 
			optionsDocUsageFormat = String.format(
					" %s", 
					new Properties.Property(Strings.concat(
							CLIPropertyKeys.OPTIONSDOC_PREFIX, 
							".", 
							OptionsDocPropertyKeys.USAGE))
							.toExpandableString()); 
		}

		String argsDocSyntaxFormat = "";
		
		if (!argsDocSyntaxEmpty) { 
			argsDocSyntaxFormat = String.format(
					" %s", 
					new Properties.Property(Strings.concat(
							CLIPropertyKeys.ARGSDOC_PREFIX, 
							".", 
							ArgsDocPropertyKeys.SYNTAX)).toExpandableString());
		}
		
		String defaultProgramUsage = String.format(
				"%1$s%2$s%3$s",
				new Properties.Property(
						CLIPropertyKeys.PROGRAM_NAME).toExpandableString(), 
				optionsDocUsageFormat,
				argsDocSyntaxFormat);
		
		return defaultProgramUsage;
	}

	/**
	 * Returns the help {@code OptionGroup} from the provided 
	 * {@code CLIImpl.Builder}.
	 *  
	 * @param builder the provided {@code CLIImpl.Builder}
	 * @return the help {@code OptionGroup}
	 */
	private static OptionGroup<Void> getHelpOptionGroup(
			final Builder builder) {
		
		List<Option> helpOptList = new ArrayList<Option>(
				builder.helpOptionList);
		
		if (helpOptList.size() == 0) {
			
			Option option = AbstractFactory.getDefault().newOption(
					OptionType.GNU_LONG, "help");
			
			helpOptList.add(option);
		}
		OptionGroupBuilder<Void> optionGroupBuilder = 
				AbstractFactory.getDefault().newOptionGroupBuilder(helpOptList);
		
		OptionGroup<Void> helpOptionGroup =	
				optionGroupBuilder.buildOptionGroup();
	
		return helpOptionGroup;
	}
	
	/**
	 * Returns the help {@code OptionGroupDoc} from the provided 
	 * {@code CLIImpl.Builder} and the provided default help {@code Option}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 * @param helpOption the provided default help {@code Option}
	 * @return the help {@code OptionGroupDoc}
	 */
	private static OptionGroupDoc getHelpOptionGroupDoc(
			final Builder builder, final Option helpOption) {
		
		List<OptionDoc> helpOptDocList = new ArrayList<OptionDoc>(
				builder.helpOptionDocList);
		String helpOptGrpDocDescription = builder.helpOptionGroupDocDescription;
		String helpOptGrpDocHelp = builder.helpOptionGroupDocHelp;
		int helpOptGrpDocOrdinal = builder.helpOptionGroupDocOrdinal;
		String helpOptGrpDocUsage = builder.helpOptionGroupDocUsage;
		
		if (helpOptDocList.size() == 0) {
			
			OptionDocBuilder optionDocBuilder = 
					AbstractFactory.getDefault().newOptionDocBuilder(
							helpOption);
			
			OptionDoc optionDoc = optionDocBuilder.buildOptionDoc();
			
			helpOptDocList.add(optionDoc);
		}
		
		OptionGroupDocBuilder optionGroupDocBuilder =
				AbstractFactory.getDefault().newOptionGroupDocBuilder(
						helpOptDocList);
		if (helpOptGrpDocDescription == null 
				|| helpOptGrpDocDescription.isEmpty()) {
			
			helpOptGrpDocDescription = "Display this help and exit";
		}
		optionGroupDocBuilder.description(helpOptGrpDocDescription);
		optionGroupDocBuilder.help(helpOptGrpDocHelp);
		optionGroupDocBuilder.ordinal(helpOptGrpDocOrdinal);
		optionGroupDocBuilder.usage(helpOptGrpDocUsage);
		
		OptionGroupDoc helpOptionGroupDoc = 
				optionGroupDocBuilder.buildOptionGroupDoc();
		
		return helpOptionGroupDoc;
	}
	
	/**
	 * Returns an {@code Options} based on the provided
	 * {@code CLIImpl.Builder}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 * 
	 * @return an {@code Options} based on the provided
	 * {@code CLIImpl.Builder}
	 */
	private static Options getOptions(final Builder builder) {
		
		Options opts = builder.options;
		
		if (opts == null) { opts = AbstractFactory.getDefault().newOptions(); }
		
		Options options = AbstractFactory.getDefault().newOptions(opts);
		
		return options;
	}
	
	/**
	 * Returns an {@code OptionsDoc} based on the provided
	 * {@code CLIImpl.Builder}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 * 
	 * @return an {@code OptionsDoc} based on the provided
	 * {@code CLIImpl.Builder}
	 */
	private static OptionsDoc getOptionsDoc(final Builder builder) {
		
		OptionsDoc optsDoc = builder.optionsDoc;
		String optsDocHelp = builder.optionsDocHelp;
		String optsDocUsage = builder.optionsDocUsage;
		
		if (optsDoc == null) { 
			optsDoc = AbstractFactory.getDefault().newOptionsDoc(optsDoc); 
		}
		
		OptionsDoc optionsDoc = 
				AbstractFactory.getDefault().newOptionsDoc(optsDoc);
		optionsDoc.setHelp(optsDocHelp);
		optionsDoc.setUsage(optsDocUsage);
		
		return optionsDoc;
	}
	
	/**
	 * Returns the version {@code OptionGroup} from the provided 
	 * {@code CLIImpl.Builder}.
	 *  
	 * @param builder the provided {@code CLIImpl.Builder}
	 * @return the version {@code OptionGroup}
	 */
	private static OptionGroup<Void> getVersionOptionGroup(
			final Builder builder) {
		
		List<Option> versionOptList = new ArrayList<Option>(
				builder.versionOptionList);
		
		if (versionOptList.size() == 0) {
			
			Option option = AbstractFactory.getDefault().newOption(
					OptionType.GNU_LONG, "version");
			
			versionOptList.add(option);
		}
		OptionGroupBuilder<Void> optionGroupBuilder = 
				AbstractFactory.getDefault().newOptionGroupBuilder(
						versionOptList);
		
		OptionGroup<Void> versionOptionGroup = 
				optionGroupBuilder.buildOptionGroup();
	
		return versionOptionGroup;
	}
	
	/**
	 * Returns the version {@code OptionGroupDoc} from the provided 
	 * {@code CLIImpl.Builder} and the provided default version {@code Option}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 * @param versionOption the provided default version {@code Option}
	 * @return the version {@code OptionGroupDoc}
	 */
	private static OptionGroupDoc getVersionOptionGroupDoc(
			final Builder builder, final Option versionOption) {
		
		List<OptionDoc> versionOptDocList = new ArrayList<OptionDoc>(
				builder.versionOptionDocList);
		String versionOptGrpDocDescription = 
				builder.versionOptionGroupDocDescription;
		String versionOptGrpDocHelp = builder.versionOptionGroupDocHelp;
		int versionOptGrpDocOrdinal = builder.versionOptionGroupDocOrdinal;
		String versionOptGrpDocUsage = builder.versionOptionGroupDocUsage;
		
		if (versionOptDocList.size() == 0) {
			
			OptionDocBuilder optionDocBuilder = 
					AbstractFactory.getDefault().newOptionDocBuilder(
							versionOption);
			
			OptionDoc optionDoc = optionDocBuilder.buildOptionDoc();
			
			versionOptDocList.add(optionDoc);
		}
		
		OptionGroupDocBuilder optionGroupDocBuilder =
				AbstractFactory.getDefault().newOptionGroupDocBuilder(
						versionOptDocList);
		if (versionOptGrpDocDescription == null 
				|| versionOptGrpDocDescription.isEmpty()) {
			
			versionOptGrpDocDescription = "Output version information and exit";
		}
		optionGroupDocBuilder.description(versionOptGrpDocDescription);
		optionGroupDocBuilder.help(versionOptGrpDocHelp);
		optionGroupDocBuilder.ordinal(versionOptGrpDocOrdinal);
		optionGroupDocBuilder.usage(versionOptGrpDocUsage);
		
		OptionGroupDoc versionOptionGroupDoc = 
				optionGroupDocBuilder.buildOptionGroupDoc();
		
		return versionOptionGroupDoc;
	}
	
	/**
	 * Updates the provided {@code CLIImpl}'s documentation properties.
	 * 
	 * @param cliImpl the provided {@code CLIImpl}
	 */
	private static void updateProperties(final CLIImpl cliImpl) {
		
		Map<String, String> properties = cliImpl.properties;
		
		OptionsDoc optionsDoc = cliImpl.optionsDoc;
		ArgsDoc argsDoc = cliImpl.argsInstanceDoc;		
		String programDescription = cliImpl.programDescription;
		String programHelp = cliImpl.programHelp;
		String programName = cliImpl.programName;
		String programUsage = cliImpl.programUsage;
		String programVersion = cliImpl.programVersion;
		
		if (optionsDoc != null) {

			Map<String, String> optionsDocProperties = 
				optionsDoc.getProperties();
			
			for (Map.Entry<String, String> optionsDocProperty
					: optionsDocProperties.entrySet()) {
				
				String optionsDocPropertyKey =
					CLIPropertyKeys.OPTIONSDOC_PREFIX + "." 
					+ optionsDocProperty.getKey();
				
				if (!properties.containsKey(optionsDocPropertyKey)) {
					
					properties.put(
							optionsDocPropertyKey, 
							optionsDocProperty.getValue());					
				}
			}
			
			List<OptionDoc> optionDocList = optionsDoc.getOptionDocs();
			
			for (int i = 0; i < optionDocList.size(); i++) {
				
				OptionDoc optionDoc = optionDocList.get(i);
				
				Map<String, String> optionDocProperties = 
					optionDoc.getProperties();
				
				for (Map.Entry<String, String> optionDocProperty 
						: optionDocProperties.entrySet()) {
					
					String optionDocPropertyKey = 
						CLIPropertyKeys.OPTIONDOC_PREFIX
						+ Integer.toString(i) + "."
						+ optionDocProperty.getKey();
					
					if (!properties.containsKey(optionDocPropertyKey)) {
						
						properties.put(
								optionDocPropertyKey, 
								optionDocProperty.getValue());
					}
				}			
			}			
			
			List<OptionGroupDoc> optionGroupDocList = 
				optionsDoc.getOptionGroupDocs();
			
			for (int i = 0; i < optionGroupDocList.size(); i++) {
				
				OptionGroupDoc optionGroupDoc = optionGroupDocList.get(i);
				
				Map<String, String> optionGroupDocProperties =
					optionGroupDoc.getProperties();
				
				for (Map.Entry<String, String> optionGroupDocProperty
						: optionGroupDocProperties.entrySet()) {
					
					String optionGroupDocPropertyKey =
						CLIPropertyKeys.OPTIONGROUPDOC_PREFIX
						+ Integer.toString(i) + "."
						+ optionGroupDocProperty.getKey();
					
					if (!properties.containsKey(optionGroupDocPropertyKey)) {
						
						properties.put(
								optionGroupDocPropertyKey, 
								optionGroupDocProperty.getValue());
					}
				}
			}			
		}
		
		if (argsDoc != null) {
			
			Map<String, String> argsDocProperties = argsDoc.getProperties();
			
			for (Map.Entry<String, String> argsDocProperty
					: argsDocProperties.entrySet()) {
				
				String argsDocPropertyKey =
					CLIPropertyKeys.ARGSDOC_PREFIX + "." 
					+ argsDocProperty.getKey();
				
				if (!properties.containsKey(argsDocPropertyKey)) {
					
					properties.put(
							argsDocPropertyKey, 
							argsDocProperty.getValue());					
				}
			}
			
			List<ArgGroupDoc> argGroupDocList = argsDoc.getArgGroupDocs();
			
			for (int i = 0; i < argGroupDocList.size(); i++) {
				
				ArgGroupDoc argGroupDoc = argGroupDocList.get(i);
				
				Map<String, String> argGroupDocProperties =
					argGroupDoc.getProperties();
				
				for (Map.Entry<String, String> argGroupDocProperty
						: argGroupDocProperties.entrySet()) {
					
					String argGroupDocPropertyKey =
						ArgsDocPropertyKeys.ARGGROUPDOC_PREFIX
						+ Integer.toString(i) + "."
						+ argGroupDocProperty.getKey();
					
					if (!properties.containsKey(argGroupDocPropertyKey)) {
						
						properties.put(
								argGroupDocPropertyKey, 
								argGroupDocProperty.getValue());
					}
				}
			}			
		}
		
		if (programDescription != null) {
			
			if (!properties.containsKey(CLIPropertyKeys.PROGRAM_DESCRIPTION)) {
				
				properties.put(
						CLIPropertyKeys.PROGRAM_DESCRIPTION, 
						programDescription);
			}
		}
		
		if (programHelp != null) {
			
			if (!properties.containsKey(CLIPropertyKeys.PROGRAM_HELP)) {
			
				properties.put(CLIPropertyKeys.PROGRAM_HELP, programHelp);
			}
		}
		
		if (programName != null) {
			
			if (!properties.containsKey(CLIPropertyKeys.PROGRAM_NAME)) {
			
				properties.put(CLIPropertyKeys.PROGRAM_NAME, programName);
			}
		}
		
		if (programUsage != null) {
			
			if (!properties.containsKey(CLIPropertyKeys.PROGRAM_USAGE)) {
			
				properties.put(CLIPropertyKeys.PROGRAM_USAGE, programUsage);
			}
		}
		
		if (programVersion != null) {
			
			if (!properties.containsKey(CLIPropertyKeys.PROGRAM_VERSION)) {
				
				properties.put(CLIPropertyKeys.PROGRAM_VERSION, programVersion);
			}
		}
	}
	
	/** 
	 * The {@code Map} for the documentation properties of this {@code CLIImpl}.
	 */
	private final Map<String, String> properties;
	
	/** The program name. */
	private final String programName;
	
	/** The program usage. */
	private final String programUsage;
	
	/** The program description. */
	private final String programDescription;
	
	/** The program help information. */
	private final String programHelp;
	
	/** The program version. */
	private final String programVersion;
	
	/** The {@code Options}. */
	private final Options options;
	
	/** The {@code OptionsDoc}. */
	private final OptionsDoc optionsDoc;

	/** The {@code Args}. */
	private final Args argsInstance;
	
	/** The {@code ArgsDoc}. */
	private final ArgsDoc argsInstanceDoc;

	/** The help {@code OptionGroup}. */
	private final OptionGroup<Void> helpOptionGroup;
	
	/** The help {@code OptionGroupDoc}. */
	private final OptionGroupDoc helpOptionGroupDoc;

	/**
	 * The boolean value to indicate if the help {@code OptionGroupDoc} 
	 * is excluded from documentation.
	 */
	private final boolean helpOptionGroupHidden;
	
	/**
	 * The boolean value to indicate if the help {@code OptionGroup} 
	 * is excluded from parsing.
	 */
	private final boolean helpOptionGroupUnused;
	
	/** The version {@code OptionGroup}. */
	private final OptionGroup<Void> versionOptionGroup;
	
	/** The version {@code OptionGroupDoc}. */
	private final OptionGroupDoc versionOptionGroupDoc;
	
	/**
	 * The boolean value to indicate if the version {@code OptionGroupDoc} 
	 * is excluded from documentation.
	 */
	private final boolean versionOptionGroupHidden;
	
	/**
	 * The boolean value to indicate if the version {@code OptionGroup} 
	 * is excluded from parsing.
	 */
	private final boolean versionOptionGroupUnused;
	
	/**
	 * Constructs a new {@code CLIImpl} with the provided 
	 * {@code CLIImpl.Builder}.
	 * 
	 * @param builder the provided {@code CLIImpl.Builder}
	 */
	private CLIImpl(final Builder builder) {
		
		String progName = builder.programName;
		String progVersion = builder.programVersion;		
		String progUsage = builder.programUsage;
		String progDescription = builder.programDescription;
		String progHelpHeader = builder.programHelpHeader;
		String progHelpBody = builder.programHelpBody;
		String progHelpFooter = builder.programHelpFooter;
		boolean helpOptGrpHidden = builder.helpOptionGroupHidden;
		boolean helpOptGrpUnused = builder.helpOptionGroupUnused;
		boolean versionOptGrpHidden = builder.versionOptionGroupHidden;
		boolean versionOptGrpUnused = builder.versionOptionGroupUnused;
		
		if (progName == null) {
			throw new NullPointerException("program name must not be null");
		}
		
		if (progName.isEmpty()) {
			throw new IllegalArgumentException(
					"program name must be a non-empty string");
		}
		
		this.properties = new Properties();
		
		this.programName = Util.expand1(
				progName, 
				AbstractFactory.getDefault().getSystemDocProperties());

		this.options = CLIImpl.getOptions(builder);
		
		this.optionsDoc = CLIImpl.getOptionsDoc(builder);
		
		this.argsInstance = CLIImpl.getArgs(builder);
		
		this.argsInstanceDoc = CLIImpl.getArgsDoc(builder);
		
		this.helpOptionGroupHidden = helpOptGrpHidden;

		this.helpOptionGroupUnused = helpOptGrpUnused;

		this.helpOptionGroup = CLIImpl.getHelpOptionGroup(builder);
		
		if (!this.helpOptionGroupUnused) {
			this.options.addOptionGroup(this.helpOptionGroup);
		}
		
		this.helpOptionGroupDoc = CLIImpl.getHelpOptionGroupDoc(
				builder, this.helpOptionGroup.getOptions().get(0));
		
		if (!this.helpOptionGroupHidden) {
			this.optionsDoc.addOptionGroupDoc(this.helpOptionGroupDoc);
		}
		
		if (progVersion == null || progVersion.isEmpty()) { 
			progVersion = ""; 
		}
		
		this.versionOptionGroupHidden = 
				progVersion.isEmpty() || versionOptGrpHidden;
		
		this.versionOptionGroupUnused = 
				progVersion.isEmpty() || versionOptGrpUnused;
		
		this.versionOptionGroup = CLIImpl.getVersionOptionGroup(builder);
		
		if (!this.versionOptionGroupUnused) {
			this.options.addOptionGroup(this.versionOptionGroup);
		}
		
		this.versionOptionGroupDoc = CLIImpl.getVersionOptionGroupDoc(
				builder, this.versionOptionGroup.getOptions().get(0));
		
		if (!this.versionOptionGroupHidden) {
			this.optionsDoc.addOptionGroupDoc(this.versionOptionGroupDoc);
		}
		
		CLIImpl.updateProperties(this);
		
		this.programVersion = Util.expand4(
				progVersion, this.properties);		
		
		CLIImpl.updateProperties(this);		
		
		if (progUsage == null || progUsage.isEmpty()) {
			progUsage = CLIImpl.getDefaultProgramUsage(this);
		}
		
		this.programUsage = Util.expand4(
				progUsage, this.properties);
		
		CLIImpl.updateProperties(this);
		
		if (progDescription == null) { progDescription = ""; }
		
		this.programDescription = Util.expand4(
				progDescription, this.properties);
		
		CLIImpl.updateProperties(this);
		
		if (progHelpBody == null || progHelpBody.isEmpty()) {
			progHelpBody = CLIImpl.getDefaultProgramHelpBody(this);
		}
		
		String progHelp = CLIImpl.getConcatProgramHelp(
				progHelpHeader, 
				progHelpBody, 
				progHelpFooter);
		
		this.programHelp = Util.expand4(
				progHelp, this.properties);
		
		CLIImpl.updateProperties(this);
	}
	
	@Override
	public Args getArgs() {
		return AbstractFactory.getDefault().newArgs(this.argsInstance);
	}
	
	@Override
	public ArgsDoc getArgsDoc() {
		return AbstractFactory.getDefault().newArgsDoc(this.argsInstanceDoc);
	}
	
	@Override
	public OptionGroup<Void> getHelpOptionGroup() {
		return this.helpOptionGroup;
	}
	
	@Override
	public OptionGroupDoc getHelpOptionGroupDoc() {
		return this.helpOptionGroupDoc;
	}
	
	@Override	
	public Options getOptions() {
		return AbstractFactory.getDefault().newOptions(this.options);
	}
	
	@Override	
	public OptionsDoc getOptionsDoc() {
		return AbstractFactory.getDefault().newOptionsDoc(this.optionsDoc);
	}

	@Override
	public String getProgramDescription() {
		return this.programDescription;
	}

	@Override
	public String getProgramHelp() {
		return this.programHelp;
	}

	@Override
	public String getProgramName() {
		return this.programName;
	}

	@Override
	public String getProgramUsage() {
		return this.programUsage;
	}
	
	@Override
	public String getProgramVersion() {
		return this.programVersion;
	}
	
	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}
	
	@Override
	public OptionGroup<Void> getVersionOptionGroup() {
		return this.versionOptionGroup;
	}

	@Override
	public OptionGroupDoc getVersionOptionGroupDoc() {
		return this.versionOptionGroupDoc;
	}
	
	@Override
	public boolean isHelpOptionGroupHidden() {
		return this.helpOptionGroupHidden;
	}
	
	@Override
	public boolean isHelpOptionGroupUnused() {
		return this.helpOptionGroupUnused;
	}
	
	@Override
	public boolean isVersionOptionGroupHidden() {
		return this.versionOptionGroupHidden;
	}
	
	@Override
	public boolean isVersionOptionGroupUnused() {
		return this.versionOptionGroupUnused;
	}

	@Override
	public CLIProcessResult process(final String[] args) throws CLIException {

		try {

			this.options.parse(args);

		} catch (OptionsException e) {

			throw new CLIException(e.getMessage(), e);
		}		
			
		if (this.versionOptionGroup.hasParsedOptionSelection()) {
			
			return CLIProcessResult.VERSION_REQUESTED;
		}
		
		if (this.helpOptionGroup.hasParsedOptionSelection()) {
			
			return CLIProcessResult.HELP_REQUESTED;
		}
		
		for (OptionGroup<?> optionGroup : this.options.getOptionGroups()) {
			
			if (optionGroup.isOptionSelectionsTerminative() 
					&& optionGroup.hasParsedOptionSelection()) {
				
				return CLIProcessResult.PROCEED;
			}
		}
		
		String[] remainingArgs = this.options.getRemainingArguments();
		
		try {
			
			this.argsInstance.parse(remainingArgs);
			
		} catch (ArgsException e) {

			throw new CLIException(e.getMessage(), e);
		}
		
		return CLIProcessResult.PROCEED;
	}
	
	@Override
	public CLIProcessResult process(
			final String[] args, final PrintStream outStream) 
			throws CLIException {
		
		CLIProcessResult result = this.process(args);
		
		switch(result) {
		
		case HELP_REQUESTED:
			outStream.println(this.programHelp);
			break;
			
		case VERSION_REQUESTED:
			outStream.println(this.programVersion);
			break;
			
		default:
			break;
		}
		
		return result;
	}

	@Override
	public CLIProcessResult process(
			final String[] args, 
			final PrintStream outStream, 
			final PrintStream errStream) {

		CLIProcessResult result = CLIProcessResult.PROCEED;
		
		try {
			
			result = this.process(args, outStream);
			
		} catch (CLIException e) {

			String message = String.format(
					"%1$s: %2$s", 
					this.programName, 
					e.getMessage());
			
			StringBuilder messageBuilder = new StringBuilder(message);
			
			messageBuilder.append('\n');
			
			if (!this.helpOptionGroupHidden && !this.helpOptionGroupUnused) {
				
				OptionDoc helpOptionDoc = 
					this.helpOptionGroupDoc.getOptionDocs().get(0);
				
				String suggestion = String.format(
						"Try `%1$s %2$s' for more information\n", 
						this.programName,
						helpOptionDoc.getUsage());
				
				messageBuilder.append(suggestion);
			}
			
			errStream.print(messageBuilder.toString());
			
			result = CLIProcessResult.ERROR;
		}
		
		return result;
	}

	@Override
	public String toString() {
		return String.format("CLI [" 
				+ "properties=%s, " 
				+ "programName=%s, " 
				+ "programUsage=%s, " 
				+ "programDescription=%s, " 
				+ "programHelp=%s, " 
				+ "programVersion=%s, " 
				+ "options=%s, " 
				+ "optionsDoc=%s, " 
				+ "args=%s, " 
				+ "argsDoc=%s, " 
				+ "helpOptionGroup=%s, " 
				+ "helpOptionGroupDoc=%s, " 
				+ "helpOptionGroupHidden=%s, " 
				+ "helpOptionGroupUnused=%s, " 
				+ "versionOptionGroup=%s, " 
				+ "versionOptionGroupDoc=%s, " 
				+ "versionOptionGroupHidden=%s, " 
				+ "versionOptionGroupUnused=%s]",
				this.properties, 
				this.programName, 
				this.programUsage, this.programDescription, 
				this.programHelp, this.programVersion,
				this.options, this.optionsDoc, 
				this.argsInstance, this.argsInstanceDoc,
				this.helpOptionGroup, this.helpOptionGroupDoc,
				this.helpOptionGroupHidden, this.helpOptionGroupUnused,
				this.versionOptionGroup, this.versionOptionGroupDoc,
				this.versionOptionGroupHidden, this.versionOptionGroupUnused);
	}
}
