/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class provides utility methods regarding default arguments.
 *  
 * @author Climax Development (climax.development@gmail.com)
 */
final class DefaultArgs {
	
	/**
	 * Returns the detail message specifying if any of the default arguments 
	 * that are invalid or cannot be parsed and the additional specified 
	 * detail message.
	 * 
	 * @param defaultArgumentList the provided {@code List} of default 
	 * arguments that are invalid or cannot be parsed
	 * 
	 * @param message the additional detail message
	 * 
	 * @return the detail message specifying if any of the default 
	 * arguments that are invalid or cannot be parsed and the additional 
	 * specified detail message
	 */
	private static String getMessage(
			final List<String> defaultArgumentList,
			final String message) {
		
		String defaultArgumentListString = 
			Lists.toString(defaultArgumentList);
		
		String m = String.format(
				"no default arguments : %s", 
				message);
		
		if (defaultArgumentList.size() == 1) {
			
			m = String.format(
					"invalid default argument %1$s : %2$s",
					defaultArgumentListString,
					message);
		}
		
		if (defaultArgumentList.size() > 1) {
			
			m = String.format(
					"invalid default arguments %1$s : %2$s",
					defaultArgumentListString,
					message);
		}				

		return m;
	}

	/**
	 * Parses the provided {@code List} of default arguments using the provided 
	 * {@code ArgsValidator} and the provided {@code ArgParser}.
	 * 
	 * @param <T> the specified parsed argument type
	 * 
	 * @param defaultArgumentList the provided {@code List} of default arguments
	 * 
	 * @param argsValidator the provided {@code ArgsValidator}
	 * 
	 * @param argParser the provided {@code ArgParser}
	 * 
	 * @return a {@code List} of parsed default arguments
	 */
	static <T> List<T> parse(
			final List<String> defaultArgumentList,
			final ArgsValidator argsValidator,
			final ArgParser<? extends T> argParser) {
		
		List<String> defaultArgList = 
			new ArrayList<String>(defaultArgumentList);
		
		if (argsValidator != null) {
			
			try {
				
				argsValidator.validate(defaultArgList);
				
			} catch (ArgsValidatorException e) {
				
				String message = DefaultArgs.getMessage(
						e.getArguments(), 
						e.getMessage());
				
				throw new DefaultArgsException(
						e.getArguments(), 
						message, 
						e);
			}
		}
		
		List<T> parsedDefaultArgumentList = new ArrayList<T>();
		
		if (argParser != null) {
			
			for (String defaultArgument : defaultArgumentList) {
				
				try {
					
					T parsedDefaultArgument = 
						argParser.parse(defaultArgument);
					
					parsedDefaultArgumentList.add(parsedDefaultArgument);
					
				} catch (ArgParserException e) {
					
					String message = DefaultArgs.getMessage(
							Arrays.asList(e.getArgument()), 
							e.getMessage());
					
					throw new DefaultArgsException(
							Arrays.asList(e.getArgument()), 
							message, 
							e);
				}						
			}
		}
		
		return parsedDefaultArgumentList;		
	}
	
	/**
	 * Constructs an instance of {@code DefaultArgs}. This constructor is 
	 * suppressed for noninstantiability outside of this class.
	 */
	private DefaultArgs() {
	}
}
