/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.Map;

/**
 * An implementation of an {@code OptionComplete}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionCompleteImpl implements OptionComplete {

	/** 
	 * A builder for an {@code OptionCompleteImpl}.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	static final class Builder implements OptionCompleteBuilder {
		
		/** The {@code OptionType}. */
		private final OptionType optionType;
		
		/** The name of the {@code Option}. */
		private final String name;
		
		/** 
		 * The boolean value to indicate if the {@code OptionDoc} is excluded 
		 * from documentation. 
		 */
		private boolean hidden;
		
		/**
		 * The boolean value to indicate if the {@code Option} is excluded 
		 * from parsing.
		 */		
		private boolean unused;
		
		/** 
		 * The ordinal of the {@code OptionDoc} in relation to other 
		 * {@code OptionDoc}s. 
		 */		
		private int ordinal;
		
		/** 
		 * The format string for the usage of the {@code Option}. 
		 */		
		private String usage;
		
		/**
		 * Constructs an {@code OptionCompleteImpl.Bulider} with the provided
		 * {@code Option.Type} and the specified name.
		 * 
		 * @param optType the provided {@code OptionType}
		 * @param n the specified name
		 */
		Builder(final OptionType optType, final String n) {
			
			this.optionType = optType;
			
			this.name = n;
			
			this.hidden = false;
			
			this.unused = false;
			
			this.ordinal = 0;
			
			this.usage = null;			
		}

		@Override
		public OptionComplete buildOptionComplete() {
			return new OptionCompleteImpl(this);
		}
		
		@Override
		public OptionDoc buildOptionDoc() {
			return new OptionCompleteImpl(this);
		}
	
		@Override
		public OptionCompleteBuilder hidden(final boolean b) {
			this.hidden = b;
			return this;
		}
		
		@Override
		public OptionCompleteBuilder ordinal(final int i) {
			this.ordinal = i;
			return this;
		}

		@Override
		public OptionCompleteBuilder unused(final boolean b) {
			this.unused = b;
			return this;
		}

		@Override
		public OptionCompleteBuilder usage(final String s) {
			this.usage = s;
			return this;
		}
	}
	
	/** 
	 * The boolean value to indicate if the {@code OptionDoc} is excluded 
	 * from documentation. 
	 */
	private final boolean hidden;
	
	/**
	 * The boolean value to indicate if the {@code Option} is excluded 
	 * from parsing.
	 */		
	private final boolean unused;
	
	/** The {@code Option}. */
	private final Option option;
	
	/** The {@code OptionDoc}. */
	private final OptionDoc optionDoc;
	
	/**
	 * Constructs a new {@code OptionCompleteImpl} with the provided 
	 * {@code OptionCompleteImpl.Builder}.
	 * 
	 * @param builder the provided {@code OptionCompleteImpl.Builder}
	 */
	private OptionCompleteImpl(final Builder builder) {
		
		OptionType optionType = builder.optionType;
		String name = builder.name;
		boolean hide = builder.hidden;		
		boolean unuse = builder.unused;
		int ordinal = builder.ordinal;
		String usage = builder.usage;
		
		Option opt = AbstractFactory.getDefault().newOption(optionType, name);
		
		OptionDocBuilder optionDocBuilder = 
				AbstractFactory.getDefault().newOptionDocBuilder(opt);
		optionDocBuilder.ordinal(ordinal);
		optionDocBuilder.usage(usage);
		
		OptionDoc optDoc = optionDocBuilder.buildOptionDoc();
		
		this.hidden = hide;
		this.unused = unuse;
		this.option = opt;
		this.optionDoc = optDoc;
	}
	
	@Override
	public String getName() {
		return this.option.getName();
	}

	@Override
	public Option getOption() {
		return this.option;
	}

	@Override
	public OptionType getOptionType() {
		return this.option.getOptionType();
	}

	@Override
	public OptionTypeDoc getOptionTypeDoc() {
		return this.optionDoc.getOptionTypeDoc();
	}

	@Override
	public int getOrdinal() {
		return this.optionDoc.getOrdinal();
	}

	@Override
	public Map<String, String> getProperties() {
		return this.optionDoc.getProperties();
	}

	@Override
	public String getUsage() {
		return this.optionDoc.getUsage();
	}

	@Override
	public boolean isHidden() {
		return this.hidden;
	}

	@Override
	public boolean isUnused() {
		return this.unused;
	}

	@Override
	public String toOptionString() {
		return this.option.toOptionString();
	}

	@Override
	public String toString() {
		return String.format("OptionComplete [" 
				+ "hidden=%s, " 
				+ "unused=%s, " 
				+ "option=%s, " 
				+ "optionDoc=%s]",
				this.hidden, 
				this.unused, 
				this.option, 
				this.optionDoc);
	}
}
