/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.Collections;
import java.util.Map;

/**
 * An implementation of an {@code OptionDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionDocImpl implements OptionDoc {
	
	/**
	 * A builder for an {@code OptionDocImpl}.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	static final class Builder implements OptionDocBuilder {
		
		/** 
		 * The ordinal of the {@code OptionDoc} in relation to other 
		 * {@code OptionDoc}s. 
		 */
		private int ordinal;
		
		/** The {@code Option}. */
		private final Option option;
		
		/** The format string for the usage of the {@code Option}. */
		private String usage;
		
		/**
		 * Constructs an {@code OptionDocImpl.Builder} with the provided
		 * {@code Option}.
		 * 
		 * @param opt the provided {@code Option}
		 */
		Builder(final Option opt) {
			
			this.ordinal = 0;
			
			this.option = opt;
			
			this.usage = null;
		}

		@Override
		public OptionDoc buildOptionDoc() {
			
			OptionDoc optionDoc = null;
			
			try {
				optionDoc = new OptionDocImpl(this);
			} catch (RuntimeException e) {
				throw new IllegalStateException(e);
			}
			
			return optionDoc;
		}

		@Override
		public OptionDocBuilder ordinal(final int i) {
			this.ordinal = i;
			return this;
		}
		
		@Override
		public OptionDocBuilder usage(final String s) {
			this.usage = s;
			return this;
		}
	}
	
	/**
	 * Updates the provided {@code OptionDocImpl}'s documentation properties.
	 * 
	 * @param optionDocImpl the provided {@code OptionDocImpl}
	 */
	private static void updateProperties(final OptionDocImpl optionDocImpl) {
		
		Map<String, String> properties = optionDocImpl.properties;
		
		Option option = optionDocImpl.option;
		
		OptionTypeDoc optionTypeDoc = optionDocImpl.optionTypeDoc;
		
		String usage = optionDocImpl.usage;
		
		if (option != null) {
			
			if (!properties.containsKey(OptionDocPropertyKeys.NAME)) {
				
				properties.put(
						OptionDocPropertyKeys.NAME, option.getName());
			}
			
			
			if (!properties.containsKey(OptionDocPropertyKeys.OPTION)) {
			
				properties.put(
						OptionDocPropertyKeys.OPTION, option.toOptionString());
			}			
		}
		
		if (optionTypeDoc != null) {
			
			Map<String, String> optionTypeDocProperties = 
					optionTypeDoc.getProperties();
			
			for (Map.Entry<String, String> optionTypeDocProperty 
					: optionTypeDocProperties.entrySet()) {
				
				String optionTypeDocPropertyKey = 
					OptionDocPropertyKeys.OPTIONTYPEDOC_PREFIX
					+ "."
					+ optionTypeDocProperty.getKey();
				
				if (!properties.containsKey(optionTypeDocPropertyKey)) {
				
					properties.put(
							optionTypeDocPropertyKey, 
							optionTypeDocProperty.getValue());
				}
				
				if (!properties.containsKey(optionTypeDocProperty.getKey())) {
				
					properties.put(
							optionTypeDocProperty.getKey(), 
							optionTypeDocProperty.getValue());
				}
			}
		}
		
		if (usage != null) {
			
			properties.put(OptionDocPropertyKeys.USAGE, usage);
		}
	}
	
	/** 
	 * The {@code Map} for the documentation properties of this 
	 * {@code OptionDocImpl}. 
	 */
	private final Map<String, String> properties;
	
	/** 
	 * The ordinal of this {@code OptionDoc} in relation to other 
	 * {@code OptionDoc}s. 
	 */
	private final int ordinal;
	
	/** The {@code Option}. */
	private final Option option;
	
	/** The {@code OptionTypeDoc}. */
	private final OptionTypeDoc optionTypeDoc;
	
	/** The usage of the {@code Option}. */
	private final String usage;
	
	/**
	 * Constructs a new {@code OptionDocImpl} with the provided
	 * {@code OptionDocImpl.Builder}.
	 * 
	 * @param builder the provided {@code OptionDocImpl.Builder}
	 */
	private OptionDocImpl(final Builder builder) {
		
		int ord = builder.ordinal;
		Option opt = builder.option;
		String use = builder.usage;
		
		if (opt == null) {
			throw new NullPointerException("option must not be null");
		}
		
		this.properties = new Properties();
		
		this.ordinal = ord;
		
		this.option = opt;
		
		OptionDocImpl.updateProperties(this);
		
		OptionType optionType = this.option.getOptionType();
		
		this.optionTypeDoc = OptionTypeDoc.of(optionType);
		
		OptionDocImpl.updateProperties(this);
		
		if (use == null || use.isEmpty()) {
			use = new Properties.Property(
					OptionDocPropertyKeys.OPTION).toExpandableString();
		}
		
		this.usage = Util.expand4(use, this.properties);
		
		OptionDocImpl.updateProperties(this);
	}
	
	@Override
	public Option getOption() {
		return this.option;
	}

	@Override
	public OptionTypeDoc getOptionTypeDoc() {
		return this.optionTypeDoc;
	}

	@Override
	public int getOrdinal() {
		return this.ordinal;
	}

	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}

	@Override
	public String getUsage() {
		return this.usage;
	}

	@Override
	public String toString() {
		return String.format("OptionDoc [" 
				+ "properties=%s, " 
				+ "ordinal=%s, " 
				+ "option=%s, " 
				+ "optionTypeDoc=%s, " 
				+ "usage=%s]",
				this.properties, 
				this.ordinal, 
				this.option, 
				this.optionTypeDoc, 
				this.usage);
	}
}
