/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.List;

/**
 * A group of mutually-exclusive {@code Option}s and their arguments of the 
 * specified parsed argument type.
 * 
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @param <T> the specified parsed argument type
 */
public interface OptionGroup<T> {

	/**
	 * Adds to this {@code OptionGroup} an {@code OptionSelection} of type 
	 * {@code String} or {@code OptionSelection}s for each sub-argument of type
	 * {@code String}. 
	 * <p>
	 * If an argument separator is defined in this {@code OptionGroup} and 
	 * the provided {@code OptionSelection}'s argument contains one or more 
	 * argument separators, an {@code OptionSelection} for each sub-argument 
	 * is created and is added instead of the original {@code OptionSelection}.
	 * </p> 
	 * 
	 * @param optionSelection the provided {@code OptionSelection} of type 
	 * {@code String}
	 */
	void addOptionSelection(
			final OptionSelection<String> optionSelection);
	
	/**
	 * Indicates if this {@code OptionGroup} accepts 
	 * {@code OptionSelection}s with non {@code null} arguments.
	 * 
	 * @return {@code true} if the non {@code null} {@code ArgsValidator} 
	 * and the non {@code null} {@code ArgParser} were provided in the 
	 * {@link org.climax.OptionGroupBuilder#argsValidator(ArgsValidator)} 
	 * method and the 
	 * {@link org.climax.OptionGroupBuilder#argParser(ArgParser)} method 
	 */
	boolean canAcceptArgument();
	
	/**
	 * Removes all {@code OptionSelection}s of type {@code String}.
	 */
	void clearOptionSelections();
	
	/**
	 * Removes all {@code OptionSelection}s of the specified parsed 
	 * argument type.
	 */	
	void clearParsedOptionSelections();	
	
	/**
	 * Returns the {@code ArgParser}.
	 * 
	 * @return the {@code ArgParser}
	 */
	ArgParser<T> getArgParser();
	
	/**
	 * Returns the argument separator.
	 * 
	 * @return the argument separator
	 */
	String getArgSeparator();
	
	/**
	 * Returns the {@code ArgsValidator}.
	 * 
	 * @return the {@code ArgsValidator}
	 */
	ArgsValidator getArgsValidator();
	
	/**
	 * Returns an unmodifiable {@code List} of the arguments of type 
	 * {@code String}.
	 * 
	 * @return an unmodifiable {@code List} of the arguments of type 
	 * {@code String}
	 */
	List<String> getArguments();
	
	/**
	 * Returns an unmodifiable {@code List} of {@code Option}s.
	 * 
	 * @return an unmodifiable {@code List} of {@code Option}s
	 */
	List<Option> getOptions();
	
	/**
	 * Returns an unmodifiable {@code List} of the 
	 * {@code OptionSelection}s of type {@code String}.
	 * 
	 * @return an unmodifiable {@code List} of the 
	 * {@code OptionSelection}s of type {@code String}
	 */
	List<OptionSelection<String>> getOptionSelections();
	
	/**
	 * Returns an unmodifiable {@code List} of arguments of the specified 
	 * parsed argument type.
	 * 
	 * @return an unmodifiable {@code List} of arguments of the specified 
	 * parsed argument type
	 */	
	List<T> getParsedArguments();
	
	/**
	 * Returns an unmodifiable {@code List} of the 
	 * {@code OptionSelection}s of the specified parsed argument type.
	 * 
	 * @return an unmodifiable {@code List} of the 
	 * {@code OptionSelection}s of the specified parsed argument type
	 */
	List<OptionSelection<T>> getParsedOptionSelections();
	
	/**
	 * Indicates if this {@code OptionGroup} has at least one argument 
	 * of type {@code String}.
	 * 
	 * @return {@code true} if this {@code OptionGroup} has at least one
	 * argument of type {@code String}
	 */
	boolean hasArgument();
	
	/**
	 * Indicates if this {@code OptionGroup} has at least one 
	 * {@code OptionSelection} of type {@code String}.
	 * 
	 * @return {@code true} if this {@code OptionGroup} has at least 
	 * one {@code OptionSelection} of type {@code String}
	 */
	boolean hasOptionSelection();
	
	/**
	 * Indicates if this {@code OptionGroup} has at least one argument 
	 * of the specified parsed argument type.
	 * 
	 * @return {@code true} if this {@code OptionGroup} has at least one 
	 * argument of the specified parsed argument type
	 */
	boolean hasParsedArgument();
	
	/**
	 * Indicates if this {@code OptionGroup} has at least one 
	 * {@code OptionSelection} of the specified parsed argument type.
	 * 
	 * @return {@code true} if this {@code OptionGroup} has at least 
	 * one {@code OptionSelection} of the specified parsed argument
	 * type
	 */
	boolean hasParsedOptionSelection();
	
	/**
	 * Indicates if this {@code OptionGroup}'s {@code OptionSelection}s 
	 * of type {@code String} terminates the parsing of any other command line 
	 * argument.
	 * 
	 * @return {@code true} if the {@link 
	 * org.climax.OptionGroupBuilder#optionSelectionsTerminative(boolean)} 
	 * method is set to {@code true}; otherwise {@code false}
	 */
	boolean isOptionSelectionsTerminative();
	
	/**
	 * Parses the {@code OptionSelection}s of type {@code String} to
	 * {@code OptionSelection}s of the specified parsed argument type.
	 * 
	 * @throws OptionGroupException if the {@code OptionSelection}s of 
	 * type {@code String} are invalid or cannot be parsed
	 */
	void parse() throws OptionGroupException;	
}
