/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An implementation of an {@code OptionGroupComplete} of the specified parsed 
 * argument type.  
 * 
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @param <T> the specified parsed argument type
 */
final class OptionGroupCompleteImpl<T> 
	implements OptionGroupComplete<T> {

	/**
	 * A builder for an {@code OptionGroupCompleteImpl} of the specified parsed 
	 * argument type.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 * 
	 * @param <T> the specified parsed argument type
	 */
	static final class Builder<T> 
		implements OptionGroupCompleteBuilder<T> {
		
		/** The {@code List} of {@code OptionComplete}s. */
		private final List<OptionComplete> optionCompleteList;
		
		/** 
		 * The boolean value to indicate if the {@code OptionGroupDoc} is 
		 * excluded from documentation. 
		 */
		private boolean hidden;
		
		/**
		 * The boolean value to indicate if the {@code OptionGroup} is excluded 
		 * from parsing.
		 */
		private boolean unused;
		
		/** 
		 * The ordinal of the {@code OptionGroupDoc} in relation to 
		 * other {@code OptionGroupDoc}s. 
		 */
		private int ordinal;
		
		/** 
		 * The format string for the usage of the {@code OptionGroup}.
		 */
		private String usage;
		
		/** 
		 * The format string for the description of the {@code OptionGroup}.
		 */
		private String description;
		
		/**
		 * The format string for the help information for the 
		 * {@code OptionGroup}.
		 */
		private String help;
		
		/** The format string for the argument separator. */
		private String argSeparator;
		
		/** The {@code ArgsValidator}. */
		private ArgsValidator argsValidator;
		
		/** The {@code ArgParser}. */
		private ArgParser<? extends T> argParser;

		/** The {@code List} of default argument format strings. */
		private final List<String> defaultArgumentList;
		
		/**
		 * The boolean value to indicate if the {@code OptionGroup}'s 
		 * {@code OptionSelection}s of type {@code String} terminates 
		 * the parsing of any other command line argument.
		 */
		private boolean optionSelectionsTerminative;
		
		/**
		 * Constructs an {@code OptionGroupCompleteImpl.Builder} with the 
		 * provided {@code List} of {@code OptionComplete}s.
		 * 
		 * @param optCompleteList the provided {@code List} of 
		 * {@code OptionComplete}s
		 */
		Builder(final List<OptionComplete> optCompleteList) {
			
			this.optionCompleteList = 
					new ArrayList<OptionComplete>(optCompleteList);
			
			this.hidden = false;
			
			this.unused = false;
			
			this.ordinal = 0;
			
			this.usage = null;
			
			this.description = null;
			
			this.help = null;
			
			this.argSeparator = null;
			
			this.argsValidator = null;
			
			this.argParser = null;
			
			this.defaultArgumentList = new ArrayList<String>();
			
			this.optionSelectionsTerminative = false;	
		}
		
		@Override
		public OptionGroupCompleteBuilder<T> argParser(
				final ArgParser<? extends T> p) {
			
			this.argParser = p;
			return this;
		}
		
		@Override
		public OptionGroupCompleteBuilder<T> argSeparator(final String s) {
			this.argSeparator = s;
			return this;
		}
		
		@Override
		public OptionGroupCompleteBuilder<T> argsValidator(
				final ArgsValidator v) {
			
			this.argsValidator = v;
			return this;
		}

		@Override
		public OptionGroup<T> buildOptionGroup() {
			return new OptionGroupCompleteImpl<T>(this);
		}
		
		@Override
		public OptionGroupComplete<T> buildOptionGroupComplete() {
			return new OptionGroupCompleteImpl<T>(this);
		}

		@Override
		public OptionGroupDoc buildOptionGroupDoc() {
			return new OptionGroupCompleteImpl<T>(this);
		}

		@Override
		public OptionGroupCompleteBuilder<T> defaultArguments(
				final List<String> l) {
			
			this.defaultArgumentList.clear();
			this.defaultArgumentList.addAll(l);
			return this;
		}

		@Override
		public OptionGroupCompleteBuilder<T> description(final String s) {
			this.description = s;
			return this;
		}		

		@Override
		public OptionGroupCompleteBuilder<T> help(final String s) {
			this.help = s;
			return this;
		}

		@Override
		public OptionGroupCompleteBuilder<T> hidden(final boolean b) {
			this.hidden = b;
			return this;
		}

		@Override
		public OptionGroupCompleteBuilder<T> optionSelectionsTerminative(
				final boolean b) {
			
			this.optionSelectionsTerminative = b;
			return this;
		}

		@Override
		public OptionGroupCompleteBuilder<T> ordinal(final int i) {
			this.ordinal = i;
			return this;
		}

		@Override
		public OptionGroupCompleteBuilder<T> unused(final boolean b) {
			this.unused = b;
			return this;
		}

		@Override
		public OptionGroupCompleteBuilder<T> usage(final String s) {
			this.usage = s;
			return this;
		}
	}
	
	/** 
	 * The boolean value to indicate if the {@code OptionGroupDoc} is 
	 * excluded from documentation. 
	 */
	private final boolean hidden;
	
	/**
	 * The boolean value to indicate if the {@code OptionGroup} is excluded 
	 * from parsing.
	 */
	private final boolean unused;
	
	/** The {@code List} of {@code OptionComplete}s. */
	private final List<OptionComplete> optionCompleteList;
	
	/** The {@code OptionGroupDoc}. */
	private final OptionGroupDoc optionGroupDoc;
	
	/** The {@code OptionGroup}. */
	private final OptionGroup<T> optionGroup;

	/**
	 * Constructs an {@code OptionGroupCompleteImpl} with the provided
	 * {@code OptionGroupCompleteImpl.Builder}.
	 * 
	 * @param builder the the provided {@code OptionGroupCompleteImpl.Builder}
	 */
	private OptionGroupCompleteImpl(final Builder<T> builder) {
		
		List<OptionComplete> optCompleteList = new ArrayList<OptionComplete>();
		
		for (OptionComplete optionComplete : builder.optionCompleteList) {
			
			if (optionComplete == null) {
				throw new IllegalStateException(
						"optionCompletes must not be null");
			}
			
			optCompleteList.add(optionComplete);
		}
		
		boolean hide = builder.hidden;		
		boolean unuse = builder.unused;
		int ordinal = builder.ordinal;
		String usage = builder.usage;
		String description = builder.description;
		String help = builder.help;
		String argSeparator = builder.argSeparator;
		ArgsValidator validator = builder.argsValidator;
		ArgParser<? extends T> parser = builder.argParser;
		List<String> defaultArgList = new ArrayList<String>(
				builder.defaultArgumentList);
		boolean optionSelectTerminative = 
				builder.optionSelectionsTerminative;
		
		List<Option> optList = new ArrayList<Option>();
		List<OptionDoc> optDocList = new ArrayList<OptionDoc>();
		
		for (OptionComplete optionComplete : optCompleteList) {
			
			if (!optionComplete.isHidden()) { 
				optDocList.add(optionComplete); 
			}
			
			if (!optionComplete.isUnused()) { 
				optList.add(optionComplete); 
			}
		}
		
		OptionGroupDocBuilder optionGroupDocBuilder = 
				AbstractFactory.getDefault().newOptionGroupDocBuilder(
						optDocList);
		optionGroupDocBuilder.description(description);
		optionGroupDocBuilder.help(help);
		optionGroupDocBuilder.ordinal(ordinal);
		optionGroupDocBuilder.usage(usage);
		
		OptionGroupDoc optGroupDoc = 
				optionGroupDocBuilder.buildOptionGroupDoc();
		
		OptionGroupBuilder<T> optionGroupBuilder = 
				AbstractFactory.getDefault().newOptionGroupBuilder(optList);
		optionGroupBuilder.argParser(parser);
		optionGroupBuilder.argSeparator(argSeparator);
		optionGroupBuilder.argsValidator(validator);
		optionGroupBuilder.defaultArguments(defaultArgList);
		optionGroupBuilder.optionSelectionsTerminative(
				optionSelectTerminative);
		
		OptionGroup<T> optGroup = optionGroupBuilder.buildOptionGroup();
		
		this.unused = unuse;
		this.hidden = hide;
		this.optionCompleteList = optCompleteList;
		this.optionGroupDoc = optGroupDoc;
		this.optionGroup = optGroup;
	}

	@Override
	public void addOptionSelection(
			final OptionSelection<String> optionSelection) {
		
		this.optionGroup.addOptionSelection(optionSelection);
	}
	
	@Override
	public boolean canAcceptArgument() {
		return this.optionGroup.canAcceptArgument();
	}

	@Override
	public void clearOptionSelections() {
		this.optionGroup.clearOptionSelections();
	}

	@Override
	public void clearParsedOptionSelections() {
		this.optionGroup.clearParsedOptionSelections();
	}
	
	@Override
	public ArgParser<T> getArgParser() {
		return this.optionGroup.getArgParser();
	}
	
	@Override
	public String getArgSeparator() {
		return this.optionGroup.getArgSeparator();
	}

	@Override
	public ArgsValidator getArgsValidator() {
		return this.optionGroup.getArgsValidator();
	}

	@Override
	public List<String> getArguments() {
		return this.optionGroup.getArguments();
	}

	@Override
	public String getDescription() {
		return this.optionGroupDoc.getDescription();
	}

	@Override
	public String getHelp() {
		return this.optionGroupDoc.getHelp();
	}

	@Override
	public List<OptionComplete> getOptionCompletes() {
		return Collections.unmodifiableList(this.optionCompleteList);
	}

	@Override
	public List<OptionDoc> getOptionDocs() {
		return this.optionGroupDoc.getOptionDocs();
	}

	@Override
	public List<Option> getOptions() {
		return this.optionGroup.getOptions();
	}

	@Override
	public List<OptionSelection<String>> getOptionSelections() {
		return this.optionGroup.getOptionSelections();
	}

	@Override
	public int getOrdinal() {
		return this.optionGroupDoc.getOrdinal();
	}

	@Override
	public List<T> getParsedArguments() {
		return this.optionGroup.getParsedArguments();
	}

	@Override
	public List<OptionSelection<T>> getParsedOptionSelections() {
		return this.optionGroup.getParsedOptionSelections();
	}

	@Override
	public Map<String, String> getProperties() {
		return this.optionGroupDoc.getProperties();
	}

	@Override
	public String getUsage() {
		return this.optionGroupDoc.getUsage();
	}

	@Override
	public boolean hasArgument() {
		return this.optionGroup.hasArgument();
	}

	@Override
	public boolean hasOptionSelection() {
		return this.optionGroup.hasOptionSelection();
	}

	@Override
	public boolean hasParsedArgument() {
		return this.optionGroup.hasParsedArgument();
	}

	@Override
	public boolean hasParsedOptionSelection() {
		return this.optionGroup.hasParsedOptionSelection();
	}

	@Override
	public boolean isHidden() {
		return this.hidden;
	}

	@Override
	public boolean isOptionSelectionsTerminative() {
		return this.optionGroup.isOptionSelectionsTerminative();
	}

	@Override
	public boolean isUnused() {
		return this.unused;
	}

	@Override
	public void parse() throws OptionGroupException {
		this.optionGroup.parse();
	}

	@Override
	public String toString() {
		return String.format("OptionGroupComplete [" 
				+ "hidden=%s, " 
				+ "unused=%s, " 
				+ "optionCompletes=%s, " 
				+ "optionGroupDoc=%s, " 
				+ "optionGroup=%s]",
				this.hidden, this.unused, 
				this.optionCompleteList, this.optionGroupDoc,
				this.optionGroup);
	}
}
