/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

/**
 * A builder for an {@code OptionGroupDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public interface OptionGroupDocBuilder {
	
	/**
	 * Builds a new {@code OptionGroupDoc}.
	 * 
	 * @return a new {@code OptionGroupDoc}
	 */
	OptionGroupDoc buildOptionGroupDoc();
	
	/**
	 * Sets the format string for the description for the 
	 * {@code OptionGroup}.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by a 
	 * dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionGroupDocPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.OptionGroupDocPropertyKeys#OPTIONDOC_PREFIX})  
	 * followed by an integer representing the index of an {@code OptionDoc}
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionDoc} and <i>n</i> is the last {@code OptionDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property key defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>{@link org.climax.OptionGroupDocPropertyKeys#USAGE}
	 * ({@value org.climax.OptionGroupDocPropertyKeys#USAGE})</td></tr> 
	 * </table>
	 *  
	 * @param s the provided format string for the description for the 
	 * {@code OptionGroup}
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */	
	OptionGroupDocBuilder description(final String s);
	
	/**
	 * Sets the format string for the help information for the 
	 * {@code OptionGroup}.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by a 
	 * dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionGroupDocPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.OptionGroupDocPropertyKeys#OPTIONDOC_PREFIX})  
	 * followed by an integer representing the index of an {@code OptionDoc}
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionDoc} and <i>n</i> is the last {@code OptionDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>{@link org.climax.OptionGroupDocPropertyKeys#DESCRIPTION}
	 * ({@value org.climax.OptionGroupDocPropertyKeys#DESCRIPTION})</td></tr>
	 * <tr><td>{@link org.climax.OptionGroupDocPropertyKeys#USAGE}
	 * ({@value org.climax.OptionGroupDocPropertyKeys#USAGE})</td></tr>  
	 * </table>
	 *  
	 * @param s the provided format string for the help information of the 
	 * {@code OptionGroup}
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */	
	OptionGroupDocBuilder help(final String s);
	
	/**
	 * Sets the ordinal of the {@code OptionGroupDoc} in relation
	 * to other {@code OptionGroupDoc}s.
	 * 
	 * @param i the provided ordinal of the {@code OptionGroupDoc}
	 * in relation to other {@code OptionGroupDoc}s
	 * 
	 * @return this builder
	 */	
	OptionGroupDocBuilder ordinal(final int i);
	
	/**
	 * Sets the format string for the usage of the {@code OptionGroup}.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by a 
	 * dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionGroupDocPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.OptionGroupDocPropertyKeys#OPTIONDOC_PREFIX})  
	 * followed by an integer representing the index of an {@code OptionDoc}
	 * (whose range is from {@code 0} to <i>n</i> where {@code 0} is the 
	 * first {@code OptionDoc} and <i>n</i> is the last {@code OptionDoc}), 
	 * a dot (.), and the {@code OptionDoc}'s property defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * </table>
	 * 
	 * @param s the provided format string for the usage of the 
	 * {@code OptionGroup}
	 * 
	 * @return this builder
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */	
	OptionGroupDocBuilder usage(final String s);
}