/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An implementation of an {@code OptionGroupDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionGroupDocImpl implements OptionGroupDoc {

	/**
	 * A builder for an {@code OptionGroupDocImpl}.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	static final class Builder implements OptionGroupDocBuilder {
		
		/** 
		 * The ordinal of the {@code OptionGroupDoc} in relation to 
		 * other {@code OptionGroupDoc}s. 
		 */
		private int ordinal;
		
		/** 
		 * The {@code List} of {@code OptionDoc}s. 
		 */
		private final List<OptionDoc> optionDocList;
		
		/** 
		 * The format string for the usage of the {@code OptionGroup}.
		 */
		private String usage;
		
		/** 
		 * The format string for the description of the {@code OptionGroup}.
		 */
		private String description;
		
		/**
		 * The format string for the help information for the 
		 * {@code OptionGroup}.
		 */
		private String help;
		
		/**
		 * Constructs an {@code OptionGroupDocImpl.Builder} with the provided
		 * {@code List} of {@code OptionDoc}s.
		 * 
		 * @param optDocList the provided {@code List} of {@code OptionDoc}s.
		 */
		Builder(final List<OptionDoc> optDocList) {
			
			this.ordinal = 0;
			
			this.optionDocList = new ArrayList<OptionDoc>(optDocList);
			
			this.usage = null;
			
			this.description = null;
			
			this.help = null;			
		}
		
		@Override
		public OptionGroupDoc buildOptionGroupDoc() {
			
			OptionGroupDoc optionGroupDoc = null;
			
			try {
				optionGroupDoc = new OptionGroupDocImpl(this);
			} catch (RuntimeException e) {
				throw new IllegalStateException(e);
			}
			
			return optionGroupDoc;
		}
		
		@Override
		public OptionGroupDocBuilder description(final String s) {
			this.description = s;
			return this;
		}
		
		@Override
		public OptionGroupDocBuilder help(final String s) {
			this.help = s;
			return this;
		}
		
		@Override
		public OptionGroupDocBuilder ordinal(final int i) {
			this.ordinal = i;
			return this;
		}
		
		@Override
		public OptionGroupDocBuilder usage(final String s) {
			this.usage = s;
			return this;
		}
	}
	
	/**
	 * Returns a default format string for the help information for the 
	 * {@code OptionGroup}.
	 * 
	 * @param optionGroupDocImpl the provided {@code OptionGroupDocImpl}
	 *  
	 * @return a default format string for the help information for the 
	 * {@code OptionGroup}
	 */
	private static String getDefaultHelp(
			final OptionGroupDocImpl optionGroupDocImpl) {
		
		List<OptionDoc> optionDocList = optionGroupDocImpl.optionDocList;
		String usage = optionGroupDocImpl.usage;
		String description = optionGroupDocImpl.description;		
		
		String usageFormat = "";
		
		if (!usage.isEmpty()) {
			
			usageFormat = String.format(
					"  %s",
					new Properties.Property(
							OptionGroupDocPropertyKeys.USAGE)
							.toExpandableString());
		}
		
		boolean usageLengthMatch = false;
		
		if (optionDocList.size() == 1) {
			
			OptionDoc optionDoc = optionDocList.get(0);
			
			int usageLength = optionDoc.getUsage().length();
			
			if (usageLength == 2) {
				
				usageLengthMatch = true;
			}
		}
		
		String descriptionFormat = "";
		
		if (!description.isEmpty()) {
			
			if (usageLengthMatch) {
				
				descriptionFormat = String.format(
						"  %s", 
						new Properties.Property(
								OptionGroupDocPropertyKeys.DESCRIPTION)
								.toExpandableString());
				
			} else {
				
				descriptionFormat = String.format(
						"\n      %s", 
						new Properties.Property(
								OptionGroupDocPropertyKeys.DESCRIPTION)
								.toExpandableString());
			}
		}
		
		String defaultHelp = String.format(
				"%1$s%2$s", 
				usageFormat, 
				descriptionFormat);
		
		return defaultHelp;
	}
	
	/**
	 * Returns a default format string for the usage of the 
	 * {@code OptionGroup}.
	 * 
	 * @param optionGroupDocImpl the provided {@code OptionGroupDocImpl}
	 * 
	 * @return a default format string for the usage of the 
	 * {@code OptionGroup}
	 */
	private static String getDefaultUsage(
			final OptionGroupDocImpl optionGroupDocImpl) {
		
		List<OptionDoc> optionDocList = optionGroupDocImpl.optionDocList;
		
		StringBuilder builder = new StringBuilder();
		
		for (int i = 0; i < optionDocList.size(); i++) {
			
			String optionDocUsageProperty = 
				new Properties.Property(Strings.concat(
						OptionGroupDocPropertyKeys.OPTIONDOC_PREFIX,
						Integer.toString(i),
						".", 
						OptionDocPropertyKeys.USAGE)).toExpandableString();
			
			
			builder.append(optionDocUsageProperty);
			
			if (i < optionDocList.size() - 1) {
				
				builder.append(", ");
			}
		}
		
		return builder.toString();
	}
	
	/**
	 * Updates the provided {@code OptionGroupDocImpl}'s documentation 
	 * properties.
	 * 
	 * @param optionGroupDocImpl the provided {@code OptionGroupDocImpl}
	 */
	private static void updateProperties(
			final OptionGroupDocImpl optionGroupDocImpl) {
		
		Map<String, String> properties = optionGroupDocImpl.properties;
		
		String description = optionGroupDocImpl.description;
		
		String help = optionGroupDocImpl.help;

		List<OptionDoc> optionDocList = optionGroupDocImpl.optionDocList;
		
		String usage = optionGroupDocImpl.usage;
		
		if (description != null) {
			
			if (!properties.containsKey(
					OptionGroupDocPropertyKeys.DESCRIPTION)) {
			
				properties.put(
						OptionGroupDocPropertyKeys.DESCRIPTION, description);
			}
		}
		
		if (help != null) {
			
			if (!properties.containsKey(OptionGroupDocPropertyKeys.HELP)) {
			
				properties.put(OptionGroupDocPropertyKeys.HELP, help);
			}
		}
		
		if (optionDocList != null) {
			
			for (int i = 0; i < optionDocList.size(); i++) {
				
				OptionDoc optionDoc = optionDocList.get(i);
				
				Map<String, String> optionDocProperties = 
					optionDoc.getProperties();
				
				for (Map.Entry<String, String> optionDocProperty 
						: optionDocProperties.entrySet()) {
					
					String optionDocPropertyKey = 
						OptionGroupDocPropertyKeys.OPTIONDOC_PREFIX
						+ Integer.toString(i)
						+ "."
						+ optionDocProperty.getKey();
					
					if (!properties.containsKey(optionDocPropertyKey)) {
						
						properties.put(
								optionDocPropertyKey, 
								optionDocProperty.getValue());
					}
				}
			}
		}
		
		if (usage != null) {
			
			if (!properties.containsKey(OptionGroupDocPropertyKeys.USAGE)) {
			
				properties.put(OptionGroupDocPropertyKeys.USAGE, usage);
			}
		}
	}
	
	/** 
	 * The {@code Map} for the documentation properties of this 
	 * {@code OptionGroupDocImpl}. 
	 */
	private final Map<String, String> properties;
	
	/** 
	 * The ordinal of this {@code OptionGroupDoc} in relation to 
	 * other {@code OptionGroupDoc}s. 
	 */
	private final int ordinal;
	
	/** 
	 * The {@code List} of {@code OptionDoc}s. 
	 */
	private final List<OptionDoc> optionDocList;
	
	/** 
	 * The usage for the {@code OptionGroup}.
	 */
	private final String usage;
	
	/** 
	 * The description for the {@code OptionGroup}.
	 */
	private final String description;
	
	/**
	 * The help information for the {@code OptionGroup}.
	 */
	private final String help;
	
	/**
	 * Constructs an {@code OptionGroupDocImpl} with the provided 
	 * {@code OptionGroupDocImpl.Builder}.
	 * 
	 * @param builder the provided {@code OptionGroupDocImpl.Builder}
	 */
	private OptionGroupDocImpl(final Builder builder) {
		
		int ord = builder.ordinal;
		List<OptionDoc> optDocList = 
			new ArrayList<OptionDoc>(builder.optionDocList);
		String use = builder.usage;
		String desc = builder.description;
		String h = builder.help;
		
		for (OptionDoc optionDoc : optDocList) {
			
			if (optionDoc == null) {
				throw new NullPointerException("optionDocs must not be null");
			}
		}
		
		Collections.sort(
				optDocList, 
				AbstractFactory.getDefault().getOptionDocComparator());
		
		this.properties = new Properties();
		
		this.ordinal = ord;
		
		this.optionDocList = new ArrayList<OptionDoc>();
		
		this.optionDocList.addAll(optDocList);
		
		OptionGroupDocImpl.updateProperties(this);
		
		if (use == null || use.isEmpty()) {
			use = OptionGroupDocImpl.getDefaultUsage(this);
		}
		
		this.usage = Util.expand4(use, this.properties);
		
		OptionGroupDocImpl.updateProperties(this);
		
		if (desc == null) { desc = ""; }
		
		this.description = Util.expand4(desc, this.properties);
		
		OptionGroupDocImpl.updateProperties(this);		
		
		if (h == null || h.isEmpty()) {
			h = OptionGroupDocImpl.getDefaultHelp(this);
		}
		
		this.help = Util.expand4(h, this.properties);
		
		OptionGroupDocImpl.updateProperties(this);
	}
	
	@Override
	public String getDescription() {
		return this.description;
	}

	@Override
	public String getHelp() {
		return this.help;
	}

	@Override
	public List<OptionDoc> getOptionDocs() {
		return Collections.unmodifiableList(this.optionDocList);
	}

	@Override
	public int getOrdinal() {
		return this.ordinal;
	}

	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}

	@Override
	public String getUsage() {
		return this.usage;
	}

	@Override
	public String toString() {
		return String.format("OptionGroupDoc [" 
				+ "properties=%s, " 
				+ "ordinal=%s, " 
				+ "optionDocs=%s, " 
				+ "usage=%s, " 
				+ "description=%s, " 
				+ "help=%s]",
				this.properties, 
				this.ordinal, 
				this.optionDocList, 
				this.usage, 
				this.description,
				this.help);
	}
	
}
