/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.climax.argparser.NullArgParser;
import org.climax.argsvalidator.NoArgsValidator;


/**
 * An implementation of an {@code OptionGroup} of the specified 
 * parsed argument type.
 * 
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @param <T> the specified parsed argument type
 */
final class OptionGroupImpl<T> implements OptionGroup<T> {

	/**
	 * A builder for an {@code OptionGroupImpl} of the specified parsed 
	 * argument type.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 *
	 * @param <T> the specified parsed argument type
	 */
	static final class Builder<T> implements OptionGroupBuilder<T> {
		
		/** The {@code List} of {@code Option}s. */
		private final List<Option> optionList;
		
		/** The format string for the argument separator. */
		private String argSeparator;
		
		/** The {@code ArgsValidator}. */
		private ArgsValidator argsValidator;
		
		/** The {@code ArgParser}. */
		private ArgParser<? extends T> argParser;
		
		/** The {@code List} of default argument format strings. */
		private final List<String> defaultArgumentList;
		
		/**
		 * The boolean value to indicate if the {@code OptionGroup}'s 
		 * {@code OptionSelection}s of type {@code String} terminates 
		 * the parsing of any other command line argument.
		 */		
		private boolean optionSelectionsTerminative;
		
		/**
		 * Constructs an {@code OptionGroupImpl.Builder} with the provided 
		 * {@code List} of {@code Option}s.
		 * 
		 * @param optList the provided {@code List} of {@code Option}s
		 */
		Builder(final List<Option> optList) {
			
			this.optionList = new ArrayList<Option>(optList);
			
			this.argSeparator = null;
			
			this.argsValidator = null;
			
			this.argParser = null;
			
			this.defaultArgumentList = new ArrayList<String>();
			
			this.optionSelectionsTerminative = false;
		}
		
		@Override
		public OptionGroupBuilder<T> argParser(final ArgParser<? extends T> p) {
			this.argParser = p;
			return this;
		}

		@Override
		public OptionGroupBuilder<T> argSeparator(final String s) {
			this.argSeparator = s;
			return this;
		}
		
		@Override
		public OptionGroupBuilder<T> argsValidator(final ArgsValidator v) {
			this.argsValidator = v;
			return this;
		}
		
		@Override
		public OptionGroup<T> buildOptionGroup() {
			
			OptionGroup<T> optionGroup = null;
			
			try {
				optionGroup = new OptionGroupImpl<T>(this);
			} catch (RuntimeException e) {
				throw new IllegalStateException(e);
			}
			
			return optionGroup;
		}
		
		@Override
		public OptionGroupBuilder<T> defaultArguments(final List<String> l) {
			this.defaultArgumentList.clear();
			this.defaultArgumentList.addAll(l);
			return this;
		}
		
		@Override
		public OptionGroupBuilder<T> optionSelectionsTerminative(
				final boolean b) {
			
			this.optionSelectionsTerminative = b;
			return this;
		}
	}
	
	/**
	 * Returns the detail message specifying if any of the arguments that are 
	 * invalid or cannot be parsed, the {@code Option}s, and the additional 
	 * specified detail message.
	 * 
	 * @param optionSelectionList the provided {@code List} of 
	 * {@code OptionSelection}s of type {@code String} are invalid or 
	 * cannot be parsed
	 * 
	 * @param message the additional detail message
	 * 
	 * @return the detail message specifying if any of the arguments that are 
	 * invalid or cannot be parsed, the {@code Option}s, and the additional 
	 * specified detail message
	 */
	private static String getMessage(
			final List<OptionSelection<String>> optionSelectionList,
			final String message) {
		
		Set<Option> optionSet = new TreeSet<Option>(
				AbstractFactory.getDefault().getOptionComparator());

		List<String> argumentList = new ArrayList<String>();		
		
		for (OptionSelection<String> optionSelection 
				: optionSelectionList) {
			
			Option option = optionSelection.getOption();
			
			String argument = optionSelection.getArgument();
			
			optionSet.add(option);
			
			if (argument != null) {
				
				argumentList.add(argument);
			}
		}

		List<String> optionStringList = new ArrayList<String>();
		
		for (Option option : optionSet) {
			optionStringList.add(option.toOptionString());
		}
		
		String optionListString = Lists.toString(optionStringList);
		
		String argumentListString = Lists.toString(argumentList);
		
		String m = String.format(
				"no arguments for %1$s : %2$s",
				optionListString,
				message);
		
		if (argumentList.size() == 1) {
			
			m = String.format(
					"invalid argument %1$s for %2$s : %3$s",
					argumentListString,
					optionListString,
					message);
		}
		
		if (argumentList.size() > 1) {
			
			m = String.format(
					"invalid arguments %1$s for %2$s : %3$s",
					argumentListString,
					optionListString,
					message);
		}
		
		return m;
	}
	
	/** The {@code List} of {@code Option}s. */
	private final List<Option> optionList;
	
	/** The argument separator. */
	private final String argSeparator;
	
	/** The {@code ArgsValidator}. */
	private final ArgsValidator argsValidator;
	
	/** The {@code ArgParser}. */
	private final ArgParser<? extends T> argParser;
	
	/** The {@code List} of default arguments. */
	private final List<String> defaultArgumentList;
	
	/** 
	 * The {@code List} of parsed default arguments of the specified parsed 
	 * argument type.
	 */
	private final List<T> parsedDefaultArgumentList;
	
	/**
	 * The boolean value to indicate if the {@code OptionGroup}'s
	 * {@code OptionSelection}s of type {@code String} terminates 
	 * the parsing of any other command line argument.
	 */		
	private final boolean optionSelectionsTerminative;
	
	/**
	 * The boolean value to indicate if this {@code OptionGroup} accepts 
	 * {@code OptionSelection}s with non {@code null} arguments.
	 */
	private final boolean canAcceptArgument;
	
	/**
	 * The {@code List} of {@code OptionSelection}s of type 
	 * {@code String}.
	 */
	private final List<OptionSelection<String>> optionSelectionList;
	
	/**
	 * The {@code List} of {@code OptionSelection}s of the specified 
	 * parsed argument type.
	 */
	private final List<OptionSelection<T>> parsedOptionSelectionList;
	
	/**
	 * Constructs an {@code OptionGroupImpl} with the provided
	 * {@code OptionGroupImpl.Builder}.
	 * 
	 * @param builder the provided {@code OptionGroupImpl.Builder}
	 */
	private OptionGroupImpl(final Builder<T> builder) {
		
		List<Option> optList = new ArrayList<Option>(builder.optionList);
		String argSep = builder.argSeparator;
		ArgsValidator validator = builder.argsValidator;
		ArgParser<? extends T> parser = builder.argParser;
		boolean canAcceptArg = false;
		List<String> defaultArgList = new ArrayList<String>();
		
		for (String defaultArgument : builder.defaultArgumentList) {
			
			if (defaultArgument == null) {
				throw new IllegalStateException(
						"default arguments must not be null");
			}
			
			String defaultArg = Util.expand1(
					defaultArgument, 
					AbstractFactory.getDefault().getSystemDocProperties());
			
			defaultArgList.add(defaultArg);
		}
		
		boolean optionSelectTerminative = 
				builder.optionSelectionsTerminative;
		
		List<T> parsedDefaultArgList = new ArrayList<T>();
		
		for (Option option : optList) {
			
			if (option == null) {
				throw new NullPointerException("options must not be null");
			}
		}
		
		Collections.sort(
				optList, 
				AbstractFactory.getDefault().getOptionComparator());
		
		if (argSep == null) {
			argSep = ArgSeparators.NON_ARG_SEPARATOR;
		}
		
		canAcceptArg = validator != null && parser != null;
		
		if (validator == null) {
			validator = NoArgsValidator.getInstance();
		}
		
		if (parser == null) {
			parser = NullArgParser.newInstance();
		}

		if (defaultArgList.size() > 0) {
			
			parsedDefaultArgList = DefaultArgs.parse(
					defaultArgList, 
					validator, 
					parser);
		}
		
		this.optionList = optList;
		
		this.argSeparator = Util.expand1(
				argSep, 
				AbstractFactory.getDefault().getSystemDocProperties());
		
		this.canAcceptArgument = canAcceptArg;
		
		this.argsValidator = validator;
		
		this.argParser = parser;
		
		this.defaultArgumentList = new ArrayList<String>(defaultArgList);
		
		this.optionSelectionList = 
				new ArrayList<OptionSelection<String>>();
		
		this.optionSelectionsTerminative = optionSelectTerminative;

		this.parsedDefaultArgumentList = new ArrayList<T>(
				parsedDefaultArgList);
		
		this.parsedOptionSelectionList = 
				new ArrayList<OptionSelection<T>>();
	}

	@Override
	public void addOptionSelection(
			final OptionSelection<String> optionSelection) {
	
		Option option = optionSelection.getOption();
		
		String argument = optionSelection.getArgument();
		
		boolean recognized = false;
		
		Comparator<Option> comparator = 
				AbstractFactory.getDefault().getOptionComparator(); 
		
		for (Option o : this.getOptions()) {
			
			if (comparator.compare(option, o) == 0) {
				recognized = true;
				break;
			}
		}
		
		if (!recognized) {
			
			String message = String.format(
					"unrecognized option '%s'", 
					option.toOptionString());
			
			throw new IllegalArgumentException(message);
		}
		
		if (!this.canAcceptArgument() && argument != null) {
			
			String message = String.format(
					"option '%s' does not allow an argument", 
					option.toOptionString());
			
			throw new IllegalArgumentException(message);
		}
		
		List<OptionSelection<String>> optionSelectList =
			new ArrayList<OptionSelection<String>>();
		
		if (argument == null) {
			
			optionSelectList.add(optionSelection);
		}
		
		if (argument != null) {

			String[] args = argument.split(this.argSeparator);
			
			if (args.length == 1) {
				
				optionSelectList.add(optionSelection);
				
			} else if (args.length > 1) {

				for (String arg : args) {
					
					OptionSelection<String> optionSelect =
							AbstractFactory.getDefault().newOptionSelection(
									option, arg);
					
					optionSelectList.add(optionSelect);					
				}
			}
		}
		
		this.optionSelectionList.addAll(optionSelectList);
	}

	@Override
	public boolean canAcceptArgument() {
		return this.canAcceptArgument;
	}

	@Override
	public void clearOptionSelections() {
		this.optionSelectionList.clear();
	}

	@Override
	public void clearParsedOptionSelections() {
		this.parsedOptionSelectionList.clear();
	}
	
	@Override
	public ArgParser<T> getArgParser() {
		@SuppressWarnings("unchecked")
		ArgParser<T> aParser = (ArgParser<T>) this.argParser;
		return aParser;
	}

	@Override
	public String getArgSeparator() {
		return this.argSeparator;
	}
	
	@Override
	public ArgsValidator getArgsValidator() {
		return this.argsValidator;
	}
	
	@Override
	public List<String> getArguments() {
		
		List<String> argumentList = new ArrayList<String>();
		
		if (this.hasOptionSelection()) {
			
			for (OptionSelection<String> optionSelection 
					: this.optionSelectionList) {
				
				String argument = optionSelection.getArgument();
				
				if (argument != null) {
					
					argumentList.add(argument);
				}
			}
			
		} else {
			
			argumentList.addAll(this.defaultArgumentList);

		}
		
		return Collections.unmodifiableList(argumentList);
	}

	@Override
	public List<Option> getOptions() {
		return Collections.unmodifiableList(this.optionList);
	}
	
	@Override
	public List<OptionSelection<String>> getOptionSelections() {
		return Collections.unmodifiableList(this.optionSelectionList);
	}

	@Override
	public List<T> getParsedArguments() {
		
		List<T> parsedArgumentList = new ArrayList<T>();
		
		if (this.hasOptionSelection()) {
			
			for (OptionSelection<T> parsedOptionSelection 
					: this.parsedOptionSelectionList) {
				
				T parsedArgument = parsedOptionSelection.getArgument();
				
				if (parsedArgument != null) {
					
					parsedArgumentList.add(parsedArgument);
				}
			}
			
		} else {

			parsedArgumentList.addAll(this.parsedDefaultArgumentList);
		}
		
		return Collections.unmodifiableList(parsedArgumentList);
	}

	@Override
	public List<OptionSelection<T>> getParsedOptionSelections() {
		return Collections.unmodifiableList(
				this.parsedOptionSelectionList);
	}
	
	@Override
	public boolean hasArgument() {		

		boolean hasArgument = false;
		
		if (this.hasOptionSelection()) {
			
			for (OptionSelection<String> optionSelection 
					: this.optionSelectionList) {
				
				String argument = optionSelection.getArgument();
				
				if (argument != null) {
					
					hasArgument = true;
					
					break;
				}
			}
			
		} else {
			
			hasArgument = this.defaultArgumentList.size() > 0;
		}
		
		return hasArgument;
	}
	
	@Override
	public boolean hasOptionSelection() {
		return this.optionSelectionList.size() > 0;
	}

	@Override
	public boolean hasParsedArgument() {
		
		boolean hasParsedArgument = false;
		
		if (this.hasParsedOptionSelection()) {
			
			for (OptionSelection<T> parsedOptionSelection
					: this.parsedOptionSelectionList) {
				
				T parsedArgument = parsedOptionSelection.getArgument();
				
				if (parsedArgument != null) {
					
					hasParsedArgument = true;
					
					break;
				}
			}
			
		} else {
			
			hasParsedArgument = this.parsedDefaultArgumentList.size() > 0;
		}
		
		return hasParsedArgument;
	}

	@Override
	public boolean hasParsedOptionSelection() {
		return this.parsedOptionSelectionList.size() > 0;
	}

	@Override
	public boolean isOptionSelectionsTerminative() {
		return this.optionSelectionsTerminative;
	}

	@Override
	public void parse() throws OptionGroupException {
		
		Option overridingOption = null;
		
		List<OptionSelection<String>> optionSelectList = 
			new ArrayList<OptionSelection<String>>();
		
		List<String> argumentList = new ArrayList<String>();
		
		for (int i = this.optionSelectionList.size() - 1; i >= 0; i--) {
			
			OptionSelection<String> optionSelection = 
					this.optionSelectionList.get(i);
			
			Option option = optionSelection.getOption();
			
			String argument = optionSelection.getArgument();
			
			if (overridingOption == null) { overridingOption = option; }
			
			Comparator<Option> comparator = 
					AbstractFactory.getDefault().getOptionComparator();
			
			if (comparator.compare(overridingOption, option) != 0) {
				continue;
			}
			
			optionSelectList.add(0, optionSelection);
			
			if (argument != null) { argumentList.add(0, argument); }
		}
		
		if (optionSelectList.size() > 0) {
			
			try {

				this.argsValidator.validate(argumentList);

			} catch (ArgsValidatorException e) {
				
				String message = OptionGroupImpl.getMessage(
						optionSelectList, 
						e.getMessage());
				
				throw new OptionGroupException(
						optionSelectList,
						message, 
						e);
			}		
		}

		List<OptionSelection<T>> parsedOptionSelectList = 
				new ArrayList<OptionSelection<T>>();
		
		for (OptionSelection<String> optionSelect 
				: optionSelectList) {

			Option option = optionSelect.getOption();
			
			String argument = optionSelect.getArgument();
			
			if (argument == null) {
				
				OptionSelection<T> parsedOptionSelect = 
						AbstractFactory.getDefault().newOptionSelection(
								option, null);
				
				parsedOptionSelectList.add(parsedOptionSelect);
			}
			
			if (argument != null) {
				
				try {

					T parsedArgument = this.argParser.parse(argument);
					
					OptionSelection<T> parsedOptionSelect = 
							AbstractFactory.getDefault().newOptionSelection(
									option, parsedArgument);
					
					parsedOptionSelectList.add(parsedOptionSelect);
					
				} catch (ArgParserException e) {
					
					@SuppressWarnings("unchecked")
					List<OptionSelection<String>> optionSelList = 
							Arrays.asList(optionSelect);
					
					String message = OptionGroupImpl.getMessage(
							optionSelList, 
							e.getMessage());
					
					throw new OptionGroupException(
							optionSelList,
							message, 
							e);
				}
			}			
		}

		this.parsedOptionSelectionList.clear();
		this.parsedOptionSelectionList.addAll(parsedOptionSelectList);
	}

	@Override
	public String toString() {
		return String.format("OptionGroup [" 
				+ "options=%s, " 
				+ "argSeparator=%s, " 
				+ "argsValidator=%s, " 
				+ "argParser=%s, " 
				+ "optionSelectionsTerminative=%s, " 
				+ "canAcceptArgument=%s, " 
				+ "optionSelections=%s, " 
				+ "parsedOptionSelections=%s]",
				this.optionList, 
				this.argSeparator, this.argsValidator, this.argParser,
				this.optionSelectionsTerminative,
				this.canAcceptArgument, this.optionSelectionList,
				this.parsedOptionSelectionList);
	}
}
