/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;


/**
 * An implementation of an {@code Option}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionImpl implements Option {
	
	/** The {@code OptionType} of this {@code Option}. */
	private final OptionType optionType;
	
	/** The name of this {@code Option}. */
	private final String optionName;
	
	/** 
	 * The {@code String} representation of this {@code Option} as it would 
	 * appear in the command line. 
	 */
	private final String optionString;

	/**
	 * Constructs a new {@code OptionImpl} with the provided {@code OptionType}
	 * and the specified name.
	 * 
	 * @param type the provided {@code OptionType}
	 * @param name the specified name
	 */
	OptionImpl(final OptionType type, final String name) {
		
		if (type == null) {
			throw new NullPointerException("OptionType must not be null");
		}
		
		if (name == null) {
			throw new NullPointerException("name must not be null");
		}
		
		if (name.isEmpty()) {
			throw new IllegalArgumentException("name must not be empty");
		}

		switch (type) {
		
		case POSIX:
			
			if (name.length() != 1) {
				throw new IllegalArgumentException(String.format(
						"name of option of OptionType %s must be one character"
						+ " long",
						type));
			}
			
			if (!Character.isLetterOrDigit(name.charAt(0))) {
				throw new IllegalArgumentException(String.format(
						"name of option of OptionType %s must be a letter or a"
						+ " digit",
						type));
			}

			break;
		
		case GNU_SHORT_LONG:
		case GNU_LONG:
			
			if (name.matches(".*\\s.*")) {
				throw new IllegalArgumentException(String.format(
						"name of option of OptionType %s must not contain any " 
						+ "whitespace characters",
						type));
			}
			
			break;
			
		default:
			throw new AssertionError(String.format(
					"Unknown %1$s: %2$s", 
					OptionType.class.getName(), 
					type));
		}
		
		this.optionType = type;
		this.optionName = name;
		this.optionString = type.getDelimiter().concat(name);
	}
	
	@Override
	public String getName() {
		return this.optionName;
	}

	@Override
	public OptionType getOptionType() {
		return this.optionType;
	}
	
	@Override
	public String toOptionString() {
		return this.optionString;
	}

	@Override
	public String toString() {
		return String.format(
				"Option [optionType=%s, name=%s, optionString=%s]",
				this.optionType, this.optionName, this.optionString);
	}
}
