/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.Collections;
import java.util.Map;

/**
 * The documentation of an {@code OptionType}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public enum OptionTypeDoc implements Doc {
	
	/** 
	 * The documentation of {@link org.climax.OptionType#POSIX}. 
	 */
	POSIX(OptionType.POSIX),
	
	/** 
	 * The documentation of {@link org.climax.OptionType#GNU_SHORT_LONG}. 
	 */
	GNU_SHORT_LONG(OptionType.GNU_SHORT_LONG),
	
	/** 
	 * The documentation of {@link org.climax.OptionType#GNU_LONG}. 
	 */
	GNU_LONG(OptionType.GNU_LONG);

	/** 
	 * Lazy initialization holder class idiom for static field for
	 * {@code OptionTypeDocsPropertiesHolder.OPTIONTYPEDOCS_PROPERTIES}.
	 */
	private static final class OptionTypeDocsPropertiesHolder {
		
		/**
		 * The unmodifiable {@code Map} for the documentation properties of all 
		 * of the {@code PackageDocImpl}s.
		 */
		static final Map<String, String> OPTIONTYPEDOCS_PROPERTIES;
		
		static {
			
			Map<String, String> optionTypeDocsProperties = new Properties();
			
			for (OptionTypeDoc optionTypeDoc : OptionTypeDoc.values()) {
				
				Map<String, String> optionTypeDocProperties = 
						optionTypeDoc.getProperties();
				
				for (Map.Entry<String, String> optionTypeDocProperty
						: optionTypeDocProperties.entrySet()) {
					
					String optionTypeDocPropertyKey = 
							optionTypeDoc.toString()
							+ "."
							+ optionTypeDocProperty.getKey();
					
					optionTypeDocsProperties.put(
							optionTypeDocPropertyKey, 
							optionTypeDocProperty.getValue());
				}
			}
			
			OPTIONTYPEDOCS_PROPERTIES = 
					Collections.unmodifiableMap(optionTypeDocsProperties);
		}
		
		/**
		 * Constructs an {@code OptionTypeDocsPropertiesHolder}. This 
		 * constructor is suppressed for noninstantiability outside of this 
		 * class.
		 */
		private OptionTypeDocsPropertiesHolder() {
		}
	}
	
	/**
	 * Returns an unmodifiable {@code Map} for the documentation properties of 
	 * all of the {@code OptionTypeDoc}s.
	 * 
	 * @return an unmodifiable {@code Map} for the documentation properties of 
	 * all of the {@code OptionTypeDoc}s
	 */
	public static Map<String, String> getOptionTypeDocsProperties() {
		return OptionTypeDocsPropertiesHolder.OPTIONTYPEDOCS_PROPERTIES;
	}
	
	/**
	 * Returns the enum constant of this type with the provided 
	 * {@code OptionType}.
	 * 
	 * @param optionType the provided {@code OptionType} of the enum constant 
	 * to be returned
	 * 
	 * @return the enum constant with the provided {@code OptionType}
	 */
	public static OptionTypeDoc of(final OptionType optionType) {
		
		for (OptionTypeDoc optionTypeDoc : OptionTypeDoc.values()) {
			
			if (optionTypeDoc.getOptionType().equals(optionType)) {
				
				return optionTypeDoc;
			}
		}
		
		return null;
	}
	
	/**
	 * The {@code Map} for the documentation properties of this 
	 * {@code OptionTypeDoc}.
	 */
	private final Map<String, String> properties;
	
	/** The {@code OptionType}. */
	private final OptionType optionType;
	
	/**
	 * Constructs an {@code OptionTypeDoc} with the provided 
	 * {@code OptionType}.
	 * 
	 * @param optType the provided {@code OptionType}
	 */
	private OptionTypeDoc(final OptionType optType) {
		
		this.properties = new Properties();
		
		this.optionType = optType;
		
		this.properties.put(
				OptionTypeDocPropertyKeys.DELIMITER, 
				this.optionType.getDelimiter());
		this.properties.put(
				OptionTypeDocPropertyKeys.ASSIGNMENT_OPERATOR, 
				this.optionType.getAssignmentOperator());
	}
	
	/**
	 * Returns the {@code OptionType}.
	 * 
	 * @return the {@code OptionType}
	 */
	public OptionType getOptionType() {
		return this.optionType;
	}

	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}
}