/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An implementation of an {@code OptionsComplete}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionsCompleteImpl 
	implements OptionsComplete {
	
	/** The {@code List} of {@code OptionGroupComplete}s. */
	private final List<OptionGroupComplete<Object>> optionGroupCompleteList;
	
	/** The {@code List} of {@code OptionComplete}s. */
	private final List<OptionComplete> optionCompleteList;
	
	/** The {@code OptionsDoc}. */
	private final OptionsDoc optionsDoc;
	
	/** The {@code Options}. */
	private final Options options;
	
	/**
	 * Constructs an {@code OptionsCompleteImpl}.
	 */
	OptionsCompleteImpl() {
		
		this.optionGroupCompleteList = 
				new ArrayList<OptionGroupComplete<Object>>();
		this.optionCompleteList = new ArrayList<OptionComplete>();
		this.optionsDoc = AbstractFactory.getDefault().newOptionsDoc();
		this.options = AbstractFactory.getDefault().newOptions();
	}
	
	/**
	 * Constructs an {@code OptionsCompleteImpl} with the provided 
	 * {@code OptionsComplete}.
	 * 
	 * @param optionsComplete the provided {@code OptionsComplete}
	 */
	OptionsCompleteImpl(final OptionsComplete optionsComplete) {
		
		this.optionGroupCompleteList = 
				new ArrayList<OptionGroupComplete<Object>>(
						optionsComplete.getOptionGroupCompletes());
		this.optionCompleteList = new ArrayList<OptionComplete>(
				optionsComplete.getOptionCompletes());
		this.optionsDoc = AbstractFactory.getDefault().newOptionsDoc(
				optionsComplete);
		this.options = AbstractFactory.getDefault().newOptions(
				optionsComplete);
	}
	
	@Override
	public void addOptionGroup(
			final OptionGroup<? extends Object> optionGroup) {
		
		this.options.addOptionGroup(optionGroup);
	}
	
	@Override
	public void addOptionGroupComplete(
			final OptionGroupComplete<? extends Object> optionGroupComplete) {
		
		if (optionGroupComplete == null) {
			throw new NullPointerException(
					"optionGroupComplete must not be null");
		}
		
		if (!optionGroupComplete.isHidden()) {
			this.optionsDoc.addOptionGroupDoc(optionGroupComplete);
		}
		
		if (!optionGroupComplete.isUnused()) {
			this.options.addOptionGroup(optionGroupComplete);
		}
		
		@SuppressWarnings("unchecked")
		OptionGroupComplete<Object> optGrpComplete =
			(OptionGroupComplete<Object>) optionGroupComplete;
		
		this.optionGroupCompleteList.add(optGrpComplete);
		
		this.optionCompleteList.addAll(optGrpComplete.getOptionCompletes());
	}
	
	@Override
	public void addOptionGroupDoc(final OptionGroupDoc optionGroupDoc) {
		this.optionsDoc.addOptionGroupDoc(optionGroupDoc);
	}
	
	@Override
	public String getHelp() {
		return this.optionsDoc.getHelp();
	}

	@Override
	public List<OptionComplete> getOptionCompletes() {
		return Collections.unmodifiableList(this.optionCompleteList);
	}

	@Override
	public List<OptionDoc> getOptionDocs() {
		return this.optionsDoc.getOptionDocs();
	}

	@Override
	public List<OptionGroupComplete<Object>> getOptionGroupCompletes() {
		return Collections.unmodifiableList(this.optionGroupCompleteList);
	}

	@Override
	public List<OptionGroupDoc> getOptionGroupDocs() {
		return this.optionsDoc.getOptionGroupDocs();
	}

	@Override
	public List<OptionGroup<Object>> getOptionGroups() {
		return this.options.getOptionGroups();
	}

	@Override
	public List<Option> getOptions() {
		return this.options.getOptions();
	}

	@Override
	public Map<String, String> getProperties() {
		return this.optionsDoc.getProperties();
	}

	@Override
	public String[] getRemainingArguments() {
		return this.options.getRemainingArguments();
	}

	@Override
	public String getUsage() {
		return this.optionsDoc.getUsage();
	}

	@Override
	public void parse(final String[] args) throws OptionsException {
		this.options.parse(args);
	}

	@Override	
	public void setHelp(final String s) {
		this.optionsDoc.setHelp(s);
	}

	@Override	
	public void setUsage(final String s) {
		this.optionsDoc.setUsage(s);
	}

	@Override
	public String toString() {
		return String.format("OptionsComplete [" 
				+ "optionGroupCompletes=%s, " 
				+ "optionCompletes=%s, " 
				+ "optionsDoc=%s, " 
				+ "options=%s]",
				this.optionGroupCompleteList, 
				this.optionCompleteList,
				this.optionsDoc, 
				this.options);
	}
}
