/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.List;

/**
 * The documentation for the {@code Options}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public interface OptionsDoc extends Doc {
	
	/**
	 * Adds an {@code OptionGroupDoc} to this {@code OptionsDoc}.
	 * 
	 * @param optionGroupDoc the provided {@code OptionGroupDoc} 
	 */
	void addOptionGroupDoc(final OptionGroupDoc optionGroupDoc);
	
	/**
	 * Returns the help information for the {@code Options}.
	 * 
	 * @return the help information for the {@code Options}
	 */
	String getHelp();
	
	/**
	 * Returns a sorted unmodifiable {@code List} of {@code OptionDoc}s. The
	 * {@code OptionDoc}s are sorted first by their {@code OptionGroupDoc}s 
	 * and then by each other. 
	 * 
	 * @return a sorted unmodifiable {@code List} of {@code OptionDoc}s
	 */	
	List<OptionDoc> getOptionDocs();
	
	/**
	 * Returns a sorted unmodifiable {@code List} of {@code OptionGroupDoc}s.
	 * 
	 * @return a sorted unmodifiable {@code List} of {@code OptionGroupDoc}s
	 */
	List<OptionGroupDoc> getOptionGroupDocs();
	
	/**
	 * Returns the usage of the {@code Options}.
	 * 
	 * @return the usage of the {@code Options}
	 */	
	String getUsage();
	
	/**
	 * Sets the format string for the help information for the {@code Options}.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by a 
	 * dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionsDocPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.OptionsDocPropertyKeys#OPTIONDOC_PREFIX}) followed 
	 * by an integer representing the index of an {@code OptionDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code OptionDoc} of the first {@code OptionGroupDoc} and <i>n</i> is 
	 * the last {@code OptionDoc} of the last {@code OptionGroupDoc}), a dot 
	 * (.), and the {@code OptionDoc}'s property defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionsDocPropertyKeys#OPTIONGROUPDOC_PREFIX}
	 * ({@value org.climax.OptionsDocPropertyKeys#OPTIONGROUPDOC_PREFIX})  
	 * followed by an integer representing the index of an 
	 * {@code OptionGroupDoc} (whose range is from {@code 0} to <i>n</i> where
	 * {@code 0} is the first {@code OptionGroupDoc} and <i>n</i> is the last 
	 * {@code OptionGroupDoc}), a dot (.), and the {@code OptionGroupDoc}'s
	 * property defined in {@link org.climax.OptionGroupDocPropertyKeys}.
	 * </td></tr>
	 * <tr><td>{@link org.climax.OptionsDocPropertyKeys#USAGE}
	 * ({@value org.climax.OptionsDocPropertyKeys#USAGE})</td></tr>  
	 * </table>
	 *  
	 * @param s the provided format string for the help information for the 
	 * {@code Options}
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */	
	void setHelp(final String s);
	
	/**
	 * Sets the format string for the usage of the {@code Options}.
	 * <p>
	 * The following are property keys that can be applied as expandable 
	 * properties within the provided format string:
	 * </p>
	 * <table border="1"><tr><th>Property Key(s)</th></tr>
	 * <tr><td>Property keys defined in {@link System#getProperties()}</td></tr>
	 * <tr><td>Property keys prefixed with the name of a package followed by a 
	 * dot (.) and the {@code PackageDoc}'s property key defined in 
	 * {@link org.climax.PackageDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with the {@code String} representation of
	 * an {@link org.climax.OptionTypeDoc} followed by a dot (.) and the 
	 * {@link org.climax.OptionTypeDoc}'s property key defined in
	 * {@link org.climax.OptionTypeDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionsDocPropertyKeys#OPTIONDOC_PREFIX}
	 * ({@value org.climax.OptionsDocPropertyKeys#OPTIONDOC_PREFIX}) followed 
	 * by an integer representing the index of an {@code OptionDoc} (whose 
	 * range is from {@code 0} to <i>n</i> where {@code 0} is the first 
	 * {@code OptionDoc} of the first {@code OptionGroupDoc} and <i>n</i> is 
	 * the last {@code OptionDoc} of the last {@code OptionGroupDoc}), a dot 
	 * (.), and the {@code OptionDoc}'s property defined in 
	 * {@link org.climax.OptionDocPropertyKeys}.</td></tr>
	 * <tr><td>Property keys prefixed with 
	 * {@link org.climax.OptionsDocPropertyKeys#OPTIONGROUPDOC_PREFIX}
	 * ({@value org.climax.OptionsDocPropertyKeys#OPTIONGROUPDOC_PREFIX})  
	 * followed by an integer representing the index of an 
	 * {@code OptionGroupDoc} (whose range is from {@code 0} to <i>n</i> where
	 * {@code 0} is the first {@code OptionGroupDoc} and <i>n</i> is the last 
	 * {@code OptionGroupDoc}), a dot (.), and the {@code OptionGroupDoc}'s
	 * property defined in {@link org.climax.OptionGroupDocPropertyKeys}.
	 * </td></tr>
	 * </table>
	 *  
	 * @param s the provided format string for the usage of the {@code Options}
	 * 
	 * @see <a href="package-summary.html#3">Expandable Properties</a> 
	 */	
	void setUsage(final String s);
}
