/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An implementation of an {@code OptionsDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionsDocImpl implements OptionsDoc {
	
	/**
	 * Returns a default format string for the help information for the
	 * {@code Options}.
	 * 
	 * @param optionsDocImpl the provided {@code OptionsDocImpl}
	 * 
	 * @return a default format string for the help information for the
	 * {@code Options}
	 */
	private static String getDefaultHelp(final OptionsDocImpl optionsDocImpl) {
		
		List<OptionGroupDoc> optionGroupDocList = 
			optionsDocImpl.optionGroupDocList;
		
		StringBuilder builder = new StringBuilder();

		for (int i = 0; i < optionGroupDocList.size(); i++) {
			
			String optionGroupDocHelpProperty =
				new Properties.Property(Strings.concat(
						OptionsDocPropertyKeys.OPTIONGROUPDOC_PREFIX,
						Integer.toString(i),
						".", 
						OptionGroupDocPropertyKeys.HELP)).toExpandableString();
			
			builder.append(optionGroupDocHelpProperty);
			
			if (i < optionGroupDocList.size() - 1) {
				
				builder.append('\n');
			}
		}
		
		return builder.toString();
	}
	
	/**
	 * Returns a default format string for the usage of the {@code Options}.
	 * 
	 * @param optionsDocImpl the provided {@code OptionsDocImpl}
	 * 
	 * @return a default format string for the usage of the {@code Options}
	 */
	private static String getDefaultUsage(final OptionsDocImpl optionsDocImpl) {
		
		List<OptionGroupDoc> optionGroupDocList =
			optionsDocImpl.optionGroupDocList;
		
		if (optionGroupDocList.size() == 0) {
			return "";
		}
		
		if (optionGroupDocList.size() == 1) {
			return "[OPTION]";
		}
		
		return "[OPTION...]";
	}
	
	/**
	 * Updates the provided {@code OptionsDocImpl}'s properties.
	 * 
	 * @param optionsDocImpl the provided {@code OptionsDocImpl}.
	 */
	private static void updateProperties(final OptionsDocImpl optionsDocImpl) {
		
		Map<String, String> properties = optionsDocImpl.properties;
		
		String help = optionsDocImpl.help;
		
		List<OptionDoc> optionDocList = optionsDocImpl.optionDocList;
		
		List<OptionGroupDoc> optionGroupDocList = 
			optionsDocImpl.optionGroupDocList;
		
		String usage = optionsDocImpl.usage;
		
		if (help != null) {
			
			properties.put(OptionsDocPropertyKeys.HELP, help);
		}
		
		if (optionDocList != null) {
			
			for (int i = 0; i < optionDocList.size(); i++) {
				
				OptionDoc optionDoc = optionDocList.get(i);
				
				Map<String, String> optionDocProperties = 
					optionDoc.getProperties();
				
				for (Map.Entry<String, String> optionDocProperty 
						: optionDocProperties.entrySet()) {
					
					String optionDocPropertyKey = 
						OptionsDocPropertyKeys.OPTIONDOC_PREFIX
						+ Integer.toString(i)
						+ "."
						+ optionDocProperty.getKey();
					
					properties.put(
							optionDocPropertyKey, 
							optionDocProperty.getValue());					
				}			
			}
		}
		
		if (optionGroupDocList != null) {
			
			for (int i = 0; i < optionGroupDocList.size(); i++) {
				
				OptionGroupDoc optionGroupDoc = optionGroupDocList.get(i);
				
				Map<String, String> optionGroupDocProperties =
					optionGroupDoc.getProperties();
				
				for (Map.Entry<String, String> optionGroupDocProperty
						: optionGroupDocProperties.entrySet()) {
					
					String optionGroupDocPropertyKey =
						OptionsDocPropertyKeys.OPTIONGROUPDOC_PREFIX
						+ Integer.toString(i)
						+ "."
						+ optionGroupDocProperty.getKey();
					
					properties.put(
							optionGroupDocPropertyKey, 
							optionGroupDocProperty.getValue());					
				}
			}
		}

		if (usage != null) {
		
			properties.put(OptionsDocPropertyKeys.USAGE, usage);
		}
	}

	/** The {@code Map} for the properties of this {@code OptionsDocImpl}. */
	private final Map<String, String> properties;
	
	/** The {@code List} of {@code OptionGroupDoc}s. */
	private final List<OptionGroupDoc> optionGroupDocList;
	
	/** The {@code List} of {@code OptionDoc}s. */
	private final List<OptionDoc> optionDocList;
	
	/** 
	 * The default format string for the usage of the {@code Options}.
	 */
	private String defaultUsage;
	
	/** 
	 * The default format string for the help information for the 
	 * {@code Options}.
	 */
	private String defaultHelp;
	
	/** 
	 * The specified format string for the usage of the {@code Options}.
	 */	
	private String specifiedUsage;
	
	/** 
	 * The specified format string for the help information for the 
	 * {@code Options}.
	 */	
	private String specifiedHelp;
	
	/** 
	 * The usage of the {@code Options}.
	 */	
	private String usage;
	
	/** 
	 * The help information for the {@code Options}.
	 */
	private String help;
	
	/**
	 * Constructs an {@code OptionsDocImpl}.
	 */
	OptionsDocImpl() {
		
		this.properties = new Properties();
		this.optionGroupDocList = new ArrayList<OptionGroupDoc>();
		this.optionDocList = new ArrayList<OptionDoc>();
		this.defaultUsage = OptionsDocImpl.getDefaultUsage(this);
		this.defaultHelp = OptionsDocImpl.getDefaultHelp(this);
		this.specifiedUsage = null;
		this.specifiedHelp = null;
		this.usage = Util.expand4(
				this.defaultUsage, this.properties);
		this.help = Util.expand4(
				this.defaultHelp, this.properties);
	}
	
	/**
	 * Constructs an {@code OptionsDocImpl} with the provided 
	 * {@code OptionsDoc}.
	 * 
	 * @param optionsDoc the provided {@code OptionsDoc} 
	 */
	OptionsDocImpl(final OptionsDoc optionsDoc) {
		
		this();
		
		this.properties.putAll(optionsDoc.getProperties());
		this.optionGroupDocList.addAll(optionsDoc.getOptionGroupDocs());
		this.optionDocList.addAll(optionsDoc.getOptionDocs());
		this.usage = optionsDoc.getUsage();
		this.help = optionsDoc.getHelp();
	}

	@Override
	public void addOptionGroupDoc(final OptionGroupDoc optionGroupDoc) {
		
		if (optionGroupDoc == null) {
			throw new NullPointerException(
					"optionGroupDoc must not be null");
		}
		
		this.optionGroupDocList.add(optionGroupDoc);
		
		Collections.sort(
				this.optionGroupDocList, 
				AbstractFactory.getDefault().getOptionGroupDocComparator());
		
		this.optionDocList.clear();
		
		for (OptionGroupDoc optGrpDoc : this.optionGroupDocList) {
			
			this.optionDocList.addAll(optGrpDoc.getOptionDocs());
		}
		
		this.update();
	}

	@Override
	public String getHelp() {
		return this.help;
	}

	@Override
	public List<OptionDoc> getOptionDocs() {
		return Collections.unmodifiableList(this.optionDocList);
	}

	@Override
	public List<OptionGroupDoc> getOptionGroupDocs() {
		return Collections.unmodifiableList(this.optionGroupDocList);
	}

	@Override
	public Map<String, String> getProperties() {
		return Collections.unmodifiableMap(this.properties);
	}

	@Override
	public String getUsage() {
		return this.usage;
	}

	@Override
	public void setHelp(final String s) {
		
		this.specifiedHelp = s;
		
		this.update();
	}
	
	@Override
	public void setUsage(final String s) {
		
		this.specifiedUsage = s;
		
		this.update();
	}

	@Override
	public String toString() {
		return String.format("OptionsDoc [" 
				+ "properties=%s, " 
				+ "optionGroupDocs=%s, " 
				+ "optionDocs=%s, " 
				+ "usage=%s, " 
				+ "help=%s]",
				this.properties, 
				this.optionGroupDocList, 
				this.optionDocList, 
				this.usage,
				this.help);
	}

	/**
	 * Updates the usage and help information for the {@code Options}.
	 */
	private void update() {
		
		this.usage = null;
		
		this.help = null;
		
		OptionsDocImpl.updateProperties(this);
		
		this.defaultUsage = OptionsDocImpl.getDefaultUsage(this);
		
		if (this.specifiedUsage != null && !this.specifiedUsage.isEmpty()) {
		
			this.usage = Util.expand4(
					this.specifiedUsage, this.properties);
			
		} else {
			
			this.usage = Util.expand4(
					this.defaultUsage, this.properties);
		}
		
		OptionsDocImpl.updateProperties(this);
		
		this.defaultHelp = OptionsDocImpl.getDefaultHelp(this);
		
		if (this.specifiedHelp != null && !this.specifiedHelp.isEmpty()) {
			
			this.help = Util.expand4(
					this.specifiedHelp, this.properties);
			
		} else {
			
			this.help = Util.expand4(
					this.defaultHelp, this.properties);
		}
		
		OptionsDocImpl.updateProperties(this);
	}
}
