/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * An implementation of an {@code Options}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class OptionsImpl implements Options {
	
	/**
	 * A command line argument handler. The constants of this enumerated type 
	 * provides a classification of command line argument handlers.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static enum ArgHandler {
		
		/** 
		 * A command line argument handler that handles a command line option. 
		 */
		OPTION_HANDLER {
			
			@Override
			void handleArgument(
					final String current, 
					final Iterator<String> remaining, 
					final Context context) 
					throws ArgHandlerException {
				
				OptionHandler optionHandler = OptionHandler.bestHandles(
						current, context);
				
				try {
				
					optionHandler.handleOption(current, remaining, context);
					
				} catch (OptionHandlerException e) {
					
					throw new ArgHandlerException(e.getMessage(), e);
				}
			}

			@Override
			boolean handles(
					final String argument, final Context context) {
				
				return !context.isParsingTerminated() 
					&& OptionHandler.bestHandles(argument, context) != null;
			}
		},
		
		/** 
		 * A command line argument handler that handles a command line argument
		 * that terminates further parsing of command line options and their 
		 * arguments. 
		 */
		TERMINATION_ARG_HANDLER {
			
			@Override
			void handleArgument(
					final String current, 
					final Iterator<String> remaining, 
					final Context context) 
					throws ArgHandlerException {
				
				context.setParsingTerminated(true);
			}

			@Override
			boolean handles(
					final String argument, final Context context) {

				return !context.isParsingTerminated() 
					&& argument.equals("--");
			}
		},
		
		/** 
		 * A command line argument handler that handles an unspecified command 
		 * line argument (does nothing). 
		 */
		UNSPECIFIED_ARG_HANDLER {

			@Override
			public void handleArgument(
					final String current, 
					final Iterator<String> remaining,
					final Context context) 
					throws ArgHandlerException {
				
			}

			@Override
			public boolean handles(
					final String argument, final Context context) {
				
				for (ArgHandler argHandler : ArgHandler.values()) {
					
					if (!argHandler.equals(UNSPECIFIED_ARG_HANDLER) 
							&& argHandler.handles(argument, context)) {
						
						return false;
					}
				}
				
				return true;
			}
			
		};

		/**
		 * Returns the enum constant that can handle the provided command line 
		 * argument and the provided {@code Context}.
		 * 
		 * @param argument the provided command line argument
		 * 
		 * @param context the provided {@code Context}
		 *  
		 * @return an enum constant that can handle the provided command line 
		 * argument and the provided {@code Context} or {@code null} if there 
		 * is no enum constant that can handle the provided command line 
		 * argument and the provided {@code Context}
		 */
		static ArgHandler bestHandles(
				final String argument, final Context context) {
			
			for (ArgHandler argHandler : ArgHandler.values()) {
				
				if (argHandler.handles(argument, context)) {
					
					return argHandler;
				}
			}
			
			return null;
		}
		
		/**
		 * Handles the provided command line argument and the provided remaining
		 * command line arguments based on this {@code ArgHandler} and the 
		 * provided {@code Context}.
		 * <p>
		 * When obtaining the next remaining command line argument by invoking
		 * the {@code Iterator#next()} method, the {@code Iterator#remove()}
		 * method must be invoked afterwards. If the remaining command line 
		 * argument is not handled directly by this method, the remaining 
		 * command line argument's {@code ArgHandler} must be determined, and 
		 * the remaining command line argument and the other remaining command 
		 * line arguments must be handled by the determined {@code ArgHandler}.
		 * </p>
		 *     
		 * @param current the provided command line argument
		 * 
		 * @param remaining the provided remaining command line arguments
		 * 
		 * @param context the provided {@code Context}
		 *  
		 * @throws ArgHandlerException if the provided command line argument 
		 * cannot be handled
		 */
		abstract void handleArgument(
				String current, Iterator<String> remaining, Context context) 
				throws ArgHandlerException;
		
		/**
		 * Indicates if the provided command line argument based on the provided
		 * {@code Context} can be handled by this {@code ArgHandler}.
		 * 
		 * @param argument the provided command line argument
		 * 
		 * @param context the provided {@code Context}
		 * 
		 * @return {@code true} if the provided command line argument based on
		 * the provided {@code Context} can be handled by this 
		 * {@code ArgHandler}
		 */
		abstract boolean handles(String argument, Context context);
	}
	
	/**
	 * Thrown when the provided command line argument cannot be handled.
	 *  
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static final class ArgHandlerException extends Exception {

		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Constructs an {@code ArgHandlerException} with the specified detail 
		 * message and the {@code Throwable} that was raised.
		 * 
		 * @param message the detail message
		 * 
		 * @param cause the {@code Throwable} that was raised
		 */		
		ArgHandlerException(
				final String message, final Throwable cause) {
			
			super(message, cause);
		}
	}
	
	/**
	 * The context for parsing command line options and their arguments.
	 *   
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static final class Context {
	
		/**
		 * The {@code Map} of the {@code Option}s and their declaring
		 * {@code OptionGroup}s.  
		 */
		private final Map<Option, OptionGroup<Object>> optionMap;
		
		/**
		 * The selected and terminative {@code OptionGroup}.
		 */
		private OptionGroup<Object> selectedTerminativeOptionGroup;
		
		/**
		 * A boolean value that indicates if further parsing of command line 
		 * options and their arguments should be terminated. 
		 */
		private boolean parsingTerminated;
		
		/**
		 * Constructs a {@code Context} with the provided {@code Map} of the 
		 * {@code Option}s and their declaring {@code OptionGroup}s.
		 * 
		 * @param optMap the provided {@code Map} of the {@code Option}s and 
		 * their declaring {@code OptionGroup}s
		 */
		Context(final Map<Option, OptionGroup<Object>> optMap) {
			
			this.optionMap = new TreeMap<Option, OptionGroup<Object>>(
					AbstractFactory.getDefault().getOptionComparator());
			this.optionMap.putAll(optMap);
			this.selectedTerminativeOptionGroup = null;
			this.parsingTerminated = false;
		}
		
		/**
		 * Returns an unmodifiable {@code Map} of the {@code Option}s and 
		 * their declaring {@code OptionGroup}s.
		 * 
		 * @return an unmodifiable {@code Map} of the {@code Option}s and 
		 * their declaring {@code OptionGroup}s
		 */
		Map<Option, OptionGroup<Object>> getOptions() {
			return Collections.unmodifiableMap(this.optionMap);
		}
		
		/**
		 * Returns the selected and terminative {@code OptionGroup}.
		 * 
		 * @return the selected and terminative {@code OptionGroup} or 
		 * {@code null} if the selected and terminative {@code OptionGroup} 
		 * has not been provided
		 */
		OptionGroup<Object> getSelectedTerminativeOptionGroup() {
			return this.selectedTerminativeOptionGroup;
		}
		
		/**
		 * Indicates if this {@code Context} has an {@code Option} of the 
		 * provided {@code OptionType}.
		 * 
		 * @param optionType the provided {@code OptionType}
		 * 
		 * @return {@code true} if this {@code Context} has an {@code Option} 
		 * of the provided {@code Option.Type}
		 */
		boolean hasOptionOf(final OptionType optionType) {
			
			for (Option option : this.optionMap.keySet()) {
				
				if (option.getOptionType().equals(optionType)) {
					
					return true;
				}
			}
			
			return false;
		}
		
		/**
		 * Returns the boolean value that indicates if further parsing of 
		 * command line options and their arguments should be terminated.
		 * 
		 * @return the boolean value that indicates if further parsing of 
		 * command line options and their arguments should be terminated
		 */
		boolean isParsingTerminated() {
			return this.parsingTerminated;
		}

		/**
		 * Sets the boolean value that indicates if further parsing of 
		 * command line options and their arguments should be terminated.
		 * 
		 * @param terminate the provided boolean value that indicates if 
		 * further parsing of command line options and their arguments should 
		 * be terminated
		 */
		void setParsingTerminated(final boolean terminate) {
			this.parsingTerminated = terminate;
		}

		/**
		 * Sets the selected and terminative {@code OptionGroup}.
		 * 
		 * @param selectedTerminativeOptGrp the provided selected and 
		 * terminative {@code OptionGroup}
		 */
		void setSelectedTerminativeOptionGroup(
				final OptionGroup<Object> selectedTerminativeOptGrp) {
			
			if (selectedTerminativeOptGrp == null) {
				throw new NullPointerException(
						"OptionGroup must not be null");
			}
			
			if (!selectedTerminativeOptGrp.hasOptionSelection()) {
				throw new IllegalArgumentException(
						"OptionGroup must have at least one OptionSelection");
			}
			
			if (!selectedTerminativeOptGrp.isOptionSelectionsTerminative()) {
				throw new IllegalArgumentException(
						"OptionGroup must have terminative OptionSelections");
			}
			
			if (!this.optionMap.values().contains(selectedTerminativeOptGrp)) {
				throw new IllegalArgumentException(
						"OptionGroup must exist in optionMap");				
			}
			
			this.selectedTerminativeOptionGroup = selectedTerminativeOptGrp;
		}
		
	}
	
	/**
	 * A command line option handler. The constants of this enumerated type 
	 * provides a classification of command line option handlers.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static enum OptionHandler {
		
		/** A command line option handler that handles a POSIX option. */
		POSIX_OPTION_HANDLER(OptionType.POSIX) {
			
			@Override
			void handleOption(
					final String current, 
					final Iterator<String> remaining, 
					final Context context) 
					throws OptionHandlerException {
				
				OptionType optType = this.getOptionType();
				
				Map<Option, OptionGroup<Object>> optionMap =  
					context.getOptions();
				
				for (int i = 1; i < current.length(); i++) {

					String name = Character.toString(current.charAt(i));
					
					Option option;
					
					try {
						
						option = AbstractFactory.getDefault().newOption(
								optType, name);
						
					} catch (IllegalArgumentException e) {

						String message = String.format(
								"invalid option -- '%s'",
								name);

						throw new OptionHandlerException(message);
					}
					
					if (!optionMap.containsKey(option)) {

						String message = String.format(
								"unrecognized option '%s'",
								option.toOptionString());

						throw new OptionHandlerException(message);
					}
					
					OptionGroup<Object> optionGroup = optionMap.get(option);
					
					String argument = null;

					String arg = null;
					
					ArgHandler argHandler = null;
					
					if (optionGroup.canAcceptArgument()) {
						
						if (i < current.length() - 1) {
							arg = current.substring(i + 1);
						}

						if (arg != null) {

							argument = arg;

							i = current.length();
						}
						
						if (arg == null && remaining.hasNext()) {
							
							arg = remaining.next();
							
							remaining.remove();
							
							argHandler = ArgHandler.bestHandles(arg, context);
							
							switch(argHandler) {
							
							case OPTION_HANDLER:
								break;
								
							case TERMINATION_ARG_HANDLER:
								
								try {
									
									argHandler.handleArgument(
											arg, remaining, context);
									
								} catch (ArgHandlerException e) {
									
									throw new OptionHandlerException(
											e.getMessage(), e);
								}
								
								if (remaining.hasNext()) {
									
									argument = remaining.next();
									
									remaining.remove();
								}
								
								break;

							case UNSPECIFIED_ARG_HANDLER:
								argument = arg;
								break;
								
							default:
								throw new AssertionError(String.format(
										"Unknown ArgHandler %s", argHandler));
							}
						}
					}
					
					OptionSelection<String> optionSelection = 
							AbstractFactory.getDefault().newOptionSelection(
									option, argument);
					
					optionGroup.addOptionSelection(optionSelection);
					
					if (context.getSelectedTerminativeOptionGroup() == null 
							&& optionGroup.isOptionSelectionsTerminative()) {

						context.setSelectedTerminativeOptionGroup(optionGroup);
					}
					
					if (argHandler != null 
							&& argHandler.equals(ArgHandler.OPTION_HANDLER)) {
						
						try {
							
							argHandler.handleArgument(arg, remaining, context);
							
						} catch (ArgHandlerException e) {
							
							throw new OptionHandlerException(e.getMessage(), e);
						}
					}
				}
			}

			@Override
			boolean handles(final String option, final Context context) {
				
				return !GNU_SHORT_LONG_OPTION_HANDLER.handles(option, context)
					&& context.hasOptionOf(this.getOptionType())
					&& option.matches("^-[^\\-].*$");
			}
		},
		
		/** 
		 * A command line option handler that handles a short GNU long option. 
		 */
		GNU_SHORT_LONG_OPTION_HANDLER(OptionType.GNU_SHORT_LONG) {
			
			@Override
			void handleOption(
					final String current, 
					final Iterator<String> remaining, 
					final Context context) 
					throws OptionHandlerException {
				
				OptionType optType = this.getOptionType();
				
				Map<Option, OptionGroup<Object>> optionMap = 
					context.getOptions();
				
				String name = current.replaceFirst(optType.getDelimiter(), "");
				
				Option option = AbstractFactory.getDefault().newOption(
						optType, name);
				
				if (!optionMap.containsKey(option)) {

					String message = String.format(
							"unrecognized option '%s'",
							option.toOptionString());

					throw new OptionHandlerException(message);
				}
				
				OptionGroup<Object> optionGroup = optionMap.get(option);
				
				String argument = null;

				String arg = null;
				
				ArgHandler argHandler = null;
				
				if (optionGroup.canAcceptArgument() && remaining.hasNext()) {
					
					arg = remaining.next();
					
					remaining.remove();
					
					argHandler = ArgHandler.bestHandles(arg, context);
					
					switch(argHandler) {
					
					case OPTION_HANDLER:
						break;
						
					case TERMINATION_ARG_HANDLER:
						
						try {
							
							argHandler.handleArgument(arg, remaining, context);
							
						} catch (ArgHandlerException e) {
							
							throw new OptionHandlerException(e.getMessage(), e);
						}
						
						if (remaining.hasNext()) {
							
							argument = remaining.next();
							
							remaining.remove();
						}
						
						break;

					case UNSPECIFIED_ARG_HANDLER:
						argument = arg;
						break;
						
					default:
						throw new AssertionError(String.format(
								"Unknown ArgHandler %s", argHandler));
					}					
				}
				
				OptionSelection<String> optionSelection = 
						AbstractFactory.getDefault().newOptionSelection(
								option, argument);
				
				optionGroup.addOptionSelection(optionSelection);
				
				if (context.getSelectedTerminativeOptionGroup() == null 
						&& optionGroup.isOptionSelectionsTerminative()) {

					context.setSelectedTerminativeOptionGroup(optionGroup);
				}
				
				if (argHandler != null 
						&& argHandler.equals(ArgHandler.OPTION_HANDLER)) {
					
					try {
						
						argHandler.handleArgument(arg, remaining, context);
						
					} catch (ArgHandlerException e) {
						
						throw new OptionHandlerException(e.getMessage(), e);
					}
				}				
			}

			@Override
			boolean handles(final String option, final Context context) {
				
				OptionType optType = this.getOptionType(); 
				
				if (context.hasOptionOf(optType)) {
					
					for (Option opt : context.getOptions().keySet()) {
						
						if (opt.getOptionType().equals(optType) 
								&& option.equals(opt.toOptionString())) {
							
							return true;
						}
					}
				}
				
				return false;
			}
		},
		
		/** A command line option handler that handles a GNU long option. */
		GNU_LONG_OPTION_HANDLER(OptionType.GNU_LONG) {
			
			@Override
			void handleOption(
					final String current, 
					final Iterator<String> remaining, 
					final Context context) 
					throws OptionHandlerException {
				
				OptionType optType = this.getOptionType();
				
				Map<Option, OptionGroup<Object>> optionMap = 
					context.getOptions();
				
				String[] splitCurrent = current.split(
						optType.getAssignmentOperator(), 2);
				
				String name = splitCurrent[0].replaceFirst(
						optType.getDelimiter(), "");
				
				Option option = AbstractFactory.getDefault().newOption(
						optType, name);
				
				if (!optionMap.containsKey(option)) {

					String message = String.format(
							"unrecognized option '%s'",
							option.toOptionString());

					throw new OptionHandlerException(message);
				}
				
				OptionGroup<Object> optionGroup = optionMap.get(option);
				
				String argument = null;

				String arg = null;
				
				ArgHandler argHandler = null;
				
				if (splitCurrent.length == 2) { 
					argument = splitCurrent[1];	
				}
				
				if (argument != null && !optionGroup.canAcceptArgument()) {
					
					String message = String.format(
							"option '%s' does not allow an argument",
							option.toOptionString());
					
					throw new OptionHandlerException(message);					
				}
				
				if (argument == null 
						&& optionGroup.canAcceptArgument() 
						&& remaining.hasNext()) {
					
					arg = remaining.next(); 
					
					remaining.remove();
					
					argHandler = ArgHandler.bestHandles(arg, context);
					
					switch(argHandler) {
					
					case OPTION_HANDLER:
						break;
						
					case TERMINATION_ARG_HANDLER:
						
						try {
							
							argHandler.handleArgument(arg, remaining, context);
							
						} catch (ArgHandlerException e) {
							
							throw new OptionHandlerException(e.getMessage(), e);
						}
						
						if (remaining.hasNext()) {
							
							argument = remaining.next();
							
							remaining.remove();
						}
						
						break;

					case UNSPECIFIED_ARG_HANDLER:
						argument = arg;
						break;
						
					default:
						throw new AssertionError(String.format(
								"Unknown ArgHandler %s", argHandler));
					}					
				}
				
				OptionSelection<String> optionSelection = 
						AbstractFactory.getDefault().newOptionSelection(
								option, argument);
				
				optionGroup.addOptionSelection(optionSelection);
				
				if (context.getSelectedTerminativeOptionGroup() == null 
						&& optionGroup.isOptionSelectionsTerminative()) {

					context.setSelectedTerminativeOptionGroup(optionGroup);
				}
				
				if (argHandler != null 
						&& argHandler.equals(ArgHandler.OPTION_HANDLER)) {
					
					try {
						
						argHandler.handleArgument(arg, remaining, context);
						
					} catch (ArgHandlerException e) {
						
						throw new OptionHandlerException(e.getMessage(), e);
					}
				}				
			}

			@Override
			boolean handles(final String option, final Context context) {
				
				return context.hasOptionOf(this.getOptionType())
					&& option.matches("^--.+$");
			}
		};
		
		/**
		 * Returns the enum constant that can handle the provided command line 
		 * option and the provided {@code Context}.
		 * 
		 * @param option the provided command line option
		 * 
		 * @param context the provided {@code Context}
		 *  
		 * @return an enum constant that can handle the provided command line 
		 * option and the provided {@code Context} or {@code null} if there is 
		 * no enum constant that can handle the provided command line option 
		 * and the provided {@code Context}
		 */
		static OptionHandler bestHandles(
				final String option, final Context context) {
			
			for (OptionHandler optionHandler : OptionHandler.values()) {
				
				if (optionHandler.handles(option, context)) {
					
					return optionHandler;
				}
			}
			
			return null;
		}
		
		/** The {@code OptionType}. */
		private final OptionType optionType;
		
		/**
		 * Constructs an {@code OptionHandler} with the provided 
		 * {@code OptionType}.
		 *  
		 * @param optType the provided {@code OptionType}
		 */
		private OptionHandler(final OptionType optType) {
			this.optionType = optType;
		}
		
		/**
		 * Returns the {@code OptionType}.
		 * 
		 * @return the {@code OptionType}
		 */
		OptionType getOptionType() {
			return this.optionType;
		}
		
		/**
		 * Handles the provided command line option and the provided remaining
		 * command line arguments based on this {@code OptionHandler} and the 
		 * provided {@code Context}.
		 * <p>
		 * When obtaining the next remaining command line argument by invoking
		 * the {@code Iterator#next()} method, the {@code Iterator#remove()}
		 * method must be invoked afterwards. If the remaining command line 
		 * argument is not handled directly by this method, the remaining 
		 * command line argument's {@code ArgHandler} must be determined, and 
		 * the remaining command line argument and the other remaining command 
		 * line arguments must be handled by the determined {@code ArgHandler}.
		 * </p>
		 *     
		 * @param current the provided command line option
		 * 
		 * @param remaining the provided remaining command line arguments
		 * 
		 * @param context the provided {@code Context}
		 *  
		 * @throws OptionHandlerException if the provided command line option 
		 * cannot be handled
		 */
		abstract void handleOption(
				String current, Iterator<String> remaining, Context context) 
				throws OptionHandlerException;
		
		/**
		 * Indicates if the provided command line option based on the provided
		 * {@code Context} can be handled by this {@code OptionHandler}.
		 * 
		 * @param option the provided command line option
		 * 
		 * @param context the provided {@code Context}
		 * 
		 * @return {@code true} if the provided command line option based on 
		 * the provided {@code Context} can be handled by this 
		 * {@code OptionHandler}
		 */		
		abstract boolean handles(String option, Context context);
	}
	
	/**
	 * Thrown when the provided command line option cannot be handled.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static final class OptionHandlerException extends Exception {

		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;

		/**
		 * Constructs an {@code OptionHandlerException} with the specified 
		 * detail message.
		 * 
		 * @param message the detail message	
		 */
		OptionHandlerException(final String message) {
			super(message);
		}
		
		/**
		 * Constructs an {@code OptionHandlerException} with the specified 
		 * detail message and the {@code Throwable} that was raised.
		 * 
		 * @param message the detail message
		 * 
		 * @param cause the {@code Throwable} that was raised
		 */
		OptionHandlerException(
				final String message, final Throwable cause) {
			
			super(message, cause);
		}
	}
	
	/**
	 * The {@code List} of {@code OptionGroup}s.
	 */
	private final List<OptionGroup<Object>> optionGroupList;
	
	/**
	 * The {@code List} of {@code Option}s.
	 */
	private final List<Option> optionList;
	
	/**
	 * The {@code List} of command line arguments that are not command line 
	 * options nor their arguments.
	 */
	private final List<String> remainingArgumentList;
	
	/**
	 * Constructs an {@code OptionsImpl}.
	 */
	OptionsImpl() {
		
		this.optionGroupList = new ArrayList<OptionGroup<Object>>();
		
		this.optionList = new ArrayList<Option>();
		
		this.remainingArgumentList = new ArrayList<String>();
	}
	
	/**
	 * Constructs an {@code OptionsImpl} with the provided {@code Options}.
	 * 
	 * @param options the provided {@code Options}
	 */
	OptionsImpl(final Options options) {
		
		this();
		
		this.optionGroupList.addAll(options.getOptionGroups());
		
		this.optionList.addAll(options.getOptions());
		
		this.remainingArgumentList.addAll(Arrays.asList(
				options.getRemainingArguments()));
	}
	
	@Override
	public void addOptionGroup(
			final OptionGroup<? extends Object> optionGroup) {
		
		if (optionGroup == null) {
			throw new NullPointerException("optionGroup must not be null");
		}
		
		@SuppressWarnings("unchecked")
		OptionGroup<Object> optGroup = (OptionGroup<Object>) optionGroup;
		
		this.optionGroupList.add(optGroup);
		
		for (Option option : optGroup.getOptions()) {
			
			this.optionList.add(option);
		}
	}

	@Override
	public List<OptionGroup<Object>> getOptionGroups() {
		return Collections.unmodifiableList(this.optionGroupList);
	}

	@Override
	public List<Option> getOptions() {
		return Collections.unmodifiableList(this.optionList);
	}

	@Override
	public String[] getRemainingArguments() {
		return this.remainingArgumentList.toArray(
				new String[this.remainingArgumentList.size()]);
	}

	@Override
	public void parse(final String[] args) throws OptionsException {
		
		List<OptionGroup<Object>> optionGroups = this.optionGroupList;

		Map<Option, OptionGroup<Object>> optionMap = 
				new TreeMap<Option, OptionGroup<Object>>(
						AbstractFactory.getDefault().getOptionComparator()); 
		
		for (OptionGroup<Object> optionGroup : optionGroups) {
		
			optionGroup.clearOptionSelections();
			optionGroup.clearParsedOptionSelections();
			
			for (Option option : optionGroup.getOptions()) {
				
				optionMap.put(option, optionGroup);
			}
		}
		
		Context context = new Context(optionMap);
		
		List<String> argumentList = new ArrayList<String>(Arrays.asList(args));

		for (Iterator<String> i = argumentList.iterator(); i.hasNext();) {
			
			String argument = i.next();
			
			ArgHandler argHandler = ArgHandler.bestHandles(argument, context);
			
			if (!argHandler.equals(ArgHandler.UNSPECIFIED_ARG_HANDLER)) {
				
				i.remove(); 
			}
			
			try {
				
				argHandler.handleArgument(argument, i, context);
				
			} catch (ArgHandlerException e) {
				
				throw new OptionsException(e.getMessage(), e);
			}			
		}
		
		OptionGroup<Object> selectedTerminativeOptionGroup =
			context.getSelectedTerminativeOptionGroup();
		
		if (selectedTerminativeOptionGroup != null) {
			
			try {
				
				selectedTerminativeOptionGroup.parse();
				
			} catch (OptionGroupException e) {
				
				throw new OptionsException(e.getMessage(), e);
			}
			
			this.remainingArgumentList.clear();
			
			return;
		}
		
		for (OptionGroup<Object> optionGroup : optionGroups) {
			
			try {
				
				optionGroup.parse();
				
			} catch (OptionGroupException e) {
				
				throw new OptionsException(e.getMessage(), e);
			}
		}
		
		this.remainingArgumentList.clear();
		this.remainingArgumentList.addAll(argumentList);		
	}

	@Override
	public String toString() {
		return String.format("Options [" 
				+ "optionGroups=%s, " 
				+ "options=%s, " 
				+ "remainingArguments=%s]",
				this.optionGroupList, 
				this.optionList, 
				this.remainingArgumentList);
	}
}
