/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * An implementation of a {@code PackageDoc}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class PackageDocImpl implements PackageDoc {
	
	/** The unmodifiable {@code List} of {@code Package}s. */
	private static List<Package> packageList;
	
	/**
	 * The unmodifiable {@code Map} that maps {@code Package}s to their 
	 * associated {@code PackageDocImpl}s.
	 */
	private static Map<Package, PackageDocImpl> packageMap;
	
	/** 
	 * The sorted unmodifiable {@code List} of all of the 
	 * {@code PackageDocImpl}s. 
	 */
	private static List<PackageDocImpl> packageDocImplList;
	
	/**
	 * The unmodifiable {@code Map} for the documentation properties of all of 
	 * the {@code PackageDocImpl}s.
	 */
	private static Map<String, String> packageDocImplsProperties;
	
	/**
	 * Returns a sorted unmodifiable {@code List} of all of the 
	 * {@code PackageDocImpl}s based on the provided {@code List} of 
	 * {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been provided to 
	 * {@link org.climax.PackageDocImpl#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 *  
	 * @return a sorted unmodifiable {@code List} of all of the 
	 * {@code PackageDocImpl}s based on the provided {@code List} of 
	 * {@code Package}s
	 */
	static List<PackageDocImpl> getPackageDocImpls() {
		if (PackageDocImpl.packageList == null) {
			PackageDocImpl.setPackages(Arrays.asList(Package.getPackages()));
		}
		return PackageDocImpl.packageDocImplList;
	}
	
	/**
	 * Returns an unmodifiable {@code Map} for the documentation properties of 
	 * all of the {@code PackageDocImpl}s based on the provided {@code List} of
	 * {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been provided to 
	 * {@link org.climax.PackageDocImpl#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 * 
	 * @return an unmodifiable {@code Map} for the documentation properties of 
	 * all of the {@code PackageDocImpl}s based on the provided {@code List} of
	 * {@code Package}s
	 */
	static Map<String, String> getPackageDocImplsProperties() {
		if (PackageDocImpl.packageList == null) {
			PackageDocImpl.setPackages(Arrays.asList(Package.getPackages()));
		}
		return PackageDocImpl.packageDocImplsProperties;
	}
	
	/**
	 * Returns an unmodifiable {@code List} of {@code Package}s based on the 
	 * provided {@code List} of {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been provided to 
	 * {@link org.climax.PackageDocImpl#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 * 
	 * @return an unmodifiable {@code List} of {@code Package}s based on the 
	 * provided {@code List} of {@code Package}s
	 */
	static List<Package> getPackages() {
		if (PackageDocImpl.packageList == null) {
			PackageDocImpl.setPackages(Arrays.asList(Package.getPackages()));
		}
		return PackageDocImpl.packageList;
	}
	
	/**
	 * Returns a {@code PackageDocImpl} based on the provided {@code Package} 
	 * that exists from the provided {@code List} of {@code Package}s.
	 * <p>
	 * If no {@code List} of {@code Package}s has been provided to 
	 * {@link org.climax.PackageDocImpl#setPackages(List)}, a {@code List} of 
	 * {@code Package}s will be created and provided.
	 * </p>
	 *  
	 * @param pkg the provided {@code Package} that exists from the 
	 * {@code List} of {@code Package}s
	 * 
	 * @return a {@code PackageDocImpl} based on the provided {@code Package} 
	 * that exists from the provided {@code List} of {@code Package}s or 
	 * {@code null} if the provided {@code Package} does not exist from the 
	 * {@code List} of {@code Package}s 
	 */
	static PackageDocImpl of(final Package pkg) {
		if (PackageDocImpl.packageList == null) {
			PackageDocImpl.setPackages(Arrays.asList(Package.getPackages()));
		}
		return PackageDocImpl.packageMap.get(pkg);
	}

	/**
	 * Sets the {@code Package}s with the provided array of {@code Package}s.
	 * 
	 * @param pkgList the provided array of {@code Package}s
	 */
	static void setPackages(final List<Package> pkgList) {
		
		Map<Package, PackageDocImpl> pkgMap = 
			new HashMap<Package, PackageDocImpl>();
		
		for (Package pkg : pkgList) {
			
			PackageDocImpl pkgDocImpl = new PackageDocImpl(pkg);
			
			pkgMap.put(pkg, pkgDocImpl);
		}
		
		List<PackageDocImpl> pkgDocImplList = 
				new ArrayList<PackageDocImpl>(pkgMap.values());
		
		Collections.sort(
				pkgDocImplList, 
				AbstractFactory.getDefault().getPackageDocComparator());
		
		Map<String, String> pkgDocImplsProperties = new Properties();

		for (PackageDocImpl pkgDocImpl : pkgDocImplList) {
			
			String pkgDocImplName = pkgDocImpl.getPackage().getName();
			
			Map<String, String> pkgDocImplProperties = 
				pkgDocImpl.getProperties();
			
			for (Map.Entry<String, String> pkgDocImplProperty 
					: pkgDocImplProperties.entrySet()) {
				
				String pkgDocImplPropertyKey = 
					pkgDocImplName 
					+ "."
					+ pkgDocImplProperty.getKey();
					
				pkgDocImplsProperties.put(
						pkgDocImplPropertyKey, 
						pkgDocImplProperty.getValue());
			}
		}		
		
		PackageDocImpl.packageList = Collections.unmodifiableList(pkgList);
		PackageDocImpl.packageMap = Collections.unmodifiableMap(pkgMap);
		PackageDocImpl.packageDocImplList = 
				Collections.unmodifiableList(pkgDocImplList);
		PackageDocImpl.packageDocImplsProperties = 
				Collections.unmodifiableMap(pkgDocImplsProperties);
	}
	
	/** 
	 * The {@code Map} for the documentation properties of this 
	 * {@code PackageDocImpl}. 
	 */
	private final Map<String, String> properties;
	
	/** The {@code Package}. */
	private final Package pkg;
	
	/**
	 * Constructs a {@code PackageDocImpl} with the provided {@code Package}.
	 * 
	 * @param p the provided {@code Package}
	 */
	private PackageDocImpl(final Package p) {
		
		this.properties = new Properties();
		
		this.pkg = p;
		
		String pkgImplTitle = this.pkg.getImplementationTitle();
		
		if (pkgImplTitle != null) {
			this.properties.put(
					PackageDocPropertyKeys.IMPL_TITLE, pkgImplTitle);
		}
		
		String pkgImplVendor = this.pkg.getImplementationVendor();
		
		if (pkgImplVendor != null) {
			this.properties.put(
					PackageDocPropertyKeys.IMPL_VENDOR, pkgImplVendor);
		}
		
		String pkgImplVersion = this.pkg.getImplementationVersion();
		
		if (pkgImplVersion != null) {
			this.properties.put(
					PackageDocPropertyKeys.IMPL_VERSION, pkgImplVersion);
		}
		
		String pkgSpecTitle = this.pkg.getSpecificationTitle();
		
		if (pkgSpecTitle != null) {
			this.properties.put(
					PackageDocPropertyKeys.SPEC_TITLE, pkgSpecTitle);
		}
		
		String pkgSpecVendor = this.pkg.getSpecificationVendor();
		
		if (pkgSpecVendor != null) {
			this.properties.put(
					PackageDocPropertyKeys.SPEC_VENDOR, pkgSpecVendor);
		}
		
		String pkgSpecVersion = this.pkg.getSpecificationVersion();
		
		if (pkgSpecVersion != null) {
			this.properties.put(
					PackageDocPropertyKeys.SPEC_VERSION, pkgSpecVersion);
		}		
	}
	
	@Override
	public Package getPackage() {
		return this.pkg;
	}
	
	@Override
	public Map<String, String> getProperties() {
		return this.properties;
	}

	@Override
	public String toString() {
		return String.format("PackageDoc [" 
				+ "properties=%s, " 
				+ "package=%s]",
				this.properties, 
				this.pkg);
	}
}
