/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.AbstractMap;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * A {@code Map} implementation that represents a persistent set of properties.
 *  
 * @author Climax Development (climax.development@gmail.com)
 */
final class Properties implements Map<String, String> {
	
	/**
	 * A {@code Map.Entry} implementation that represents a property.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	static final class Property implements Map.Entry<String, String> {

		/** 
		 * The {@code Map.Entry} that contains the property's key and its 
		 * associated value.
		 */
		private final Map.Entry<String, String> entry;
		
		/**
		 * Constructs a {@code Properties.Property} with the provided 
		 * {@code Map.Entry}.
		 * 
		 * @param e the provided {@code Map.Entry}
		 * 
		 * @throws NullPointerException if the provided {@code Map.Entry}, its 
		 * key, or its value is {@code null}
		 */
		Property(final Map.Entry<? extends String, ? extends String> e) {
			this(e.getKey(), e.getValue());
		}
		
		/**
		 * Constructs a {@code Properties.Property} with the provided key.
		 * 
		 * @param key the provided key
		 * @throws NullPointerException if the provided key is {@code null}
		 */
		Property(final String key) {
			this(key, "");
		}
		
		/**
		 * Constructs a {@code Properties.Property} with the provided key and 
		 * value.
		 * 
		 * @param key the provided key
		 * 
		 * @param value the provided value
		 * 
		 * @throws NullPointerException if the provided key or value is 
		 * {@code null}
		 */
		Property(final String key, final String value) {
			
			if (key == null) {
				throw new NullPointerException("key must not be null");
			}
			
			if (value == null) {
				throw new NullPointerException("value must not be null");
			}
			
			this.entry = 
				new AbstractMap.SimpleEntry<String, String>(key, value);
		}
		
		@Override
		public boolean equals(final Object o) {
			return this.entry.equals(o);
		}
		
		@Override
		public String getKey() {
			return this.entry.getKey();
		}

		@Override
		public String getValue() {
			return this.entry.getValue();
		}
		
		@Override
		public int hashCode() {
			return this.entry.hashCode();
		}

		/**
		 * Replaces the value corresponding to this {@code Properties.Property} 
		 * with the specified value.
		 * 
		 * @param value new value to be stored in this 
		 * {@code Properties.Property}
		 * 
		 * @return the old value corresponding to the 
		 * {@code Properties.Property}
		 * 
		 * @throws NullPointerException if value is {@code null}
		 */
		@Override
		public String setValue(final String value) {
			
			if (value == null) {
				throw new NullPointerException("value must not be null");
			}
			
			return this.entry.setValue(value);
		}
		
		/**
		 * Returns an expandable {@code String} representation of this 
		 * {@code Properties.Property}. This implementation returns the string 
		 * representation of a dollar sign ($) followed by a left French brace 
		 * ({) followed by this {@code Properties.Property}'s key followed by a 
		 * right French brace (}).
		 * 
		 * @return a {@code String} representation of this map entry
		 */
		public String toExpandableString() {
			return String.format("${%s}", this.entry.getKey());
		}
		
		@Override
		public String toString() {
			return this.entry.toString();
		}
	}
	
	/**
	 * The {@code Map} that contains the property keys and their associated
	 * values.
	 */
	private final Map<String, String> map;
	
	/**
	 * Constructs an instance of {@code Properties}.
	 */
	Properties() {
		
		this.map = new HashMap<String, String>();
	}
	
	/**
	 * Constructs an instance of {@code Properties} with the provided 
	 * {@code Map}.
	 * 
	 * @param m the provided {@code Map}
	 * 
	 * @throws NullPointerException if the provided {@code Map} or a key or 
	 * value of a {@code Map.Entry} of the provided {@code Map} is {@code null}
	 */
	Properties(final Map<? extends String, ? extends String> m) {
		
		this();
		
		this.putAll(m);
	}
	
	@Override
	public void clear() {
		this.map.clear();
	}

	@Override
	public boolean containsKey(final Object key) {
		return this.map.containsKey(key);
	}

	@Override
	public boolean containsValue(final Object value) {
		return this.map.containsValue(value);
	}

	@Override
	public Set<Map.Entry<String, String>> entrySet() {
		return this.map.entrySet();
	}

	@Override
	public boolean equals(final Object o) {
		return this.map.equals(o);
	}
	
	@Override
	public String get(final Object key) {
		return this.map.get(key);
	}

	@Override
	public int hashCode() {
		return this.map.hashCode();
	}
	
	@Override
	public boolean isEmpty() {
		return this.map.isEmpty();
	}

	@Override
	public Set<String> keySet() {
		return this.map.keySet();
	}
	
	/**
	 * Associates the provided value with the provided key in this instance 
	 * of {@code Properties}. If the instance of {@code Properties} previously 
	 * contained a mapping for the key, the old value is replaced by the 
	 * provided value. 
	 * 
	 * @param key the provided key
	 * 
	 * @param value the provided value
	 * 
	 * @return the previous value associated with the provided key or 
	 * {@code null} if there was no mapping for the provided key
	 * 
	 * @throws NullPointerException if the provided key or value is 
	 * {@code null}
	 */
	@Override
	public String put(final String key, final String value) {
		
		if (key == null) {
			throw new NullPointerException("key must not be null");
		}
		
		if (value == null) {
			throw new NullPointerException("value must not be null");
		}
		
		return this.map.put(key, value);
	}

	/**
	 * Copies all of the mappings from the provided {@code Map} to this 
	 * instance of {@code Properties}. These mappings will replace any mappings 
	 * that this instance of {@code Properties} had for any of the keys 
	 * currently in the provided {@code Map}.
	 * 
	 * @param m the provided {@code Map}
	 * 
	 * @throws NullPointerException if the provided {@code Map} or a key or 
	 * value of a {@code Map.Entry} of the provided {@code Map} is {@code null}
	 */
	@Override
	public void putAll(final Map<? extends String, ? extends String> m) {
		
		if (m == null) {
			throw new NullPointerException("map must not be null");
		}
		
		for (Map.Entry<? extends String, ? extends String> entry 
				: m.entrySet()) {
			
			this.put(entry.getKey(), entry.getValue());
		}
	}

	@Override
	public String remove(final Object key) {
		return this.map.remove(key);
	}

	@Override
	public int size() {
		return this.map.size();
	}

	@Override
	public String toString() {
		return this.map.toString();
	}
	
	@Override
	public Collection<String> values() {
		return this.map.values();
	}
}
