/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.Map;

/**
 * This class expands within a provided string expandable properties whose keys 
 * and associated values are defined by a provided {@code Map}. 
 *    
 * @author Climax Development (climax.development@gmail.com)
 */
final class StringExpander {
	
	/**
	 * Expands within the provided string the expandable properties whose keys 
	 * and associated values are defined by the provided {@code Map}.
	 * <p>
	 * If the key of an expandable property construct does not exist in the 
	 * provided {@code Map}, the expandable property construct is not expanded 
	 * (remains the same).
	 * </p>
	 * <p>
	 * If the value of an expandable property contains any expandable property
	 * constructs, those constructs will not be expanded (remain the same).
	 * </p>
	 *  
	 * @param string the provided string
	 * 
	 * @param map the provided {@code Map} that defines the properties' keys 
	 * and their associated values
	 * 
	 * @return the expanded string
	 *  
	 * @throws NullPointerException if the provided string is {@code null} or 
	 * the provided {@code Map} is {@code null}
	 * 
	 * @throws StringExpansionException if an error in expanding a 
	 * {@code String} is encountered
	 */	
	static String expand(final String string, final Map<String, String> map) {
		
		if (string == null) {
			throw new NullPointerException("string must not be null");
		}
		
		if (map == null) {
			throw new NullPointerException("map must not be null");
		}

		Map<String, String> m = new Properties(map);
		
		StringBuilder builder = new StringBuilder(string);
		
		if (builder.length() > 0) {
			StringExpander.expand(builder, 0, m);
		}
		
		return builder.toString();
	}

	/**
	 * Expands within the provided {@code StringBuilder} starting at the 
	 * provided index the expandable properties whose keys and associated 
	 * values are defined by the provided {@code Map}.
	 * 
	 * @param builder the provided {@code StringBuilder}
	 * 
	 * @param start the provided starting index (inclusive)
	 * 
	 * @param map the provided {@code Map}
	 * 
	 * @return the ending index (exclusive) of the first expanded property or
	 * the ending index (exclusive) of the provided {@code StringBuilder}
	 * 
	 * @throws StringExpansionException if an error in expanding a 
	 * {@code String} is encountered
	 */
	private static int expand(
			final StringBuilder builder, 
			final int start,
			final Map<String, String> map) {
		
		assert (builder != null);
		
		assert (start >= 0 && start < builder.length());
		
		assert (map != null);
		
		boolean isStartOfPropertyKey = 
			start >= 2 && builder.substring(start - 2, start).equals("${");
			
		String expandableProperty = "";
		
		if (isStartOfPropertyKey) {
			
			expandableProperty = builder.substring(start - 2);
		}
		
		int i = start;
		
		do {
			
			char ch = builder.charAt(i);
			
			if (ch == '$' && i < builder.length() - 1) {
				
				char nextChar = builder.charAt(i + 1);
				
				switch(nextChar) {
				
				case '$':
					builder.replace(i, i + 2, "$");
					break;
					
				case '{':
					i = StringExpander.expand(builder, i + 2, map) - 1;
					break;
					
				case '}':
					builder.replace(i, i + 2, "}");
					break;
					
				default:
					break;
				}
			}
			
			if (ch == '}' && isStartOfPropertyKey) {
				
				String propertyKey = builder.substring(start, i);
				
				if (propertyKey.startsWith(
						StringFormatter.FORMAT_PROTOCOL)
						|| propertyKey.startsWith(
								StringFormatter.LFORMAT_PROTOCOL)) {
					
					try {
						
						String replacement = StringFormatter.format(
								propertyKey, map);
						
						builder.replace(start - 2, i + 1, replacement);
						
						return start - 2 + replacement.length();
						
					} catch (IllegalArgumentException e) {
						
						String expandableProp = builder.substring(
								start - 2, i + 1);
						
						throw new StringExpansionException(
								expandableProp, 
								StringExpander.getMessage(
										expandableProp, 
										e.getMessage()), 
								e);
					}
					
				} else if (map.containsKey(propertyKey)) {
					
					String replacement = map.get(propertyKey);
					
					builder.replace(start - 2, i + 1, replacement);
					
					return start - 2 + replacement.length();
				}
				
				return i + 1;
			}
			
			i++;
			
		} while (i < builder.length());
		
		if (isStartOfPropertyKey) {
			
			throw new StringExpansionException(
					expandableProperty,
					"incomplete expandable property");
		}
		
		return i;
	}
	
	/**
	 * Returns the detail message specifying the provided substring that 
	 * causes the error and the additional specified detail message.
	 * 
	 * @param substring the provided substring that causes the error
	 * 
	 * @param message the additional detail message (can be {@code null}).
	 * 
	 * @return the detail message specifying the provided substring that 
	 * causes the error and the additional specified detail message
	 */
	private static String getMessage(
			final String substring, final String message) {
		
		String m = String.format(
				"error in substring: '%s'", 
				substring);
		
		if (message != null) {
			m = String.format(
					"error in substring: '%1$s' : %2$s", 
					substring, 
					message);
		}
		
		return m;
	}
	
	/**
	 * Constructs a {@code StringExpander}. This constructor is suppressed for 
	 * noninstantiability outside of this class.
	 */
	private StringExpander() { 
	}
}
