/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.IllegalFormatException;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * This class returns formatted strings based on the provided format string 
 * that starts with {@code format:} or {@code lformat:} and the provided 
 * {@code Map} that defines the properties' keys and their associated values.
 *    
 * @author Climax Development (climax.development@gmail.com)
 */
final class StringFormatter {

	/**
	 * A {@code String} conversion type. The constants of this enumerated 
	 * type provide a simple classification of the {@code String} conversion 
	 * types.
	 *  
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static enum ConversionType {
		
		/** Conversion to {@code Boolean}. */
		BOOLEAN("boolean") {
			
			@Override
			Object convert(final String value) {
				return Boolean.valueOf(value);
			}
			
		},
		
		/** Conversion to {@code Byte}. */
		BYTE("byte") {

			@Override
			Object convert(final String value) {
				
				Object converted = null;
				
				try {
					
					converted = Byte.valueOf(value);
					
				} catch (NumberFormatException e) {
					
					throw new IllegalArgumentException(e.getMessage(), e);
				}
				
				return converted;
			}

		},
		
		/** Conversion to {@code Character}. */
		CHARACTER("char") {
			
			@Override
			Object convert(final String value) {
				
				if (value.length() != 1) {
					throw new IllegalArgumentException(
							"value must be one character in length");
				}
				
				return Character.valueOf(value.charAt(0));
			}
			
		},
		
		/** Conversion to {@code Date}. */
		DATE("date") {
			
			@Override
			Object convert(final String value) {
				
				if (value.equals("now")) {
					
					return new Date();
				}
				
				Object converted = null;
				
				Throwable cause = null;
				
				DateFormat dateFormat1 =
					new SimpleDateFormat("yyyyMMdd", Locale.US);
				
				DateFormat dateFormat2 =
					new SimpleDateFormat("yyyyMMddHHmmss", Locale.US);
				
				DateFormat dateFormat3 =
					new SimpleDateFormat("yyyyMMddHHmmssSSSZ", Locale.US);
				
				try {
					
					converted = dateFormat1.parse(value);
					
				} catch (ParseException e) {
					
					cause = e; 
				}
				
				if (converted != null) { return converted; }
				
				try {
					
					converted = dateFormat2.parse(value);
					
				} catch (ParseException e) {
					
					cause = e; 
				}
				
				if (converted != null) { return converted; }
				
				try {
					
					converted = dateFormat3.parse(value);
					
				} catch (ParseException e) {
					
					cause = e;	
				}
				
				if (converted != null) { return converted; }
				
				if (converted == null && cause != null) {
					
					throw new IllegalArgumentException(
							cause.getMessage(), cause);
				}
				
				return converted;
			}
			
		},
		
		/** Conversion to {@code Double}. */
		DOUBLE("double") {
			
			@Override
			Object convert(final String value) {
				
				Object converted = null;
				
				try {
					
					converted = Double.valueOf(value);
					
				} catch (NumberFormatException e) {
					
					throw new IllegalArgumentException(e.getMessage(), e);
				}
				
				return converted;
			}
			
		},
		
		/** Conversion to {@code Float}. */
		FLOAT("float") {
			
			@Override
			Object convert(final String value) {
				
				Object converted = null;
				
				try {
					
					converted = Float.valueOf(value);
					
				} catch (NumberFormatException e) {
					
					throw new IllegalArgumentException(e.getMessage(), e);
				}
				
				return converted;
			}
			
		},
		
		/** Conversion to {@code Integer}. */
		INTEGER("int") {
			
			@Override
			Object convert(final String value) {
				
				Object converted = null;
				
				try {
					
					converted = Integer.valueOf(value);
					
				} catch (NumberFormatException e) {
					
					throw new IllegalArgumentException(e.getMessage(), e);
				}
				
				return converted;
			}
			
		},
		
		/** Conversion to {@code Long}. */
		LONG("long") {
			
			@Override
			Object convert(final String value) {
				
				Object converted = null;
				
				try {
					
					converted = Long.valueOf(value);
					
				} catch (NumberFormatException e) {
					
					throw new IllegalArgumentException(e.getMessage(), e);
				}
				
				return converted;
			}
			
		},
		
		/** Conversion to {@code Short}. */
		SHORT("short") {
			
			@Override
			Object convert(final String value) {
				
				Object converted = null;
				
				try {
					
					converted = Short.valueOf(value);
					
				} catch (NumberFormatException e) {
					
					throw new IllegalArgumentException(e.getMessage(), e);
				}
				
				return converted;
			}
			
		},
		
		/** Conversion to {@code String}. */
		STRING("string") {
			
			@Override
			Object convert(final String value) {
				return value;
			}
			
		};

		/**
		 * Returns the enum constant of this type with the provided 
		 * {@code String} representation.
		 *  
		 * @param conversion the provided {@code String} representation 
		 * of the enum constant to be returned
		 * 
		 * @return the enum constant with the provided {@code String} 
		 * representation
		 */
		static ConversionType of(final String conversion) {
			
			for (ConversionType conversionType : ConversionType.values()) {
				
				if (conversionType.toString().equals(conversion)) {
					
					return conversionType;
				}
			}
			
			return null;
		}
		
		/** 
		 * The {@code String} representation of this {@code ConversionType}. 
		 */
		private final String string;
		
		/**
		 * Constructs a {@code Conversion} with the provided {@code String} 
		 * representation.
		 *  
		 * @param s the provided {@code String} representation
		 */
		private ConversionType(final String s) {
			this.string = s;
		}
		
		/**
		 * Converts the provided string to a specified {@code Object} based
		 * on this {@code ConversionType}.
		 * 
		 * @param value the provided string
		 * 
		 * @return a specified {@code Object} based on this 
		 * {@code ConversionType}
		 * 
		 * @throws IllegalArgumentException if the provided string can not be
		 * converted
		 */
		abstract Object convert(String value);
		
		@Override
		public String toString() {
			return this.string;
		}
	}
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code StringMapHolder.STRING_MAP}.
	 */	
	private static final class StringMapHolder {

		/**
		 * A {@code Map} that maps {@code String} representations of 
		 * {@code Locale}s to their {@code Locale}s.
		 */
		static final Map<String, Locale> STRING_MAP;
		
		static {
			
			Map<String, Locale> stringMap = new HashMap<String, Locale>();
			
			for (Locale locale : Locale.getAvailableLocales()) {
				
				stringMap.put(locale.toString(), locale);
			}
			
			STRING_MAP = Collections.unmodifiableMap(stringMap);
		}
		
		/**
		 * Constructs a {@code StringMapHolder}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */
		private StringMapHolder() {
		}
	}
	
	/** The protocol for formatting under the default {@code Locale}. */
	static final String FORMAT_PROTOCOL = "format";
	
	/** The protocol for formatting under a specified {@code Locale}. */
	static final String LFORMAT_PROTOCOL = "lformat";
	
	/**
	 * Converts the provided {@code List} of arguments to a {@code List} of 
	 * converted {@code Object}s.
	 *  
	 * @param map the the provided {@code Map} that defines the properties' 
	 * keys and their associated values
	 * 
	 * @param argList the provided {@code List} of arguments
	 * 
	 * @return a {@code List} of converted {@code Object}s
	 * 
	 * @throws IllegalArgumentException if one of the arguments is invalid 
	 * or one of the arguments can not be converted
	 */
	private static List<Object> convert(
			final Map<String, String> map,
			final List<String> argList) {
		
		assert (map != null);
		
		assert (argList != null);
		
		StringBuilder builder = new StringBuilder();
		builder.append(
				"argument '%s' must have one of the following syntaxes:\n");
		builder.append("\tc=conversion,p=propertykey\n");
		builder.append("\tc=conversion,p=propertykey,v=value\n");
		builder.append("\tc=conversion,v=value\n");
		builder.append("\tp=propertykey\n");
		builder.append("\tp=propertykey,v=value\n");
		builder.append("\tv=value\n");
		
		List<Object> convertedArgList = new ArrayList<Object>();
		
		for (String arg : argList) {
			
			String m = String.format(builder.toString(), arg);
			
			String conversion = null;
			
			String propertyKey = null;
			
			String value = null;
			
			List<String> subArgList = 
				StringFormatter.split('\\', ',', arg);
			
			for (int i = 0; i < subArgList.size(); i++) {
				
				String subArg = subArgList.get(i);
				
				if (subArg.startsWith("c=")) {
					
					if (i == 0) {
					
						conversion = subArg.split("=", 2)[1];
						
					} else {
						
						throw new IllegalArgumentException(m);
					}
					
				} else if (subArg.startsWith("p=")) {

					if ((i == 0 || conversion != null) 
							&& propertyKey == null && value == null) {
						
						propertyKey = subArg.split("=", 2)[1];
						
					} else {
						
						throw new IllegalArgumentException(m);
					}
					
				} else if (subArg.startsWith("v=")) {
					
					if ((i == 0 || conversion != null || propertyKey != null)
							&& value == null) {
						
						value = subArg.split("=", 2)[1];
						
					} else {
						
						throw new IllegalArgumentException(m);
					}
					
				} else {
					
					throw new IllegalArgumentException(m); 
				}
			}
			
			ConversionType conversionType = ConversionType.STRING;
			
			if (conversion != null) {
				
				conversionType = ConversionType.of(conversion);

				if (conversionType == null) {
					
					String message = String.format(
							"unknown conversion in argument '%1$s': %2$s",
							arg,
							conversion);
					
					throw new IllegalArgumentException(message);
				}
			}
			
			if (propertyKey != null) {
				
				if (map.containsKey(propertyKey)) {
				
					value = map.get(propertyKey);
					
				} else if (value == null) {
					
					String message = String.format(
							"unknown property key in argument '%1$s': %2$s",
							arg,
							propertyKey);
					
					throw new IllegalArgumentException(message);
				}
			}
			
			if (value == null) {
				
				String message = String.format(
						"required value or property key in argument '%s'", 
						arg);
				
				throw new IllegalArgumentException(message);
			}
			
			Object converted = null;
			
			try {
				
				converted = conversionType.convert(value);
				
			} catch (IllegalArgumentException e) {
				
				String message = String.format(
						"can not convert from argument '%1$s' value '%2$s' to "
						+ "'%3$s': %4$s",
						arg,
						value,
						conversionType,
						e.getMessage());
				
				throw new IllegalArgumentException(message, e);
			}
			
			convertedArgList.add(converted);			
		}
		
		return convertedArgList;
	}
	
	/**
	 * Returns a formatted string based on the provided format string that 
	 * starts with {@code format:} or {@code lformat:} and the provided 
	 * {@code Map} that defines the properties' keys and their associated 
	 * values.
	 * 
	 * @param string the provided format string that starts with 
	 * {@code format:} or {@code lformat:}
	 * 
	 * @param map the provided {@code Map} that defines the properties' keys 
	 * and their associated values 
	 * 
	 * @return a formatted string
	 * 
	 * @throws NullPointerException if the provided format string is 
	 * {@code null} or the provided {@code Map} is {@code null}
	 * 
	 * @throws IllegalArgumentException if the provided format string is 
	 * invalid
	 */
	static String format(final String string, final Map<String, String> map) {
		
		if (string == null) {
			throw new NullPointerException("string must not be null");
		}
		
		if (map == null) {
			throw new NullPointerException("map must not be null");
		}

		Map<String, String> m = new Properties(map);
		
		String[] strings = string.split(":", 2);
		
		if (strings.length != 2) { 
			
			throw new IllegalArgumentException("illegal syntax");
		}
		
		String protocol = strings[0];
		
		if (!protocol.equals(StringFormatter.FORMAT_PROTOCOL) 
				&& !protocol.equals(
						StringFormatter.LFORMAT_PROTOCOL)) {
			
			String message = String.format(
					"unknown protocol: %s", protocol);
			
			throw new IllegalArgumentException(message);
		}
		
		String s = strings[1];
		
		List<String> argList = StringFormatter.split('\\', ';', s);
		
		Locale locale = Locale.getDefault();
		
		String localeString = locale.toString();
		
		if (protocol.equals(StringFormatter.LFORMAT_PROTOCOL)) {
			
			localeString = argList.remove(0);
			
			locale = StringMapHolder.STRING_MAP.get(localeString);
		}
		
		if (locale == null) {
			
			String message = String.format(
					"unknown locale: %s", localeString);
			
			throw new IllegalArgumentException(message);
		}
		
		if (argList.size() == 0) {
			
			throw new IllegalArgumentException("required format");
		}
		
		String format = argList.remove(0);
		
		List<Object> formatArgList = new ArrayList<Object>();
		
		try {
			
			List<Object> convertedArgList = 
				StringFormatter.convert(m, argList);
			
			formatArgList.addAll(convertedArgList);
			
		} catch (IllegalArgumentException e) {
			
			throw new IllegalArgumentException(e.getMessage(), e);
		}
		
		String formattedString = null;
		
		try {
			
			formattedString = String.format(
					locale, format, formatArgList.toArray());
			
		} catch (IllegalFormatException e) {
			
			throw new IllegalArgumentException(e.getMessage(), e);
		}
		
		return formattedString;
	}
	
	/**
	 * Splits the provided string around characters matching the specified 
	 * delimiter character.
	 * 
	 * @param escape the specified escape character
	 * 
	 * @param delimiter the specified delimiter character
	 * 
	 * @param string the provided string to be split
	 * 
	 * @return a {@code List} of strings computed by splitting the provided
	 * string around characters matching the specified delimiter character
	 */
	private static List<String> split(
			final char escape, final char delimiter, final String string) {
		
		assert (escape != delimiter);
		
		assert (string != null);
		
		List<String> argList = new ArrayList<String>();
		
		StringBuilder builder = new StringBuilder();
		
		for (int i = 0; i < string.length(); i++) {
		
			char ch = string.charAt(i);
			
			if (ch == escape && i < string.length() - 1) {
				
				char nextChar = string.charAt(i + 1);
				
				if (nextChar == escape || nextChar == delimiter) {
					
					builder.append(nextChar);
					i++;
					
				} else {
					
					builder.append(ch);
				}
				
			} else if (ch == delimiter) {
				
				argList.add(builder.toString());
				
				builder.delete(0, builder.length());
				
			} else {
				
				builder.append(ch);
			}
		}
		
		argList.add(builder.toString());
		
		return argList;		
	}
	
	/**
	 * Constructs a {@code StringFormatter}. This constructor is 
	 * suppressed for noninstantiability outside of this class.
	 */
	private StringFormatter() { 
	}
}
