/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class provides utility methods regarding strings.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class Strings {

	/**
	 * Concatenates a provided {@code List} of {@code String}s.
	 * 
	 * @param stringList the provided {@code List} of {@code String}s.
	 * 
	 * @return a {@code String} that represents the concatenation of the
	 * provided {@code List} of {@code String}s
	 * 
	 * @throws NullPointerException if one of the {@code String}s from the
	 * provided {@code List} of {@code String}s is {@code null}
	 */
	private static String concat(final List<String> stringList) {
		
		for (String string : stringList) {
			
			if (string == null) {
				
				throw new NullPointerException("strings must not be null");
			}
		}
		
		StringBuilder builder = new StringBuilder();
		
		for (String string : stringList) {
			
			builder.append(string);
		}
		
		return builder.toString();
	}
	
	/**
	 * Concatenates two provided {@code String}s.
	 * 
	 * @param s1 the first provided {@code String}
	 * 
	 * @param s2 the second provided {@code String}
	 * 
	 * @return a {@code String} that represents the concatenation of the
	 * two provided {@code String}s
	 * 
	 * @throws NullPointerException if one of the {@code String}s is 
	 * {@code null}
	 */
	public static String concat(final String s1, final String s2) {
		return Strings.concat(Arrays.asList(s1, s2));
	}
	
	/**
	 * Concatenates three provided {@code String}s. 
	 * 
	 * @param s1 the first provided {@code String}
	 * 
	 * @param s2 the second provided {@code String}
	 * 
	 * @param s3 the third provided {@code String}
	 * 
	 * @return a {@code String} that represents the concatenation of the
	 * three provided {@code String}s
	 * 
	 * @throws NullPointerException if one of the {@code String}s is 
	 * {@code null}
	 */
	public static String concat(
			final String s1, final String s2, final String s3) {
		
		return Strings.concat(Arrays.asList(s1, s2, s3));
	}
	
	/**
	 * Concatenates three provided {@code String}s and additional 
	 * {@code String}s.
	 * 
	 * @param s1 the first provided {@code String}
	 * 
	 * @param s2 the second provided {@code String}
	 * 
	 * @param s3 the third provided {@code String}
	 * 
	 * @param strings the additional {@code String}s
	 * 
	 * @return a {@code String} that represents the concatenation of the
	 * three provided {@code String}s and additional {@code String}s
	 * 
	 * @throws NullPointerException if one of the {@code String}s is 
	 * {@code null}
	 */
	public static String concat(
			final String s1, 
			final String s2, 
			final String s3, 
			final String... strings) {

		List<String> stringList = new ArrayList<String>();
		
		stringList.addAll(Arrays.asList(s1, s2, s3));
		
		stringList.addAll(Arrays.asList(strings));
		
		return Strings.concat(stringList);
	}
	
	/**
	 * Constructs an instance of {@code Strings}. This constructor is 
	 * suppressed for noninstantiability outside of this class.
	 */
	private Strings() {
	}
}
