/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argacceptor;

import org.climax.ArgAcceptor;
import org.climax.ArgAcceptorContext;

/**
 * An {@code ArgAcceptor} implementation that can accept an argument whose 
 * index is within a range of acceptable indices.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class ArgIndexRangeAcceptor implements ArgAcceptor {
	
	/** The {@code ArgAcceptor}. */
	private final ArgAcceptor argAcceptor;
	
	/**
	 * Constructs an {@code ArgIndexRangeAcceptor} with the provided
	 * minimum index and the provided maximum index.
	 * 
	 * @param minArgIndex the provided minimum index, inclusive
	 * @param maxArgIndex the provided maximum index, inclusive
	 */
	public ArgIndexRangeAcceptor(
			final int minArgIndex, final int maxArgIndex) {
		
		if (minArgIndex < 0) {
			throw new IndexOutOfBoundsException(
					"minArgIndex must be at least 0");
		}
		
		if (maxArgIndex < 0) {
			throw new IndexOutOfBoundsException(
					"maxArgIndex must be at least 0");			
		}
		
		if (minArgIndex > maxArgIndex) {
			throw new IndexOutOfBoundsException(
					"minArgIndex must be less than or equal to maxArgIndex");
		}
		
		ArgAcceptor minArgIndexAcceptor = 
			new MinArgIndexAcceptor(minArgIndex);
		
		ArgAcceptor maxArgIndexAcceptor =
			new MaxArgIndexAcceptor(maxArgIndex);
		
		ArgAcceptor logicalArgAcceptor = 
			new ConditionalArgAcceptor(
					ConditionalArgAcceptor.Operation.AND,
					minArgIndexAcceptor,
					maxArgIndexAcceptor);
			
		this.argAcceptor = logicalArgAcceptor;
	}
	
	@Override
	public boolean canAccept(final ArgAcceptorContext context) {
		return this.argAcceptor.canAccept(context);
	}

}
