/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argacceptor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.climax.ArgAcceptor;
import org.climax.ArgAcceptorContext;


/**
 * An {@code ArgAcceptor} implementation that can accept an argument based on
 * the criteria from the provided {@code ArgAcceptor}s. 
 *  
 * @author Climax Development (climax.development@gmail.com)
 */
public final class ConditionalArgAcceptor implements ArgAcceptor {

	/**
	 * A boolean conditional operation type. The constants of this enumerated 
	 * type provides a classification of boolean conditional operations.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	public static enum Operation {
		
		/** 
		 * Operation that returns true if all operands are true; 
		 * false if otherwise. 
		 */
		AND {

			@Override
			boolean canProceed(final boolean operand) {
				
				boolean proceed = false;
				
				if (operand) { proceed = true; }
				
				return proceed;
			}
			
		},
		
		/** 
		 * Operation that returns true if at least one operand is true; 
		 * false if no operands are true. 
		 */
		OR {

			@Override
			boolean canProceed(final boolean operand) {
				
				boolean proceed = false;
				
				if (!operand) { proceed = true; }
				
				return proceed;
			}
			
		};
		
		/**
		 * Indicates from the provided operand if this {@code Operation} can 
		 * proceed.
		 * 
		 * @param operand the provided operand
		 * 
		 * @return {@code true} or {@code false} depending on the provided 
		 * operand and this {@code Operation}
		 */
		abstract boolean canProceed(boolean operand);
	}
	
	/** The {@code ConditionalArgAcceptor.Operation}. */
	private final Operation operation;
	
	/** The {@code List} of {@code ArgAcceptor}s. */
	private final List<ArgAcceptor> argAcceptorList;
	
	/**
	 * Constructs a {@code ConditionalArgAcceptor} with the provided
	 * {@code ConditionalArgAcceptor.Operation} and two provided 
	 * {@code ArgAcceptor}s. 
	 * 
	 * @param op the provided {@code ConditionalArgAcceptor.Operation}
	 * @param acceptor1 the first provided {@code ArgAcceptor}
	 * @param acceptor2 the second provided {@code ArgAcceptor}
	 */
	public ConditionalArgAcceptor(
			final Operation op,
			final ArgAcceptor acceptor1,
			final ArgAcceptor acceptor2) {
		
		if (op == null) {
			throw new NullPointerException("Operation must not be null");
		}
		
		if (acceptor1 == null || acceptor2 == null) {
			throw new NullPointerException("ArgAcceptors must not be null");
		}
		
		List<ArgAcceptor> acceptorList = new ArrayList<ArgAcceptor>();
		acceptorList.addAll(Arrays.asList(acceptor1, acceptor2));
		
		this.operation = op;
		this.argAcceptorList = new ArrayList<ArgAcceptor>(acceptorList);
	}
	
	/**
	 * Constructs a {@code ConditionalArgAcceptor} with the provided
	 * {@code ConditionalArgAcceptor.Operation}, two provided 
	 * {@code ArgAcceptor}s, and additional {@code ArgAcceptor}s. 
	 * 
	 * @param op the provided {@code ConditionalArgAcceptor.Operation}
	 * @param acceptor1 the first provided {@code ArgAcceptor}
	 * @param acceptor2 the second provided {@code ArgAcceptor}
	 * @param additionalAcceptors the additional {@code ArgAcceptor}s
	 */
	public ConditionalArgAcceptor(
			final Operation op,
			final ArgAcceptor acceptor1,
			final ArgAcceptor acceptor2,
			final ArgAcceptor... additionalAcceptors) {
		
		this(op, acceptor1, acceptor2);
		
		for (ArgAcceptor acceptor : additionalAcceptors) {
			if (acceptor == null) {
				throw new NullPointerException("ArgAcceptors must not be null");
			}
		}
		
		this.argAcceptorList.addAll(Arrays.asList(additionalAcceptors));
	}
	
	@Override
	public boolean canAccept(final ArgAcceptorContext context) {
		
		boolean canAcceptArgument = false;

		for (ArgAcceptor argAcceptor : this.argAcceptorList) {
			
			canAcceptArgument = argAcceptor.canAccept(context);
			
			if (!this.operation.canProceed(canAcceptArgument)) {
				break;
			}
		}
		
		return canAcceptArgument;
	}

}
