/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argacceptor;

import java.util.Arrays;

import org.climax.ArgAcceptor;
import org.climax.ArgAcceptorContext;


/**
 * An {@code ArgAcceptor} implementation that can accept a limited number of
 * arguments throughout multiple invocations.
 * <p> 
 * This {@code ArgAcceptor} keeps a copy of the {@code ArgAcceptorContext} from 
 * the previous invocation. The previous {@code ArgAcceptorContext} and the 
 * current {@code ArgAcceptorContext} are compared to determine if the number 
 * of accepted arguments needs to be reset. If there is a difference in their 
 * comparison, the number of accepted arguments is reset.
 * </p>
 * <p>
 * The definition of their comparison is determined first by their lengths of 
 * their returning {@code String} arrays from their methods 
 * {@link org.climax.ArgAcceptorContext#getArguments()}. If their lengths are 
 * the same, then each of their arguments from their methods 
 * {@link org.climax.ArgAcceptorContext#getArguments()} are compared by index 
 * position. If there is no index position at which their arguments differ, 
 * then each of their availability statuses from their methods 
 * {@link org.climax.ArgAcceptorContext#getArgumentsAvailability()} are 
 * compared by index position. A difference is accepted if--at a specified 
 * index--an availability status from the previous {@code ArgAcceptorContext} 
 * is {@code false} and an availability status of the current 
 * {@code ArgAcceptorContext} is {@code true}.
 * </p>
 *  
 * @author Climax Development (climax.development@gmail.com)
 */
public final class LimitedArgAcceptor implements ArgAcceptor {
	
	/**
	 * An implementation of an {@code ArgAcceptorContext}.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static final class ArgAcceptorContextImpl 
		implements ArgAcceptorContext {

		/** The arguments. */
		private final String[] arguments;
		/** The availability status of each argument. */
		private final boolean[] argumentsAvailability;
		/** The index of the argument in question. */
		private final int index;
		
		/**
		 * Constructs an {@code ArgAcceptorContextImpl} with the provided 
		 * {@code ArgAcceptorContext}.
		 * 
		 * @param context the provided {@code ArgAcceptorContext}
		 */
		ArgAcceptorContextImpl(final ArgAcceptorContext context) {
			this.arguments = Arrays.copyOf(
					context.getArguments(), 
					context.getArguments().length);
			this.argumentsAvailability = Arrays.copyOf(
					context.getArgumentsAvailability(), 
					context.getArgumentsAvailability().length);
			this.index = context.getIndex();
		}
		
		@Override
		public String[] getArguments() {
			return Arrays.copyOf(this.arguments, this.arguments.length);
		}

		@Override
		public boolean[] getArgumentsAvailability() {
			return Arrays.copyOf(
					this.argumentsAvailability, 
					this.argumentsAvailability.length);
		}

		@Override
		public int getIndex() {
			return this.index;
		}
		
	}
	
	/** The current number of accepted arguments. */
	private int acceptedArgumentCount;
	
	/** The maximum number of accepted arguments.*/
	private final int maximumAcceptedArgumentCount;
	
	/** The copy of the previous {@code ArgAcceptorContext}. */
	private ArgAcceptorContext previousContext;
	
	/**
	 * Constructs a {@code LimitedArgAcceptor} with the provided maximum
	 * number of accepted arguments.
	 * 
	 * @param maxAcceptedArgCount the provided maximum number of accepted 
	 * arguments
	 */
	public LimitedArgAcceptor(final int maxAcceptedArgCount) {
		
		if (maxAcceptedArgCount < 0) {
			throw new IllegalArgumentException(
					"maxAcceptedArgCount must be at least 0");
		}
		
		this.acceptedArgumentCount = 0;
		this.maximumAcceptedArgumentCount = maxAcceptedArgCount;
		this.previousContext = null;
	}

	@Override
	public boolean canAccept(final ArgAcceptorContext context) {
		
		ArgAcceptorContext currentContext = new ArgAcceptorContextImpl(context);
		
		if (this.previousContext != null) {
			
			String[] previousArguments = this.previousContext.getArguments();
			
			String[] currentArguments = currentContext.getArguments();
			
			boolean resetAcceptedArgumentCount = false;
			
			// a change in the length of the current arguments
			if (previousArguments.length != currentArguments.length) {
				
				resetAcceptedArgumentCount = true;
				
			} else {
				
				/* 
				 * a change in the argument of the argument in the current 
				 * argument
				 */
				for (int i = 0; i < previousArguments.length; i++) {
					
					String previousArgument = previousArguments[i];
						
					String currentArgument = currentArguments[i];
					
					if (!previousArgument.equals(currentArgument)) {
						
						resetAcceptedArgumentCount = true;
						
						break;
						
					}
				}
				
				if (!resetAcceptedArgumentCount) {
					
					boolean[] prevArgumentsAvailability =
							this.previousContext.getArgumentsAvailability();

					boolean[] currArgumentsAvailability = 
							currentContext.getArgumentsAvailability();
					
					/* 
					 * a retroactive change in the availability status of 
					 * argument in the current arguments (from false to true)
					 */
					for (int i = 0; i < prevArgumentsAvailability.length; i++) {
						
						boolean previouslyAvailable = 
								prevArgumentsAvailability[i];
						
						boolean currentlyAvailable = 
								currArgumentsAvailability[i];
						
						if (!previouslyAvailable && currentlyAvailable) {
							
							resetAcceptedArgumentCount = true;
							
							break;
						}
					}					
				}
			}
			
			if (resetAcceptedArgumentCount) {
				
				this.acceptedArgumentCount = 0;
			}
		}
		
		boolean canAcceptArgument = false;
		
		if (this.acceptedArgumentCount < this.maximumAcceptedArgumentCount) {
			
			canAcceptArgument = true;
			
			this.acceptedArgumentCount++;
		}
		
		this.previousContext = currentContext;
		
		return canAcceptArgument;		
	}

}
