/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.math.BigDecimal;
import java.math.MathContext;
import java.util.Comparator;

/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code BigDecimal}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class BigDecimalArgParser extends NumberArgParser<BigDecimal> 
	implements ParameterlessArgParser<BigDecimal> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final BigDecimalArgParser ARG_PARSER =
			new BigDecimalArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}

	/**
	 * A {@code Comparator} that compares two {@code BigDecimal}s. 
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	private static final class BigDecimalComparator 
		implements Comparator<BigDecimal> {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final BigDecimalComparator COMPARATOR =
				new BigDecimalComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/**
		 * Returns a {@code BigDecimalComparator}.
		 * 
		 * @return a {@code BigDecimalComparator}
		 */		
		public static BigDecimalComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}
		
		/**
		 * Constructs a {@code BigDecimalComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private BigDecimalComparator() {
		}
		
		@Override
		public int compare(
				final BigDecimal bigDecimal1, final BigDecimal bigDecimal2) {

			if (bigDecimal1 == getPseudoNegativeInfinity()
					&& bigDecimal2 == getPseudoNegativeInfinity()) {
				
				return 0;
			}
			
			if (bigDecimal1 == getPseudoPositiveInfinity()
					&& bigDecimal2 == getPseudoPositiveInfinity()) {
				
				return 0;
			}
			
			if (bigDecimal1 == getPseudoNegativeInfinity()) {
				return -1;
			}
			
			if (bigDecimal2 == getPseudoNegativeInfinity()) {
				return 1;
			}
			
			if (bigDecimal1 == getPseudoPositiveInfinity()) {
				return 1;
			}
			
			if (bigDecimal2 == getPseudoPositiveInfinity()) {
				return -1;
			}
			
			return bigDecimal1.compareTo(bigDecimal2);
		}
		
	}
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code PseudoNegativeInfinityHolder.PSEUDO_NEGATIVE_INFINITY}.
	 */
	private static final class PseudoNegativeInfinityHolder {
		
		/** 
		 * The {@code BigDecimal} whose identity is negative infinity when 
		 * compared to other {@code BigDecimal}s (used as the default minimum 
		 * value). 
		 */
		static final BigDecimal PSEUDO_NEGATIVE_INFINITY = new BigDecimal("-1");
		
		/**
		 * Constructs a {@code PseudoNegativeInfinityHolder}. This constructor 
		 * is suppressed for noninstantiability outside of this class.
		 */
		private PseudoNegativeInfinityHolder() {
		}
	}
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code PseudoPositiveInfinityHolder.PSEUDO_POSITIVE_INFINITY}.
	 */
	private static final class PseudoPositiveInfinityHolder {
		
		/** 
		 * The {@code BigDecimal} whose identity is positive infinity when 
		 * compared to other {@code BigDecimal}s (used as the default maximum 
		 * value). 
		 */
		static final BigDecimal PSEUDO_POSITIVE_INFINITY = new BigDecimal("1");
		
		/**
		 * Constructs a {@code PseudoPositiveInfinityHolder}. This constructor 
		 * is suppressed for noninstantiability outside of this class.
		 */
		private PseudoPositiveInfinityHolder() {
		}
	}
	
	/** 
	 * The default {@code MathContext} to be used in interpreting the argument. 
	 */
	private static final MathContext DEFAULT_MATH_CONTEXT = 
			MathContext.UNLIMITED;

	/**
	 * Returns a {@code BigDecimalArgParser}.
	 * 
	 * @return a {@code BigDecimalArgParser}
	 */
	public static BigDecimalArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}
	
	/** 
	 * Returns the {@code BigDecimal} whose identity is negative infinity when 
	 * compared to other {@code BigDecimal}s (used as the default minimum
	 * value).
	 * 
	 * @return the {@code BigDecimal} whose identity is negative infinity when 
	 * compared to other {@code BigDecimal}s (used as the default minimum
	 * value) 
	 */
	public static BigDecimal getPseudoNegativeInfinity() {
		return PseudoNegativeInfinityHolder.PSEUDO_NEGATIVE_INFINITY;
	}
	
	/** 
	 * Returns the {@code BigDecimal} whose identity is positive infinity when 
	 * compared to other {@code BigDecimal}s (used as the default maximum
	 * value).
	 * 
	 * @return the {@code BigDecimal} whose identity is positive infinity when 
	 * compared to other {@code BigDecimal}s (used as the default maximum
	 * value) 
	 */
	public static BigDecimal getPseudoPositiveInfinity() {
		return PseudoPositiveInfinityHolder.PSEUDO_POSITIVE_INFINITY;
	}
	
	/** The {@code MathContext} to be used in interpreting the argument. */
	private final MathContext mathContext;
	
	/**
	 * Constructs a {@code BigDecimalArgParser}.
	 */
	public BigDecimalArgParser() {
		
		this(
				getPseudoNegativeInfinity(), 
				getPseudoPositiveInfinity(), 
				BigDecimalArgParser.DEFAULT_MATH_CONTEXT); 
	}
	
	/**
	 * Constructs a {@code BigDecimalArgParser} with the provided minimum value
	 * and the provided maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public BigDecimalArgParser(
			final BigDecimal minValue, final BigDecimal maxValue) {
		
		this(minValue, maxValue, BigDecimalArgParser.DEFAULT_MATH_CONTEXT);
	}
	
	/**
	 * Constructs a {@code BigDecimalArgParser} with the provided minimum 
	 * value, the provided maximum value, and the provided {@code MathContext}.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 * @param mc the provided {@code MathContext}
	 */
	public BigDecimalArgParser(
			final BigDecimal minValue, 
			final BigDecimal maxValue, 
			final MathContext mc) {
		
		super(BigDecimalComparator.getInstance(), minValue, maxValue);
		
		if (minValue == getPseudoNegativeInfinity() 
				&& maxValue == getPseudoNegativeInfinity()) {
			
			throw new IllegalArgumentException(
					"minValue and maxValue cannot both be pseudo negative " 
					+ "infinity");
		}
		
		if (minValue == getPseudoPositiveInfinity() 
				&& maxValue == getPseudoPositiveInfinity()) {
			
			throw new IllegalArgumentException(
					"minValue and maxValue cannot both be pseudo positive " 
					+ "infinity");
		}
		
		if (mc == null) {
			throw new NullPointerException("MathContext must not be null");
		}
		
		this.mathContext = mc;		
	}
	
	/**
	 * Constructs a {@code BigDecimalArgParser} with the provided 
	 * {@code MathContext}.
	 * 
	 * @param mc the provided {@code MathContext}
	 */
	public BigDecimalArgParser(final MathContext mc) {
		
		this(
				getPseudoNegativeInfinity(), 
				getPseudoPositiveInfinity(), 
				mc);
	}

	@Override
	protected BigDecimal parseNumber(final String string) {
		
		BigDecimal d = null;
		
		try {
			
			d = new BigDecimal(string, this.mathContext);
			
		} catch (NumberFormatException e) {
			
			BigDecimal minValue = this.getMinValue();
			BigDecimal maxValue = this.getMaxValue();
			
			if (minValue == getPseudoNegativeInfinity() 
					&& maxValue == getPseudoPositiveInfinity()) {
				
				throw new NumberFormatException(
						"argument must be a floating point number");
				
			} else if (minValue == getPseudoNegativeInfinity()) {
				
				String message = String.format(
						"argument must be a floating point number" 
						+ " no more than %1$s",
						maxValue);
				
				throw new NumberFormatException(message);
				
			} else if (maxValue == getPseudoPositiveInfinity()) {	
				
				String message = String.format(
						"argument must be a floating point number" 
						+ " no less than %1$s",
						minValue);
				
				throw new NumberFormatException(message);
				
			} else {
				
				String message = String.format(
						"argument must be a floating point number" 
						+ " no less than %1$s and no more than %2$s",
						minValue,
						maxValue);
				
				throw new NumberFormatException(message);				
			}
		}
		
		return d;
	}

}
