/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.math.BigInteger;
import java.util.Comparator;

/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code BigInteger}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class BigIntegerArgParser extends NumberArgParser<BigInteger>
		implements ParameterlessArgParser<BigInteger> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final BigIntegerArgParser ARG_PARSER =
			new BigIntegerArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code BigInteger}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class BigIntegerComparator 
		implements Comparator<BigInteger> {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final BigIntegerComparator COMPARATOR =
				new BigIntegerComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/**
		 * Returns a {@code BigIntegerComparator}.
		 * 
		 * @return a {@code BigIntegerComparator}
		 */		
		public static BigIntegerComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}

		/**
		 * Constructs a {@code BigIntegerComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private BigIntegerComparator() {
		}

		@Override
		public int compare(
				final BigInteger bigInteger1, final BigInteger bigInteger2) {

			if (bigInteger1 == getPseudoNegativeInfinity()
					&& bigInteger2 == getPseudoNegativeInfinity()) {
				
				return 0;
			}
			
			if (bigInteger1 == getPseudoPositiveInfinity()
					&& bigInteger2 == getPseudoPositiveInfinity()) {
				
				return 0;
			}
			
			if (bigInteger1 == getPseudoNegativeInfinity()) {
				return -1;
			}
			
			if (bigInteger2 == getPseudoNegativeInfinity()) {
				return 1;
			}
			
			if (bigInteger1 == getPseudoPositiveInfinity()) {
				return 1;
			}
			
			if (bigInteger2 == getPseudoPositiveInfinity()) {
				return -1;
			}
			
			return bigInteger1.compareTo(bigInteger2);
		}
	}
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code PseudoNegativeInfinityHolder.PSEUDO_NEGATIVE_INFINITY}.
	 */
	private static final class PseudoNegativeInfinityHolder {
		
		/** 
		 * The {@code BigInteger} whose identity is negative infinity when 
		 * compared to other {@code BigInteger}s (used as the default minimum 
		 * value). 
		 */
		static final BigInteger PSEUDO_NEGATIVE_INFINITY = new BigInteger("-1");
		
		/**
		 * Constructs a {@code PseudoNegativeInfinityHolder}. This constructor 
		 * is suppressed for noninstantiability outside of this class.
		 */
		private PseudoNegativeInfinityHolder() {
		}
	}

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code PseudoPositiveInfinityHolder.PSEUDO_POSITIVE_INFINITY}.
	 */
	private static final class PseudoPositiveInfinityHolder {
		
		/** 
		 * The {@code BigInteger} whose identity is positive infinity when 
		 * compared to other {@code BigInteger}s (used as the default maximum 
		 * value). 
		 */
		static final BigInteger PSEUDO_POSITIVE_INFINITY = new BigInteger("1");
		
		/**
		 * Constructs a {@code PseudoPositiveInfinityHolder}. This constructor 
		 * is suppressed for noninstantiability outside of this class.
		 */
		private PseudoPositiveInfinityHolder() {
		}
	}
	
	/** The default radix to be used in interpreting the argument. */
	private static final int DEFAULT_RADIX = 10;
	
	/**
	 * Returns a {@code BigIntegerArgParser}.
	 * 
	 * @return a {@code BigIntegerArgParser}
	 */	
	public static BigIntegerArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}
	
	/** 
	 * Returns the {@code BigInteger} whose identity is negative infinity when 
	 * compared to other {@code BigInteger}s (used as the default minimum
	 * value).
	 * 
	 * @return the {@code BigInteger} whose identity is negative infinity when 
	 * compared to other {@code BigInteger}s (used as the default minimum
	 * value) 
	 */
	public static BigInteger getPseudoNegativeInfinity() {
		return PseudoNegativeInfinityHolder.PSEUDO_NEGATIVE_INFINITY;
	}

	/** 
	 * Returns the {@code BigInteger} whose identity is positive infinity when 
	 * compared to other {@code BigInteger}s (used as the default maximum
	 * value).
	 * 
	 * @return the {@code BigInteger} whose identity is positive infinity when 
	 * compared to other {@code BigInteger}s (used as the default maximum
	 * value) 
	 */
	public static BigInteger getPseudoPositiveInfinity() {
		return PseudoPositiveInfinityHolder.PSEUDO_POSITIVE_INFINITY;
	}	
	
	/** The radix to be used in interpreting the argument. */
	private final int radix;

	/**
	 * Constructs a {@code BigIntegerArgParser}.
	 */
	public BigIntegerArgParser() {
		
		this(
				getPseudoNegativeInfinity(), 
				getPseudoPositiveInfinity(), 
				BigIntegerArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code BigIntegerArgParser} with the provided minimum value
	 * and the provided maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public BigIntegerArgParser(
			final BigInteger minValue, final BigInteger maxValue) {
		
		this(minValue, maxValue, BigIntegerArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code BigIntegerArgParser} with the provided minimum value,
	 * the provided maximum value, and the provided radix to be used in 
	 * interpreting the argument.
	 * 
	 * @param minValue the provided minimum value 
	 * @param maxValue the provided maximum value
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public BigIntegerArgParser(
			final BigInteger minValue, 
			final BigInteger maxValue, 
			final int r) {
		
		super(BigIntegerComparator.getInstance(), minValue, maxValue);
		
		if (minValue == getPseudoNegativeInfinity() 
				&& maxValue == getPseudoNegativeInfinity()) {
			
			throw new IllegalArgumentException(
					"minValue and maxValue cannot both be pseudo negative " 
					+ "infinity");
		}
		
		if (minValue == getPseudoPositiveInfinity() 
				&& maxValue == getPseudoPositiveInfinity()) {
			
			throw new IllegalArgumentException(
					"minValue and maxValue cannot both be pseudo positive " 
					+ "infinity");
		}

		this.radix = r;
	}
	
	/**
	 * Constructs a {@code BigIntegerArgParser} with the provided radix to be 
	 * used in interpreting the argument.
	 * 
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public BigIntegerArgParser(final int r) {
		
		this(
				getPseudoNegativeInfinity(), 
				getPseudoPositiveInfinity(), 
				r);
	}

	@Override
	protected BigInteger parseNumber(final String string) {
		
		BigInteger i = null;
		
		try {
			
			i = new BigInteger(string, this.radix);
			
		} catch (NumberFormatException e) {
			
			BigInteger minValue = this.getMinValue();
			BigInteger maxValue = this.getMaxValue();
			
			if (minValue == getPseudoNegativeInfinity() 
					&& maxValue == getPseudoPositiveInfinity()) {
				
				throw new NumberFormatException(
						"argument must be an integer");
				
			} else if (minValue == getPseudoNegativeInfinity()) {
				
				String message = String.format(
						"argument must be an integer no more than %1$s",
						maxValue);
				
				throw new NumberFormatException(message);
				
			} else if (maxValue == getPseudoPositiveInfinity()) {	
				
				String message = String.format(
						"argument must be an integer no less than %1$s",
						minValue);
				
				throw new NumberFormatException(message);
				
			} else {
				
				String message = String.format(
						"argument must be an integer" 
						+ " no less than %1$s and no more than %2$s",
						minValue.toString(this.radix),
						maxValue.toString(this.radix));
				
				throw new NumberFormatException(message);
			}
		}
		
		return i;
	}
}
