/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.Serializable;
import java.util.Comparator;


/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code Byte}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class ByteArgParser extends NumberArgParser<Byte> 
	implements ParameterlessArgParser<Byte> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final ByteArgParser ARG_PARSER =
			new ByteArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code Byte}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class ByteComparator 
		implements Comparator<Byte>, Serializable {

		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final ByteComparator COMPARATOR =
				new ByteComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Returns a {@code ByteComparator}.
		 * 
		 * @return a {@code ByteComparator}
		 */		
		public static ByteComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}
		
		/**
		 * Constructs a {@code ByteComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */
		private ByteComparator() {
		}
		
		@Override
		public int compare(final Byte byte1, final Byte byte2) {
			
			return byte1.compareTo(byte2);
		}
	}
	
	/** The default radix to be used in interpreting the argument. */
	private static final int DEFAULT_RADIX = 10;

	/**
	 * Returns a {@code ByteArgParser}.
	 * 
	 * @return a {@code ByteArgParser}
	 */	
	public static ByteArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}
	
	/** The radix to be used in interpreting the argument. */
	private final int radix;
	
	/**
	 * Constructs a {@code ByteArgParser}.
	 */
	public ByteArgParser() {
		
		this(
				Byte.valueOf(Byte.MIN_VALUE), 
				Byte.valueOf(Byte.MAX_VALUE), 
				ByteArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code ByteArgParser} with the provided minimum value and
	 * the provided maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public ByteArgParser(final Byte minValue, final Byte maxValue) {
		
		this(minValue, maxValue, ByteArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code ByteArgParser} with the provided minimum value,
	 * the provided maximum value, and the provided radix to be used in 
	 * interpreting the argument.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public ByteArgParser(
			final Byte minValue, final Byte maxValue, final int r) {
		
		super(ByteComparator.getInstance(), minValue, maxValue);
		
		this.radix = r;
	}

	/**
	 * Constructs a {@code ByteArgParser} with the provided radix to be used in
	 * interpreting the argument.
	 * 
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public ByteArgParser(final int r) {
		
		this(
				Byte.valueOf(Byte.MIN_VALUE), 
				Byte.valueOf(Byte.MAX_VALUE), 
				r);
	}
	
	@Override
	protected Byte parseNumber(final String string) {

		Byte b = null;
		
		try {
			
			b = Byte.valueOf(string, this.radix);
			
		} catch (NumberFormatException e) {
			
			String message = String.format(
					"argument must be an integer" 
					+ " no less than %1$s and no more than %2$s",
					Integer.toString(this.getMinValue(), this.radix),
					Integer.toString(this.getMaxValue(), this.radix));
			
			throw new NumberFormatException(message);
		}
		
		return b;
	}

}
