/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import org.climax.ArgParserException;

/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code Class}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class ClassArgParser implements ParameterlessArgParser<Class<?>> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final ClassArgParser ARG_PARSER =
			new ClassArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * Returns a {@code ClassArgParser}.
	 * 
	 * @return a {@code ClassArgParser}
	 */	
	public static ClassArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/**
	 * The boolean value that indicates that the class must be initialized.
	 */
	private final boolean initializeClass;
	
	/**
	 * The {@code ClassLoader} from which the class must be loaded.
	 */
	private final ClassLoader classLoader;
	
	/**
	 * Constructs a {@code ClassArgParser}.
	 */
	public ClassArgParser() {
		this(true, ClassArgParser.class.getClassLoader());
	}
	
	/**
	 * Constructs a {@code ClassArgParser} with the provided boolean value
	 * that indicates that the class must be initialized and the provided
	 * {@code ClassLoader} from which the class must be loaded.
	 * 
	 * @param initialize the provided boolean value that indicates that the 
	 * class must be initialized
	 * 
	 * @param loader the provided {@code ClassLoader} from which the class 
	 * must be loaded
	 */
	public ClassArgParser(
			final boolean initialize, final ClassLoader loader) {
		
		this.initializeClass = initialize;
		this.classLoader = loader;
	}
	
	@Override
	public Class<?> parse(final String arg) throws ArgParserException {
		
		Class<?> cls = null;
		
		try {
			
			cls = Class.forName(arg, this.initializeClass, this.classLoader);
			
		} catch (ClassNotFoundException e) {
			
			String message = String.format("cannot find class %s", arg);
			
			throw new ArgParserException(arg, message, e);
			
		} 
		
		return cls;
	}

}
