/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.climax.ArgParserException;


/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code Date}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class DateArgParser implements ParameterlessArgParser<Date> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final DateArgParser ARG_PARSER =
			new DateArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * Returns a {@code DateArgParser}.
	 * 
	 * @return a {@code DateArgParser}
	 */	
	public static DateArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/** The {@code DateFormat}. */
	private final DateFormat dateFormat;
	
	/**
	 * Constructs a {@code DateArgParser}.
	 */
	public DateArgParser() {
		this(DateFormat.getDateTimeInstance());
	}
	
	/**
	 * Constructs a {@code DateArgParser} with the provided {@code DateFormat}.
	 * 
	 * @param format the provided {@code DateFormat}
	 */
	public DateArgParser(final DateFormat format) {
		
		this.dateFormat = format;
	}
	
	@Override
	public Date parse(final String arg) throws ArgParserException {
		
		Date date = null;
		
		try {
			
			date = this.dateFormat.parse(arg);
			
		} catch (ParseException e) {
			
			String message = e.getMessage();
			
			if (this.dateFormat instanceof SimpleDateFormat) {
				
				message = String.format(
						"argument must be in the following format: %1$s"
						+ " (example: %2$s)", 
						((SimpleDateFormat) this.dateFormat).toPattern(),
						this.dateFormat.format(new Date()));
			}
			
			throw new ArgParserException(arg, message, e);
		}
		
		return date;
	}

}
