/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.Serializable;
import java.util.Comparator;


/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code Double}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class DoubleArgParser extends NumberArgParser<Double> 
	implements ParameterlessArgParser<Double> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final DoubleArgParser ARG_PARSER =
			new DoubleArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code Double}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class DoubleComparator 
		implements Comparator<Double>, Serializable {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final DoubleComparator COMPARATOR =
				new DoubleComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Returns a {@code DoubleComparator}.
		 * 
		 * @return a {@code DoubleComparator}
		 */		
		public static DoubleComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}
		
		/**
		 * Constructs a {@code DoubleComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private DoubleComparator() {
		}

		@Override
		public int compare(final Double double1, final Double double2) {
			
			return double1.compareTo(double2);
		}
	}
	
	/**
	 * Returns a {@code DoubleArgParser}.
	 * 
	 * @return a {@code DoubleArgParser}
	 */	
	public static DoubleArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/**
	 * Constructs a {@code DoubleArgParser}.
	 */
	public DoubleArgParser() {
		
		this(
				Double.valueOf(Double.MIN_VALUE), 
				Double.valueOf(Double.MAX_VALUE));
	}
	
	/**
	 * Constructs a {@code DoubleArgParser} with the provided minimum value
	 * and the provided maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public DoubleArgParser(final Double minValue, final Double maxValue) {
		
		super(DoubleComparator.getInstance(), minValue, maxValue);
	}

	@Override
	protected Double parseNumber(final String string) {

		Double d = null;
		
		try {
			
			d = Double.valueOf(string);
			
		} catch (NumberFormatException e) {
			
			String message = String.format(
					"argument must be a floating point number" 
					+ " no less than %1$s and no more than %2$s",
					this.getMinValue(),
					this.getMaxValue());
			
			throw new NumberFormatException(message);
		}
		
		return d;
	}

}
