/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.lang.ref.WeakReference;
import java.util.Arrays;
import java.util.Map;
import java.util.WeakHashMap;

import org.climax.ArgParser;
import org.climax.ArgParserException;


/**
 * An {@code ArgParser} implementation that parses an argument as an
 * {@code Enum} of the specified enum type.
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <E> the specified enum type
 */
public final class EnumArgParser<E extends Enum<E>> implements ArgParser<E> {
	
	/**
	 * Lazy initialization holder class idiom for static field
	 * {@code ClassMapHolder.CLASS_MAP}.
	 */
	private static final class ClassMapHolder {
		
		/**
		 * A {@code Map} that maps existing enum classes to existing
		 * {@code EnumArgParser}s.
		 */
		static final Map<Class<? extends Enum<?>>, 
				WeakReference<EnumArgParser<? extends Enum<?>>>> 
		CLASS_MAP = new WeakHashMap<Class<? extends Enum<?>>, 
				WeakReference<EnumArgParser<? extends Enum<?>>>>();
		
		/**
		 * Constructs a {@code ClassMapHolder}. This constructor is suppressed 
		 * for noninstantiability outside of this class.
		 */
		private ClassMapHolder() {
		}
	}
	
	/**
	 * Returns an {@code EnumArgParser} of the specified enum type based on the 
	 * provided enum class.
	 * 
	 * @param <E> the specified enum type
	 * @param enumClass the provided enum class
	 * @return an {@code EnumArgParser} of the specified enum type
	 */
	public static <E extends Enum<E>> EnumArgParser<E> getInstance(
			final Class<E> enumClass) {
		
		EnumArgParser<E> enumArgParser = null;
		
		Map<Class<? extends Enum<?>>, 
				WeakReference<EnumArgParser<? extends Enum<?>>>> 
		classMap = ClassMapHolder.CLASS_MAP;
		
		if (!classMap.containsKey(enumClass)) {
			
			enumArgParser = new EnumArgParser<E>(enumClass);
			
			WeakReference<EnumArgParser<? extends Enum<?>>> 
			weakRef = new WeakReference<EnumArgParser<? extends Enum<?>>>(
					enumArgParser);
			
			classMap.put(enumClass, weakRef);
			
		} else {
			
			@SuppressWarnings("unchecked")
			EnumArgParser<E> eArgParser =
				(EnumArgParser<E>) classMap.get(enumClass).get();
			
			enumArgParser = eArgParser;
		}
		
		return enumArgParser;
	}

	/** The enum class. */
	private Class<E> enumClass;
	
	/**
	 * Constructs an {@code EnumArgParser} with the provided enum class.
	 * 
	 * @param enumCls the provided enum class
	 */
	public EnumArgParser(final Class<E> enumCls) {
		
		if (enumCls == null) {
			throw new NullPointerException("enum class must not be null");
		}
		
		if (!enumCls.isEnum()) {
			throw new IllegalArgumentException("class must be an enum");
		}
		
		this.enumClass = enumCls;
	}

	@Override
	public E parse(final String arg) throws ArgParserException {
		
		E e = null;
		
		try {
			
			e = Enum.valueOf(this.enumClass, arg);
			
		} catch (IllegalArgumentException ex) {
			
			String enumConstantsString =
				Arrays.toString(this.enumClass.getEnumConstants());
			
			enumConstantsString = enumConstantsString.substring(
					1, enumConstantsString.length() - 1);
				
			String message = String.format(
					"argument must be one of the following: %s", 
					enumConstantsString);
			
			throw new ArgParserException(arg, message, ex);			
		}
		
		return e;
	}
}
