/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.File;
import java.util.EnumSet;
import java.util.Set;

import org.climax.ArgParserException;


/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code File}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class FileArgParser implements ParameterlessArgParser<File> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final FileArgParser ARG_PARSER =
			new FileArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A file permission type. The constants of this enumerated type
	 * provides a classification of file permissions.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	public static enum Permission {
		
		/** Permission that allows a file to be executed. */
		EXECUTE {

			@Override
			boolean hasPermission(final File file) {
				return file.canExecute();
			}
		},
		
		/** Permission that allows a file to be read. */
		READ {

			@Override
			boolean hasPermission(final File file) {
				return file.canRead();
			}
		},
		
		/** Permission that allows a file to be written. */
		WRITE {

			@Override
			boolean hasPermission(final File file) {
				return file.canWrite();
			}
		};
		
		/**
		 * Indicates if the provided {@code File} has the file permission.
		 * 
		 * @param file the provided {@code File}
		 * 
		 * @return {@code true} if the provided {@code File} has the file
		 * permission
		 */
		abstract boolean hasPermission(File file);
	}

	/**
	 * A file type. The constants of this enumerated type provides a 
	 * classification of file types.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */
	public static enum Type {
		
		/** Directory. */
		DIRECTORY {

			@Override
			boolean isType(final File file) {
				return file.isDirectory();
			}
		},
		
		/** File. */
		FILE {

			@Override
			boolean isType(final File file) {
				return file.isFile();
			}
		};
		
		/**
		 * Indicates if the provided {@code File} is described by this 
		 * {@code Type}.
		 * 
		 * @param file the provided {@code File}
		 * 
		 * @return {@code true} if the provided {@code File} is described by 
		 * this {@code Type}
		 */
		abstract boolean isType(File file);
	}
	
	/**
	 * Returns a {@code FileArgParser}.
	 * 
	 * @return a {@code FileArgParser}
	 */	
	public static FileArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/** The boolean value to indicate if the {@code File} must exist. */
	private final boolean fileMustExist;
	
	/** The {@code Set} of allowed {@code Type}s. */
	private final Set<Type> allowedTypeSet;
	
	/** The {@code Set} of required {@code Permission}s. */
	private final Set<Permission> requiredPermissionSet;
	
	/**
	 * Constructs a {@code FileArgParser}.
	 */
	public FileArgParser() {
		
		this(
				false,
				EnumSet.allOf(Type.class),
				EnumSet.noneOf(Permission.class));
	}
	
	/**
	 * Constructs a {@code FileArgParser} with the provided boolean value 
	 * to indicate if the {@code File} must exist.
	 * 
	 * @param mustExist the provided boolean value to indicate if the 
	 * {@code File} must exist
	 */
	public FileArgParser(final boolean mustExist) {
		
		this(
				mustExist,
				EnumSet.allOf(Type.class),
				EnumSet.noneOf(Permission.class));
	}
	
	/**
	 * Constructs a {@code FileArgParser} with the provided boolean value 
	 * to indicate if the {@code File} must exist, the provided {@code Set} 
	 * of allowed {@code Type}s, and the provided {@code Set} of required 
	 * {@code Permission}s.
	 * 
	 * @param mustExist the provided boolean value to indicate if the 
	 * {@code File} must exist
	 * 
	 * @param allowedTypes the provided {@code Set} of allowed {@code Type}s
	 * 
	 * @param requiredPermissions the provided {@code Set} of required 
	 * {@code Permission}s
	 */
	public FileArgParser(
			final boolean mustExist,
			final Set<Type> allowedTypes,
			final Set<Permission> requiredPermissions) {
		
		if (allowedTypes == null) {
			throw new NullPointerException(
					"set of allowed types must not be null");
		}
		
		if (allowedTypes.size() == 0) {
			throw new IllegalArgumentException(
					"set of allowed types must not be empty");
		}
		
		for (Type allowedType : allowedTypes) {
			if (allowedType == null) {
				throw new NullPointerException(
						"allowed types in set must not be null");				
			}
		}
		
		if (requiredPermissions == null) {
			throw new NullPointerException(
					"set of required permissions must not be null");
		}
		
		for (Permission requiredPermission : requiredPermissions) {
			if (requiredPermission == null) {
				throw new NullPointerException(
						"required permissions in set must not be null");
			}
		}
		
		this.fileMustExist = mustExist;
		this.allowedTypeSet = EnumSet.copyOf(allowedTypes);		
		this.requiredPermissionSet = EnumSet.copyOf(requiredPermissions);
	}
	
	/**
	 * Constructs a {@code FileArgParser} with the provided {@code Set} 
	 * of allowed {@code Type}s and the provided {@code Set} of required 
	 * {@code Permission}s.
	 * 
	 * @param allowedTypes the provided {@code Set} of allowed {@code Type}s
	 * 
	 * @param requiredPermissions the provided {@code Set} of required 
	 * {@code Permission}s
	 */
	public FileArgParser(
			final Set<Type> allowedTypes,
			final Set<Permission> requiredPermissions) {
		
		this(
				false,
				allowedTypes,
				requiredPermissions);		
	}
	
	@Override
	public File parse(final String arg) throws ArgParserException {

		File file = new File(arg);
		
		if (!file.exists() && this.fileMustExist) {
			
			String message = String.format(
					"cannot find `%s'", file);
			
			throw new ArgParserException(arg, message);			
		}
		
		if (!file.exists() && !this.fileMustExist) {
			
			return file;
		}
		
		Set<Type> acceptedTypeSet = EnumSet.noneOf(Type.class);
		
		for (Type allowedType : this.allowedTypeSet) {
			
			if (allowedType.isType(file)) {
				
				acceptedTypeSet.add(allowedType);
			}
		}
		
		if (acceptedTypeSet.size() == 0) {
			
			String allowedTypeSetString = this.allowedTypeSet.toString();
			
			allowedTypeSetString = 
				allowedTypeSetString.substring(
					1, allowedTypeSetString.length() - 1);
			
			String message = String.format(
					"`%1$s' must be one of the following" 
					+ " file type(s): %2$s", 
					file,
					allowedTypeSetString);
			
			throw new ArgParserException(arg, message);			
		}
		
		Set<Permission> missingPermissionSet =
			EnumSet.noneOf(Permission.class);
		
		for (Permission requiredPermission : this.requiredPermissionSet) {
			
			if (!requiredPermission.hasPermission(file)) {
				
				missingPermissionSet.add(requiredPermission);
			}
		}
		
		if (missingPermissionSet.size() > 0) {
			
			String missingPermissionSetString = missingPermissionSet.toString();
			
			missingPermissionSetString = missingPermissionSetString.substring(
					1, missingPermissionSetString.length() - 1);
			
			String message = String.format(
					"`%1$s' does not have the following" 
					+ " file permission(s): %2$s", 
					file,
					missingPermissionSetString);
			
			throw new ArgParserException(arg, message);			
		}
		
		return file;
	}

}
