/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import org.climax.ArgParserException;


/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code FileInputStream}. This implementation can parse a hyphen 
 * ("<code>-</code>") as a {@code FileInputStream} of standard input.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class FileInputStreamArgParser 
	implements ParameterlessArgParser<FileInputStream> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final FileInputStreamArgParser ARG_PARSER =
			new FileInputStreamArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}

	/**
	 * Returns a {@code FileInputStreamArgParser}.
	 * 
	 * @return a {@code FileInputStreamArgParser}
	 */
	public static FileInputStreamArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/**
	 * Constructs a {@code FileInputStreamArgParser}.
	 */
	public FileInputStreamArgParser() {
	}
	
	@Override
	public FileInputStream parse(final String arg) throws ArgParserException {
		
		FileInputStream fileInputStream = null;
		
		if (arg.equals("-")) {
			
			try {
			
				fileInputStream = new FileInputStream(FileDescriptor.in);
				
			} catch (SecurityException e) {
			
				String message = "read access is denied to standard input";
				
				throw new ArgParserException(arg, message, e);
			}
			
		} else {
			
			File file = new File(arg);
			
			if (!file.exists()) {
				
				String message = String.format("cannot find `%s'", file);
				
				throw new ArgParserException(arg, message);
			}
			
			if (file.isDirectory()) {
				
				String message = String.format(
						"`%s' must be a file, not a directory", file);
				
				throw new ArgParserException(arg, message);
			}
			
			try {
				
				fileInputStream = new FileInputStream(file);
				
			} catch (FileNotFoundException e) {
				
				throw new ArgParserException(arg, e.getMessage(), e);

			} catch (SecurityException e) {
				
				String message = String.format(
						"read access is denied to `%s'", arg);
				
				throw new ArgParserException(arg, message, e);
			}
		}
		
		return fileInputStream;
	}

}
