/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import org.climax.ArgParserException;


/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code FileOutputStream}. This implementation can parse a hyphen 
 * ("<code>-</code>") as a {@code FileOutputStream} of standard output.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class FileOutputStreamArgParser 
	implements ParameterlessArgParser<FileOutputStream> {
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final FileOutputStreamArgParser ARG_PARSER =
			new FileOutputStreamArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}

	/**
	 * Returns a {@code FileOutputStreamArgParser}.
	 * 
	 * @return a {@code FileOutputStreamArgParser}
	 */
	public static FileOutputStreamArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/** 
	 * The boolean value to indicate if the bytes will be written to the end 
	 * of the file rather than the beginning.
	 */
	private final boolean appendBytesToFile;
	
	/**
	 * Constructs a {@code FileOutputStreamArgParser}.
	 */
	public FileOutputStreamArgParser() {
		this(false);
	}
	
	/**
	 * Constructs a {@code FileOutputStreamArgParser} with the provided boolean
	 * value to indicate if the bytes will be written to the end of the file 
	 * rather than the beginning.
	 * 
	 * @param append the provided boolean value to indicate if the bytes will 
	 * be written to the end of the file rather than the beginning
	 */
	public FileOutputStreamArgParser(final boolean append) {
		this.appendBytesToFile = append;
	}
	
	@Override
	public FileOutputStream parse(final String arg) throws ArgParserException {
		
		FileOutputStream fileOutputStream = null;
		
		if (arg.equals("-")) {
			
			try {
				
				fileOutputStream = new FileOutputStream(FileDescriptor.out);
				
			} catch (SecurityException e) {
				
				String message = "write access is denied to standard output";
				
				throw new ArgParserException(arg, message, e);
			}
			
		} else {
			
			File file = new File(arg);
			
			if (file.exists() && file.isDirectory()) {
				
				String message = String.format(
						"`%s' must be a file, not a directory", file);
				
				throw new ArgParserException(arg, message);
			}

			try {
				
				fileOutputStream = new FileOutputStream(
						file, this.appendBytesToFile);
				
			} catch (FileNotFoundException e) {

				throw new ArgParserException(arg, e.getMessage(), e);
				
			} catch (SecurityException e) {
				
				String message = String.format(
						"write access is denied to `%s'", arg);
				
				throw new ArgParserException(arg, message, e);
			}
		}
		
		return fileOutputStream;
	}

}
