/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileNotFoundException;
import java.io.FileReader;

import org.climax.ArgParserException;

/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code FileReader}. This implementation can parse a hyphen 
 * ("<code>-</code>") as a {@code FileReader} of standard input.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class FileReaderArgParser implements
		ParameterlessArgParser<FileReader> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final FileReaderArgParser ARG_PARSER =
			new FileReaderArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * Returns a {@code FileReaderArgParser}.
	 * 
	 * @return a {@code FileReaderArgParser}
	 */
	public static FileReaderArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/**
	 * Constructs a {@code FileReaderArgParser}.
	 */
	public FileReaderArgParser() {
	}
	
	@Override
	public FileReader parse(final String arg) throws ArgParserException {
		
		FileReader fileReader = null;
		
		if (arg.equals("-")) {
		
			fileReader = new FileReader(FileDescriptor.in);
			
		} else {
			
			File file = new File(arg);
			
			if (!file.exists()) {
				
				String message = String.format("cannot find `%s'", file);
				
				throw new ArgParserException(arg, message);
			}
			
			if (file.isDirectory()) {
				
				String message = String.format(
						"`%s' must be a file, not a directory", file);
				
				throw new ArgParserException(arg, message);
			}
			
			try {
				
				fileReader = new FileReader(file);
				
			} catch (FileNotFoundException e) {
				
				throw new ArgParserException(arg, e.getMessage(), e);
			}			
		}
		
		return fileReader;
	}
}
