/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileWriter;
import java.io.IOException;

import org.climax.ArgParserException;

/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code FileWriter}. This implementation can parse a hyphen 
 * ("<code>-</code>") as a {@code FileWriter} of standard output.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class FileWriterArgParser implements
		ParameterlessArgParser<FileWriter> {
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final FileWriterArgParser ARG_PARSER =
			new FileWriterArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}

	/**
	 * Returns a {@code FileWriterArgParser}.
	 * 
	 * @return a {@code FileWriterArgParser}
	 */
	public static FileWriterArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/** 
	 * The boolean value to indicate if the data will be written to the end 
	 * of the file rather than the beginning.
	 */
	private final boolean appendDataToFile;
	
	/**
	 * Constructs a {@code FileWriterArgParser}.
	 */
	public FileWriterArgParser() {
		this(false);
	}
	
	/**
	 * Constructs a {@code FileWriterArgParser} with the provided boolean
	 * value to indicate if the data will be written to the end of the file 
	 * rather than the beginning.
	 * 
	 * @param append the provided boolean value to indicate if the data will 
	 * be written to the end of the file rather than the beginning
	 */	
	public FileWriterArgParser(final boolean append) {
		this.appendDataToFile = append;
	}
	
	@Override
	public FileWriter parse(final String arg) throws ArgParserException {
		
		FileWriter fileWriter = null;
		
		if (arg.equals("-")) {
		
			fileWriter = new FileWriter(FileDescriptor.out);
			
		} else {
			
			File file = new File(arg);
			
			if (file.exists() && file.isDirectory()) {
				
				String message = String.format(
						"`%s' must be a file, not a directory", file);
				
				throw new ArgParserException(arg, message);
			}
			
			try {
				
				fileWriter = new FileWriter(file, this.appendDataToFile);
				
			} catch (IOException e) {
				
				throw new ArgParserException(arg, e.getMessage(), e);
			}
		}
		
		return fileWriter;
	}

}
