/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.Serializable;
import java.util.Comparator;


/**
 * An {@code ArgParser} implementation that parses an argument as a 
 * {@code Float}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class FloatArgParser extends NumberArgParser<Float> 
	implements ParameterlessArgParser<Float> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final FloatArgParser ARG_PARSER =
			new FloatArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code Float}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class FloatComparator
		implements Comparator<Float>, Serializable {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final FloatComparator COMPARATOR =
				new FloatComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Returns a {@code FloatComparator}.
		 * 
		 * @return a {@code FloatComparator}
		 */		
		public static FloatComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}

		/**
		 * Constructs a {@code FloatComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private FloatComparator() {
		}

		@Override
		public int compare(final Float float1, final Float float2) {
			
			return float1.compareTo(float2);
		}
	}
	
	/**
	 * Returns a {@code FloatArgParser}.
	 * 
	 * @return a {@code FloatArgParser}
	 */	
	public static FloatArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/**
	 * Constructs a {@code FloatArgParser}.
	 */
	public FloatArgParser() {
		
		this(Float.valueOf(Float.MIN_VALUE), Float.valueOf(Float.MAX_VALUE));
	}
	
	/**
	 * Constructs a {@code FloatArgParser} with the provided minimum value and
	 * the provided maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public FloatArgParser(final Float minValue, final Float maxValue) {
		
		super(FloatComparator.getInstance(), minValue, maxValue);
	}

	@Override
	protected Float parseNumber(final String string) {

		Float f = null;
		
		try {
			
			f = Float.valueOf(string);
			
		} catch (NumberFormatException e) {
			
			String message = String.format(
					"argument must be a floating point number" 
					+ " no less than %1$s and no more than %2$s",
					this.getMinValue(),
					this.getMaxValue());
			
			throw new NumberFormatException(message);
		}
		
		return f;
	}

}
