/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.Serializable;
import java.util.Comparator;


/**
 * An {@code ArgParser} implementation that parses an argument as an
 * {@code Integer}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class IntegerArgParser extends NumberArgParser<Integer> 
	implements ParameterlessArgParser<Integer> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final IntegerArgParser ARG_PARSER =
			new IntegerArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code Integer}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class IntegerComparator 
		implements Comparator<Integer>, Serializable {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final IntegerComparator COMPARATOR =
				new IntegerComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Returns an {@code IntegerComparator}.
		 * 
		 * @return an {@code IntegerComparator}
		 */		
		public static IntegerComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}

		/**
		 * Constructs an {@code IntegerComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private IntegerComparator() {
		}

		@Override
		public int compare(final Integer integer1, final Integer integer2) {
			
			return integer1.compareTo(integer2);
		}
	}
	
	/** The default radix to be used in interpreting the argument. */
	private static final int DEFAULT_RADIX = 10;

	/**
	 * Returns an {@code IntegerArgParser}.
	 * 
	 * @return an {@code IntegerArgParser}
	 */	
	public static IntegerArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}	
	
	/** The radix to be used in interpreting the argument. */
	private final int radix;
	
	/**
	 * Constructs an {@code IntegerArgParser}.
	 */
	public IntegerArgParser() {
		
		this(
				Integer.valueOf(Integer.MIN_VALUE), 
				Integer.valueOf(Integer.MAX_VALUE), 
				IntegerArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs an {@code IntegerArgParser} with the provided radix to be 
	 * used in interpreting the argument.
	 * 
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public IntegerArgParser(final int r) {
		
		this(
				Integer.valueOf(Integer.MIN_VALUE), 
				Integer.valueOf(Integer.MAX_VALUE), 
				r);
	}
	
	/**
	 * Constructs an {@code IntegerArgParser} with the provided minimum value
	 * and the provided maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public IntegerArgParser(final Integer minValue, final Integer maxValue) {
		
		this(minValue, maxValue, IntegerArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs an {@code IntegerArgParser} with the provided minimum value,
	 * the provided maximum value, and the provided radix to be used in 
	 * interpreting the argument.
	 * 
	 * @param minValue the provided minimum value 
	 * @param maxValue the provided maximum value
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public IntegerArgParser(
			final Integer minValue, final Integer maxValue, final int r) {
		
		super(IntegerComparator.getInstance(), minValue, maxValue);
		
		this.radix = r;
	}

	@Override
	protected Integer parseNumber(final String string) {
		
		Integer i = null;
		
		try {
			
			i = Integer.valueOf(string, this.radix);
			
		} catch (NumberFormatException e) {
			
			String message = String.format(
					"argument must be an integer" 
					+ " no less than %1$s and no more than %2$s",
					Integer.toString(this.getMinValue(), this.radix),
					Integer.toString(this.getMaxValue(), this.radix));
			
			throw new NumberFormatException(message);
		}
		
		return i;
	}

}
