/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.Serializable;
import java.util.Comparator;


/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code Long}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class LongArgParser extends NumberArgParser<Long> 
	implements ParameterlessArgParser<Long> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final LongArgParser ARG_PARSER =
			new LongArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code Long}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class LongComparator
		implements Comparator<Long>, Serializable {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final LongComparator COMPARATOR =
				new LongComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Returns a {@code LongComparator}.
		 * 
		 * @return a {@code LongComparator}
		 */		
		public static LongComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}
		
		/**
		 * Constructs a {@code LongComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private LongComparator() {
		}

		@Override
		public int compare(final Long long1, final Long long2) {
			
			return long1.compareTo(long2);
		}
		
	}
	
	/** The default radix to be used in interpreting the argument. */
	private static final int DEFAULT_RADIX = 10;

	/**
	 * Returns a {@code LongArgParser}.
	 * 
	 * @return a {@code LongArgParser}
	 */	
	public static LongArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}	
	
	/** The radix to be used in interpreting the argument. */
	private final int radix;

	/**
	 * Constructs a {@code LongArgParser}.
	 */
	public LongArgParser() {
		
		this(
				Long.valueOf(Long.MIN_VALUE), 
				Long.valueOf(Long.MAX_VALUE), 
				LongArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code LongArgParser} with the provided radix to be used 
	 * in interpreting the argument.
	 * 
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public LongArgParser(final int r) {
		
		this(
				Long.valueOf(Long.MIN_VALUE), 
				Long.valueOf(Long.MAX_VALUE), 
				r);
	}
	
	/**
	 * Constructs a {@code LongArgParser} with the provided minimum value
	 * and the provided maximum value.
	 *  
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public LongArgParser(final Long minValue, final Long maxValue) {
		
		this(minValue, maxValue, LongArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code LongArgParser} with the provided minimum value,
	 * the provided maximum value, and the provided radix to be used in 
	 * interpreting the argument.
	 * 
	 * @param minValue the provided minimum value 
	 * @param maxValue the provided maximum value
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public LongArgParser(
			final Long minValue, final Long maxValue, final int r) {
		
		super(LongComparator.getInstance(), minValue, maxValue);
		
		this.radix = r;
	}

	@Override
	protected Long parseNumber(final String string) {
		
		Long l = null;
		
		try {
			
			l = Long.valueOf(string, this.radix);
			
		} catch (NumberFormatException e) {
			
			String message = String.format(
					"argument must be an integer" 
					+ " no less than %1$s and no more than %2$s",
					Long.toString(this.getMinValue(), this.radix),
					Long.toString(this.getMaxValue(), this.radix));
			
			throw new NumberFormatException(message);
		}
		
		return l;
	}

}
