/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.util.Comparator;

import org.climax.ArgParser;
import org.climax.ArgParserException;


/**
 * An {@code ArgParser} that parses an argument as a {@code Number} of the
 * specified number type.
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <N> the specified number type
 */
public abstract class NumberArgParser<N extends Number> 
	implements ArgParser<N> {
	
	/** The {@code Comparator}. */
	private final Comparator<N> comparator;
	
	/** The minimum value. */
	private final N minimumValue;
	
	/** The maximum value. */
	private final N maximumValue;
	
	/**
	 * Constructs a {@code NumberArgParser} with the provided 
	 * {@code Comparator}, the provided minimum value, and the provided 
	 * maximum value.
	 * 
	 * @param c the provided {@code Comparator}
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	protected NumberArgParser(
			final Comparator<N> c, final N minValue, final N maxValue) {
		
		this.comparator = c;
		
		if (this.comparator.compare(minValue, maxValue) > 0) {
			throw new IllegalArgumentException(
					"minValue must be less than or equal to maxValue");
		}		
		
		this.minimumValue = minValue;
		this.maximumValue = maxValue;
	}
	
	/**
	 * Returns the maximum value.
	 * 
	 * @return the maximum value
	 */
	protected final N getMaxValue() {
		return this.maximumValue;
	}
	
	/**
	 * Returns the minimum value.
	 * 
	 * @return the minimum value
	 */
	protected final N getMinValue() {
		return this.minimumValue;
	}
	
	@Override
	public final N parse(final String arg) throws ArgParserException {
		
		N number = null;
		
		try {
			
			number = this.parseNumber(arg);
			
		} catch (NumberFormatException e) {
			
			throw new ArgParserException(arg, e.getMessage(), e);
		}

		if (this.comparator.compare(this.minimumValue, this.maximumValue) == 0 
				&& this.comparator.compare(number, this.minimumValue) != 0) {
			
			String message = String.format(
					"argument must be %s", 
					this.minimumValue);
			
			throw new ArgParserException(arg, message);
		}
		
		if (this.comparator.compare(this.minimumValue, this.maximumValue) != 0 
				&& this.comparator.compare(number, this.minimumValue) < 0) {
			
			String message = String.format(
					"argument must be no less than %s", 
					this.minimumValue);
			
			throw new ArgParserException(arg, message);			
		}
		
		if (this.comparator.compare(this.minimumValue, this.maximumValue) != 0 
				&& this.comparator.compare(number, this.maximumValue) > 0) {
			
			String message = String.format(
					"argument must be no more than %s", 
					this.maximumValue);
			
			throw new ArgParserException(arg, message);			
		}
		
		return number;
	}
	
	/**
	 * Parses the provided string as a number. 
	 * <p>
	 * A {@code NumberFormatException} may be thrown if the provided string 
	 * cannot be parsed as a number or the number is out of bounds.
	 * </p>
	 * 
	 * @param string the provided string
	 * @return a number represented by the provided string
	 */
	protected abstract N parseNumber(String string);
}
