/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import java.io.Serializable;
import java.util.Comparator;


/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code Short}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class ShortArgParser extends NumberArgParser<Short> 
	implements ParameterlessArgParser<Short> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final ShortArgParser ARG_PARSER =
			new ShortArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * A {@code Comparator} that compares two {@code Short}s.
	 * 
	 * @author Climax Development (climax.development@gmail.com)
	 */	
	private static final class ShortComparator
		implements Comparator<Short>, Serializable {
		
		/**
		 * Lazy initialization holder class idiom for static field for
		 * {@code ComparatorHolder.COMPARATOR}.
		 */
		private static final class ComparatorHolder {
			
			/** The {@code Comparator}. */
			static final ShortComparator COMPARATOR =
				new ShortComparator();
			
			/**
			 * Constructs a {@code ComparatorHolder}. This constructor is 
			 * suppressed for noninstantiability outside of this class.
			 */			
			private ComparatorHolder() {
			}
		}
		
		/** The default serial version UID. */
		private static final long serialVersionUID = 1L;
		
		/**
		 * Returns a {@code ShortComparator}.
		 * 
		 * @return a {@code ShortComparator}
		 */		
		public static ShortComparator getInstance() {
			return ComparatorHolder.COMPARATOR;
		}

		/**
		 * Constructs a {@code ShortComparator}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private ShortComparator() {
		}

		@Override
		public int compare(final Short short1, final Short short2) {

			return short1.compareTo(short2);
		}
	}
	
	/** The default radix to be used in interpreting the argument. */
	private static final int DEFAULT_RADIX = 10;

	/**
	 * Returns a {@code ShortArgParser}.
	 * 
	 * @return a {@code ShortArgParser}
	 */	
	public static ShortArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}	
	
	/** The radix to be used in interpreting the argument. */
	private final int radix;
	
	/**
	 * Constructs a {@code ShortArgParser}.
	 */
	public ShortArgParser() {
		
		this(
				Short.valueOf(Short.MIN_VALUE), 
				Short.valueOf(Short.MAX_VALUE), 
				ShortArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code ShortArgParser} with the provided radix to be 
	 * used in interpreting the argument.
	 * 
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public ShortArgParser(final int r) {
		
		this(
				Short.valueOf(Short.MIN_VALUE), 
				Short.valueOf(Short.MAX_VALUE), 
				r);
	}
	
	/**
	 * Constructs a {@code ShortArgParser} with the provided minimum value
	 * and the maximum value.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 */
	public ShortArgParser(final Short minValue, final Short maxValue) {
		
		this(minValue, maxValue, ShortArgParser.DEFAULT_RADIX);
	}
	
	/**
	 * Constructs a {@code ShortArgParser} with the provided minimum value, 
	 * the provided maximum value, and the provided radix to be used in 
	 * interpreting the argument.
	 * 
	 * @param minValue the provided minimum value
	 * @param maxValue the provided maximum value
	 * @param r the provided radix to be used in interpreting the argument
	 */
	public ShortArgParser(
			final Short minValue, final Short maxValue, final int r) {
		
		super(ShortComparator.getInstance(), minValue, maxValue);
		
		this.radix = r;
	}

	@Override
	protected Short parseNumber(final String string) {
		
		Short s = null;
		
		try {
			
			s = Short.valueOf(string, this.radix);
			
		} catch (NumberFormatException e) {
			
			String message = String.format(
					"argument must be an integer" 
					+ " no less than %1$s and no more than %2$s",
					Integer.toString(this.getMinValue(), this.radix),
					Integer.toString(this.getMaxValue(), this.radix));
			
			throw new NumberFormatException(message);
		}
		
		return s;
	}

}
