/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argparser;

import org.climax.ArgParserException;

/**
 * An {@code ArgParser} implementation that parses an argument as a
 * {@code String}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class StringArgParser implements ParameterlessArgParser<String> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgParserHolder.ARG_PARSER}.
	 */
	private static final class ArgParserHolder {
		
		/** The {@code ArgParser}. */
		static final StringArgParser ARG_PARSER =
			new StringArgParser();
		
		/**
		 * Constructs an {@code ArgParserHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgParserHolder() {
		}
	}
	
	/**
	 * Returns a {@code StringArgParser}.
	 * 
	 * @return a {@code StringArgParser}
	 */	
	public static StringArgParser getInstance() {
		return ArgParserHolder.ARG_PARSER;
	}

	/** The boolean value to indicate if the string can not be empty. */
	private final boolean stringCannotBeEmpty;
	
	/**
	 * Constructs a {@code StringArgParser}.
	 */
	public StringArgParser() {
		this(false);
	}
	
	/**
	 * Constructs a {@code StringArgParser} with the provided boolean value
	 * to indicate if the string can not be empty.
	 * 
	 * @param cannotBeEmpty the provided boolean value to indicate if the 
	 * string can not be empty
	 */
	public StringArgParser(final boolean cannotBeEmpty) {
		this.stringCannotBeEmpty = cannotBeEmpty;
	}
	
	@Override
	public String parse(final String arg) throws ArgParserException {
		
		if (arg.isEmpty() && this.stringCannotBeEmpty) {
			
			String message = "argument cannot be empty";
			
			throw new ArgParserException(arg, message);
		}
		
		return arg;
	}
}
