/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argsvalidator;

import java.util.List;

import org.climax.ArgsValidator;
import org.climax.ArgsValidatorException;

/**
 * An {@code ArgsValidator} implementation that validates for arguments 
 * distinct from each other.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class DistinctArgsValidator extends AbstractArgsValidator 
	implements ParameterlessArgsValidator {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ArgsValidatorHolder.ARGS_VALIDATOR}.
	 */
	private static final class ArgsValidatorHolder {

		/** The {@code ArgsValidator}. */
		static final DistinctArgsValidator ARGS_VALIDATOR =
			new DistinctArgsValidator();
		
		/**
		 * Constructs an {@code ArgsValidatorHolder}. This constructor is
		 * suppressed for noninstantiability outside of this class.
		 */
		private ArgsValidatorHolder() {
		}
	}
	
	/**
	 * Returns a {@code DistinctArgsValidator}.
	 * 
	 * @return a {@code DistinctArgsValidator}
	 */	
	public static DistinctArgsValidator getInstance() {
		return ArgsValidatorHolder.ARGS_VALIDATOR;
	}
	
	/**
	 * Constructs a {@code DistinctArgsValidator}.
	 */
	public DistinctArgsValidator() {
		this(null);
	}
	
	/**
	 * Constructs a {@code DistinctArgsValidator} with the provided 
	 * {@code ArgsValidator}.
	 * 
	 * @param validator the provided {@code ArgsValidator}
	 */
	public DistinctArgsValidator(final ArgsValidator validator) {
		super(validator);
	}

	@Override
	protected void validateArguments(final List<String> argList)
			throws ArgsValidatorException {
		
		for (int i = 0; i < argList.size(); i++) {
			
			String arg = argList.get(i);
			
			boolean argsNotDistinct = false;
			
			for (int j = i + 1; j < argList.size(); j++) {
				
				String nextArg = argList.get(j);
				
				if (arg.equals(nextArg)) {
					
					argsNotDistinct = true;
				}				
			}
			
			if (argsNotDistinct) {
				
				throw new ArgsValidatorException(
						argList,
						"arguments must be distinct from each other");
			}			
		}
	}

}
