/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argsvalidator;

import java.util.List;

import org.climax.ArgsValidator;
import org.climax.ArgsValidatorException;

/**
 * An {@code ArgsValidator} implementation that validates for the number of
 * arguments no more than a certain number.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class MaxArgCountValidator extends AbstractArgsValidator {

	/** The maximum number of arguments. */
	private final int maximumArgumentCount;
	
	/**
	 * Constructs a {@code MaxArgCountValidator} with the provided maximum 
	 * number of arguments.
	 * 
	 * @param maxArgCount the provided maximum number of arguments
	 */
	public MaxArgCountValidator(final int maxArgCount) {
		
		this(maxArgCount, null);
	}
	
	/**
	 * Constructs a {@code MaxArgCountValidator} with the provided maximum
	 * number of arguments and the provided {@code ArgsValidator}.
	 * 
	 * @param maxArgCount the provided maximum number of arguments
	 * @param validator the provided {@code ArgsValidator}
	 */
	public MaxArgCountValidator(
			final int maxArgCount, final ArgsValidator validator) {
		
		super(validator);
		
		if (maxArgCount < 0) {
			throw new IllegalArgumentException(
					"maxArgCount must be no less than 0");
		}
		
		this.maximumArgumentCount = maxArgCount;		
	}

	@Override
	protected void validateArguments(final List<String> argList)
			throws ArgsValidatorException {
		
		if (argList.size() > this.maximumArgumentCount) {
			
			String message = String.format(
					"no more than %s specified argument(s) required", 
					this.maximumArgumentCount);
			
			throw new ArgsValidatorException(argList, message);
		}
	}

}
