/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.argsvalidator;

import java.util.List;

import org.climax.ArgsValidator;
import org.climax.ArgsValidatorException;

/**
 * An {@code ArgsValidator} implementation that validates for the number of
 * arguments no less than a certain number.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public final class MinArgCountValidator extends AbstractArgsValidator {

	/** The minimum number of arguments. */
	private final int minimumArgumentCount;
	
	/**
	 * Constructs a {@code MinArgCountValidator} with the provided minimum 
	 * number of arguments.
	 * 
	 * @param minArgCount the provided minimum number of arguments
	 */	
	public MinArgCountValidator(final int minArgCount) {
		
		this(minArgCount, null);
	}
	
	/**
	 * Constructs a {@code MinArgCountValidator} with the provided minimum
	 * number of arguments and the provided {@code ArgsValidator}.
	 * 
	 * @param minArgCount the provided minimum number of arguments
	 * @param validator the provided {@code ArgsValidator}
	 */	
	public MinArgCountValidator(
			final int minArgCount, final ArgsValidator validator) {
		
		super(validator);
		
		if (minArgCount < 0) {
			throw new IllegalArgumentException(
					"minArgCount must be no less than 0");
		}
		
		this.minimumArgumentCount = minArgCount;
	}

	@Override
	protected void validateArguments(final List<String> argList)
			throws ArgsValidatorException {
		
		if (argList.size() < this.minimumArgumentCount) {
			
			String message = String.format(
					"no less than %s specified argument(s) required", 
					this.minimumArgumentCount);
			
			throw new ArgsValidatorException(argList, message);
		}
	}

}
