/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

/**
 * Contains all of the interfaces and classes for parsing command line 
 * arguments.
 * <dl>
 * <dt><strong>Table of Contents</strong></dt>
 * <dt>1. <a href="#1">Architecture</a></dt>
 * <dd>
 * <dl>
 * <dt>1.1. <a href="#1.1">Representation Objects</a></dt>
 * <dt>1.2. <a href="#1.2">Documentation Objects</a></dt>
 * <dt>1.3. <a href="#1.3">Complete Objects</a></dt>
 * </dl>
 * </dd>
 * <dt>2. <a href="#2">Client Entry Points</a></dt>
 * <dt>3. <a href="#3">Expandable Properties</a></dt>
 * <dd>
 * <dl>
 * <dt>3.1. <a href="#3.1">Interpreting Literal Dollar Signs and French 
 * Braces</a></dt>
 * <dt>3.2. <a href="#3.2">Nested Expandable Properties</a></dt>
 * <dt>3.3. <a href="#3.3">Formatting Expansion of Expandable Properties with 
 * ${format:} or ${lformat:}</a></dt>
 * <dd>
 * <dl>
 * <dt>3.3.1. <a href="#3.3.1">Interpreting Literal Semicolons and Commas</a>
 * </dt>
 * </dl>
 * </dd>
 * </dl>
 * </dd>
 * </dl>
 * <h3>1. <a name="1">Architecture</a></h3>
 * <p>
 * This package is organized into three types of objects: representation 
 * objects, documentation objects, and complete objects.
 * </p>
 * <h4>1.1. <a name="1.1">Representation Objects</a></h4>
 * <p>
 * Representation objects represent elements of the command line interface.
 * </p>
 * <p>
 * The following diagram displays the following representation objects and 
 * their associations.
 * <table>
 * <tr align="center">
 * <td colspan="2">{@link org.climax.CLI}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 1</td><td>&darr; 1</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.Options}</td><td>{@link org.climax.Args}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 0...*</td><td>&darr; 0...*</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionGroup}</td><td>{@link org.climax.ArgGroup}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 0...*</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.Option}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 1</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionType}</td>
 * </tr>
 * </table> 
 * </p>
 * <h4>1.2. <a name="1.2">Documentation Objects</a></h4>
 * <p>
 * Documentation objects represent the documentation of the representation 
 * objects.
 * </p> 
 * <p>
 * The following diagram displays the following documentation objects and their 
 * associations.
 * <table>
 * <tr align="center">
 * <td colspan="2">{@link org.climax.CLI}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 1</td><td>&darr; 1</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionsDoc}</td><td>{@link org.climax.ArgsDoc}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 0...*</td><td>&darr; 0...*</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionGroupDoc}</td><td>{@link org.climax.ArgGroupDoc}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 0...*</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionDoc}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 1</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionTypeDoc}</td>
 * </tr>
 * </table> 
 * </p>
 * <h4>1.3. <a name="1.3">Complete Objects</a></h4>
 * <p>
 * Complete objects are both representation objects and documentation objects.
 * </p>
 * <p>
 * The following diagram displays the following complete objects and their 
 * associations.
 * <table>
 * <tr align="center">
 * <td colspan="2">{@link org.climax.CLI}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 1</td><td>&darr; 1</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionsComplete}</td><td>{@link org.climax.ArgsComplete}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 0...*</td><td>&darr; 0...*</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionGroupComplete}</td><td>{@link org.climax.ArgGroupComplete}</td>
 * </tr>
 * <tr align="center">
 * <td>&darr; 0...*</td>
 * </tr>
 * <tr align="center">
 * <td>{@link org.climax.OptionComplete}</td>
 * </tr>
 * </table> 
 * </p> 
 * <h3>2. <a name="2">Client Entry Points</a></h3>
 * <p>
 * {@link org.climax.CLI} is the primary client entry-point to this package.
 * </p>
 * <p>
 * {@link org.climax.Args}, {@link org.climax.ArgsComplete}, 
 * {@link org.climax.Options} and {@link org.climax.OptionsComplete} are the 
 * secondary client entry-points to this package.
 * </p>
 * <p>
 * The aforementioned client entry-points can be created by using the 
 * {@link org.climax.AbstractFactory}. 
 * </p>
 * <h3>3. <a name="3">Expandable Properties</a></h3>
 * <p>
 * An expandable property within a provided format string is expressed under 
 * the following syntax similar to Apache Ant properties: <code>${key}</code> 
 * where <i>key</i> is the property's key. If the property of that key exists, 
 * the expandable property is expanded to the property's value. If the 
 * property of that key does not exist, the expandable property construct is 
 * not expanded (remains the same). If the value of a property contains any 
 * expandable property constructs, those constructs will not be expanded 
 * (remain the same).
 * </p>
 * <p>
 * Expandable properties are each defined as either of the following: system 
 * properties or additional properties defined for a provided format string.
 * </p>
 * <p>
 * All system properties can be expressed as expandable properties in any 
 * provided format string. For example, the system property 
 * <code>os.name</code> expressed as the expandable property 
 * <code>${os.name}</code> expands to the name of the operating system. For a 
 * list of system properties, see the Javadoc of 
 * {@link java.lang.System#getProperties()}.
 * </p>
 * <p>
 * Additional properties defined for a provided format string can also be 
 * expressed as expandable properties in that provided format string. For a 
 * list of acceptable additional properties defined for a provided format 
 * string, see the Javadoc of any method that requires a provided format 
 * string.
 * </p>  
 * <h4>3.1. <a name="3.1">Interpreting Literal Dollar Signs and French 
 * Braces</a></h4>
 * <p>
 * A literal dollar sign ("<code>$</code>") can be interpreted by preceding it 
 * with another dollar sign. Multiple adjacent literal dollar signs will need 
 * to be interpreted by preceding each dollar sign with another dollar sign. 
 * If a dollar sign is apart from other dollar signs and French braces 
 * ("<code>{</code>", "<code>}</code>"), it is interpreted as a literal dollar 
 * sign.
 * </p>
 * <p>
 * A literal left French brace ("<code>{</code>") is interpreted if it is not 
 * preceded by a dollar sign or if it is preceded by a dollar sign to be 
 * interpreted as literal ("<code>$${</code>").
 * </p>
 * <p>
 * A literal right French brace ("<code>}</code>") outside all or within any  
 * expandable property constructs can be interpreted by preceding it with 
 * a dollar sign. If a right French brace is outside all expandable property 
 * constructs and is not preceded by a dollar sign, it is interpreted as a 
 * literal right French brace.
 * </p>
 * <h4>3.2. <a name="3.2">Nested Expandable Properties</a></h4>
 * <p>
 * Expandable properties can also have nested expandable properties. Consider 
 * the following expandable property as an example:
 * <blockquote>
 * <code>${key1${key2}${key3${key1}}}</code>
 * </blockquote>
 * Let's assume that the associated values of <i>key1</i>, <i>key2</i>, 
 * <i>key31</i>, and <i>key1231</i> are 1, 2, 31, and 1231 respectively.
 * </p>
 * <p>
 * The following describes the process of how the expandable property and its 
 * nested expandable properties are expanded:
 * </p>
 * <ol>
 * <li>The expandable property is encountered 
 * ( <code>${key1${key2}${key3${key1}}}</code> ) but is not yet expanded.
 * <blockquote>
 * <code>${key1${key2}${key3${key1}}}</code>
 * </blockquote></li>
 * <li>The first nested expandable property is encountered 
 * ( <code>${key2}</code> ) and is expanded to {@code 2}. The expandable 
 * property becomes:
 * <blockquote>
 * <code>${key12${key3${key1}}}</code>
 * </blockquote></li>
 * <li>Another nested expandable property is encountered
 * ( <code>${key3${key1}}</code> ) but is not yet expanded.
 * <blockquote>
 * <code>${key12${key3${key1}}}</code>
 * </blockquote></li>
 * <li>Within the nested expandable property ( <code>${key3${key1}}</code> ), 
 * another nested expandable property is encountered ( <code>${key1}</code> ) 
 * and is expanded to {@code 1}. The expandable property becomes:
 * <blockquote>
 * <code>${key12${key31}}</code>
 * </blockquote></li> 
 * <li>The nested expandable property ( <code>${key31}</code> ) is 
 * expanded to {@code 31}. The expandable property becomes:
 * <blockquote>
 * <code>${key1231}</code>
 * </blockquote></li>
 * <li>The expandable property ( <code>${key1231}</code> ) is expanded to 
 * {@code 1231}. The expandable property becomes:
 * <blockquote>
 * {@code 1231}
 * </blockquote></li> 
 * </ol>
 * <h4>3.3. <a name="3.3">Formatting Expansion of Expandable Properties with 
 * ${format:} or ${lformat:}</a></h4>
 * <p>
 * The expansion of expandable properties can be formatted using the 
 * <code>${format:}</code> operation or the <code>${lformat:}</code> operation.
 * </p>
 * <p>
 * The provided operations must have either of the following syntaxes:
 * <blockquote>
 * <code>${format:format[;argument1[;argument2[...]]]}</code>
 * </blockquote>
 * <blockquote>
 * <code>${lformat:locale;format[;argument1[;argument2[...]]]}</code>
 * </blockquote>
 * The required <i>locale</i> argument for {@code lformat} is the 
 * {@code String} representation of the specified {@code Locale}.
 * </p>
 * <p>
 * The required <i>format</i> argument is the printf-sytle format whose 
 * syntax is defined by {@link java.util.Formatter}.
 * </p>
 * <p>
 * The optional <i>argument</i> or <i>arguments</i> are the arguments 
 * referenced by the format specifiers within the <i>format</i> argument. 
 * The <i>arguments</i> must each have one of the following syntaxes:
 * </p>
 * <ul>
 * <li>{@code c=conversion,p=propertykey}</li>
 * <li>{@code c=conversion,p=propertykey,v=value}</li>
 * <li>{@code c=conversion,v=value}</li>
 * <li>{@code p=propertykey}</li>
 * <li>{@code p=propertykey,v=value}</li>
 * <li>{@code v=value}</li>
 * </ul>
 * <p>
 * The <i>conversion</i> sub-argument specifies the type of {@code Object}
 * to which the value retrieved from the <i>propertykey</i> sub-argument or 
 * the value from the <i>value</i> sub-argument is converted. The 
 * <i>conversion</i> sub-argument can be one of the following:
 * </p>
 * <ul>
 * <li>{@code boolean}</li>
 * <li>{@code byte}</li>
 * <li>{@code char}</li>
 * <li>{@code date}
 * <p>
 * The value retrieved from the <i>propertykey</i> sub-argument or the value 
 * from the <i>value</i> sub-argument must be in one of the following formats 
 * (See {@link java.text.SimpleDateFormat} for details):
 * </p>
 * <ul>
 * <li>yyyyMMdd</li>
 * <li>yyyyMMddHHmmss</li>
 * <li>yyyyMMddHHmmssSSSZ</li>
 * </ul>
 * <p>
 * If the value retrieved from the <i>propertykey</i> sub-argument or the 
 * value from the <i>value</i> sub-argument is "{@code now}", the value is 
 * converted to the current Date.
 * </p>
 * </li>  
 * <li>{@code double}</li>
 * <li>{@code float}</li>
 * <li>{@code int}</li>
 * <li>{@code long}</li>
 * <li>{@code short}</li>
 * <li>{@code string}
 * <p>
 * If no <i>conversion</i> sub-argument is provided, this is the implicit 
 * default <i>conversion</i> sub-argument.
 * </p>
 * </li> 
 * </ul>
 * <p>
 * The <i>propertykey</i> sub-argument is the key of a property that retrieves 
 * the value of that property.
 * </p>
 * <p>
 * The <i>value</i> sub-argument is the provided value in case the 
 * <i>propertykey</i> sub-argument is not present or in case the key of the 
 * property specified by the <i>propertykey</i> sub-argument does not exist.
 * </p> 
 * <h5>3.3.1. <a name="3.3.1">Interpreting Literal Semicolons and Commas</a>
 * </h5>
 * <p>
 * For the arguments <i>locale</i>, <i>format</i>, and <i>argument</i>, 
 * a literal semicolon ("<code>;</code>") can be interpreted by preceding it 
 * with a backslash ("<code>\\</code>"). Multiple adjacent literal backslashes 
 * will need to be interpreted by preceding each backslash with another 
 * backslash. If a backslash is apart from other backslashes and semicolons, 
 * it is interpreted as a literal backslash.
 * </p>
 * <p>
 * For the sub-arguments of an <i>argument</i>, a literal comma 
 * ("<code>,</code>") can be interpreted by preceding the literal comma with a 
 * backslash ("<code>\\</code>"). Because the literal semicolons and literal 
 * backslashes are interpreted for the <i>argument</i>, multiple adjacent 
 * literal backslashes will need to be interpreted by preceding each 
 * backslash with three backslashes. If a backslash is apart from other 
 * backslashes, semicolons, and commas, it is interpreted as a literal 
 * backslash.
 * </p>  
 */
package org.climax;