/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import org.climax.ArgAcceptor;
import org.climax.argacceptor.AfterFirstArgIndexAcceptor;
import org.climax.argacceptor.BeforeLastArgIndexAcceptor;
import org.climax.argacceptor.FirstArgIndexAcceptor;
import org.climax.argacceptor.LastArgIndexAcceptor;
import org.climax.argacceptor.NoArgAcceptor;
import org.climax.argacceptor.SingleArgAcceptor;
import org.climax.argacceptor.UnlimitedArgAcceptor;

/**
 * An {@code ArgAcceptor} class type. The constants of this 
 * enumerated type provides a classification of {@code ArgAcceptor} 
 * classes.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
enum ArgAcceptorClassType {

	/** {@code AfterFirstArgIndexAcceptor} class. */
	AFTER_FIRST_ARG_INDEX_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return AfterFirstArgIndexAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return AfterFirstArgIndexAcceptor.getInstance();
		}
	},
	
	/** {@code BeforeLastArgIndexAcceptor} class. */
	BEFORE_LAST_ARG_INDEX_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return BeforeLastArgIndexAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return BeforeLastArgIndexAcceptor.getInstance();
		}
	},
	
	/** {@code DummyArgAcceptor} class. */
	DUMMY_ARG_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return DummyArgAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return null;
		}
	},
	
	/** {@code FirstArgIndexAcceptor} class. */
	FIRST_ARG_INDEX_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return FirstArgIndexAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return FirstArgIndexAcceptor.getInstance();
		}
	},
	
	/** {@code LastArgIndexAcceptor} class. */
	LAST_ARG_INDEX_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return LastArgIndexAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return LastArgIndexAcceptor.getInstance();
		}
	},
	
	/** {@code NoArgAcceptor} class. */
	NO_ARG_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return NoArgAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return NoArgAcceptor.getInstance();
		}
	},
	
	/** {@code SingleArgAcceptor} class. */
	SINGLE_ARG_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return SingleArgAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return SingleArgAcceptor.newInstance();
		}
	},
	
	/** {@code UnlimitedArgAcceptor} class. */
	UNLIMITED_ARG_ACCEPTOR {
		@Override
		boolean describes(final Class<? extends ArgAcceptor> argAcceptorClass) {
			return UnlimitedArgAcceptor.class == argAcceptorClass;
		}

		@Override
		ArgAcceptor getArgAcceptor() {
			return UnlimitedArgAcceptor.getInstance();
		}
	};
	
	/**
	 * Returns the enum constant that describes the type of 
	 * {@code ArgAcceptor} class from the provided {@code ArgAcceptor} 
	 * class.
	 *   
	 * @param argAcceptorClass the provided {@code ArgAcceptor} class
	 * 
	 * @return an enum constant that describes the type of 
	 * {@code ArgAcceptor} class from the provided {@code ArgAcceptor} 
	 * class or {@code null} if there is no enum constant that 
	 * describes the type of {@code ArgAcceptor} class from the provided 
	 * {@code ArgAcceptor} class
	 */
	static ArgAcceptorClassType bestDescribes(
			final Class<? extends ArgAcceptor> argAcceptorClass) {
		
		for (ArgAcceptorClassType argAcceptorClassType 
				: ArgAcceptorClassType.values()) {
			
			if (argAcceptorClassType.describes(argAcceptorClass)) {
				
				return argAcceptorClassType;
			}
		}
		
		return null;
	}
	
	/**
	 * Indicates if the provided {@code ArgAcceptor} class is described by
	 * this {@code ArgAcceptorClassType}.
	 * 
	 * @param argAcceptorClass the provided {@code ArgAcceptor} class
	 * 
	 * @return {@code true} if the provided {@code ArgAcceptor} class is 
	 * described by this {@code ArgAcceptorClassType}
	 */
	abstract boolean describes(
			final Class<? extends ArgAcceptor> argAcceptorClass);
	
	/**
	 * Returns an {@code ArgAcceptor} based on the provided class 
	 * and this {@code ArgAcceptorClassType}.
	 * 
	 * @return an {@code ArgAcceptor}
	 */
	abstract ArgAcceptor getArgAcceptor();
}
