/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import org.climax.ArgGroupComplete;


/**
 * A {@code Method} whose return type is {@code ArgGroupComplete}.
 * <p>
 * The underlying method represented by this 
 * {@code ArgGroupCompleteReturnTypeMethod} returns an 
 * {@code ArgGroupComplete}. The returned {@code ArgGroupComplete} is defined 
 * by the following:
 * </p>
 * <ul>
 * <li>The type argument of the {@code ArgGroupComplete} return type</li>
 * <li>The method's name</li>
 * <li>The method's optional {@code ArgGroupCompleteSpec} annotation</li>
 * <li>The provided {@code ParameterlessArgsValidator} class for the 
 * {@code ArgGroupCompleteSpec} annotation</li>
 * <li>The provided {@code ParameterlessArgAcceptor} class for the 
 * {@code ArgGroupCompleteSpec} annotation</li>
 * </ul>
 * <p>
 * The type argument of the {@code ArgGroupComplete} return type determines the 
 * appropriate default {@code ArgParser} for the {@code ArgGroupComplete}. The 
 * following lists the valid types for determining a default {@code ArgParser}:
 * </p>
 * <ul>
 * <li>{@code java.io.File}</li>
 * <li>{@code java.io.FileInputStream}</li>
 * <li>{@code java.io.FileOutputStream}</li>
 * <li>{@code java.io.FileReader}</li>
 * <li>{@code java.io.FileWriter}</li>
 * <li>{@code java.lang.Boolean}</li>
 * <li>{@code java.lang.Byte}</li>
 * <li>{@code java.lang.Character}</li>
 * <li>{@code java.lang.Class}</li>
 * <li>{@code java.lang.Double}</li>
 * <li>{@code java.lang.Enum} types</li>
 * <li>{@code java.lang.Float}</li>
 * <li>{@code java.lang.Integer}</li>
 * <li>{@code java.lang.Long}</li>
 * <li>{@code java.lang.Short}</li>
 * <li>{@code java.lang.String}</li>
 * <li>{@code java.lang.Void}</li>
 * <li>{@code java.math.BigDecimal}</li>
 * <li>{@code java.math.BigInteger}</li>
 * <li>{@code java.util.Date}</li>
 * </ul>
 * <p>
 * If the type argument of the {@code ArgGroupComplete} return type is not any 
 * of the aforementioned types and there is no provided custom 
 * {@code ParameterlessArgParser} class for the {@code ArgGroupCompleteSpec} 
 * annotation, an {@code InvalidMethodException} is thrown.  
 * </p>
 * <p>
 * The method's name determines the appropriate default argument type for the 
 * {@code ArgGroupComplete}. If the name of the method begins with 
 * "{@code get}" or "{@code is}", the rest of the name of the method is used. 
 * If the name of the method does not begin with "{@code get}" or "{@code is}",
 * the entire name of the method is used. 
 * </p>
 * <p>
 * The method's optional {@code ArgGroupCompleteSpec} annotation is used to 
 * provide further specification of the {@code ArgGroupComplete}. It can also 
 * be used to override any of the defaults determined by the type argument of 
 * the {@code ArgGroupComplete} return type, the method's name, and the 
 * provided {@code ParameterlessArgsValidator} class.
 * </p>
 * <p>
 * The provided {@code ParameterlessArgsValidator} class for the 
 * {@code ArgGroupCompleteSpec} annotation not only provides the 
 * {@code ParameterlessArgsValidator} for the {@code ArgGroupComplete}, but it 
 * also determines the default format string for the syntax for the 
 * {@code ArgGroupComplete}. The following lists the 
 * {@code ParameterlessArgsValidator} classes that determines the default 
 * format string for the syntax for the {@code ArgGroupComplete}: 
 * </p>
 * <ul>
 * <li>{@code org.climax.argsvalidator.DistinctArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.NoArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.NoneOrSingleArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.PassiveArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.SingleArgValidator}</li>
 * <li>{@code org.climax.argsvalidator.SingleOrMultipleArgsValidator}</li>
 * </ul>
 * <p>
 * If a {@code ParameterlessArgsValidator} class has not been provided, a 
 * {@code SingleArgValidator} class is provided.
 * </p>
 * <p>
 * If the provided {@code ParameterlessArgsValidator} class is not any of the 
 * aforementioned classes, the default format string for the syntax for the 
 * {@code ArgGroupComplete} is a single argument named after the 
 * {@code ArgGroupComplete}'s argument type.  
 * </p>
 * <p>
 * The provided {@code ParameterlessArgAcceptor} class for the 
 * {@code ArgGroupCompleteSpec} annotation provides the 
 * {@code ParameterlessArgAcceptor} for the {@code ArgGroupComplete}. If a 
 * {@code ParameterlessArgAcceptor} class has not been provided, a 
 * {@code SingleArgAcceptor} class is provided.
 * </p>
 * 
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @see ArgGroupCompleteSpec
 */
public final class ArgGroupCompleteReturnTypeMethod {
	
	/** The {@code Method}. */
	private final Method method;
	
	/** The target {@code Type}. */
	private final Type targetType;
	
	/** The {@code ArgGroupCompleteSpec}. */
	private final ArgGroupCompleteSpec argGroupCompleteSpec;
	
	/**
	 * Constructs an {@code ArgGroupCompleteReturnTypeMethod} with the provided 
	 * {@code Method}.
	 * 
	 * @param meth the provided {@code Method}
	 */
	ArgGroupCompleteReturnTypeMethod(final Method meth) {
		
		if (meth == null) {
			
			throw new NullPointerException("method must not be null");
		}
		
		Type methodReturnType = meth.getGenericReturnType();
		
		Type type = null;
		
		if (methodReturnType instanceof Class<?>) {
			
			Class<?> cls = (Class<?>) methodReturnType;
			
			if (cls == ArgGroupComplete.class) {
				
				type = Object.class;
			}
			
		} else if (methodReturnType instanceof ParameterizedType) {
		
			ParameterizedType parameterizedType = 
				(ParameterizedType) methodReturnType;
			
			Type rawType = parameterizedType.getRawType();
			
			if (rawType instanceof Class<?>) {
			
				Class<?> cls = (Class<?>) rawType;
				
				if (cls == ArgGroupComplete.class) {
					
					type = 
						parameterizedType.getActualTypeArguments()[0];
				}
			}
			
		}
		
		if (type == null) {
			
			String message = String.format(
					"return type must be %s", 
					ArgGroupComplete.class.getName());
			
			throw new InvalidMethodException(meth, message);			
		}
		
		ArgGroupCompleteSpec arGroupCompleteSpec = null;
		
		if (meth.isAnnotationPresent(ArgGroupCompleteSpec.class)) {
			
			arGroupCompleteSpec = meth.getAnnotation(
					ArgGroupCompleteSpec.class);
		}
						
		this.method = meth;
		this.targetType = type;
		this.argGroupCompleteSpec = arGroupCompleteSpec;
	}
	
	/**
	 * Returns the {@code Method} view of this 
	 * {@code ArgGroupCompleteReturnTypeMethod}.
	 * 
	 * @return the {@code Method} view of this 
	 * {@code ArgGroupCompleteReturnTypeMethod}
	 */	
	public Method asMethod() {
		return this.method;
	}
	
	/**
	 * Constructs a new {@code ArgGroupComplete} based off of this 
	 * {@code ArgGroupCompleteReturnTypeMethod}.
	 * 
	 * @return a new {@code ArgGroupComplete} based off of this 
	 * {@code ArgGroupCompleteReturnTypeMethod}
	 */
	public ArgGroupComplete<Object> newArgGroupComplete() {
		
		TypesBasedArgGroupCompleteBuilder typesBasedArgGroupCompleteBuilder =
				new TypesBasedArgGroupCompleteBuilder(
						Methods.getAccessorSubjectName(this.method),
						this.targetType,
						TypesType.TYPE);
		
		if (this.argGroupCompleteSpec != null) {
			
			typesBasedArgGroupCompleteBuilder.argGroupCompleteSpec(
					this.argGroupCompleteSpec);
		}
		
		return typesBasedArgGroupCompleteBuilder.build();
	}

	@Override
	public String toString() {
		return String.format("ArgGroupCompleteReturnTypeMethod [" 
				+ "method=%s]",
				this.method);
	}
}
