/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import org.climax.ArgParser;
import org.climax.argparser.BigDecimalArgParser;
import org.climax.argparser.BigIntegerArgParser;
import org.climax.argparser.BooleanArgParser;
import org.climax.argparser.ByteArgParser;
import org.climax.argparser.CharacterArgParser;
import org.climax.argparser.ClassArgParser;
import org.climax.argparser.DateArgParser;
import org.climax.argparser.DoubleArgParser;
import org.climax.argparser.EnumArgParser;
import org.climax.argparser.FileArgParser;
import org.climax.argparser.FileInputStreamArgParser;
import org.climax.argparser.FileOutputStreamArgParser;
import org.climax.argparser.FileReaderArgParser;
import org.climax.argparser.FileWriterArgParser;
import org.climax.argparser.FloatArgParser;
import org.climax.argparser.IntegerArgParser;
import org.climax.argparser.LongArgParser;
import org.climax.argparser.ParameterlessArgParser;
import org.climax.argparser.ShortArgParser;
import org.climax.argparser.StringArgParser;
import org.climax.argparser.VoidArgParser;

/**
 * This class provides utility methods regarding {@code ArgParser}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ArgParsers {

	/**
	 * Returns the default {@code ArgParser} based on the provided 
	 * {@code Class}.
	 * 
	 * @param cls the provided {@code Class}
	 * @return the default {@code ArgParser}
	 */
	static ArgParser<? extends Object> getDefault(final Class<?> cls) {
		
		ArgParser<? extends Object> argParser = null;
		
		ClassType classType = ClassType.bestDescribes(cls);
		
		if (classType == null) {
			String message = String.format(
					"Unable to determine default %1$s for %2$s",
					ArgParser.class.getName(), 
					cls);

			throw new IllegalArgumentException(message);
		}
		
		switch (classType) {
		case BIG_DECIMAL:
			argParser = BigDecimalArgParser.getInstance();
			break;
		case BIG_INTEGER:
			argParser = BigIntegerArgParser.getInstance();
			break;
		case BOOLEAN:
		case BOOLEAN_PRIMITIVE:
			argParser = BooleanArgParser.getInstance();
			break;
		case BYTE:
		case BYTE_PRIMITIVE:
			argParser = ByteArgParser.getInstance();
			break;
		case CHARACTER:
		case CHARACTER_PRIMITIVE:
			argParser = CharacterArgParser.getInstance();
			break;
		case CLASS:
			argParser = ClassArgParser.getInstance();
			break;
		case DATE:
			argParser = DateArgParser.getInstance();
			break;
		case DOUBLE:
		case DOUBLE_PRIMITIVE:
			argParser = DoubleArgParser.getInstance();
			break;
		case ENUM:
			@SuppressWarnings({ "rawtypes", "unchecked" })
			Class<Enum> enumCls = (Class<Enum>) cls;
			@SuppressWarnings({ "rawtypes", "unchecked" })
			ArgParser<Enum> parser = EnumArgParser.getInstance(enumCls); 
			argParser = parser;
			break;
		case FILE:
			argParser = FileArgParser.getInstance();
			break;
		case FILE_INPUT_STREAM:
			argParser = FileInputStreamArgParser.getInstance();
			break;
		case FILE_OUTPUT_STREAM:
			argParser = FileOutputStreamArgParser.getInstance();
			break;
		case FILE_READER:
			argParser = FileReaderArgParser.getInstance();
			break;
		case FILE_WRITER:
			argParser = FileWriterArgParser.getInstance();
			break;
		case FLOAT:
		case FLOAT_PRIMITIVE:
			argParser = FloatArgParser.getInstance();
			break;
		case INTEGER:
		case INTEGER_PRIMITIVE:
			argParser = IntegerArgParser.getInstance();
			break;
		case LONG:
		case LONG_PRIMITIVE:
			argParser = LongArgParser.getInstance();
			break;
		case SHORT:
		case SHORT_PRIMITIVE:
			argParser = ShortArgParser.getInstance();
			break;
		case STRING:
			argParser = StringArgParser.getInstance();
			break;
		case VOID:
		case VOID_PRIMITIVE:
			argParser = VoidArgParser.getInstance();
			break;
		default:
			String message = String.format(
					"Unable to determine default %1$s for %2$s",
					ArgParser.class.getName(), 
					cls);

			throw new IllegalArgumentException(message);
		}
		
		return argParser;
	}
	
	/**
	 * Returns the default {@code ArgParser} based on the provided {@code Type}.
	 * 
	 * @param type the provided {@code Type}
	 * @return the default {@code ArgParser}
	 */
	static ArgParser<? extends Object> getDefault(final Type type) {
		
		ArgParser<? extends Object> argParser = null;
		
		TypeType typeType = TypeType.bestDescribes(type);
		
		if (typeType == null) {
			String message = String.format(
					"Unable to determine default %1$s for %2$s",
					ArgParser.class.getName(), 
					type);

			throw new IllegalArgumentException(message);
		}
		
		switch (typeType) {
		case CLASS:
			Class<?> cls = (Class<?>) type;
			argParser = ArgParsers.getDefault(cls);
			break;
		case PARAMETERIZED_TYPE:
			ParameterizedType parameterizedType = (ParameterizedType) type;
			Type rawType = parameterizedType.getRawType();
			argParser = ArgParsers.getDefault(rawType);
			break;
		default:
			String message = String.format(
					"Unable to determine default %1$s for %2$s",
					ArgParser.class.getName(), 
					type);

			throw new IllegalArgumentException(message);
		}
		
		return argParser;
	}
	
	/**
	 * Returns an {@code ArgParser} based on the provided 
	 * {@code ParameterlessArgParser} class.
	 * 
	 * @param argParserClass the provided {@code ParameterlessArgParser} 
	 * class
	 * 
	 * @return an {@code ArgParser}
	 */
	static ArgParser<? extends Object> getInstance(
			final Class<? extends ParameterlessArgParser<?>> argParserClass) {
		
		ArgParserClassType argParserClassType = 
				ArgParserClassType.bestDescribes(argParserClass);
		
		if (argParserClassType == null) {
			return Objects.getInstance(argParserClass);
		}
		
		return argParserClassType.getArgParser();
	}
	
	/**
	 * Constructs an instance of {@code ArgParsers}. This 
	 * constructor is suppressed for noninstantiability outside of 
	 * this class.
	 */
	private ArgParsers() {
	}
}
