/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.ref.WeakReference;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import org.climax.AbstractFactory;
import org.climax.ArgGroupComplete;
import org.climax.ArgsComplete;

/**
 * An interface {@code Class} of a specified type that is or extends 
 * {@code ArgsComplete}.
 * <p>
 * This {@code ArgsCompleteInterface} returns an instance of the specified 
 * type. The instance of the specified type is defined by the following:
 * </p>
 * <ul>
 * <li>The interface class's methods whose return type is 
 * {@code ArgGroupComplete}</li>
 * <li>The interface class's optional {@code ArgsCompleteSpec} annotation</li>
 * </ul>
 * <p>
 * The interface class's methods whose return type is {@code ArgGroupComplete} 
 * each define an {@code ArgGroupCompleteReturnTypeMethod}. If one of the 
 * methods' return type is not {@code ArgGroupComplete}, an 
 * {@code InvalidMethodException} is thrown.
 * </p>
 * <p>
 * The interface class's optional {@code ArgsCompleteSpec} annotation is used 
 * to provide further specification of the {@code ArgsComplete}.
 * </p>       
 *  
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <A> the specified type that is or extends {@code ArgsComplete}
 * 
 * @see ArgGroupCompleteReturnTypeMethod
 * 
 * @see ArgsCompleteSpec
 */
public final class ArgsCompleteInterface<A extends ArgsComplete> {
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ClassMapHolder.CLASS_MAP}.
	 */
	private static final class ClassMapHolder {
		
		/**
		 * A {@code Map} that maps existing {@code ArgsComplete} classes to 
		 * existing {@code ArgsCompleteInterface}s.
		 */
		static final Map<Class<? extends ArgsComplete>, 
			WeakReference<ArgsCompleteInterface<? extends ArgsComplete>>> 
		CLASS_MAP = new WeakHashMap<Class<? extends ArgsComplete>, 
			WeakReference<ArgsCompleteInterface<? extends ArgsComplete>>>();
		
		/**
		 * Constructs a {@code ClassMapHolder}. This constructor is suppressed 
		 * for noninstantiability outside of this class.
		 */
		private ClassMapHolder() {
		}
	}

	
	/**
	 * Returns an {@code ArgsCompleteInterface} of the specified type based on 
	 * the provided interface class.
	 * 
	 * @param <A> the specified type that extends {@code ArgsComplete}
	 * 
	 * @param intrface the provided interface class
	 * 
	 * @return an {@code ArgsCompleteInterface} of the specified type
	 */
	public static <A extends ArgsComplete> ArgsCompleteInterface<A> getInstance(
			final Class<A> intrface) {
		
		ArgsCompleteInterface<A> argsCompleteInterface = null;
		
		Map<Class<? extends ArgsComplete>, 
			WeakReference<ArgsCompleteInterface<? extends ArgsComplete>>> 
		classMap = ClassMapHolder.CLASS_MAP;
		
		if (!classMap.containsKey(intrface)) {
			
			argsCompleteInterface = new ArgsCompleteInterface<A>(intrface);
			
			WeakReference<ArgsCompleteInterface<? extends ArgsComplete>> 
			weakRef = new 
			WeakReference<ArgsCompleteInterface<? extends ArgsComplete>>(
					argsCompleteInterface);
			
			classMap.put(intrface, weakRef);
			
		} else {
			
			@SuppressWarnings("unchecked")
			ArgsCompleteInterface<A> arsCompleteInterface = 
				(ArgsCompleteInterface<A>) classMap.get(intrface).get();
			
			argsCompleteInterface = arsCompleteInterface;
		}
		
		return argsCompleteInterface;
	}

	/** The interface class. */
	private final Class<A> intrface;
	
	/** The dynamic proxy class constructor. */
	private final Constructor<? extends A> proxyClassConstructor;
	
	/** 
	 * The {@code List} of {@code Method}s from the class {@code ArgsComplete}.
	 */
	private final List<Method> methodList;
	
	/** The {@code List} of {@code ArgGroupCompleteReturnTypeMethod}s. */
	private final List<ArgGroupCompleteReturnTypeMethod> argGroupCompleteReturnTypeMethodList;
	
	/** The {@code ArgsCompleteSpec}. */
	private final ArgsCompleteSpec argsCompleteSpec;
	
	/**
	 * Constructs an {@code ArgsCompleteInterface} with the provided interface
	 * class.
	 * 
	 * @param i the provided interface class
	 */
	private ArgsCompleteInterface(final Class<A> i) {
		
		if (i == null) {
			
			throw new NullPointerException("class must not be null");
		}
		
		if (!i.isInterface()) {
			
			throw new IllegalArgumentException("class must be an interface");
		}
		
		if (!ArgsComplete.class.isAssignableFrom(i)) {
			
			String message = String.format(
					"class must extend %s", 
					ArgsComplete.class.getName());
			
			throw new IllegalArgumentException(message);
		}
		
		@SuppressWarnings("unchecked")
		Class<? extends A> proxyClass = 
			(Class<? extends A>) Proxy.getProxyClass(i.getClassLoader(), i);
		
		Constructor<? extends A> proxyClassCtor = null;
		
		try {
			
			proxyClassCtor = 
				proxyClass.getConstructor(InvocationHandler.class);
			
		} catch (SecurityException e) {
			throw new AssertionError(e);
		} catch (NoSuchMethodException e) {
			throw new AssertionError(e);
		}
		
		List<Method> methList = Arrays.asList(ArgsComplete.class.getMethods());
		
		List<ArgGroupCompleteReturnTypeMethod> argGroupCompleteReturnTypeMethList = 
				new ArrayList<ArgGroupCompleteReturnTypeMethod>();
		
		Method[] methods = i.getMethods();

		Arrays.sort(methods, MethodComparator.getInstance());		
		
		for (Method method : methods) {
			
			if (methList.contains(method)) {
				
				continue;
			}
			
			Class<?> methodReturnType = method.getReturnType();
			
			if (methodReturnType == ArgGroupComplete.class) {
				
				ArgGroupCompleteReturnTypeMethod m =
					new ArgGroupCompleteReturnTypeMethod(method);
				
				argGroupCompleteReturnTypeMethList.add(m);
				
				continue;
			}
			
			String message = String.format(
					"return type must be %s", 
					ArgGroupComplete.class.getName());
			
			throw new InvalidMethodException(method, message);
		}

		ArgsCompleteSpec arsCompleteSpec = null;
		
		if (i.isAnnotationPresent(ArgsCompleteSpec.class)) {
			
			arsCompleteSpec = i.getAnnotation(ArgsCompleteSpec.class);
		}
		
		this.intrface = i;
		this.proxyClassConstructor = proxyClassCtor;
		this.methodList = methList;
		this.argGroupCompleteReturnTypeMethodList = 
				argGroupCompleteReturnTypeMethList;
		this.argsCompleteSpec = arsCompleteSpec;
	}
	
	/**
	 * Returns the {@code Class} view of this {@code ArgsCompleteInterface}.
	 * 
	 * @return the {@code Class} view of this {@code ArgsCompleteInterface}
	 */
	public Class<A> asClass() {
		return this.intrface;
	}
	
	/**
	 * Returns an unmodifiable {@code List} of 
	 * {@code ArgGroupCompleteReturnTypeMethod}s.
	 * 
	 * @return an unmodifiable {@code List} of 
	 * {@code ArgGroupCompleteReturnTypeMethod}s
	 */
	public List<ArgGroupCompleteReturnTypeMethod> getArgGroupCompleteReturnTypeMethods() {
		return Collections.unmodifiableList(
				this.argGroupCompleteReturnTypeMethodList);
	}
	
	/**
	 * Returns a new instance of the specified type.
	 *  
	 * @return a new instance of the specified type
	 */
	public A newArgsComplete() {
		
		ArgsComplete argsComplete = 
				AbstractFactory.getDefault().newArgsComplete();
		
		Map<Method, InvocationHandler> methodMap = 
				new HashMap<Method, InvocationHandler>();
		
		for (ArgGroupCompleteReturnTypeMethod argGroupCompleteReturnTypeMethod
				: this.argGroupCompleteReturnTypeMethodList) {
			
			Method method = argGroupCompleteReturnTypeMethod.asMethod();
			
			ArgGroupComplete<Object> argGroupComplete = 
					argGroupCompleteReturnTypeMethod.newArgGroupComplete();
			
			argsComplete.addArgGroupComplete(argGroupComplete);
			
			methodMap.put(
					method, 
					new ReturnOnlyInvocationHandler(argGroupComplete));
		}
		
		for (Method method : this.methodList) {
			
			methodMap.put(method, new ObjectInvocationHandler(argsComplete));
		}
		
		if (this.argsCompleteSpec != null) {
			
			argsComplete.setSyntax(this.argsCompleteSpec.syntax());
		}
		
		InvocationHandler handler = new ForwardingInvocationHandler(methodMap);
		
		A arsComplete = null;
		
		try {
			
			arsComplete = this.proxyClassConstructor.newInstance(handler);
			
		} catch (IllegalArgumentException e) {
			throw new AssertionError(e);
		} catch (InstantiationException e) {
			throw new AssertionError(e);
		} catch (IllegalAccessException e) {
			throw new AssertionError(e);
		} catch (InvocationTargetException e) {
			throw new AssertionError(e);
		}		
		
		return arsComplete;		
	}

	@Override
	public String toString() {
		return String.format("ArgsCompleteInterface [" 
				+ "interface=%s, " 
				+ "argGroupCompleteReturnTypeMethods=%s]",
				this.intrface, 
				this.argGroupCompleteReturnTypeMethodList);
	}
}
