/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import org.climax.ArgsValidator;
import org.climax.argsvalidator.DistinctArgsValidator;
import org.climax.argsvalidator.NoArgsValidator;
import org.climax.argsvalidator.NoneOrSingleArgValidator;
import org.climax.argsvalidator.PassiveArgsValidator;
import org.climax.argsvalidator.SingleArgValidator;
import org.climax.argsvalidator.SingleOrMultipleArgsValidator;

/**
 * An {@code ArgsValidator} class type. The constants of this 
 * enumerated type provides a classification of {@code ArgsValidator} 
 * classes.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
enum ArgsValidatorClassType {

	/** {@code DistinctArgsValidator} class. */
	DISTINCT_ARGS_VALIDATOR {

		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {

			return DistinctArgsValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return DistinctArgsValidator.getInstance();
		}
		
	},
	
	/** {@code DummyArgsValidator} class. */
	DUMMY_ARGS_VALIDATOR {

		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {
			
			return DummyArgsValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return null;
		}
		
	},
	
	/** {@code NoArgsValidator} class. */
	NO_ARGS_VALIDATOR {
		
		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {
			
			return NoArgsValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return NoArgsValidator.getInstance();
		}
	},
	
	/** {@code NoneOrSingleArgValidator} class. */
	NONE_OR_SINGLE_ARG_VALIDATOR {
		
		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {
			
			return NoneOrSingleArgValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return NoneOrSingleArgValidator.getInstance();
		}
	},
	
	/** {@code PassiveArgsValidator} class. */
	PASSIVE_ARGS_VALIDATOR {
		
		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {

			return PassiveArgsValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return PassiveArgsValidator.getInstance();
		}
	},
	
	/** {@code SingleArgValidator} class. */
	SINGLE_ARG_VALIDATOR {
		
		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {

			return SingleArgValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return SingleArgValidator.getInstance();
		}
	},
	
	/** {@code SingleOrMultipleArgsValidator} class. */
	SINGLE_OR_MULTIPLE_ARGS_VALIDATOR {
		
		@Override
		boolean describes(
				final Class<? extends ArgsValidator> argsValidatorClass) {

			return SingleOrMultipleArgsValidator.class == argsValidatorClass;
		}

		@Override
		ArgsValidator getArgsValidator() {
			return SingleOrMultipleArgsValidator.getInstance();
		}
	};
	
	/**
	 * Returns the enum constant that describes the type of 
	 * {@code ArgsValidator} class from the provided {@code ArgsValidator} 
	 * class.
	 *   
	 * @param argsValidatorClass the provided {@code ArgsValidator} class
	 * 
	 * @return an enum constant that describes the type of 
	 * {@code ArgsValidator} class from the provided {@code ArgsValidator} 
	 * class or {@code null} if there is no enum constant that describes the 
	 * type of {@code ArgsValidator} class from the provided 
	 * {@code ArgsValidator} class
	 */
	static ArgsValidatorClassType bestDescribes(
			final Class<? extends ArgsValidator> argsValidatorClass) {
		
		for (ArgsValidatorClassType argsValidatorClassType 
				: ArgsValidatorClassType.values()) {
			
			if (argsValidatorClassType.describes(argsValidatorClass)) {
				
				return argsValidatorClassType;
			}
		}
		
		return null;
	}
	
	/**
	 * Indicates if the provided {@code ArgsValidator} class is described by
	 * this {@code ArgsValidatorClassType}.
	 * 
	 * @param argsValidatorClass the provided {@code ArgsValidator} class
	 * 
	 * @return {@code true} if the provided {@code ArgsValidator} class is 
	 * described by this {@code ArgsValidatorClassType}
	 */
	abstract boolean describes(
			Class<? extends ArgsValidator> argsValidatorClass);
	
	/**
	 * Returns an {@code ArgsValidator} based on this 
	 * {@code ArgsValidatorClassType}.
	 * 
	 * @return an {@code ArgsValidator}
	 */
	abstract ArgsValidator getArgsValidator();
}
