/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import org.climax.ArgsValidator;
import org.climax.argsvalidator.NoArgsValidator;
import org.climax.argsvalidator.ParameterlessArgsValidator;
import org.climax.argsvalidator.PassiveArgsValidator;
import org.climax.argsvalidator.SingleArgValidator;
import org.climax.argsvalidator.SingleOrMultipleArgsValidator;

/**
 * This class provides utility methods regarding {@code ArgsValidator}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ArgsValidators {
	
	/**
	 * Returns the default {@code ArgsValidator} based on the provided 
	 * {@code TypesType}.
	 * 
	 * @param typesType the provided {@code TypesType}
	 * @return the default {@code ArgsValidator}
	 */
	static ArgsValidator getDefault(final TypesType typesType) {
		
		ArgsValidator argsValidator = null;
		
		switch (typesType) {
		case ARRAYTYPE:
			argsValidator = PassiveArgsValidator.getInstance();
			break;
		case LISTTYPE:
			argsValidator = PassiveArgsValidator.getInstance();
			break;
		case NONE:
			argsValidator = NoArgsValidator.getInstance();
			break;
		case TYPE:
			argsValidator = SingleArgValidator.getInstance();
			break;
		case TYPE_ARRAYTYPE:
			argsValidator = SingleOrMultipleArgsValidator.getInstance();
			break;
		case TYPE_LISTTYPE:
			argsValidator = SingleOrMultipleArgsValidator.getInstance();
			break;
		default:
			throw new AssertionError(String.format(
					"Unknown %1$s: %2$s", 
					TypesType.class.getName(), 
					typesType));
		}
		
		return argsValidator;
	}
	
	/**
	 * Returns an {@code ArgsValidator} based on the provided 
	 * {@code ParameterlessArgsValidator} class.
	 * 
	 * @param argsValidatorClass the provided 
	 * {@code ParameterlessArgsValidator} class
	 * 
	 * @return an {@code ArgsValidator}
	 */
	static ArgsValidator getInstance(
			final Class<? extends ParameterlessArgsValidator> argsValidatorClass) {
		
		ArgsValidatorClassType argsValidatorClassType =
				ArgsValidatorClassType.bestDescribes(argsValidatorClass);
		
		if (argsValidatorClassType == null) {
			return Objects.getInstance(argsValidatorClass);
		}
		
		return argsValidatorClassType.getArgsValidator();
	}
	
	/**
	 * Constructs an instance of {@code ArgsValidators}. This 
	 * constructor is suppressed for noninstantiability outside of this 
	 * class.
	 */
	private ArgsValidators() {
	}
}
