/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.ref.WeakReference;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import org.climax.AbstractFactory;
import org.climax.ArgGroupComplete;
import org.climax.ArgsComplete;
import org.climax.CLI;
import org.climax.OptionGroupComplete;
import org.climax.OptionsComplete;

/**
 * An interface {@code Class} of a specified type that is or extends 
 * {@code CLI}.
 * <p>
 * This {@code CLIInterface} returns an instance of the specified type. 
 * The instance of the specified type is defined by the following: 
 * </p>
 * <ul>
 * <li>The interface class's name</li>
 * <li>The interface class's methods whose return type is either 
 * {@code ArgGroupComplete} or {@code OptionGroupComplete}</li>
 * <li>The interface class's optional {@code CLISpec} annotation</li>
 * </ul>
 * <p> 
 * The interface class's name determines the default program name. 
 * </p>
 * <p>
 * The interface class's methods whose return type is either 
 * {@code ArgGroupComplete} or {@code OptionGroupComplete} each define an 
 * {@code ArgGroupCompleteReturnTypeMethod} or an 
 * {@code OptionGroupCompleteReturnTypeMethod}. If one of the methods' return 
 * type is neither {@code ArgGroupComplete} or {@code OptionGroupComplete}, an 
 * {@code InvalidMethodException} is thrown.
 * </p>
 * <p>
 * The interface class's optional {@code CLISpec} annotation is used to provide
 * further specification of the {@code CLI}. It can also be used to override 
 * the default program name determined by the class's name.
 * </p>
 *   
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <C> the specified type that is or extends {@code CLI}
 * 
 * @see ArgGroupCompleteReturnTypeMethod
 *  
 * @see CLISpec
 * 
 * @see OptionGroupCompleteReturnTypeMethod
 */
public final class CLIInterface<C extends CLI> {
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ClassMapHolder.CLASS_MAP}.
	 */
	private static final class ClassMapHolder {
		
		/**
		 * A {@code Map} that maps existing {@code CLI} classes to 
		 * existing {@code CLIInterface}s.
		 */
		static final Map<Class<? extends CLI>, 
			WeakReference<CLIInterface<? extends CLI>>> 
		CLASS_MAP = new WeakHashMap<Class<? extends CLI>, 
			WeakReference<CLIInterface<? extends CLI>>>();
		
		/**
		 * Constructs a {@code ClassMapHolder}. This constructor is suppressed 
		 * for noninstantiability outside of this class.
		 */
		private ClassMapHolder() {
		}
	}

	/**
	 * Returns a {@code CLIInterface} of the specified type based on the 
	 * provided interface class.
	 * 
	 * @param <C> the specified type that extends {@code CLI}
	 * 
	 * @param intrface the provided interface class
	 * 
	 * @return a {@code CLIInterface} of the specified type
	 */
	public static <C extends CLI> CLIInterface<C> getInstance(
			final Class<C> intrface) {
		
		CLIInterface<C> cliInterface = null;
		
		Map<Class<? extends CLI>, 
			WeakReference<CLIInterface<? extends CLI>>> 
		classMap = ClassMapHolder.CLASS_MAP;
		
		if (!classMap.containsKey(intrface)) {
			
			cliInterface = new CLIInterface<C>(intrface);
			
			WeakReference<CLIInterface<? extends CLI>> weakRef = 
				new WeakReference<CLIInterface<? extends CLI>>(
					cliInterface);
			
			classMap.put(intrface, weakRef);
			
		} else {
			
			@SuppressWarnings("unchecked")
			CLIInterface<C> cInterface = 
				(CLIInterface<C>) classMap.get(intrface).get();
			
			cliInterface = cInterface;
		}
		
		return cliInterface;
	}

	/** The interface class. */
	private final Class<C> intrface;
	
	/** The dynamic proxy class constructor. */
	private final Constructor<? extends C> proxyClassConstructor;
	
	/**
	 * The {@code List} of {@code Method}s from the class 
	 * {@code OptionsComplete}.
	 */
	private final List<Method> methodList;
	
	/** The {@code List} of {@code OptionGroupCompleteReturnTypeMethod}s. */
	private final List<OptionGroupCompleteReturnTypeMethod> optionGroupCompleteReturnTypeMethodList;
	
	/** The {@code List} of {@code ArgGroupCompleteReturnTypeMethod}s. */
	private final List<ArgGroupCompleteReturnTypeMethod> argGroupCompleteReturnTypeMethodList;
	
	/**
	 * Constructs a {@code CLIInterface} with the provided interface
	 * class.
	 * 
	 * @param i the provided interface class
	 */
	private CLIInterface(final Class<C> i) {
		
		if (i == null) {
			
			throw new NullPointerException("class must not be null");
		}
		
		if (!i.isInterface()) {
			
			throw new IllegalArgumentException("class must be an interface");
		}
		
		if (!CLI.class.isAssignableFrom(i)) {
			
			String message = String.format(
					"class must extend %s", 
					CLI.class.getName());
			
			throw new IllegalArgumentException(message);
		}
		
		@SuppressWarnings("unchecked")
		Class<? extends C> proxyClass = 
			(Class<? extends C>) Proxy.getProxyClass(i.getClassLoader(), i);
		
		Constructor<? extends C> proxyClassCtor = null;
		
		try {
			
			proxyClassCtor = 
				proxyClass.getConstructor(InvocationHandler.class);
			
		} catch (SecurityException e) {
			throw new AssertionError(e);
		} catch (NoSuchMethodException e) {
			throw new AssertionError(e);
		}
		
		List<Method> methList = Arrays.asList(CLI.class.getMethods());
		
		List<OptionGroupCompleteReturnTypeMethod> optionGroupCompleteReturnTypeMethList = 
				new ArrayList<OptionGroupCompleteReturnTypeMethod>();
		
		List<ArgGroupCompleteReturnTypeMethod> argGroupCompleteReturnTypeMethList = 
				new ArrayList<ArgGroupCompleteReturnTypeMethod>();
		
		Method[] methods = i.getMethods();

		Arrays.sort(methods, MethodComparator.getInstance());		
		
		for (Method method : methods) {
			
			if (methList.contains(method)) {
				
				continue;
			}
			
			Class<?> methodReturnType = method.getReturnType();
			
			if (methodReturnType == OptionGroupComplete.class) {
				
				OptionGroupCompleteReturnTypeMethod m =
					new OptionGroupCompleteReturnTypeMethod(method);
				
				optionGroupCompleteReturnTypeMethList.add(m);
				
				continue;
			}
			
			if (methodReturnType == ArgGroupComplete.class) {
				
				ArgGroupCompleteReturnTypeMethod m =
					new ArgGroupCompleteReturnTypeMethod(method);
				
				argGroupCompleteReturnTypeMethList.add(m);
				
				continue;
			}
			
			String message = String.format(
					"return type must be %1$s or %2$s", 
					OptionGroupComplete.class.getName(),
					ArgGroupComplete.class.getName());
			
			throw new InvalidMethodException(method, message);
		}
		
		this.intrface = i;
		this.proxyClassConstructor = proxyClassCtor;
		this.methodList = methList;
		this.optionGroupCompleteReturnTypeMethodList = 
				optionGroupCompleteReturnTypeMethList;
		this.argGroupCompleteReturnTypeMethodList = 
				argGroupCompleteReturnTypeMethList;
		
	}
	
	/**
	 * Returns the {@code Class} view of this {@code CLIInterface}.
	 * 
	 * @return the {@code Class} view of this {@code CLIInterface}
	 */
	public Class<C> asClass() {
		return this.intrface;
	}
	
	/**
	 * Returns an unmodifiable {@code List} of 
	 * {@code ArgGroupCompleteReturnTypeMethod}s.
	 * 
	 * @return an unmodifiable {@code List} of 
	 * {@code ArgGroupCompleteReturnTypeMethod}s
	 */
	public List<ArgGroupCompleteReturnTypeMethod> getArgGroupCompleteReturnTypeMethods() {
		return Collections.unmodifiableList(
				this.argGroupCompleteReturnTypeMethodList);
	}
	
	/**
	 * Returns an unmodifiable {@code List} of 
	 * {@code OptionGroupCompleteReturnTypeMethod}s.
	 * 
	 * @return an unmodifiable {@code List} of 
	 * {@code OptionGroupCompleteReturnTypeMethod}s
	 */
	public List<OptionGroupCompleteReturnTypeMethod> getOptionGroupCompleteReturnTypeMethods() {
		return Collections.unmodifiableList(
				this.optionGroupCompleteReturnTypeMethodList);
	}
	
	/**
	 * Returns a new instance of the specified type.
	 *  
	 * @return a new instance of the specified type
	 */
	public C newCLI() {
		
		OptionsComplete optionsComplete =
				AbstractFactory.getDefault().newOptionsComplete();
		
		ArgsComplete argsComplete =
				AbstractFactory.getDefault().newArgsComplete();
		
		Map<Method, InvocationHandler> methodMap = 
				new HashMap<Method, InvocationHandler>();
		
		ExistingOptionSet.clear();
		
		try {
			
			for (OptionGroupCompleteReturnTypeMethod optGrpCompleteReturnTypeMethod
					: this.optionGroupCompleteReturnTypeMethodList) {
				
				Method method = optGrpCompleteReturnTypeMethod.asMethod();
				
				OptionGroupComplete<Object> optionGroupComplete = 
						optGrpCompleteReturnTypeMethod.newOptionGroupComplete();
				
				optionsComplete.addOptionGroupComplete(optionGroupComplete);
				
				methodMap.put(
						method, 
						new ReturnOnlyInvocationHandler(optionGroupComplete));
			}
			
		} finally {
			
			ExistingOptionSet.clear();
		}
		
		for (ArgGroupCompleteReturnTypeMethod argGroupCompleteReturnTypeMethod
				: this.argGroupCompleteReturnTypeMethodList) {
			
			Method method = argGroupCompleteReturnTypeMethod.asMethod();
			
			ArgGroupComplete<Object> argGroupComplete = 
					argGroupCompleteReturnTypeMethod.newArgGroupComplete();
			
			argsComplete.addArgGroupComplete(argGroupComplete);
			
			methodMap.put(
					method, 
					new ReturnOnlyInvocationHandler(argGroupComplete));
		}
		
		ClassBasedCLIBuilder classBasedCLIBuilder = 
				new ClassBasedCLIBuilder(this.intrface);
		classBasedCLIBuilder.argsComplete(argsComplete);
		classBasedCLIBuilder.optionsComplete(optionsComplete);
		
		CLI cli = classBasedCLIBuilder.buildCLI();
		
		for (Method method : this.methodList) {
			
			methodMap.put(method, new ObjectInvocationHandler(cli));
		}
		
		InvocationHandler handler = new ForwardingInvocationHandler(methodMap);
		
		C cl = null;
		
		try {
			
			cl = this.proxyClassConstructor.newInstance(handler);
			
		} catch (IllegalArgumentException e) {
			throw new AssertionError(e);
		} catch (InstantiationException e) {
			throw new AssertionError(e);
		} catch (IllegalAccessException e) {
			throw new AssertionError(e);
		} catch (InvocationTargetException e) {
			throw new AssertionError(e);
		}		
		
		return cl;
	}

	@Override
	public String toString() {
		return String.format("CLIInterface [" 
				+ "interface=%s, " 
				+ "optionGroupCompleteReturnTypeMethods=%s, " 
				+ "argGroupCompleteReturnTypeMethods=%s]",
				this.intrface, 
				this.optionGroupCompleteReturnTypeMethodList, 
				this.argGroupCompleteReturnTypeMethodList);
	}
}
