/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Provides further specification of the {@code CLI}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @see CLIInterface
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface CLISpec {
	
	/**
	 * The format string for the syntax for the {@code Args}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.ArgsDoc#setSyntax(String)} method.
	 * </p> 
	 */	
	String argsDocSyntax() default "";
	
	/** 
	 * The format string for the description of the help {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#description(String)} method.
	 * </p> 
	 */
	String helpOptionGroupDocDescription() default "";
	
	/** 
	 * The format string for the help {@code OptionGroup}'s help information. 
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#help(String)} method.
	 * </p>
	 */	
	String helpOptionGroupDocHelp() default "";
	
	/** 
	 * The ordinal of the help {@code OptionGroupDoc} in relation to other 
	 * {@code OptionGroupDoc}s. 
	 */	
	int helpOptionGroupDocOrdinal() default Integer.MAX_VALUE;
	
	/**
	 * The format string for the usage of the help {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#usage(String)} method.
	 * </p>
	 */	
	String helpOptionGroupDocUsage() default ""; 
	
	/**
	 * The boolean value to indicate if the help {@code OptionGroupDoc} is 
	 * excluded from documentation.
	 */
	boolean helpOptionGroupHidden() default false;

	/**
	 * The boolean value to indicate if the help {@code OptionGroup} is 
	 * excluded from parsing.
	 */	
	boolean helpOptionGroupUnused() default false;
	
	/**
	 * The {@code OptionCompleteSpec}s for the help options. 
	 */
	OptionCompleteSpec[] helpOptions() default { };
	
	/** 
	 * The format string for the help information for the {@code Options}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionsDoc#setHelp(String)} method.
	 * </p>  
	 */	
	String optionsDocHelp() default "";
	
	/**
	 * The format string for the usage of the {@code Options}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionsDoc#setUsage(String)} method.
	 * </p>
	 */	
	String optionsDocUsage() default "";
	
	/** 
	 * The format string for the program description. 
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.CLIBuilder#programDescription(String)} method.
	 * </p> 
	 */
	String programDescription() default "";
	
	/** 
	 * The format string for the program help information body.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.CLIBuilder#programHelpBody(String)} method.
	 * </p>  
	 */
	String programHelpBody() default "";
	
	/** 
	 * The format string for the program help information footer.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.CLIBuilder#programHelpBody(String)} method.
	 * </p>  
	 */
	String programHelpFooter() default "";
	
	/** 
	 * The format string for the program help information header.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.CLIBuilder#programHelpBody(String)} method.
	 * </p>  
	 */
	String programHelpHeader() default "";
	
	/** 
	 * The format string for the program name.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.AbstractFactory#newCLIBuilder(String)} method.
	 * </p>  
	 */
	String programName() default "";
	
	/** 
	 * The format string for the program usage.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.CLIBuilder#programUsage(String)} method.
	 * </p>  
	 */
	String programUsage() default "";
	
	/** 
	 * The format string for the program version information.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.CLIBuilder#programVersion(String)} method.
	 * </p>  
	 */
	String programVersion() default "";
	
	/** 
	 * The format string for the description of the version {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#description(String)} method.
	 * </p> 
	 */	
	String versionOptionGroupDocDescription() default "";
	
	/** 
	 * The format string for the version {@code OptionGroup}'s help information.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#help(String)} method.
	 * </p> 
	 */	
	String versionOptionGroupDocHelp() default "";
	
	/** 
	 * The ordinal of the version {@code OptionGroupDoc} in relation to other 
	 * {@code OptionGroupDoc}s. 
	 */	
	int versionOptionGroupDocOrdinal() default Integer.MAX_VALUE;
	
	/**
	 * The format string for the usage of the version {@code OptionGroup}.
	 * <p>
	 * The property keys that can be applied as expandable properties 
	 * within the provided format string are found in the
	 * {@link org.climax.OptionGroupDocBuilder#usage(String)} method.
	 * </p> 
	 */	
	String versionOptionGroupDocUsage() default "";
	
	/**
	 * The boolean value to indicate if the version {@code OptionGroupDoc} is 
	 * excluded from documentation.
	 */	
	boolean versionOptionGroupHidden() default false;
	
	/**
	 * The boolean value to indicate if the version {@code OptionGroup} is 
	 * excluded from parsing.
	 */	
	boolean versionOptionGroupUnused() default false;
	
	/**
	 * The {@code OptionCompleteSpec}s for the version options. 
	 */
	OptionCompleteSpec[] versionOptions() default { };
}
