/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.climax.AbstractFactory;
import org.climax.ArgsComplete;
import org.climax.CLI;
import org.climax.CLIBuilder;
import org.climax.OptionComplete;
import org.climax.OptionsComplete;

/**
 * A builder for a {@code CLI} based on the provided {@code Class}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ClassBasedCLIBuilder {

	/**
	 * Returns the program name based on the provided {@code Class}.
	 * 
	 * @param cls the provided {@code Class}
	 * @return the program name
	 */
	private static String getProgramName(final Class<?> cls) {
		
		String potentialProgramName = cls.getSimpleName();
		
		Pattern pattern = Pattern.compile("[A-Za-z0-9][A-Z][a-z0-9]");

		Matcher matcher = pattern.matcher(potentialProgramName);

		StringBuilder builder = new StringBuilder(potentialProgramName);

		int offset = 0;

		while (matcher.find()) {

			int start = matcher.start() + offset;
			int end = matcher.end() + offset;

			String match = builder.substring(start, end);

			StringBuilder replacement = new StringBuilder(match);

			replacement.insert(1, "-");

			builder.replace(start, end, replacement.toString());

			offset++;
		}

		return builder.toString().toLowerCase();		
	}
	
	/** The {@code Class}. */
	private Class<?> cls;
	
	/** The {@code ArgsComplete}. */
	private ArgsComplete argsComplete;
	
	/** The {@code OptionsComplete}. */
	private OptionsComplete optionsComplete;
	
	/**
	 * Constructs a {@code ClassBasedCLIBuilder} with the provided 
	 * {@code Class}.
	 * 
	 * @param c the provided {@code Class}
	 */
	ClassBasedCLIBuilder(final Class<?> c) {
		
		if (c == null) {
			throw new NullPointerException("class must not be null");
		}
		
		this.cls = c;
		this.argsComplete = null;
		this.optionsComplete = null;
	}
	
	/**
	 * Sets the {@code ArgsComplete} with the provided 
	 * {@code ArgsComplete}.
	 * 
	 * @param a the provided {@code ArgsComplete}
	 * @return this builder
	 */
	ClassBasedCLIBuilder argsComplete(final ArgsComplete a) {
		this.argsComplete = AbstractFactory.getDefault().newArgsComplete(a);
		return this;
	}
	
	/**
	 * Builds a new {@code CLI}.
	 * 
	 * @return a new {@code CLI}
	 */
	CLI buildCLI() {
		
		CLISpec cliSpec = null;
		
		if (this.cls.isAnnotationPresent(CLISpec.class)) {
			cliSpec = this.cls.getAnnotation(CLISpec.class);
		}
		
		String argsDocSyntax = "";
		String helpOptionGroupDocDescription = "";
		String helpOptionGroupDocHelp = "";
		int helpOptionGroupDocOrdinal = Integer.MAX_VALUE;
		String helpOptionGroupDocUsage = "";
		boolean helpOptionGroupHidden = false;
		boolean helpOptionGroupUnused = false;
		OptionCompleteSpec[] helpOptions = new OptionCompleteSpec[] { };
		String optionsDocHelp = "";
		String optionsDocUsage = "";
		String programDescription = "";
		String programHelpBody = "";
		String programHelpFooter = "";
		String programHelpHeader = "";
		String programName = "";
		String programUsage = "";
		String programVersion = "";
		String versionOptionGroupDocDescription = "";
		String versionOptionGroupDocHelp = "";
		int versionOptionGroupDocOrdinal = Integer.MAX_VALUE;
		String versionOptionGroupDocUsage = "";
		boolean versionOptionGroupHidden = false;
		boolean versionOptionGroupUnused = false;
		OptionCompleteSpec[] versionOptions = new OptionCompleteSpec[] { };
		
		if (cliSpec != null) {
			
			argsDocSyntax = cliSpec.argsDocSyntax();
			helpOptionGroupDocDescription = 
					cliSpec.helpOptionGroupDocDescription();
			helpOptionGroupDocHelp = cliSpec.helpOptionGroupDocHelp();
			helpOptionGroupDocOrdinal = cliSpec.helpOptionGroupDocOrdinal();
			helpOptionGroupDocUsage = cliSpec.helpOptionGroupDocUsage();
			helpOptionGroupHidden = cliSpec.helpOptionGroupHidden();
			helpOptionGroupUnused = cliSpec.helpOptionGroupUnused();
			helpOptions = cliSpec.helpOptions();
			optionsDocHelp = cliSpec.optionsDocHelp();
			optionsDocUsage = cliSpec.optionsDocUsage();
			programDescription = cliSpec.programDescription();
			programHelpBody = cliSpec.programHelpBody();
			programHelpFooter = cliSpec.programHelpFooter();
			programHelpHeader = cliSpec.programHelpHeader();
			programName = cliSpec.programName();
			programUsage = cliSpec.programUsage();
			programVersion = cliSpec.programVersion();
			versionOptionGroupDocDescription = 
					cliSpec.versionOptionGroupDocDescription();
			versionOptionGroupDocHelp = cliSpec.versionOptionGroupDocHelp();
			versionOptionGroupDocOrdinal = 
					cliSpec.versionOptionGroupDocOrdinal();
			versionOptionGroupDocUsage = cliSpec.versionOptionGroupDocUsage();
			versionOptionGroupHidden = cliSpec.versionOptionGroupHidden();
			versionOptionGroupUnused = cliSpec.versionOptionGroupUnused();
			versionOptions = cliSpec.versionOptions();
		}
		
		List<OptionComplete> helpOptionCompleteList = 
				new ArrayList<OptionComplete>();
		
		for (OptionCompleteSpec helpOptionCompleteSpec : helpOptions) {
			
			TypesBasedOptionCompleteBuilder typesBasedOptionCompleteBuilder = 
					new TypesBasedOptionCompleteBuilder(
							"help", Void.class, TypesType.NONE);
			typesBasedOptionCompleteBuilder.optionCompleteSpec(
					helpOptionCompleteSpec);
			
			OptionComplete helpOptionComplete = 
					typesBasedOptionCompleteBuilder.buildOptionComplete();
			
			helpOptionCompleteList.add(helpOptionComplete);
		}
		
		if (programName.isEmpty()) {
			programName = ClassBasedCLIBuilder.getProgramName(this.cls);
		}
		
		List<OptionComplete> versionOptionCompleteList = 
				new ArrayList<OptionComplete>();
		
		for (OptionCompleteSpec versionOptionCompleteSpec : versionOptions) {
			
			TypesBasedOptionCompleteBuilder typesBasedOptionCompleteBuilder = 
					new TypesBasedOptionCompleteBuilder(
							"version", Void.class, TypesType.NONE);
			typesBasedOptionCompleteBuilder.optionCompleteSpec(
					versionOptionCompleteSpec);
			
			OptionComplete versionOptionComplete = 
					typesBasedOptionCompleteBuilder.buildOptionComplete();
			
			versionOptionCompleteList.add(versionOptionComplete);
		}
		
		AbstractFactory factory = AbstractFactory.getDefault();
		
		CLIBuilder cliBuilder = factory.newCLIBuilder(programName);
		cliBuilder.argsComplete(this.argsComplete);
		cliBuilder.argsDocSyntax(argsDocSyntax);
		cliBuilder.helpOptionCompletes(helpOptionCompleteList);
		cliBuilder.helpOptionGroupDocDescription(helpOptionGroupDocDescription);
		cliBuilder.helpOptionGroupDocHelp(helpOptionGroupDocHelp);
		cliBuilder.helpOptionGroupDocOrdinal(helpOptionGroupDocOrdinal);
		cliBuilder.helpOptionGroupDocUsage(helpOptionGroupDocUsage);
		cliBuilder.helpOptionGroupHidden(helpOptionGroupHidden);
		cliBuilder.helpOptionGroupUnused(helpOptionGroupUnused);
		cliBuilder.optionsComplete(this.optionsComplete);
		cliBuilder.optionsDocHelp(optionsDocHelp);
		cliBuilder.optionsDocUsage(optionsDocUsage);
		cliBuilder.programDescription(programDescription);
		cliBuilder.programHelpBody(programHelpBody);
		cliBuilder.programHelpFooter(programHelpFooter);
		cliBuilder.programHelpHeader(programHelpHeader);
		cliBuilder.programUsage(programUsage);
		cliBuilder.programVersion(programVersion);
		cliBuilder.versionOptionCompletes(versionOptionCompleteList);
		cliBuilder.versionOptionGroupDocDescription(
				versionOptionGroupDocDescription);
		cliBuilder.versionOptionGroupDocHelp(versionOptionGroupDocHelp);
		cliBuilder.versionOptionGroupDocOrdinal(versionOptionGroupDocOrdinal);
		cliBuilder.versionOptionGroupDocUsage(versionOptionGroupDocUsage);
		cliBuilder.versionOptionGroupHidden(versionOptionGroupHidden);
		cliBuilder.versionOptionGroupUnused(versionOptionGroupUnused);
		
		CLI cli = cliBuilder.buildCLI();
		
		return cli;
	}
	
	/**
	 * Sets the {@code OptionsComplete} with the provided 
	 * {@code OptionsComplete}.
	 * 
	 * @param o the provided {@code OptionsComplete}
	 * @return this builder
	 */
	ClassBasedCLIBuilder optionsComplete(final OptionsComplete o) {
		this.optionsComplete = 
				AbstractFactory.getDefault().newOptionsComplete(o);
		return this;
	}
}
