/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;


/**
 * A {@code Class} type. The constants of this enumerated type provides a 
 * classification of {@code Class}es.
 *  
 * @author Climax Development (climax.development@gmail.com)
 */
enum ClassType {
	
	/** {@code BigDecimal} class. */
	BIG_DECIMAL {

		@Override
		boolean describes(final Class<?> cls) {
			return BigDecimal.class == cls;
		}
		
	},
	
	/** {@code BigInteger} class. */
	BIG_INTEGER {

		@Override
		boolean describes(final Class<?> cls) {
			return BigInteger.class == cls;
		}
		
	},
	
	/** {@code Boolean} class. */
	BOOLEAN {

		@Override 
		boolean describes(final Class<?> cls) {
			return Boolean.class == cls;
		}
		
	},
	
	/** {@code boolean} class. */
	BOOLEAN_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return boolean.class == cls;
		}

	},
	
	/** {@code Byte} class. */
	BYTE {

		@Override 
		boolean describes(final Class<?> cls) {
			return Byte.class == cls;
		}

	},
	
	/** {@code byte} class. */
	BYTE_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return byte.class == cls;
		}

	},
	
	/** {@code Character} class. */
	CHARACTER {

		@Override boolean describes(final Class<?> cls) {
			return Character.class == cls;
		}

	},
	
	/** {@code char} class. */
	CHARACTER_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return char.class == cls;
		}

	},
	
	/** {@code Class} class. */
	CLASS {

		@Override 
		boolean describes(final Class<?> cls) {
			return Class.class == cls;
		}

	},
	
	/** {@code Date} class. */
	DATE {

		@Override 
		boolean describes(final Class<?> cls) {
			return Date.class == cls;
		}
		
	},
	
	/** {@code Double} class. */
	DOUBLE {

		@Override 
		boolean describes(final Class<?> cls) {
			return Double.class == cls;
		}

	},
	
	/** {@code double} class. */
	DOUBLE_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return double.class == cls;
		}

	},
	
	/** {@code Enum} class. */
	ENUM {

		@Override 
		boolean describes(final Class<?> cls) {
			return cls != null && cls.isEnum();
		}

	},
	
	/** {@code File} class. */
	FILE {

		@Override 
		boolean describes(final Class<?> cls) {
			return File.class == cls;
		}

	},
	
	/** {@code FileInputStream} class. */
	FILE_INPUT_STREAM {

		@Override
		boolean describes(final Class<?> cls) {
			return FileInputStream.class == cls;
		}

	},
	
	/** {@code FileOutputStream} class. */
	FILE_OUTPUT_STREAM {

		@Override
		boolean describes(final Class<?> cls) {
			return FileOutputStream.class == cls;
		}

	},
	
	/** {@code FileReader} class. */
	FILE_READER {

		@Override
		boolean describes(final Class<?> cls) {
			return FileReader.class == cls;
		}

	},
	
	/** {@code FileWriter} class. */
	FILE_WRITER {

		@Override
		boolean describes(final Class<?> cls) {
			return FileWriter.class == cls;
		}

	},
	
	/** {@code Float} class. */
	FLOAT {

		@Override 
		boolean describes(final Class<?> cls) {
			return Float.class == cls;
		}

	},
	
	/** {@code float} class. */
	FLOAT_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return float.class == cls;
		}

	},
	
	/** {@code Integer} class. */
	INTEGER {

		@Override 
		boolean describes(final Class<?> cls) {
			return Integer.class == cls;
		}

	},
	
	/** {@code int} class. */
	INTEGER_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return int.class == cls;
		}

	},
	
	/** {@code Long} class. */
	LONG {

		@Override 
		boolean describes(final Class<?> cls) {
			return Long.class == cls;
		}

	},
	
	/** {@code long} class. */
	LONG_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return long.class == cls;
		}

	},
	
	/** {@code Short} class. */
	SHORT {

		@Override 
		boolean describes(final Class<?> cls) {
			return Short.class == cls;
		}

	},
	
	/** {@code short} class. */
	SHORT_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return short.class == cls;
		}

	},
	
	/** {@code String} class. */
	STRING {

		@Override 
		boolean describes(final Class<?> cls) {
			return String.class == cls;
		}

	},
	
	/** {@code Void} class. */
	VOID {

		@Override 
		boolean describes(final Class<?> cls) {
			return Void.class == cls;
		}

	},
	
	/** {@code void} class. */
	VOID_PRIMITIVE {

		@Override
		boolean describes(final Class<?> cls) {
			return void.class == cls;
		}

	};
	
	/**
	 * Returns the enum constant that describes the type of {@code Class} 
	 * from the provided {@code Class}.
	 * 
	 * @param cls the provided {@code Class}
	 * 
	 * @return an enum constant that describes the type of {@code Class} 
	 * from the provided {@code Class} or {@code null} if there is no enum 
	 * constant that describes the type of {@code Class} from the provided 
	 * {@code Class}
	 */
	static ClassType bestDescribes(final Class<?> cls) {
		
		for (ClassType clsType : ClassType.values()) {
			
			if (clsType.describes(cls)) {
				
				return clsType;
			}
		}
		
		return null;
	}
	
	/**
	 * Indicates if the provided {@code Class} is described by this 
	 * {@code ClassType}.
	 * 
	 * @param cls the provided {@code Class}
	 * 
	 * @return {@code true} if the provided {@code Class} is described by this 
	 * {@code ClassType}
	 */
	abstract boolean describes(Class<?> cls);
}