/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.util.Collections;
import java.util.Set;
import java.util.TreeSet;

import org.climax.AbstractFactory;
import org.climax.Option;

/**
 * This class provides utility methods for interacting with 
 * the {@code Set} of existing {@code Option}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class ExistingOptionSet {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code OptionSetHolder.OPTION_SET}.
	 */
	private static final class OptionSetHolder {
		
		/** The {@code Set} of {@code Option}s. */
		static final Set<Option> OPTION_SET = new TreeSet<Option>(
				AbstractFactory.getDefault().getOptionComparator());
		
		/**
		 * Constructs an {@code OptionSetHolder}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */		
		private OptionSetHolder() {
		}
	}

	/**
	 * Adds the provided {@code Option} to the {@code Set} of 
	 * existing {@code Option}s.
	 * 
	 * @param option the provided {@code Option}
	 * 
	 * @return {@code true} if the {@code Set} of existing 
	 * {@code Option}s did not contain the provided {@code Option}
	 */
	static boolean add(final Option option) {
		return OptionSetHolder.OPTION_SET.add(option);
	}
	
	/**
	 * Removes all of the {@code Option}s from the {@code Set}.
	 */
	static void clear() {
		OptionSetHolder.OPTION_SET.clear();
	}
	
	/**
	 * Returns an unmodifiable {@code Set} of existing 
	 * {@code Option}s.
	 * 
	 * @return an unmodifiable {@code Set} of existing 
	 * {@code Option}s
	 */
	static Set<Option> get() {
		return Collections.unmodifiableSet(OptionSetHolder.OPTION_SET);
	}
	
	/**
	 * Constructs an instance of {@code ExistingOptionSet}. This 
	 * constructor is suppressed for noninstantiability outside of 
	 * this class.
	 */
	private ExistingOptionSet() {
	}
}
