/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Method;
import java.util.Comparator;

/**
 * A {@code Comparator} that compares two {@code Method}s first by their 
 * names and then by their parameter types.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class MethodComparator implements Comparator<Method> {

	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code MethodComparatorHolder.METHOD_COMPARATOR}.
	 */
	private static final class MethodComparatorHolder {
		
		/** The {@code MethodComparator}. */
		static final MethodComparator METHOD_COMPARATOR = 
			new MethodComparator();
		
		/**
		 * Constructs a {@code MethodComparatorHolder}. This constructor is 
		 * suppressed for noninstantiability outside of this class.
		 */			
		private MethodComparatorHolder() {
		}
	}

	/**
	 * Returns a {@code MethodComparator}.
	 * 
	 * @return a {@code MethodComparator}
	 */
	static MethodComparator getInstance() {
		return MethodComparatorHolder.METHOD_COMPARATOR;
	}
	
	/**
	 * Constructs a {@code MethodComparator}. This constructor is 
	 * suppressed for noninstantiability outside of this class.
	 */
	private MethodComparator() {
	}
	
	@Override
	public int compare(final Method method1, final Method method2) {

		int result = 0;

		result = method1.getName().compareTo(method2.getName());
		
		if (result != 0) { return result; }

		Class<?>[] parameterTypes1 = method1.getParameterTypes();
		Class<?>[] parameterTypes2 = method2.getParameterTypes();
		
		int size1 = parameterTypes1.length;
		int size2 = parameterTypes2.length;
		
		int sizeResult = size1 - size2;
		
		int smallestSize = size1;
		
		if (sizeResult > 0) {
			smallestSize = size2;
		}
		
		for (int i = 0; i < smallestSize; i++) {
			
			Class<?> parameterType1 = parameterTypes1[i];
			Class<?> parameterType2 = parameterTypes2[i];
			
			result = parameterType1.getName().compareTo(
					parameterType2.getName());
			
			if (result != 0) { return result; }
		}
		
		return sizeResult;
	}
}