/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class provides utility methods regarding methods.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class Methods {
	
	/**
	 * Returns the name of the subject of the provided accessor method.
	 * If the provided method is not an accessor method, the name of the 
	 * method is returned.
	 * 
	 * @param method the provided accessor method
	 * 
	 * @return the name of the subject of the provided accessor method or 
	 * the name of the provided method if the provided method is not an
	 * accessor method 
	 */
	static String getAccessorSubjectName(final Method method) {

		String methodName = method.getName();

		Pattern pattern = Pattern.compile("^(get|is).+");

		Matcher matcher = pattern.matcher(methodName);

		StringBuilder builder = new StringBuilder(methodName);

		if (matcher.find()) {

			int start = matcher.start(1);
			int end = matcher.end(1);

			builder.replace(start, end, "");
		}
		
		return builder.toString();		
	}
	
	/**
	 * Returns the name of the subject of the provided mutator method.
	 * If the provided method is not a mutator method, the name of the 
	 * method is returned.
	 * 
	 * @param method the provided mutator method
	 * 
	 * @return the name of the subject of the provided mutator method or 
	 * the name of the provided method if the provided method is not an
	 * mutator method 
	 */	
	static String getMutatorSubjectName(final Method method) {

		String methodName = method.getName();

		Pattern pattern = Pattern.compile("^(set).+");

		Matcher matcher = pattern.matcher(methodName);

		StringBuilder builder = new StringBuilder(methodName);

		if (matcher.find()) {

			int start = matcher.start(1);
			int end = matcher.end(1);

			builder.replace(start, end, "");
		}
		
		return builder.toString();		
	}
	
	/**
	 * Indicates if the provided method is a public, static, parameterless 
	 * method that returns an instance of the method's declaring class.
	 *   
	 * @param method the provided method
	 * 
	 * @return {@code true} if the provided method is a public, static, 
	 * parameterless method that returns an instance of the method's declaring 
	 * class
	 */
	static boolean isStaticFactory(final Method method) {

		boolean isPublic = Modifier.isPublic(method.getModifiers());
		boolean isStatic = Modifier.isStatic(method.getModifiers());
		boolean isParameterless = method.getParameterTypes().length == 0;		
		boolean returnsDeclaringClassInstance = 
			method.getReturnType() == method.getDeclaringClass();

		
		return isPublic && isStatic && isParameterless 
			&& returnsDeclaringClassInstance;
	}
	
	/**
	 * Constructs an instance of {@code Methods}. This constructor is 
	 * suppressed for noninstantiability outside of this class.
	 */	
	private Methods() {
	}
}
