/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * This class provides utility methods regarding {@code Object}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class Objects {
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ClassMapHolder.CLASS_MAP}.
	 */
	private static final class ClassMapHolder {
		
		/**
		 * A {@code Map} that maps existing classes to existing instances.
		 */
		static final Map<Class<?>, Object> CLASS_MAP = 
			new WeakHashMap<Class<?>, Object>();
		
		/**
		 * Constructs a {@code ClassMapHolder}. This constructor is suppressed 
		 * for noninstantiability outside of this class.
		 */		
		private ClassMapHolder() {
		}
	}

	/**
	 * Returns an instance of the specified type based on the provided extended
	 * class of the specified type.
	 * <p>
	 * If an instance was already created from the public, parameterless 
	 * default constructor of the provided class annotated with the 
	 * {@code Singleton} annotation, then that instance is returned. 
	 * Otherwise, an instance is retrieved from invoking the provided class's 
	 * first public, static, parameterless method that returns an instance of 
	 * the method's declaring class and is annotated with the 
	 * {@code StaticFactory} annotation. If there is no such method, an 
	 * instance is retrieved from invoking the provided class's first public, 
	 * static, parameterless method that returns an instance of the method's 
	 * declaring class (no {@code StaticFactory} annotation). If there is no 
	 * such method or the instance is {@code null}, the instance is retrieved 
	 * from invoking the provided class's public, parameterless default 
	 * constructor. If the provided class is annotated with the 
	 * {@code Singleton} annotation, the instance is kept when the same 
	 * class is provided upon a later invocation of this method. If there is 
	 * no public, parameterless default constructor, a 
	 * {@code InvalidClassException} is thrown.
	 * </p>
	 * 
	 * @param <T> the specified type
	 * 
	 * @param cls the provided extended class of the specified type
	 * 
	 * @return an instance of the specified type
	 */
	static <T> T getInstance(final Class<? extends T> cls) {
		
		Map<Class<?>, Object> classMap = ClassMapHolder.CLASS_MAP;

		if (classMap.containsKey(cls)) {
			
			return cls.cast(classMap.get(cls));
		}
		
		Method method = null;
		
		for (Method m : cls.getMethods()) {
			
			if (m.isAnnotationPresent(StaticFactory.class)) {
				
				if (Methods.isStaticFactory(m)) {
					
					method = m;
					
					break;
				}
			}
		}
		
		if (method == null) {
			
			for (Method m : cls.getMethods()) {
				
				if (Methods.isStaticFactory(m)) {
					
					method = m;
					
					break;
				}				
			}
		}
		
		T t = null;
		
		if (method != null) {
			
			method.setAccessible(true);
			
			try {
				
				t = cls.cast(method.invoke(null));
				
			} catch (IllegalArgumentException e) {
				throw new AssertionError(e);
			} catch (IllegalAccessException e) {
				throw new AssertionError(e);
			} catch (InvocationTargetException e) {
				throw new AssertionError(e);
			}
			
		} 
		
		if (method == null || t == null) {
			
			Constructor<? extends T> ctor = null;
			
			try {
				
				ctor = cls.getConstructor();
				
			} catch (NoSuchMethodException e) {
				
				String message = String.format(
						"must have either a public, static, parameterless" 
						+ " method or a public, parameterless default" 
						+ " constructor to return a non null instance" 
						+ " of %s", 
						cls.getName());
				
				throw new InvalidClassException(cls, message);
			}
			
			ctor.setAccessible(true);
			
			try {
				
				t = ctor.newInstance();
				
			} catch (IllegalArgumentException e) {
				throw new AssertionError(e);
			} catch (InstantiationException e) {
				throw new AssertionError(e);
			} catch (IllegalAccessException e) {
				throw new AssertionError(e);
			} catch (InvocationTargetException e) {
				throw new AssertionError(e);
			}
			
			if (cls.isAnnotationPresent(Singleton.class)) {
				
				classMap.put(cls, t);
			}

		}
		
		return t;
	}

	/**
	 * Constructs an instance of {@code Objects}. This constructor is 
	 * suppressed for noninstantiability outside of this class.
	 */
	private Objects() {
	}
}
