/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.climax.AbstractFactory;
import org.climax.ArgSeparators;
import org.climax.ArgsValidator;
import org.climax.Option;
import org.climax.OptionDocPropertyKeys;
import org.climax.OptionType;
import org.climax.OptionTypeDocPropertyKeys;


/**
 * An {@code OptionComplete} type. The constants of this enumerated type 
 * provide a simple classification of {@code OptionComplete} types.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
enum OptionCompleteType {
	
	/** An {@code OptionComplete} type of {@link OptionType#POSIX}. */
	POSIX(OptionType.POSIX) {

		@Override
		String getName(final String potentialName) {
			
			String name = Character.toString(Character.toLowerCase(
					potentialName.charAt(0)));
			
			String names = "aAbBcCdDeEfFgGhHiIjJkKlLmM" 
				+ "nNoOpPqQrRsStTuUvVwWxXyYzZ" 
				+ "0123456789"; 

			Set<Option> existingOptionSet =	ExistingOptionSet.get();
			
			int index = names.indexOf(name);	
			
			for (int i = index; i < names.length(); i++) {
				
				String anotherName = Character.toString(names.charAt(i));
				
				Option option = AbstractFactory.getDefault().newOption(
						this.getOptionType(), anotherName);
				
				if (!existingOptionSet.contains(option)) {
					
					name = anotherName;
					
					break;
				}
			}
			
			return name;
		}

		@Override
		String getUsage(
				final Type commonComponentType,
				final TypesType typesType,
				final Class<? extends ArgsValidator> argsValidatorClass, 
				final String argSeparator) {
			
			String optionDocOptionProperty = 
				"${" + OptionDocPropertyKeys.OPTION + "}";
			
			String optionUsage = optionDocOptionProperty;
			
			ArgsValidatorClassType argsValidatorClassType = 
					ArgsValidatorClassType.bestDescribes(argsValidatorClass);
			
			if (argsValidatorClassType == null
					|| argsValidatorClassType.equals(
							ArgsValidatorClassType.DUMMY_ARGS_VALIDATOR)) {
				
				switch (typesType) {
				case ARRAYTYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.PASSIVE_ARGS_VALIDATOR;
					break;
				case LISTTYPE:
					argsValidatorClassType =
						ArgsValidatorClassType.PASSIVE_ARGS_VALIDATOR;
					break;
				case NONE:
					argsValidatorClassType = 
						ArgsValidatorClassType.NO_ARGS_VALIDATOR;
					break;
				case TYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.SINGLE_ARG_VALIDATOR;
					break;
				case TYPE_ARRAYTYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.SINGLE_OR_MULTIPLE_ARGS_VALIDATOR;
					break;
				case TYPE_LISTTYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.SINGLE_OR_MULTIPLE_ARGS_VALIDATOR;
					break;
				default:
					throw new AssertionError(String.format(
							"Unknown %1$s: %2$s", 
							TypesType.class.getName(), 
							typesType));
				}
			}
			
			String argumentSyntax = 
					OptionCompleteType.getArgumentSyntax(commonComponentType);
			
			String argSep = argSeparator;
			
			if (argSep == null || argSep.isEmpty()) {
				argSep = ArgSeparators.NON_ARG_SEPARATOR;
			}
			
			switch(argsValidatorClassType) {
			case NO_ARGS_VALIDATOR:
				optionUsage = optionDocOptionProperty;
				break;
			case NONE_OR_SINGLE_ARG_VALIDATOR:
				optionUsage = String.format(
						"%1$s[ %2$s]", 
						optionDocOptionProperty,
						argumentSyntax);				
				break;
			case SINGLE_ARG_VALIDATOR:
				optionUsage = String.format(
						"%1$s %2$s", 
						optionDocOptionProperty,
						argumentSyntax);				
				break;
			case SINGLE_OR_MULTIPLE_ARGS_VALIDATOR:
				if (argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s %2$s [%1$s %2$s [...]]", 
							optionDocOptionProperty,
							argumentSyntax);
				}
				if (!argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s %2$s[%3$s%2$s[...]]", 
							optionDocOptionProperty,
							argumentSyntax,
							argSep);
				}
				break;
			case DISTINCT_ARGS_VALIDATOR:
			case PASSIVE_ARGS_VALIDATOR:
				if (argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s [%2$s] [%1$s [%2$s] [...]]", 
							optionDocOptionProperty,
							argumentSyntax);					
				}
				if (!argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s [%2$s[%3$s%2$s[...]]]", 
							optionDocOptionProperty,
							argumentSyntax,
							argSep);
				}				
				break;
			default:
				throw new AssertionError(String.format(
						"Unknown %1$s: %2$s", 
						ArgsValidatorClassType.class.getName(), 
						argsValidatorClassType));
			}
			
			return optionUsage;
		}
		
	},
	
	/** An {@code OptionComplete} type of {@link OptionType#GNU_SHORT_LONG}. */
	GNU_SHORT_LONG(OptionType.GNU_SHORT_LONG) {

		@Override
		String getName(final String potentialName) {
			return GNU_LONG.getName(potentialName);
		}

		@Override
		String getUsage(
				final Type commonComponentType,
				final TypesType typesType,
				final Class<? extends ArgsValidator> argsValidatorClass, 
				final String argSeparator) {

			return POSIX.getUsage(
					commonComponentType, 
					typesType, 
					argsValidatorClass, 
					argSeparator);
		}
		
	},
	
	/** An {@code OptionComplete} type of {@link OptionType#GNU_LONG}. */
	GNU_LONG(OptionType.GNU_LONG) {

		@Override
		String getName(final String potentialName) {
			
			Pattern pattern = Pattern.compile("[A-Za-z0-9][A-Z][a-z0-9]");

			Matcher matcher = pattern.matcher(potentialName);

			StringBuilder builder = new StringBuilder(potentialName);

			int offset = 0;

			while (matcher.find()) {

				int start = matcher.start() + offset;
				int end = matcher.end() + offset;

				String match = builder.substring(start, end);

				StringBuilder replacement = new StringBuilder(match);

				replacement.insert(1, "-");

				builder.replace(start, end, replacement.toString());

				offset++;
			}

			return builder.toString().toLowerCase();
		}

		@Override
		String getUsage(
				final Type commonComponentType,
				final TypesType typesType,
				final Class<? extends ArgsValidator> argsValidatorClass, 
				final String argSeparator) {
			
			String optionDocOptionProperty = 
				"${" + OptionDocPropertyKeys.OPTION + "}";
			
			String optionUsage = optionDocOptionProperty;
			
			ArgsValidatorClassType argsValidatorClassType = 
					ArgsValidatorClassType.bestDescribes(argsValidatorClass);
			
			if (argsValidatorClassType == null
					|| argsValidatorClassType.equals(
							ArgsValidatorClassType.DUMMY_ARGS_VALIDATOR)) {
				
				switch (typesType) {
				case ARRAYTYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.PASSIVE_ARGS_VALIDATOR;
					break;
				case LISTTYPE:
					argsValidatorClassType =
						ArgsValidatorClassType.PASSIVE_ARGS_VALIDATOR;
					break;
				case NONE:
					argsValidatorClassType = 
						ArgsValidatorClassType.NO_ARGS_VALIDATOR;
					break;
				case TYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.SINGLE_ARG_VALIDATOR;
					break;
				case TYPE_ARRAYTYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.SINGLE_OR_MULTIPLE_ARGS_VALIDATOR;
					break;
				case TYPE_LISTTYPE:
					argsValidatorClassType = 
						ArgsValidatorClassType.SINGLE_OR_MULTIPLE_ARGS_VALIDATOR;
					break;
				default:
					throw new AssertionError(String.format(
							"Unknown %1$s: %2$s", 
							TypesType.class.getName(), 
							typesType));
				}
			}
			
			String assignmentOperator =
				"${" + OptionTypeDocPropertyKeys.ASSIGNMENT_OPERATOR + "}";
			
			String argumentSyntax = 
					OptionCompleteType.getArgumentSyntax(commonComponentType);
			
			String argSep = argSeparator;
			
			if (argSep == null || argSep.isEmpty()) {
				argSep = ArgSeparators.NON_ARG_SEPARATOR;
			}

			switch(argsValidatorClassType) {
			case NO_ARGS_VALIDATOR:
				optionUsage = optionDocOptionProperty;
				break;
			case NONE_OR_SINGLE_ARG_VALIDATOR:
				optionUsage = String.format(
						"%1$s[%2$s%3$s]", 
						optionDocOptionProperty,
						assignmentOperator,
						argumentSyntax);				
				break;
			case SINGLE_ARG_VALIDATOR:
				optionUsage = String.format(
						"%1$s%2$s%3$s", 
						optionDocOptionProperty,
						assignmentOperator,
						argumentSyntax);				
				break;
			case SINGLE_OR_MULTIPLE_ARGS_VALIDATOR:
				if (argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s%2$s%3$s [%1$s%2$s%3$s [...]]", 
							optionDocOptionProperty,
							assignmentOperator,
							argumentSyntax);
				}
				if (!argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s%2$s%3$s[%4$s%3$s[...]]", 
							optionDocOptionProperty,
							assignmentOperator,
							argumentSyntax,
							argSep);
				}
				break;
			case DISTINCT_ARGS_VALIDATOR:
			case PASSIVE_ARGS_VALIDATOR:
				if (argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s[%2$s%3$s] [%1$s[%2$s%3$s] [...]]", 
							optionDocOptionProperty,
							assignmentOperator,
							argumentSyntax);					
				}
				if (!argSep.equals(ArgSeparators.NON_ARG_SEPARATOR)) {
					optionUsage = String.format(
							"%1$s[%2$s%3$s[%4$s%3$s[...]]]", 
							optionDocOptionProperty,
							assignmentOperator,
							argumentSyntax,
							argSep);
				}				
				break;
			default:
				throw new AssertionError(String.format(
						"Unknown %1$s: %2$s", 
						ArgsValidatorClassType.class.getName(), 
						argsValidatorClassType));
			}
			
			return optionUsage;
		}
		
	};
	
	/**
	 * Returns the argument syntax based on the provided {@code Class}.
	 * 
	 * @param cls the provided {@code Class}
	 * @return the argument syntax
	 */
	private static String getArgumentSyntax(final Class<?> cls) {
		
		String argumentSyntax = "STRING";
		
		ClassType classType = ClassType.bestDescribes(cls);
		
		if (classType == null) {
			return argumentSyntax;
		}
		
		switch (classType) {
		case BIG_DECIMAL:
			argumentSyntax = "BIG_DECIMAL";
			break;
		case BIG_INTEGER:
			argumentSyntax = "BIG_INTEGER";
			break;
		case BOOLEAN:
		case BOOLEAN_PRIMITIVE:
			argumentSyntax = String.format(
					"%1$s|%2$s", 
					Boolean.TRUE.toString().toUpperCase(), 
					Boolean.FALSE.toString().toUpperCase());
			break;
		case BYTE:
		case BYTE_PRIMITIVE:
			argumentSyntax = "BYTE";
			break;
		case CHARACTER:
		case CHARACTER_PRIMITIVE:
			argumentSyntax = "CHARACTER";
			break;
		case CLASS:
			argumentSyntax = "CLASS";
			break;
		case DATE:
			argumentSyntax = "DATE";
			break;
		case DOUBLE:
		case DOUBLE_PRIMITIVE:
			argumentSyntax = "DOUBLE";
			break;
		case ENUM:
			StringBuilder builder = new StringBuilder();
			
			Object[] enumConstants = cls.getEnumConstants();
			
			for (int i = 0; i < enumConstants.length; i++) {
			
				Object enumConstant = enumConstants[i];
				
				String enumConstantString = String.valueOf(enumConstant);
				
				builder.append(enumConstantString);
				
				if (i < enumConstants.length - 1) {
					
					builder.append('|');
				}
			}
			argumentSyntax = builder.toString();
			break;
		case FILE:
		case FILE_INPUT_STREAM:
		case FILE_OUTPUT_STREAM:
		case FILE_READER:
		case FILE_WRITER:
			argumentSyntax = "FILE";
			break;
		case FLOAT:
		case FLOAT_PRIMITIVE:
			argumentSyntax = "FLOAT";
			break;
		case INTEGER:
		case INTEGER_PRIMITIVE:
			argumentSyntax = "INTEGER";
			break;
		case LONG:
		case LONG_PRIMITIVE:
			argumentSyntax = "LONG";
			break;
		case SHORT:
		case SHORT_PRIMITIVE:
			argumentSyntax = "SHORT";
			break;
		default:
			break;
		}
		
		return argumentSyntax;		
	}
	
	/**
	 * Returns the argument syntax based on the provided {@code Type}.
	 * 
	 * @param type the provided {@code Type}
	 * @return the argument syntax
	 */
	private static String getArgumentSyntax(final Type type) {
		
		String argumentSyntax = "STRING";
		
		TypeType typeType = TypeType.bestDescribes(type);
		
		if (typeType == null) {
			return argumentSyntax;
		}
		
		switch (typeType) {
		case CLASS:
			Class<?> cls = (Class<?>) type;
			argumentSyntax = OptionCompleteType.getArgumentSyntax(cls);
			break;
		case PARAMETERIZED_TYPE:
			ParameterizedType parameterizedType = (ParameterizedType) type;
			Type rawType = parameterizedType.getRawType();
			argumentSyntax = OptionCompleteType.getArgumentSyntax(rawType);
			break;
		default:
			break;
		}
		
		return argumentSyntax;
	}
	
	/**
	 * Returns the enum constant of this type with the provided 
	 * {@code OptionType}.
	 * 
	 * @param optionType the provided {@code OptionType} of the enum constant 
	 * to be returned
	 * 
	 * @return the enum constant with the provided {@code OptionType}
	 */
	static OptionCompleteType of(final OptionType optionType) {
		
		for (OptionCompleteType optionCompleteType 
				: OptionCompleteType.values()) {
			
			if (optionCompleteType.getOptionType().equals(optionType)) {
				
				return optionCompleteType;
			}
		}
		
		return null;
	}
	
	/** The {@code OptionType}. */
	private final OptionType optionType;
	
	/**
	 * Constructs an {@code OptionCompleteType} with the provided 
	 * {@code OptionType}.
	 *  
	 * @param optType the provided {@code OptionType}
	 */
	private OptionCompleteType(final OptionType optType) {
		this.optionType = optType;
	}
	
	/**
	 * Returns the name for an {@code Option} based on this 
	 * {@code OptionCompleteType} and the provided potential name.
	 * 
	 * @param potentialName the provided potential name
	 * @return the name for an {@code Option}
	 */
	abstract String getName(final String potentialName);
	
	/**
	 * Returns the {@code OptionType}.
	 * 
	 * @return the {@code OptionType}
	 */
	OptionType getOptionType() {
		return this.optionType;
	}
	
	/**
	 * Returns the format string for the usage for an {@code Option} based on 
	 * this {@code OptionCompleteType}, the provided common component 
	 * {@code Type}, the provided {@code TypesType}, the provided 
	 * {@code ArgsValidator} class and the provided format string for the 
	 * argument separator.
	 * 
	 * @param commonComponentType the provided common component 
	 * {@code Type}
	 * 
	 * @param typesType the provided {@code TypesType}
	 * 
	 * @param argsValidatorClass the provided {@code ArgsValidator} class
	 *  (can be {@code null})
	 *  
	 * @param argSeparator the provided format string of the argument 
	 * separator (can be {@code null})
	 * 
	 * @return the format string for the usage for an {@code Option}
	 */
	abstract String getUsage(
			Type commonComponentType,
			TypesType typesType,
			final Class<? extends ArgsValidator> argsValidatorClass, 
			final String argSeparator);
}