/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import org.climax.OptionGroupComplete;


/**
 * A {@code Method} whose return type is {@code OptionGroupComplete}.
 * <p>
 * The underlying method represented by this 
 * {@code OptionGroupCompleteReturnTypeMethod} returns an 
 * {@code OptionGroupComplete}. The returned {@code OptionGroupComplete} is 
 * defined by the following:
 * </p>
 * <ul>
 * <li>The type argument of the {@code OptionGroupComplete} return type</li>
 * <li>The method's name</li>
 * <li>The method's optional {@code OptionGroupCompleteSpec} annotation</li>
 * <li>The provided {@code ParameterlessArgsValidator} class for the 
 * {@code OptionGroupCompleteSpec} annotation</li>
 * <li>The provided format string for the argument separator for the 
 * {@code OptionGroupCompleteSpec} annotation</li>
 * </ul>
 * <p>
 * The type argument of the {@code OptionGroupComplete} return type determines 
 * the appropriate default {@code ArgParser} for the 
 * {@code OptionGroupComplete}. The following lists the valid types for 
 * determining a default {@code ArgParser}:
 * </p>
 * <ul>
 * <li>{@code java.io.File}</li>
 * <li>{@code java.io.FileInputStream}</li>
 * <li>{@code java.io.FileOutputStream}</li>
 * <li>{@code java.io.FileReader}</li>
 * <li>{@code java.io.FileWriter}</li>
 * <li>{@code java.lang.Boolean}</li>
 * <li>{@code java.lang.Byte}</li>
 * <li>{@code java.lang.Character}</li>
 * <li>{@code java.lang.Class}</li>
 * <li>{@code java.lang.Double}</li>
 * <li>{@code java.lang.Enum} types</li>
 * <li>{@code java.lang.Float}</li>
 * <li>{@code java.lang.Integer}</li>
 * <li>{@code java.lang.Long}</li>
 * <li>{@code java.lang.Short}</li>
 * <li>{@code java.lang.String}</li>
 * <li>{@code java.lang.Void}<p>This type is a reserved type for indicating 
 * that the {@code OptionGroupComplete} will not accept an argument.</p></li>
 * <li>{@code java.math.BigDecimal}</li>
 * <li>{@code java.math.BigInteger}</li>
 * <li>{@code java.util.Date}</li>
 * </ul>
 * <p>
 * If the type argument of the {@code OptionGroupComplete} return type is not 
 * any of the aforementioned types and there is no provided custom 
 * {@code ParameterlessArgParser} class for the {@code OptionGroupCompleteSpec}
 * annotation, an {@code InvalidMethodException} is thrown.  
 * </p>
 * <p>
 * The type argument of the {@code OptionGroupComplete} return type also 
 * determines the default argument syntax for the usage of the 
 * {@code OptionComplete}(s).
 * </p>
 * <p>
 * The method's name determines the appropriate default name(s) for the 
 * {@code OptionComplete}(s) for the {@code OptionGroupComplete}. If the name 
 * of the method begins with "{@code get}" or "{@code is}", the rest of the 
 * name of the method is used. If the name of the method does not begin with 
 * "{@code get}" or "{@code is}", the entire name of the method is used. 
 * </p>
 * <p>
 * The method's optional {@code OptionGroupCompleteSpec} annotation is used to 
 * provide further specification of the {@code OptionGroupComplete}. It can 
 * also be used to override any of the defaults determined by the type argument
 * of the {@code OptionGroupComplete} return type, the method's name, and the 
 * provided {@code ParameterlessArgsValidator} class.
 * </p>
 * <p>
 * The provided {@code ParameterlessArgsValidator} class for the 
 * {@code OptionGroupCompleteSpec} annotation not only provides 
 * {@code ParameterlessArgsValidator} for the {@code OptionGroupComplete}, but 
 * it also determines the default format string for the usage of the 
 * {@code OptionComplete}(s). The following lists the 
 * {@code ParameterlessArgsValidator} classes that determines the default 
 * format string for the usage of the {@code OptionComplete}(s): 
 * </p>
 * <ul>
 * <li>{@code org.climax.argsvalidator.DistinctArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.NoArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.NoneOrSingleArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.PassiveArgsValidator}</li>
 * <li>{@code org.climax.argsvalidator.SingleArgValidator}</li>
 * <li>{@code org.climax.argsvalidator.SingleOrMultipleArgsValidator}</li>
 * </ul>
 * <p>
 * If a {@code ParameterlessArgsValidator} class has not been provided, a 
 * {@code SingleArgValidator} class is provided.
 * </p>
 * <p>
 * If the provided {@code ParameterlessArgsValidator} class is not any of the 
 * aforementioned classes, the default format string for the usage of the 
 * {@code OptionComplete}(s) is just the {@code String} representation of the 
 * {@code OptionComplete}(s) with no provided associative argument syntax(es).  
 * </p>
 * <p>
 * Depending on the provided {@code ParameterlessArgsValidator} class and the 
 * provided format string for the argument separator for the 
 * {@code OptionGroupCompleteSpec} annotation, the provided format string for 
 * the argument separator may also further determine the default format string 
 * for the usage of the {@code OptionComplete}(s). If the format string for the 
 * argument separator is not 
 * {@link org.climax.ArgSeparators#NON_ARG_SEPARATOR} and the provided 
 * {@code ParameterlessArgsValidator} class is either the 
 * {@code DistinctArgsValidator} class, the {@code PassiveArgsValidator} class,
 * or the {@code SingleOrMultipleArgsValidator} class, the default format 
 * string for the usage of the {@code OptionComplete}(s) will include use of 
 * the argument separator.   
 * </p>
 *  
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @see OptionGroupCompleteSpec
 */
public final class OptionGroupCompleteReturnTypeMethod {

	/** The {@code Method}. */
	private final Method method;
	
	/** The target {@code Type}. */
	private final Type targetType;
	
	/** The {@code OptionGroupCompleteSpec}. */
	private final OptionGroupCompleteSpec optionGroupCompleteSpec;
	
	/**
	 * Constructs an {@code OptionGroupCompleteReturnTypeMethod} with the 
	 * provided {@code Method}.
	 * 
	 * @param meth the provided {@code Method}
	 */
	OptionGroupCompleteReturnTypeMethod(final Method meth) {
		
		if (meth == null) {
			throw new NullPointerException("method must not be null");
		}
		
		Type methodReturnType = meth.getGenericReturnType();
		
		Type type = null;
		
		if (methodReturnType instanceof Class<?>) {
			
			Class<?> cls = (Class<?>) methodReturnType;
			
			if (cls == OptionGroupComplete.class) {
				
				type = Object.class;
			}
			
		} else if (methodReturnType instanceof ParameterizedType) {
		
			ParameterizedType parameterizedType = 
				(ParameterizedType) methodReturnType;
			
			Type rawType = parameterizedType.getRawType();
			
			if (rawType instanceof Class<?>) {
			
				Class<?> cls = (Class<?>) rawType;
				
				if (cls == OptionGroupComplete.class) {
					
					type = 
						parameterizedType.getActualTypeArguments()[0];
				}
			}
			
		}
		
		if (type == null) {
			
			String message = String.format(
					"return type must be %s", 
					OptionGroupComplete.class.getName());
			
			throw new InvalidMethodException(meth, message);			
		}
		
		OptionGroupCompleteSpec optGroupCompleteSpec = null;
		
		if (meth.isAnnotationPresent(OptionGroupCompleteSpec.class)) {
			
			optGroupCompleteSpec = meth.getAnnotation(
					OptionGroupCompleteSpec.class);
		}
		
		this.method = meth;
		this.targetType = type;
		this.optionGroupCompleteSpec = optGroupCompleteSpec;
	}
	
	/**
	 * Returns the {@code Method} view of this 
	 * {@code OptionGroupCompleteReturnTypeMethod}.
	 * 
	 * @return the {@code Method} view of this 
	 * {@code OptionGroupCompleteReturnTypeMethod}
	 */
	public Method asMethod() {
		return this.method;
	}
	
	/**
	 * Returns a new {@code OptionGroupComplete} based off of this 
	 * {@code OptionGroupCompleteReturnTypeMethod}.
	 * 
	 * @return a new {@code OptionGroupComplete} based off of this 
	 * {@code OptionGroupCompleteReturnTypeMethod}
	 */
	public OptionGroupComplete<Object> newOptionGroupComplete() {
		
		TypesType typesType = TypesType.TYPE;
		
		if (this.targetType == Void.class) {
			typesType = TypesType.NONE;
		}
		
		TypesBasedOptionGroupCompleteBuilder typesBasedOptionGroupCompleteBuilder =
				new TypesBasedOptionGroupCompleteBuilder(
						Methods.getAccessorSubjectName(this.method),
						this.targetType,
						typesType);
		
		if (this.optionGroupCompleteSpec != null) {

			typesBasedOptionGroupCompleteBuilder.optionGroupCompleteSpec(
					this.optionGroupCompleteSpec);
		}
		
		return typesBasedOptionGroupCompleteBuilder.build();
	}
	
	@Override
	public String toString() {
		return String.format("OptionGroupCompleteReturnTypeMethod [" 
				+ "method=%s]",
				this.method);
	}
}
