/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.climax.ArgSeparators;
import org.climax.argparser.ParameterlessArgParser;
import org.climax.argsvalidator.ParameterlessArgsValidator;


/**
 * Provides further specification of the {@code OptionGroupComplete}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 * 
 * @see OptionGroupCompleteReturnTypeMethod
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface OptionGroupCompleteSpec {
	
	/**
	 * The {@code ArgParser} class.
	 */
	Class<? extends ParameterlessArgParser<? extends Object>> argParserClass() 
		default DummyArgParser.class;
	
	/**
	 * The format string for the argument separator.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupBuilder#argSeparator(String)} method.
	 * </p> 
	 */
	String argSeparator() default ArgSeparators.NON_ARG_SEPARATOR;
	
	/**
	 * The {@code ArgsValidator} class. 
	 */
	Class<? extends ParameterlessArgsValidator> argsValidatorClass() 
		default DummyArgsValidator.class;
	
	/**
	 * The default argument format strings.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format strings are found in the 
	 * {@link org.climax.OptionGroupBuilder#defaultArguments(java.util.List)} 
	 * method.
	 * </p>
	 */	
	String[] defaultArguments() default { };
	
	/**
	 * The format string for the description of the group of {@code Option}s.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#description(String)} method.
	 * </p> 
	 */	
	String description() default "";
	
	/**
	 * The format string for the help information for the group of 
	 * {@code Option}s.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#help(String)} method. 
	 * </p>
	 */	
	String help() default "";
	
	/**
	 * The boolean value to indicate if the {@code OptionGroupDoc} is 
	 * excluded from documentation.
	 */	
	boolean hidden() default false;
	
	/**
	 * The {@code OptionCompleteSpec}s for the options.
	 */	
	OptionCompleteSpec[] options() default { };
	
	/**
	 * The boolean value to indicate if the {@code OptionGroup}'s
	 * {@code OptionSelection}s of type {@code String} terminates 
	 * the parsing of any other command line argument.
	 */	
	boolean optionSelectionsTerminative() default false;
	
	/**
	 * The ordinal of the {@code OptionGroupDoc} in relation to other 
	 * {@code OptionGroupDoc}s.
	 */	
	int ordinal() default 0;
	
	/**
	 * The boolean value to indicate if the {@code OptionGroup} is excluded 
	 * from parsing.
	 */	
	boolean unused() default false;
	
	/**
	 * The format string for the usage of the group of {@code Option}s.
	 * <p>
	 * The property keys that can be applied as expandable properties  
	 * within the provided format string are found in the 
	 * {@link org.climax.OptionGroupDocBuilder#usage(String)} method. 
	 * </p>
	 */	
	String usage() default "";
}
