/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.ref.WeakReference;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import org.climax.AbstractFactory;
import org.climax.OptionGroupComplete;
import org.climax.OptionsComplete;

/**
 * An interface {@code Class} of a specified type that is or extends 
 * {@code OptionsComplete}.
 * <p>
 * This {@code OptionsCompleteInterface} returns an instance of the specified 
 * type. The instance of the specified type is defined by the following:
 * </p>
 * <ul>
 * <li>The interface class's methods whose return type is 
 * {@code OptionGroupComplete}</li>
 * <li>The interface class's optional {@code OptionsCompleteSpec} annotation
 * </li>
 * </ul>
 * <p>
 * The interface class's methods whose return type is 
 * {@code OptionGroupComplete} each define an 
 * {@code OptionGroupCompleteReturnTypeMethod}. If one of the methods' return 
 * type is not {@code OptionGroupComplete}, an {@code InvalidMethodException} is
 * thrown.
 * </p>
 * <p>
 * The interface class's optional {@code OptionsCompleteSpec} annotation is 
 * used to provide further specification of the {@code OptionsComplete}.
 * </p>       
 * 
 * 
 * @author Climax Development (climax.development@gmail.com)
 *
 * @param <O> the specified type that is or extends {@code OptionsComplete}
 * 
 * @see OptionGroupCompleteReturnTypeMethod
 * 
 * @see OptionsCompleteSpec
 */
public final class OptionsCompleteInterface<O extends OptionsComplete> {
	
	/**
	 * Lazy initialization holder class idiom for static field for
	 * {@code ClassMapHolder.CLASS_MAP}.
	 */
	private static final class ClassMapHolder {
		
		/**
		 * A {@code Map} that maps existing {@code OptionsComplete} classes to 
		 * existing {@code OptionsCompleteInterface}s.
		 */
		static final Map<Class<? extends OptionsComplete>, 
			WeakReference<OptionsCompleteInterface<? extends OptionsComplete>>> 
		CLASS_MAP = new WeakHashMap<Class<? extends OptionsComplete>, 
			WeakReference<OptionsCompleteInterface<? extends OptionsComplete>>>();
		
		/**
		 * Constructs a {@code ClassMapHolder}. This constructor is suppressed 
		 * for noninstantiability outside of this class.
		 */
		private ClassMapHolder() {
		}
	}
	
	/**
	 * Returns an {@code OptionsCompleteInterface} of the specified type based 
	 * on the provided interface class.
	 * 
	 * @param <O> the specified type that extends {@code OptionsComplete}
	 * 
	 * @param intrface the provided interface class
	 * 
	 * @return an {@code OptionsCompleteInterface} of the specified type
	 */
	public static <O extends OptionsComplete> OptionsCompleteInterface<O> getInstance(
			final Class<O> intrface) {
		
		OptionsCompleteInterface<O> optionsCompleteInterface = null;
		
		Map<Class<? extends OptionsComplete>, 
			WeakReference<OptionsCompleteInterface<? extends OptionsComplete>>> 
		classMap = ClassMapHolder.CLASS_MAP;
		
		if (!classMap.containsKey(intrface)) {
			
			optionsCompleteInterface = new OptionsCompleteInterface<O>(
					intrface);
			
			WeakReference<OptionsCompleteInterface<? extends OptionsComplete>> 
			weakRef = new 
			WeakReference<OptionsCompleteInterface<? extends OptionsComplete>>(
					optionsCompleteInterface);
			
			classMap.put(intrface, weakRef);
			
		} else {
			
			@SuppressWarnings("unchecked")
			OptionsCompleteInterface<O> optsCompleteInterface = 
				(OptionsCompleteInterface<O>) classMap.get(intrface).get();
			
			optionsCompleteInterface = optsCompleteInterface;
		}
		
		return optionsCompleteInterface;
	}

	/** The interface class. */
	private final Class<O> intrface;
	
	/** The dynamic proxy class constructor. */
	private final Constructor<? extends O> proxyClassConstructor;
	
	/**
	 * The {@code List} of {@code Method}s from the class 
	 * {@code OptionsComplete}.
	 */
	private final List<Method> methodList;
	
	/** The {@code List} of {@code OptionGroupCompleteReturnTypeMethod}s. */
	private final List<OptionGroupCompleteReturnTypeMethod> optionGroupCompleteReturnTypeMethodList;
	
	/** The {@code OptionsCompleteSpec}. */
	private final OptionsCompleteSpec optionsCompleteSpec;
	
	/**
	 * Constructs an {@code OptionsCompleteInterface} with the provided 
	 * interface class.
	 * 
	 * @param i the provided interface class
	 */
	private OptionsCompleteInterface(final Class<O> i) {
		
		if (i == null) {
			
			throw new NullPointerException("class must not be null");
		}
		
		if (!i.isInterface()) {
			
			throw new IllegalArgumentException("class must be an interface");
		}
		
		
		if (!OptionsComplete.class.isAssignableFrom(i)) {
			
			String message = String.format(
					"class must extend %s", 
					OptionsComplete.class.getName());
			
			throw new IllegalArgumentException(message);
		}
		
		@SuppressWarnings("unchecked")
		Class<? extends O> proxyClass = 
			(Class<? extends O>) Proxy.getProxyClass(i.getClassLoader(), i);
		
		Constructor<? extends O> proxyClassCtor = null;
		
		try {
			
			proxyClassCtor = 
				proxyClass.getConstructor(InvocationHandler.class);
			
		} catch (SecurityException e) {
			throw new AssertionError(e);
		} catch (NoSuchMethodException e) {
			throw new AssertionError(e);
		}
		
		List<Method> methList = Arrays.asList(
				OptionsComplete.class.getMethods());

		List<OptionGroupCompleteReturnTypeMethod> optionGroupCompleteReturnTypeMethList = 
				new ArrayList<OptionGroupCompleteReturnTypeMethod>();
		
		Method[] methods = i.getMethods();

		Arrays.sort(methods, MethodComparator.getInstance());		
		
		for (Method method : methods) {
			
			if (methList.contains(method)) {
				
				continue;
			}
			
			Class<?> methodReturnType = method.getReturnType();
			
			if (methodReturnType == OptionGroupComplete.class) {
				
				OptionGroupCompleteReturnTypeMethod m =
					new OptionGroupCompleteReturnTypeMethod(method);
				
				optionGroupCompleteReturnTypeMethList.add(m);
				
				continue;
			}
			
			String message = String.format(
					"return type must be %s", 
					OptionGroupComplete.class.getName());
			
			throw new InvalidMethodException(method, message);			
		}
		
		OptionsCompleteSpec optsCompleteSpec = null;
		
		if (i.isAnnotationPresent(OptionsCompleteSpec.class)) {
			
			optsCompleteSpec = i.getAnnotation(OptionsCompleteSpec.class);
		}
		
		this.intrface = i;
		this.proxyClassConstructor = proxyClassCtor;
		this.methodList = methList;
		this.optionGroupCompleteReturnTypeMethodList = 
				optionGroupCompleteReturnTypeMethList;
		this.optionsCompleteSpec = optsCompleteSpec;
	}
	
	/**
	 * Returns the {@code Class} view of this {@code OptionsCompleteInterface}.
	 * 
	 * @return the {@code Class} view of this {@code OptionsCompleteInterface}
	 */
	public Class<O> asClass() {
		return this.intrface;
	}
	
	/**
	 * Returns an unmodifiable {@code List} of 
	 * {@code OptionGroupCompleteReturnTypeMethod}s.
	 * 
	 * @return an unmodifiable {@code List} of 
	 * {@code OptionGroupCompleteReturnTypeMethod}s
	 */
	public List<OptionGroupCompleteReturnTypeMethod> getOptionGroupCompleteReturnTypeMethods() {
		return Collections.unmodifiableList(
				this.optionGroupCompleteReturnTypeMethodList);
	}
	
	/**
	 * Returns a new instance of the specified type.
	 * 
	 * @return a new instance of the specified type
	 */
	public O newOptionsComplete() {
		
		OptionsComplete optionsComplete =
				AbstractFactory.getDefault().newOptionsComplete();
		
		Map<Method, InvocationHandler> methodMap = 
				new HashMap<Method, InvocationHandler>();
		
		ExistingOptionSet.clear();
		
		try {
			
			for (OptionGroupCompleteReturnTypeMethod optGrpCompleteReturnTypeMethod
					: this.optionGroupCompleteReturnTypeMethodList) {
				
				Method method = optGrpCompleteReturnTypeMethod.asMethod();
				
				OptionGroupComplete<Object> optionGroupComplete = 
						optGrpCompleteReturnTypeMethod.newOptionGroupComplete();
				
				optionsComplete.addOptionGroupComplete(optionGroupComplete);
				
				methodMap.put(
						method, 
						new ReturnOnlyInvocationHandler(optionGroupComplete));
			}
			
		} finally {
			
			ExistingOptionSet.clear();
		}
		
		for (Method method : this.methodList) {
			
			methodMap.put(method, new ObjectInvocationHandler(optionsComplete));
		}
		
		if (this.optionsCompleteSpec != null) {
			
			optionsComplete.setHelp(this.optionsCompleteSpec.help());
			optionsComplete.setUsage(this.optionsCompleteSpec.usage());
		}
		
		InvocationHandler handler = new ForwardingInvocationHandler(methodMap);
		
		O optsComplete = null;
		
		try {
			
			optsComplete = this.proxyClassConstructor.newInstance(handler);
			
		} catch (IllegalArgumentException e) {
			throw new AssertionError(e);
		} catch (InstantiationException e) {
			throw new AssertionError(e);
		} catch (IllegalAccessException e) {
			throw new AssertionError(e);
		} catch (InvocationTargetException e) {
			throw new AssertionError(e);
		}		
		
		return optsComplete;		
	}

	@Override
	public String toString() {
		return String.format("OptionsCompleteInterface [" 
				+ "interface=%s, " 
				+ "optionGroupCompleteReturnTypeMethods=%s]",
				this.intrface, 
				this.optionGroupCompleteReturnTypeMethodList);
	}
}
