/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.GenericArrayType;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;
import java.lang.reflect.WildcardType;


/**
 * A {@code Type} type. The constants of this enumerated type provides a 
 * classification of {@code Type}s.
 *  
 * @author Climax Development (climax.development@gmail.com)
 */
enum TypeType {

	/** {@code Class}. */
	CLASS {

		@Override 
		boolean describes(final Type type) {
			return type != null && type instanceof Class<?>;
		}
		
	},
	
	/** {@code GenericArrayType}. */
	GENERIC_ARRAY_TYPE {

		@Override 
		boolean describes(final Type type) {
			return type != null && type instanceof GenericArrayType;
		}

	},
	
	/** {@code ParameterizedType}. */
	PARAMETERIZED_TYPE {

		@Override 
		boolean describes(final Type type) {
			return type != null && type instanceof ParameterizedType;
		}

	},
	
	/** {@code TypeVariable}. */
	TYPE_VARIABLE {

		@Override 
		boolean describes(final Type type) {
			return type != null && type instanceof TypeVariable<?>;
		}

	},
	
	/** {@code WildcardType}. */
	WILDCARD_TYPE {

		@Override
		boolean describes(final Type type) {
			return type != null && type instanceof WildcardType;
		}

	};

	/**
	 * Returns the enum constant that describes the type of {@code Type} from
	 * the provided {@code Type}.
	 * 
	 * @param type the provided {@code Type}
	 * 
	 * @return an enum constant that describes the type of {@code Type} 
	 * from the provided {@code Type} or {@code null} if there is no enum 
	 * constant that describes the type of {@code Type} from the provided 
	 * {@code Type}
	 */
	static TypeType bestDescribes(final Type type) {
		
		for (TypeType typeType : TypeType.values()) {
			
			if (typeType.describes(type)) {
				
				return typeType;
			}
		}
		
		return null;
	}
	
	/**
	 * Indicates if the provided {@code Type} is described by this 
	 * {@code TypeType}.
	 * 
	 * @param type the provided {@code Type}
	 * 
	 * @return {@code true} if the provided {@code Type} is described by this 
	 * {@code TypeType} 
	 */
	abstract boolean describes(Type type);
}