/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Type;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.climax.AbstractFactory;
import org.climax.ArgAcceptor;
import org.climax.ArgGroupComplete;
import org.climax.ArgGroupCompleteBuilder;
import org.climax.ArgGroupDocPropertyKeys;
import org.climax.ArgParser;
import org.climax.ArgsValidator;
import org.climax.argacceptor.ParameterlessArgAcceptor;
import org.climax.argparser.ParameterlessArgParser;
import org.climax.argsvalidator.ParameterlessArgsValidator;

/**
 * A builder for an {@code ArgGroupComplete} based on the array of 
 * {@code Type}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class TypesBasedArgGroupCompleteBuilder {

	/**
	 * Returns the argument type based on the provided potential 
	 * argument type.
	 * 
	 * @param potentialArgumentType the provided potential argument 
	 * type
	 * 
	 * @return the argument type
	 */
	private static String getArgumentType(
			final String potentialArgumentType) {
		
		Pattern pattern = Pattern.compile("[A-Za-z0-9][A-Z][a-z0-9]");

		Matcher matcher = pattern.matcher(potentialArgumentType);

		StringBuilder builder = new StringBuilder(potentialArgumentType);

		int offset = 0;

		while (matcher.find()) {

			int start = matcher.start() + offset;
			int end = matcher.end() + offset;

			String match = builder.substring(start, end);

			StringBuilder replacement = new StringBuilder(match);

			replacement.insert(1, "_");

			builder.replace(start, end, replacement.toString());

			offset++;
		}

		return builder.toString().toUpperCase();		
	}
	
	/**
	 * Returns the format string of the syntax of an {@code ArgGroup} based 
	 * on the provided {@code TypesType} and the provided 
	 * {@code ArgsValidator}.
	 * 
	 * @param typesType the provided {@code TypesType}
	 * 
	 * @param argsValidatorClass the provided {@code ArgsValidator} class (can 
	 * be {@code null})
	 * 
	 * @return the format string of the syntax of an {@code ArgGroup}
	 */
	private static String getSyntax(
			final TypesType typesType,
			final Class<? extends ArgsValidator> argsValidatorClass) {

		String argGroupDocArgumentTypeProperty = 
			"${" + ArgGroupDocPropertyKeys.ARGUMENT_TYPE + "}";
		
		String argGroupSyntax = argGroupDocArgumentTypeProperty;
		
		ArgsValidatorClassType argsValidatorClassType = 
			ArgsValidatorClassType.bestDescribes(argsValidatorClass);
		
		if (argsValidatorClassType == null
				|| argsValidatorClassType.equals(
						ArgsValidatorClassType.DUMMY_ARGS_VALIDATOR)) {
			
			switch (typesType) {
			case ARRAYTYPE:
				argsValidatorClassType = 
					ArgsValidatorClassType.PASSIVE_ARGS_VALIDATOR;
				break;
			case LISTTYPE:
				argsValidatorClassType =
					ArgsValidatorClassType.PASSIVE_ARGS_VALIDATOR;
				break;
			case NONE:
				argsValidatorClassType = 
					ArgsValidatorClassType.NO_ARGS_VALIDATOR;
				break;
			case TYPE:
				argsValidatorClassType = 
					ArgsValidatorClassType.SINGLE_ARG_VALIDATOR;
				break;
			case TYPE_ARRAYTYPE:
				argsValidatorClassType = 
					ArgsValidatorClassType.SINGLE_OR_MULTIPLE_ARGS_VALIDATOR;
				break;
			case TYPE_LISTTYPE:
				argsValidatorClassType = 
					ArgsValidatorClassType.SINGLE_OR_MULTIPLE_ARGS_VALIDATOR;
				break;
			default:
				throw new AssertionError(String.format(
						"Unknown %1$s: %2$s", 
						TypesType.class.getName(), 
						typesType));
			}
		}
		
		switch(argsValidatorClassType) {
		case NO_ARGS_VALIDATOR:
			argGroupSyntax = "";
			break;
		case NONE_OR_SINGLE_ARG_VALIDATOR:
			argGroupSyntax = String.format(
					"[%s]", 
					argGroupDocArgumentTypeProperty);
			break;
		case SINGLE_ARG_VALIDATOR:
			argGroupSyntax = argGroupDocArgumentTypeProperty;
			break;
		case SINGLE_OR_MULTIPLE_ARGS_VALIDATOR:
			argGroupSyntax = String.format(
					"%s...", 
					argGroupDocArgumentTypeProperty);
			break;
		case DISTINCT_ARGS_VALIDATOR:
		case PASSIVE_ARGS_VALIDATOR:
			argGroupSyntax = String.format(
					"[%s...]", 
					argGroupDocArgumentTypeProperty);
			break;
		default:
			throw new AssertionError(String.format(
					"Unknown %1$s: %2$s", 
					ArgsValidatorClassType.class.getName(), 
					argsValidatorClassType));			
		}

		return argGroupSyntax;
	}
	
	/** The potential argument type. */
	private String potentialArgumentType;
	
	/** The common component {@code Type}. */
	private Type commonComponentType;
	
	/** The {@code TypesType}. */
	private TypesType typesType;
	
	/** The {@code ArgGroupCompleteSpec}. */
	private ArgGroupCompleteSpec argGroupCompleteSpec;
	
	/**
	 * Constructs an {@code TypesBasedOptionGroupCompleteBuilder} with the 
	 * provided potential argument type, the provided common component 
	 * {@code Type}, and the provided {@code TypesType}.
	 * 
	 * @param potentialArgType the provided potential argument type
	 * @param commonCompType the provided common component {@code Type}
	 * @param tType the provided {@code TypesType}
	 */
	TypesBasedArgGroupCompleteBuilder(
			final String potentialArgType, 
			final Type commonCompType, 
			final TypesType tType) {
		
		if (potentialArgType == null) {
			throw new NullPointerException(
					"potential argument type must not be null");
		}
		
		if (potentialArgType.isEmpty()) {
			throw new IllegalArgumentException(
					"potential argument type must not be empty");
		}
		
		if (commonCompType == null) {
			throw new NullPointerException(
					"common component type must not be null");
		}
		
		if (tType == null) {
			throw new NullPointerException(
					"TypesType must not be null");
		}
		
		this.potentialArgumentType = potentialArgType;
		this.commonComponentType = commonCompType;
		this.typesType = tType;
		this.argGroupCompleteSpec = null;		
	}
	
	/**
	 * Sets the {@code ArgGroupCompleteSpec} with the provided 
	 * {@code ArgGroupCompleteSpec}.
	 * 
	 * @param spec the provided {@code ArgGroupCompleteSpec}
	 * @return this builder
	 */
	TypesBasedArgGroupCompleteBuilder argGroupCompleteSpec(
			final ArgGroupCompleteSpec spec) {
		
		this.argGroupCompleteSpec = spec;
		return this;
	}
	
	/**
	 * Builds a new {@code ArgGroupComplete}.
	 * 
	 * @return a new {@code ArgGroupComplete}
	 */
	ArgGroupComplete<Object> build() {
		
		Class<? extends ParameterlessArgAcceptor> argAcceptorClass = 
				DummyArgAcceptor.class;
		
		Class<? extends ParameterlessArgParser<? extends Object>> argParserClass = 
				DummyArgParser.class;
		
		Class<? extends ParameterlessArgsValidator> argsValidatorClass = 
				DummyArgsValidator.class;
		
		String argumentType = "";
		
		String[] defaultArguments = new String[] { };
		
		boolean hidden = false;
		
		int ordinal = 0;
		
		String syntax = "";
		
		boolean unused = false;
		
		if (this.argGroupCompleteSpec != null) {
			
			argAcceptorClass = this.argGroupCompleteSpec.argAcceptorClass();
			argParserClass = this.argGroupCompleteSpec.argParserClass();
			argsValidatorClass = this.argGroupCompleteSpec.argsValidatorClass();
			argumentType = this.argGroupCompleteSpec.argumentType();
			defaultArguments = this.argGroupCompleteSpec.defaultArguments();
			hidden = this.argGroupCompleteSpec.hidden();
			ordinal = this.argGroupCompleteSpec.ordinal();
			syntax = this.argGroupCompleteSpec.syntax();
			unused = this.argGroupCompleteSpec.unused();
		}
		
		ArgAcceptor argAcceptor = ArgAcceptors.getInstance(argAcceptorClass);
		
		if (argAcceptor == null) {
			argAcceptor = ArgAcceptors.getDefault(this.typesType);
		}
		
		ArgParser<? extends Object> argParser = 
				ArgParsers.getInstance(argParserClass);
		
		if (argParser == null) {
			argParser = ArgParsers.getDefault(this.commonComponentType);
		}
		
		ArgsValidator argsValidator = 
				ArgsValidators.getInstance(argsValidatorClass);
		
		if (argsValidator == null) {
			argsValidator = ArgsValidators.getDefault(this.typesType);
		}
		
		if (argumentType.isEmpty()) {
			argumentType = TypesBasedArgGroupCompleteBuilder.getArgumentType(
					this.potentialArgumentType);
		}
		
		if (syntax.isEmpty()) {
			syntax = TypesBasedArgGroupCompleteBuilder.getSyntax(
					this.typesType, 
					argsValidatorClass);
		}
		
		AbstractFactory factory = AbstractFactory.getDefault();
		
		ArgGroupCompleteBuilder<Object> argGroupCompleteBuilder =
				factory.newArgGroupCompleteBuilder(
						argumentType, argParser);
		argGroupCompleteBuilder.argAcceptor(argAcceptor);
		argGroupCompleteBuilder.argsValidator(argsValidator);
		argGroupCompleteBuilder.defaultArguments(
				Arrays.asList(defaultArguments));
		argGroupCompleteBuilder.hidden(hidden);
		argGroupCompleteBuilder.ordinal(ordinal);
		argGroupCompleteBuilder.syntax(syntax);
		argGroupCompleteBuilder.unused(unused);
		
		ArgGroupComplete<Object> argGroupComplete = 
				argGroupCompleteBuilder.buildArgGroupComplete();
		
		return argGroupComplete;
	}
}
