/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Type;

import org.climax.AbstractFactory;
import org.climax.ArgSeparators;
import org.climax.OptionComplete;
import org.climax.OptionCompleteBuilder;
import org.climax.OptionType;
import org.climax.argsvalidator.ParameterlessArgsValidator;

/**
 * A builder for an {@code OptionComplete} based on the array of {@code Type}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class TypesBasedOptionCompleteBuilder {
	
	/** The potential name of the {@code Option}. */
	private String potentialOptionName;
	
	/** The common component {@code Type}. */
	private Type commonComponentType;
	
	/** The {@code TypesType}. */
	private TypesType typesType;
	
	/** The {@code OptionCompleteSpec}. */
	private OptionCompleteSpec optionCompleteSpec;
	
	/** The {@code OptionGroupCompleteSpec}. */
	private OptionGroupCompleteSpec optionGroupCompleteSpec;
	
	/**
	 * Constructs an {@code TypesBasedOptionCompleteBuilder} with the 
	 * provided potential name of the {@code Option}, the 
	 * provided common component {@code Type}, and the provided 
	 * {@code TypesType}.
	 * 
	 * @param potentialOptName the provided potential name of 
	 * the {@code Option}
	 * 
	 * @param commonCompType the provided common component {@code Type}
	 * 
	 * @param tType the provided {@code TypesType}
	 */
	TypesBasedOptionCompleteBuilder(
			final String potentialOptName, 
			final Type commonCompType, 
			final TypesType tType) {
		
		if (potentialOptName == null) {
			throw new NullPointerException(
					"potential option name must not be null");
		}
		
		if (potentialOptName.isEmpty()) {
			throw new IllegalArgumentException(
					"potential option name must not be empty");
		}
		
		if (commonCompType == null) {
			throw new NullPointerException(
					"common component type must not be null");
		}
		
		if (tType == null) {
			throw new NullPointerException(
					"TypesType must not be null");
		}
		
		this.potentialOptionName = potentialOptName;
		this.commonComponentType = commonCompType;
		this.typesType = tType;
		this.optionCompleteSpec = null;
		this.optionGroupCompleteSpec = null;
	}
	
	/**
	 * Builds a new {@code OptionComplete}.
	 * 
	 * @return a new {@code OptionComplete}
	 */
	OptionComplete buildOptionComplete() {
		
		String argSeparator = ArgSeparators.NON_ARG_SEPARATOR;
		
		Class<? extends ParameterlessArgsValidator> argsValidatorClass = 
				DummyArgsValidator.class;
		
		boolean hidden = false;
		
		String name = "";
		
		OptionType optionType = OptionType.GNU_LONG;
		
		int ordinal = 0;
		
		boolean unused = false;
		
		String usage = "";
		
		if (this.optionCompleteSpec != null) {
			
			hidden = this.optionCompleteSpec.hidden();
			name = this.optionCompleteSpec.name();
			optionType = this.optionCompleteSpec.optionType();
			ordinal = this.optionCompleteSpec.ordinal();
			unused = this.optionCompleteSpec.unused();
			usage = this.optionCompleteSpec.usage();
		}
		
		if (this.optionGroupCompleteSpec != null) {
			
			argSeparator = this.optionGroupCompleteSpec.argSeparator();
			argsValidatorClass = 
					this.optionGroupCompleteSpec.argsValidatorClass();
		}
		
		OptionCompleteType optionCompleteType = 
				OptionCompleteType.of(optionType);
		
		if (name.isEmpty()) {
			name = optionCompleteType.getName(this.potentialOptionName);
		}
		
		if (usage.isEmpty()) {
			
			usage = optionCompleteType.getUsage(
					this.commonComponentType, 
					this.typesType, 
					argsValidatorClass, 
					argSeparator);
		}
		
		AbstractFactory factory = AbstractFactory.getDefault();
		
		OptionCompleteBuilder optionCompleteBuilder = 
				factory.newOptionCompleteBuilder(optionType, name);
		optionCompleteBuilder.hidden(hidden);
		optionCompleteBuilder.ordinal(ordinal);
		optionCompleteBuilder.unused(unused);
		optionCompleteBuilder.usage(usage);
		
		OptionComplete optionComplete = 
				optionCompleteBuilder.buildOptionComplete();
		
		ExistingOptionSet.add(optionComplete);
		
		return optionComplete;
	}
	
	/**
	 * Sets the {@code OptionCompleteSpec} with the provided 
	 * {@code OptionCompleteSpec}.
	 * 
	 * @param spec the provided {@code OptionCompleteSpec}
	 * @return this builder
	 */
	TypesBasedOptionCompleteBuilder optionCompleteSpec(
			final OptionCompleteSpec spec) {
		
		this.optionCompleteSpec = spec;
		return this;
	}
	
	/**
	 * Sets the {@code OptionGroupCompleteSpec} with the provided 
	 * {@code OptionGroupCompleteSpec}.
	 * 
	 * @param spec the provided {@code OptionGroupCompleteSpec}
	 * @return this builder
	 */
	TypesBasedOptionCompleteBuilder optionGroupCompleteSpec(
			final OptionGroupCompleteSpec spec) {
		
		this.optionGroupCompleteSpec = spec;
		return this;
	}
	
}
