/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.climax.AbstractFactory;
import org.climax.ArgParser;
import org.climax.ArgSeparators;
import org.climax.ArgsValidator;
import org.climax.OptionComplete;
import org.climax.OptionGroupComplete;
import org.climax.OptionGroupCompleteBuilder;
import org.climax.argparser.ParameterlessArgParser;
import org.climax.argsvalidator.ParameterlessArgsValidator;

/**
 * A builder for an {@code OptionGroupComplete} based on the array of 
 * {@code Type}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
final class TypesBasedOptionGroupCompleteBuilder {
	
	/** The potential name of the {@code Option}(s). */
	private String potentialOptionName;
	
	/** The common component {@code Type}. */
	private Type commonComponentType;
	
	/** The {@code TypesType}. */
	private TypesType typesType;
	
	/** The {@code OptionGroupCompleteSpec}. */
	private OptionGroupCompleteSpec optionGroupCompleteSpec;

	/**
	 * Constructs an {@code TypesBasedOptionGroupCompleteBuilder} with the 
	 * provided potential name of the {@code Option}(s), the 
	 * provided common component {@code Type}, and the provided 
	 * {@code TypesType}.
	 * 
	 * @param potentialOptName the provided potential name of the 
	 * {@code Option}(s)
	 * 
	 * @param commonCompType the provided common component {@code Type}
	 * 
	 * @param tType the provided {@code TypesType}
	 */
	TypesBasedOptionGroupCompleteBuilder(
			final String potentialOptName, 
			final Type commonCompType, 
			final TypesType tType) {
		
		if (potentialOptName == null) {
			throw new NullPointerException(
					"potential option name must not be null");
		}
		
		if (potentialOptName.isEmpty()) {
			throw new IllegalArgumentException(
					"potential option name must not be empty");
		}
		
		if (commonCompType == null) {
			throw new NullPointerException(
					"common component type must not be null");
		}
		
		if (tType == null) {
			throw new NullPointerException(
					"TypesType must not be null");
		}
		
		this.potentialOptionName = potentialOptName;
		this.commonComponentType = commonCompType;
		this.typesType = tType;
		this.optionGroupCompleteSpec = null;
	}

	/**
	 * Builds a new {@code OptionGroupComplete}.
	 * 
	 * @return a new {@code OptionGroupComplete}
	 */
	OptionGroupComplete<Object> build() {
		
		Class<? extends ParameterlessArgParser<? extends Object>> argParserClass = 
				DummyArgParser.class;
		
		String argSeparator = ArgSeparators.NON_ARG_SEPARATOR;
		
		Class<? extends ParameterlessArgsValidator> argsValidatorClass = 
				DummyArgsValidator.class;
		
		String[] defaultArguments = new String[] { };
		
		String description = "";
			
		String help = "";
		
		boolean hidden = false;
		
		OptionCompleteSpec[] options = new OptionCompleteSpec[] { };
		
		boolean optionSelectionsTerminative = false;
		
		int ordinal = 0;
		
		boolean unused = false;
		
		String usage = "";
		
		if (this.optionGroupCompleteSpec != null) {
			
			argParserClass = this.optionGroupCompleteSpec.argParserClass();
			argSeparator = this.optionGroupCompleteSpec.argSeparator();
			argsValidatorClass = 
					this.optionGroupCompleteSpec.argsValidatorClass();
			defaultArguments = this.optionGroupCompleteSpec.defaultArguments();
			description = this.optionGroupCompleteSpec.description();
			help = this.optionGroupCompleteSpec.help();
			hidden = this.optionGroupCompleteSpec.hidden();
			options = this.optionGroupCompleteSpec.options();
			optionSelectionsTerminative = 
					this.optionGroupCompleteSpec.optionSelectionsTerminative();
			ordinal = this.optionGroupCompleteSpec.ordinal();
			unused = this.optionGroupCompleteSpec.unused();
			usage = this.optionGroupCompleteSpec.usage();
		}
		
		ArgParser<? extends Object> argParser = 
				ArgParsers.getInstance(argParserClass);
		
		if (argParser == null && !this.typesType.equals(TypesType.NONE)) {
			argParser = ArgParsers.getDefault(this.commonComponentType);
		}
		
		ArgsValidator argsValidator = 
				ArgsValidators.getInstance(argsValidatorClass);
		
		if (argsValidator == null && !this.typesType.equals(TypesType.NONE)) {
			argsValidator = ArgsValidators.getDefault(this.typesType);
		}
		
		List<OptionComplete> optionCompleteList = 
				new ArrayList<OptionComplete>();
		
		if (options.length == 0) {
			
			TypesBasedOptionCompleteBuilder typesBasedOptionCompleteBuilder = 
					new TypesBasedOptionCompleteBuilder(
							this.potentialOptionName, 
							this.commonComponentType, 
							this.typesType);
			typesBasedOptionCompleteBuilder.optionGroupCompleteSpec(
					this.optionGroupCompleteSpec);
			
			OptionComplete optionComplete = 
					typesBasedOptionCompleteBuilder.buildOptionComplete();
			
			optionCompleteList.add(optionComplete);
			
		} else {
			
			for (OptionCompleteSpec optionCompleteSpec : options) {
				
				TypesBasedOptionCompleteBuilder typesBasedOptionCompleteBuilder = 
						new TypesBasedOptionCompleteBuilder(
								this.potentialOptionName, 
								this.commonComponentType,
								this.typesType);
				typesBasedOptionCompleteBuilder.optionCompleteSpec(
						optionCompleteSpec);
				typesBasedOptionCompleteBuilder.optionGroupCompleteSpec(
						this.optionGroupCompleteSpec);
				
				OptionComplete optionComplete = 
						typesBasedOptionCompleteBuilder.buildOptionComplete();
				
				optionCompleteList.add(optionComplete);				
			}
		}
		
		AbstractFactory factory = AbstractFactory.getDefault();
		
		OptionGroupCompleteBuilder<Object> optionGroupCompleteBuilder = 
				factory.newOptionGroupCompleteBuilder(optionCompleteList);
		optionGroupCompleteBuilder.argParser(argParser);
		optionGroupCompleteBuilder.argSeparator(argSeparator);
		optionGroupCompleteBuilder.argsValidator(argsValidator);
		optionGroupCompleteBuilder.defaultArguments(
				Arrays.asList(defaultArguments));
		optionGroupCompleteBuilder.description(description);
		optionGroupCompleteBuilder.help(help);
		optionGroupCompleteBuilder.hidden(hidden);
		optionGroupCompleteBuilder.optionSelectionsTerminative(
				optionSelectionsTerminative);
		optionGroupCompleteBuilder.ordinal(ordinal);
		optionGroupCompleteBuilder.unused(unused);
		optionGroupCompleteBuilder.usage(usage);
		
		OptionGroupComplete<Object> optionGroupComplete = 
				optionGroupCompleteBuilder.buildOptionGroupComplete();
		
		return optionGroupComplete;
	}
	
	/**
	 * Sets the {@code OptionGroupCompleteSpec} with the provided 
	 * {@code OptionGroupCompleteSpec}.
	 * 
	 * @param spec the provided {@code OptionGroupCompleteSpec}
	 * @return this builder
	 */
	TypesBasedOptionGroupCompleteBuilder optionGroupCompleteSpec(
			final OptionGroupCompleteSpec spec) {
		
		this.optionGroupCompleteSpec = spec;
		return this;
	}
		
}
