/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax.reflect;

import java.lang.reflect.Array;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Arrays;
import java.util.List;

/**
 * A {@code Type}s type. The constants of this enumerated type provides a 
 * classification of types of {@code Type}s.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
enum TypesType {

	/** An array {@code Type}. */
	ARRAYTYPE {
		
		@Override
		Object[] convert(final Type commonComponentType, final Object[] args) {

			Class<?> cls = null;

			TypeType typeType = TypeType.bestDescribes(commonComponentType);
			
			if (typeType != null) {
				
				switch (typeType) {
				
				case CLASS:
					cls = (Class<?>) commonComponentType;
					break;
					
				case PARAMETERIZED_TYPE:
					
					ParameterizedType parameterizedType = 
						(ParameterizedType) commonComponentType;
					
					Type rawType = parameterizedType.getRawType();
					
					TypeType tType = TypeType.bestDescribes(rawType);
					
					if (tType != null) {
						
						switch (tType) {
						
						case CLASS:
							cls = (Class<?>) rawType;
							break;
							
						default:
							break;
						}
					}
					
					break;
					
				default:
					break;
				}
			}
			
			if (cls == null) {
			
				// can't figure out the class. ask the Objects if any.
				
				if (args.length > 0) {
					
					for (Object a : args) {
						
						if (a != null) {
							
							if (cls == null) {
								
								cls = a.getClass();
								
							} else {
								
								Class<?> c = a.getClass();
								
								if (c.isAssignableFrom(cls)) {
									
									cls = c;
								}
							}
						}
					}
				}
			}
			
			Object arg = null;
			
			if (cls != null) {
				
				arg = Array.newInstance(cls, args.length);
				
				for (int i = 0; i < args.length; i++) {
					
					Array.set(arg, i, args[i]);
				}
			}
			
			return new Object[] { arg };
		}

		@Override
		boolean describes(final Type[] types) {
			
			if (types == null) {
				return false;
			}
			
			if (types.length != 1) {
				return false;
			}
			
			Type type = types[0];
			
			TypeType typeType = TypeType.bestDescribes(type);
			
			if (!TypeType.CLASS.equals(typeType)
					&& !TypeType.GENERIC_ARRAY_TYPE.equals(typeType)) {
				
				return false;
			}
			
			if (TypeType.CLASS.equals(typeType)) {
				Class<?> cls = (Class<?>) type;
				if (!cls.isArray()) { return false; }
			}
			
			return true;
		}

		@Override
		Type getCommonComponentType(final Type[] types) {
			
			if (!this.describes(types)) { 
				throw new IllegalArgumentException("invalid types"); 
			}
			
			Type commonComponentType = null;
			
			Type type = types[0];
			
			TypeType typeType = TypeType.bestDescribes(type);
			
			if (TypeType.CLASS.equals(typeType)) {
				Class<?> cls = (Class<?>) type;
				commonComponentType = cls.getComponentType();
			}
			
			if (TypeType.GENERIC_ARRAY_TYPE.equals(typeType)) {
				GenericArrayType genericArrayType = (GenericArrayType) type;
				commonComponentType = 
						genericArrayType.getGenericComponentType();
			}
			
			return commonComponentType;
		}

		@Override
		String toTypesTypeString() {
			return "T[]";
		}
	},
	
	/** A {@code List} {@code Type}. */
	LISTTYPE {
		
		@Override
		Object[] convert(final Type commonComponentType, final Object[] args) {

			Object arg = Arrays.asList(args);
			
			return new Object[] { arg };
		}

		@Override
		boolean describes(final Type[] types) {
			
			if (types == null) {
				return false;
			}
			
			if (types.length != 1) {
				return false;
			}
			
			Type type = types[0];
			
			TypeType typeType = TypeType.bestDescribes(type);

			if (!TypeType.PARAMETERIZED_TYPE.equals(typeType)) {
				return false;
			}
			
			ParameterizedType parameterizedType = (ParameterizedType) type;
			
			Type rawType = parameterizedType.getRawType();
			
			if (!List.class.equals(rawType)) { return false; }
			
			return true;
		}

		@Override
		Type getCommonComponentType(final Type[] types) {
			
			if (!this.describes(types)) { 
				throw new IllegalArgumentException("invalid types"); 
			}
			
			ParameterizedType parameterizedType = (ParameterizedType) types[0];
			
			Type[] typeArguments = parameterizedType.getActualTypeArguments();
			
			return typeArguments[0];
		}

		@Override
		String toTypesTypeString() {
			return String.format("%s<T>", List.class.getName());
		}
	},
	
	/** No {@code Type}s. */
	NONE {
		
		@Override
		Object[] convert(final Type commonComponentType, final Object[] args) {
			return new Object[] { };
		}

		@Override
		boolean describes(final Type[] types) {
			return types != null && types.length == 0;
		}

		@Override
		Type getCommonComponentType(final Type[] types) {
			
			if (!this.describes(types)) { 
				throw new IllegalArgumentException("invalid types"); 
			}
			
			return null;
		}

		@Override
		String toTypesTypeString() {
			return "";
		}
	},
	
	/** A {@code Type}. */
	TYPE {
		
		@Override
		Object[] convert(final Type commonComponentType, final Object[] args) {
			
			Object arg = null;
			
			if (args.length > 0) {
				arg = args[0];
			}

			return new Object[] { arg };
		}

		@Override
		boolean describes(final Type[] types) {
			return !ARRAYTYPE.describes(types) 
					&& !LISTTYPE.describes(types)
					&& types != null
					&& types.length == 1;
		}

		@Override
		Type getCommonComponentType(final Type[] types) {
			
			if (!this.describes(types)) { 
				throw new IllegalArgumentException("invalid types"); 
			}
			
			return types[0];
		}

		@Override
		String toTypesTypeString() {
			return "T";
		}
	},
	
	/** A {@code Type} and an array {@code Type} of the same {@code Type}. */
	TYPE_ARRAYTYPE {
		
		@Override
		Object[] convert(final Type commonComponentType, final Object[] args) {

			Object firstArg = null;
			
			int remainingArgsStartIndex = 0;
			
			if (args.length > 0) {
				
				firstArg = args[0];
				
				remainingArgsStartIndex = 1;
			}
			
			Object[] remainingArgs = Arrays.copyOfRange(
					args, remainingArgsStartIndex, args.length);
			
			Object secondArg = ARRAYTYPE.convert(
					commonComponentType, remainingArgs)[0];

			return new Object[] { firstArg, secondArg };
		}

		@Override
		boolean describes(final Type[] types) {
			
			if (types == null) {
				return false;
			}

			if (types.length != 2) { 
				return false; 
			}
			
			Type firstType = types[0];
			
			Type secondType = types[1];
			
			TypesType typesType = TypesType.bestDescribes(
					new Type[] { secondType });
			
			if (!TypesType.ARRAYTYPE.equals(typesType)) {
				return false;
			}
			
			Type commonComponentType = 
					TypesType.ARRAYTYPE.getCommonComponentType(
							new Type[] { secondType });
			
			if (!firstType.equals(commonComponentType)) { return false; }
			
			return true;
		}

		@Override
		Type getCommonComponentType(final Type[] types) {
			
			if (!this.describes(types)) { 
				throw new IllegalArgumentException("invalid types"); 
			}
			
			return types[0];
		}

		@Override
		String toTypesTypeString() {
			return "T, T[]";
		}
	},
	
	/** 
	 * A {@code Type} and a {@code List} {@code Type} of the same {@code Type}. 
	 */
	TYPE_LISTTYPE {
		
		@Override
		Object[] convert(final Type commonComponentType, final Object[] args) {
			
			Object firstArg = null;
			
			int remainingArgsStartIndex = 0;
			
			if (args.length > 0) {
				
				firstArg = args[0];
				
				remainingArgsStartIndex = 1;
			}
			
			Object[] remainingArgs = Arrays.copyOfRange(
					args, remainingArgsStartIndex, args.length);
			
			Object secondArg = LISTTYPE.convert(
					commonComponentType, remainingArgs)[0];
			
			return new Object[] { firstArg, secondArg };
		}

		@Override
		boolean describes(final Type[] types) {
			
			if (types == null) {
				return false;
			}
			
			if (types.length != 2) { 
				return false; 
			}
			
			Type firstType = types[0];
			
			Type secondType = types[1];
			
			TypesType typesType = TypesType.bestDescribes(
					new Type[] { secondType });
			
			if (!TypesType.LISTTYPE.equals(typesType)) {
				return false;
			}
			
			Type commonComponentType = 
					TypesType.LISTTYPE.getCommonComponentType(
							new Type[] { secondType });
			
			if (!firstType.equals(commonComponentType)) { return false; }

			return true;
		}

		@Override
		Type getCommonComponentType(final Type[] types) {
			
			if (!this.describes(types)) { 
				throw new IllegalArgumentException("invalid types"); 
			}
			
			return types[0];
		}

		@Override
		String toTypesTypeString() {
			return String.format("T, %s<T>", List.class.getName());
		}
	};
	
	/**
	 * Returns the enum constant that describes the type of {@code Type}s 
	 * from the provided array of {@code Type}s. 
	 * 
	 * @param types the provided array of {@code Type} parameters
	 * 
	 * @return an enum constant that describes the type of {@code Type}s 
	 * from the provided array of {@code Type}s or {@code null} if there 
	 * is no enum constant that describes the type of {@code Type}s from 
	 * the provided array of {@code Type}s
	 */
	static TypesType bestDescribes(final Type[] types) {
		
		for (TypesType typesType : TypesType.values()) {
			
			if (typesType.describes(types)) {
				
				return typesType;
			}
		}
		
		return null;
	}
	
	/**
	 * Converts the provided {@code Object} array of arguments of the provided  
	 * common component {@code Type} into an {@code Object} array that 
	 * conforms to the array of {@code Type}s described by this 
	 * {@code TypesType}.
	 * 
	 * @param commonComponentType the provided common component {@code Type}
	 * 
	 * @param args the provided {@code Object} array of arguments of the  
	 * common component type
	 * 
	 * @return an {@code Object} array that conforms to the array of 
	 * {@code Type}s described by this {@code TypesType}
	 */
	abstract Object[] convert(
			final Type commonComponentType, final Object[] args);
	
	/**
	 * Indicates if the provided array of {@code Type}s is described by this 
	 * {@code TypesType}.
	 * 
	 * @param types the provided array of {@code Type}s
	 * 
	 * @return {@code true} if the provided array of {@code Type}s is 
	 * described by this {@code TypesType}
	 */
	abstract boolean describes(final Type[] types);
	
	/**
	 * Returns the common component {@code Type} based on the provided array of 
	 * {@code Type}s and this {@code TypesType}.
	 * 
	 * @param types the provided array of {@code Type}s
	 * 
	 * @return the common component {@code Type}
	 */
	abstract Type getCommonComponentType(final Type[] types);
	
	/**
	 * Returns the {@code String} representation of the {@code Type}s in 
	 * generic form described by this {@code TypesType}.
	 * 
	 * @return the {@code String} representation of the {@code Type}s in 
	 * generic form described by this {@code TypesType}
	 */
	abstract String toTypesTypeString();
}
