/*
 * Climax - A Java command line argument parser framework
 * Copyright (c) 2010-2013 Climax contributors
 * See the file LICENSE.txt for copying permission.
 */

package org.climax;

import static org.junit.Assert.assertEquals;

import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * JUnit Test Case for {@code StringFormatter}.
 * 
 * @author Climax Development (climax.development@gmail.com)
 */
public class StringFormatterTest {

	/** */
	private Map<String, String> map;
	
	/** */
	@Before
	public final void setUp() {
		
		this.map = new HashMap<String, String>();
		// boolean
		this.map.put("true.booleanvalue", "true");
		// byte
		this.map.put("byte.minvalue", "-128");
		this.map.put("byte.maxvalue", "127");
		// char
		this.map.put("char.minvalue", "\u0000");
		this.map.put("char.maxvalue", "\uFFFF");
		this.map.put("null", "\0");
		this.map.put("backslash", "\\");
		this.map.put("semicolon", ";");
		this.map.put("comma", ",");
		// date
		this.map.put("today", "now");
		this.map.put("date1", "20110302");
		this.map.put("date2", "201103021304");
		this.map.put("date3", "201103021304957-0500");
		// double
		this.map.put("double.minvalue", "2.2250738585072014E-308");
		this.map.put("double.maxvalue", "1.7976931348623157E308");
		this.map.put("e", "2.718281828459045");
		this.map.put("pi", "3.141592653589793");
		// float
		this.map.put("float.minvalue", "1.401298464324817E-45");
		this.map.put("float.maxvalue", "3.4028234663852886E38");
		// int
		this.map.put("int.minvalue", "-2147483648");
		this.map.put("int.maxvalue", "2147483647");
		this.map.put("bigquestion.answer", "42");
		// long
		this.map.put("long.minvalue", "-9223372036854775808");
		this.map.put("long.maxvalue", "9223372036854775807");
		// short
		this.map.put("short.minvalue", "-32768");
		this.map.put("short.maxvalue", "32767");
		// string
		this.map.put("greeting", "hello");
		this.map.put("valediction", "goodbye");
	}
	
	/** */
	@After
	public final void tearDown() {
		this.map = null;
	}
	
	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat01() {
		
		String string = "format:%4$2s %3$2s %2$2s %1$2s;v=a;v=b;v=c;v=d";
		String expected = " d  c  b  a";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat02() {
		
		String string = "lformat:fr_FR;e = %+10.4f;c=double,p=e";
		String expected = "e =    +2,7183";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat03() {
		
		String string = "format:format";
		String expected = "format";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat04() {
		
		String string = "format:Alice\\;Bob\\;Jane";
		String expected = "Alice;Bob;Jane";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat05() {
		
		String string = 
			"format:A backslash (\\) and a double backslash (\\\\\\\\)";
		String expected = "A backslash (\\) and a double backslash (\\\\)";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat06() {
		
		String string = 
			"format:A backslash (%1$s) and a double backslash (%2$s);" 
			+ "v=\\\\;v=\\\\\\\\\\\\\\\\";
		String expected = "A backslash (\\) and a double backslash (\\\\)";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat07() {
		
		String string = 
			"format:A comma (%1$s) and a semicolon (%2$s);v=\\,;v=\\;";
		String expected = "A comma (,) and a semicolon (;)";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat08() {
		
		String string = 
			"format:The answer to the big question is %d;"
			+ "c=byte,p=bigquestion.answer";
		String expected = "The answer to the big question is 42";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat09() {
		
		String string = "format:Pi is exactly %.0f!;c=double,p=pi";
		String expected = "Pi is exactly 3!";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat10() {
		
		String string = "lformat:en_US;%1$tB %1$te, %1$tY;c=date,p=date1";
		String expected = "March 2, 2011";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat11() {
		
		String string = "format:%1$tY/%1$tm/%1$td;c=date,p=today";
		String expected = new java.text.SimpleDateFormat("yyyy/MM/dd")
			.format(new java.util.Date());
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test
	public final void testFormat12() {
		
		String string = "format:You say %1$s and I say %2$s;"
			+ "p=valediction;p=greeting";
		String expected = "You say goodbye and I say hello";
		assertEquals(
				expected, 
				StringFormatter.format(string, this.map));

	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException01() {
		StringFormatter.format("faux", this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException02() {
		StringFormatter.format("faux:foofoo", this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException03() {
		StringFormatter.format("lformat:foofoo", this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException04() {
		StringFormatter.format("lformat:fr_FR", this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException05() {
		StringFormatter.format(
				"format:%1$s %2$s %3$s %4$s;v=You;v=are;foofoo;v=years", 
				this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException06() {
		StringFormatter.format(
				"format:%1$s %2$s %3$s %4$s;v=You;v=are;c=foofoo;v=years", 
				this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException07() {
		StringFormatter.format(
				"format:%1$s %2$s %3$s %4$s;v=You;v=are;c=int;v=years", 
				this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException08() {
		StringFormatter.format(
				"format:%1$s %2$s %3$s %4$s;v=You;v=are;p=foo;v=years", 
				this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testFormatForIllegalArgumentException09() {
		StringFormatter.format(
				"format:%1$s %2$s %3$s %4$s;v=You;v=are;c=int,v=foo;v=years", 
				this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = NullPointerException.class)
	public final void testFormatForNullPointerException01() {
		StringFormatter.format(null, this.map);
	}

	/**
	 * Test method for {@link 
	 * org.climax.StringFormatter#format(java.lang.String, java.util.Map)}.
	 */
	@Test(expected = NullPointerException.class)
	public final void testFormatForNullPointerException02() {
		StringFormatter.format("format:format", null);
	}

}
